@@ -74,7 +74,6 @@
 #include "nsIAtom.h"
 #include "nsHTMLAtoms.h"
 #include "nsLayoutAtoms.h"
-#include "nsINameSpaceManager.h"
 #include "nsGUIEvent.h"
 
 #include "nsILink.h"
@@ -97,7 +96,8 @@
 #include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
 #include "nsString.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 
 // IFrame Helpers
 #include "nsIDocShell.h"
@@ -112,8 +112,6 @@
 #include "nsIDOMCharacterData.h"
 #endif
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
 //#define DEBUG_LEAKS
 
 PRUint32 nsAccessible::gInstanceCount = 0;
@@ -132,6 +130,9 @@ nsAccessibleTreeWalker::nsAccessibleTreeWalker(nsIWeakReference* aPresShell, nsI
 
   if (mState.siblingIndex < 0)
     mState.siblingList = nsnull;
+  NS_ASSERTION(mState.siblingList || mState.siblingIndex < 0,
+    "Accessible tree walker initialization error, "
+    "can't have index into null sibling list");
 
   if (aWalkAnonContent) {
     nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
@@ -142,6 +143,7 @@ nsAccessibleTreeWalker::nsAccessibleTreeWalker(nsIWeakReference* aPresShell, nsI
     }
   }
   MOZ_COUNT_CTOR(nsAccessibleTreeWalker);
+  mInitialState = mState;  // deep copy
 }
 
 nsAccessibleTreeWalker::~nsAccessibleTreeWalker()
@@ -205,6 +207,8 @@ void nsAccessibleTreeWalker::GetSiblings(nsIDOMNode *aOneOfTheSiblings)
   if (NS_SUCCEEDED(GetFullTreeParentNode(aOneOfTheSiblings, getter_AddRefs(node)))) {
     GetKids(node);
     if (mState.siblingList) {      // Init index by seeing how far we are into list
+      if (mState.domNode == mInitialState.domNode)
+        mInitialState = mState; // deep copy, we'll use sibling info for caching
       while (NS_SUCCEEDED(mState.siblingList->Item(mState.siblingIndex, getter_AddRefs(node))) && node != mState.domNode) {
         NS_ASSERTION(node, "Something is terribly wrong - the child is not in it's parent's children!");
         ++mState.siblingIndex;
@@ -431,7 +435,7 @@ PRBool nsAccessibleTreeWalker::GetAccessible()
 //-----------------------------------------------------
 nsAccessible::nsAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell): mDOMNode(aNode), mPresShell(aShell), mSiblingIndex(eSiblingsUninitialized)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
 #ifdef NS_DEBUG_X
    {
@@ -492,6 +496,30 @@ NS_IMETHODIMP nsAccessible::GetAccName(nsAString& _retval)
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP nsAccessible::GetAccDescription(nsAString& aDescription)
+{
+  // There are 3 conditions that make an accessible have no accDescription:
+  // 1. it's a text node; or
+  // 2. it doesn't have an accName; or
+  // 3. its title attribute equals to its accName nsAutoString name; 
+  nsCOMPtr<nsITextContent> textContent(do_QueryInterface(mDOMNode));
+  if (!textContent) {
+    nsAutoString name;
+    GetAccName(name);
+    if (!name.IsEmpty()) {
+      // If there's already a name, we'll expose a description.if it's different than the name
+      // If there is no name, then we know the title should really be exposed there
+      nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(mDOMNode));
+      if (elt)
+        elt->GetAttribute(NS_LITERAL_STRING("title"), aDescription);
+      if (!elt || aDescription == name)
+        aDescription.Truncate();
+    }
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAccessible::GetAccKeyboardShortcut(nsAString& _retval)
 {
   static PRInt32 gGeneralAccesskeyModifier = -1;  // magic value of -1 indicates unitialized state
@@ -503,12 +531,12 @@ NS_IMETHODIMP nsAccessible::GetAccKeyboardShortcut(nsAString& _retval)
     if (accesskey.IsEmpty())
       return NS_OK;
 
-    if (gGeneralAccesskeyModifier == -1) {  // Need to initialize cached global accesskey pref
+    if (gGeneralAccesskeyModifier == -1) {
+      // Need to initialize cached global accesskey pref
       gGeneralAccesskeyModifier = 0;
-      nsresult result;
-      nsCOMPtr<nsIPref> prefService(do_GetService(kPrefCID, &result));
-      if (NS_SUCCEEDED(result) && prefService)
-        prefService->GetIntPref("ui.key.generalAccessKey", &gGeneralAccesskeyModifier);
+      nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+      if (prefBranch)
+        prefBranch->GetIntPref("ui.key.generalAccessKey", &gGeneralAccesskeyModifier);
     }
     nsAutoString propertyKey;
     switch (gGeneralAccesskeyModifier) {
@@ -527,7 +555,7 @@ NS_IMETHODIMP nsAccessible::GetAccKeyboardShortcut(nsAString& _retval)
 
 NS_IMETHODIMP nsAccessible::GetAccId(PRInt32 *aAccId)
 {
-  *aAccId = - NS_REINTERPRET_CAST(PRInt32, (mDOMNode.get()));
+  *aAccId = - NS_PTR_TO_INT32(mDOMNode.get());
   return NS_OK;
 }
 
@@ -571,8 +599,12 @@ NS_IMETHODIMP nsAccessible::GetAccNextSibling(nsIAccessible * *aAccNextSibling)
   if (NS_SUCCEEDED(walker.GetNextSibling())) {
     *aAccNextSibling = walker.mState.accessible;
     NS_ADDREF(*aAccNextSibling);
-    mSiblingList = walker.mState.siblingList;
-    (*aAccNextSibling)->CacheOptimizations(mParent, walker.mState.siblingIndex, mSiblingList);
+    // Use last walker state to cache data on next accessible
+    (*aAccNextSibling)->CacheOptimizations(mParent, walker.mState.siblingIndex, 
+                                           walker.mState.siblingList);
+    // Use first walker state to cache data on this accessible
+    CacheOptimizations(mParent, walker.mInitialState.siblingIndex,
+                       walker.mInitialState.siblingList);
   }
 
   return NS_OK;  
@@ -588,8 +620,12 @@ NS_IMETHODIMP nsAccessible::GetAccPreviousSibling(nsIAccessible * *aAccPreviousS
   if (NS_SUCCEEDED(walker.GetPreviousSibling())) {
     *aAccPreviousSibling = walker.mState.accessible;
     NS_ADDREF(*aAccPreviousSibling);
-    mSiblingList = walker.mState.siblingList;
-    (*aAccPreviousSibling)->CacheOptimizations(mParent, walker.mState.siblingIndex, mSiblingList);
+    // Use last walker state to cache data on prev accessible
+    (*aAccPreviousSibling)->CacheOptimizations(mParent, walker.mState.siblingIndex, 
+                                           walker.mState.siblingList);
+    // Use first walker state to cache data on this accessible
+    CacheOptimizations(mParent, walker.mInitialState.siblingIndex,
+                       walker.mInitialState.siblingList);
   }
 
   return NS_OK;  
@@ -662,11 +698,13 @@ nsresult nsAccessible::GetFullKeyName(const nsAString& aModifierName, const nsAS
   return NS_OK;
 }
 
-PRBool nsAccessible::IsPartiallyVisible() 
+PRBool nsAccessible::IsPartiallyVisible(PRBool *aIsOffscreen) 
 {
   // We need to know if at least a kMinPixels around the object is visible
   // Otherwise it will be marked STATE_OFFSCREEN and STATE_INVISIBLE
   
+  *aIsOffscreen = PR_FALSE;
+
   const PRUint16 kMinPixels  = 12;
 
   // Set up the variables we need, return false if we can't get at them all
@@ -688,6 +726,16 @@ PRBool nsAccessible::IsPartiallyVisible()
   if (!frame) 
     return PR_FALSE;
 
+  // If visibility:hidden or visibility:collapsed then mark with STATE_INVISIBLE
+  nsCOMPtr<nsIStyleContext> styleContext;
+  frame->GetStyleContext(getter_AddRefs(styleContext));
+  if (styleContext) {
+    const nsStyleVisibility* vis = 
+      (const nsStyleVisibility*)styleContext->GetStyleData(eStyleStruct_Visibility);
+    if (!vis || !vis->IsVisible())
+      return PR_FALSE;
+  }
+
   nsCOMPtr<nsIPresContext> presContext;
   shell->GetPresContext(getter_AddRefs(presContext));
   if (!presContext)
@@ -711,13 +759,19 @@ PRBool nsAccessible::IsPartiallyVisible()
 
   float p2t;
   presContext->GetPixelsToTwips(&p2t);
-  PRBool isVisible = PR_FALSE;
-  viewManager->IsRectVisible(containingView, relFrameRect, NS_STATIC_CAST(PRUint16, (kMinPixels * p2t)), &isVisible);
+  nsRectVisibility rectVisibility;
+  viewManager->GetRectVisibility(containingView, relFrameRect, 
+                                 NS_STATIC_CAST(PRUint16, (kMinPixels * p2t)), 
+                                 &rectVisibility);
 
-  return isVisible;
+  if (rectVisibility == nsRectVisibility_kVisible)
+    return PR_TRUE;
+
+  *aIsOffscreen = PR_TRUE;
+  return PR_FALSE;
 }
 
-NS_IMETHODIMP nsAccessible::GetFocusedElement(nsIDOMElement **aFocusedElement) 
+NS_IMETHODIMP nsAccessible::GetFocusedNode(nsIDOMNode **aFocusedNode) 
 {
   nsCOMPtr<nsIFocusController> focusController;
   nsCOMPtr<nsIDocument> document;
@@ -725,6 +779,8 @@ NS_IMETHODIMP nsAccessible::GetFocusedElement(nsIDOMElement **aFocusedElement)
   if (content)
     content->GetDocument(*getter_AddRefs(document));
 
+  if (!document)
+    document = do_QueryInterface(mDOMNode);
   if (document) {
     nsCOMPtr<nsIScriptGlobalObject> ourGlobal;
     document->GetScriptGlobalObject(getter_AddRefs(ourGlobal));
@@ -734,11 +790,24 @@ NS_IMETHODIMP nsAccessible::GetFocusedElement(nsIDOMElement **aFocusedElement)
   }
 
   if (focusController) {
+    nsCOMPtr<nsIDOMNode> focusedNode;
     nsCOMPtr<nsIDOMElement> focusedElement; 
     focusController->GetFocusedElement(getter_AddRefs(focusedElement));
-    *aFocusedElement = focusedElement;
-    if (focusedElement) {
-      NS_ADDREF(*aFocusedElement);
+    if (!focusedElement) {
+      nsCOMPtr<nsIDOMWindowInternal> windowInternal;
+      focusController->GetFocusedWindow(getter_AddRefs(windowInternal));
+      nsCOMPtr<nsIDOMWindow> window(do_QueryInterface(windowInternal));
+      if (window) {
+        nsCOMPtr<nsIDOMDocument> domDoc;
+        window->GetDocument(getter_AddRefs(domDoc));
+        focusedNode = do_QueryInterface(domDoc);
+      }
+    }
+    else
+      focusedNode = do_QueryInterface(focusedElement);
+    if (focusedNode) {
+      *aFocusedNode = focusedNode;
+      NS_ADDREF(*aFocusedNode);
       return NS_OK;
     }
   }
@@ -763,15 +832,19 @@ NS_IMETHODIMP nsAccessible::GetAccState(PRUint32 *aAccState)
       *aAccState |= STATE_UNAVAILABLE;
     else { 
       *aAccState |= STATE_FOCUSABLE;
-      nsCOMPtr<nsIDOMElement> focusedElement;
-      if (NS_SUCCEEDED(GetFocusedElement(getter_AddRefs(focusedElement))) && focusedElement == currElement)
+      nsCOMPtr<nsIDOMNode> focusedNode;
+      if (NS_SUCCEEDED(GetFocusedNode(getter_AddRefs(focusedNode))) && focusedNode == mDOMNode)
         *aAccState |= STATE_FOCUSED;
     }
   }
 
   // Check if STATE_OFFSCREEN bitflag should be turned on for this object
-  if (!IsPartiallyVisible())
-    *aAccState |= STATE_OFFSCREEN | STATE_INVISIBLE;
+  PRBool isOffscreen;
+  if (!IsPartiallyVisible(&isOffscreen)) {
+    *aAccState |= STATE_INVISIBLE;
+    if (isOffscreen)
+      *aAccState |= STATE_OFFSCREEN;
+  }
 
   return rv;
 }
@@ -783,9 +856,8 @@ NS_IMETHODIMP nsAccessible::GetAccFocused(nsIAccessible **aAccFocused)
 
   nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
 
-  nsCOMPtr<nsIDOMElement> focusedElement;
-  if (accService && NS_SUCCEEDED(GetFocusedElement(getter_AddRefs(focusedElement)))) {
-    nsCOMPtr<nsIDOMNode> focusedNode(do_QueryInterface(focusedElement));
+  nsCOMPtr<nsIDOMNode> focusedNode;
+  if (accService && NS_SUCCEEDED(GetFocusedNode(getter_AddRefs(focusedNode)))) {
     nsCOMPtr<nsIAccessible> accessible;
     if (NS_SUCCEEDED(accService->GetAccessibleFor(focusedNode, getter_AddRefs(accessible)))) {
       *aAccFocused = accessible;
@@ -1141,10 +1213,9 @@ NS_IMETHODIMP nsAccessible::AccRemoveSelection()
 NS_IMETHODIMP nsAccessible::AccTakeSelection()
 {
   nsCOMPtr<nsISelectionController> control(do_QueryReferent(mPresShell));
- if (!control) {
-     return NS_ERROR_FAILURE;  
-  }
-
+  if (!control)
+    return NS_ERROR_FAILURE;  
+ 
   nsCOMPtr<nsISelection> selection;
   nsresult rv = control->GetSelection(nsISelectionController::SELECTION_NORMAL, getter_AddRefs(selection));
   if (NS_FAILED(rv))
@@ -1189,9 +1260,8 @@ NS_IMETHODIMP nsAccessible::AccTakeSelection()
 NS_IMETHODIMP nsAccessible::AccTakeFocus()
 { 
   nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-  if (!shell) {
-     return NS_ERROR_FAILURE;  
-  }
+  if (!shell)
+    return NS_ERROR_FAILURE;  
 
   nsCOMPtr<nsIPresContext> context;
   shell->GetPresContext(getter_AddRefs(context));
@@ -1205,7 +1275,8 @@ NS_IMETHODIMP nsAccessible::AppendStringWithSpaces(nsAString *aFlatString, const
 {
   // Insert spaces to insure that words from controls aren't jammed together
   if (!textEquivalent.IsEmpty()) {
-    aFlatString->Append(NS_LITERAL_STRING(" "));
+    if (!aFlatString->IsEmpty())
+      aFlatString->Append(NS_LITERAL_STRING(" "));
     aFlatString->Append(textEquivalent);
     aFlatString->Append(NS_LITERAL_STRING(" "));
   }
@@ -1231,6 +1302,7 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
     elt->GetAttribute(NS_LITERAL_STRING("value"), textEquivalent); // Prefer value over tooltiptext
     if (textEquivalent.IsEmpty())
       elt->GetAttribute(NS_LITERAL_STRING("tooltiptext"), textEquivalent);
+    textEquivalent.CompressWhitespace();
     return AppendStringWithSpaces(aFlatString, textEquivalent);
   }
 
@@ -1260,16 +1332,18 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
             const nsStyleDisplay* display = (const nsStyleDisplay*)styleContext->GetStyleData(eStyleStruct_Display);
             if (display->IsBlockLevel() || display->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL) {
               isHTMLBlock = PR_TRUE;
-              aFlatString->Append(NS_LITERAL_STRING(" "));
+              if (!aFlatString->IsEmpty())
+                aFlatString->Append(NS_LITERAL_STRING(" "));
             }
           }
         }
       }
       nsAutoString text;
       textContent->CopyText(text);
+      text.CompressWhitespace();
       if (text.Length()>0)
         aFlatString->Append(text);
-      if (isHTMLBlock)
+      if (isHTMLBlock && !aFlatString->IsEmpty())
         aFlatString->Append(NS_LITERAL_STRING(" "));
     }
     return NS_OK;
@@ -1277,7 +1351,7 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
 
   nsCOMPtr<nsIDOMHTMLBRElement> brElement(do_QueryInterface(aContent));
   if (brElement) {   // If it's a line break, insert a space so that words aren't jammed together
-    aFlatString->Append(NS_LITERAL_STRING(" "));
+    aFlatString->Append(NS_LITERAL_STRING("\r\n"));
     return NS_OK;
   }
 
@@ -1295,12 +1369,18 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
     elt->GetAttribute(NS_LITERAL_STRING("alt"), textEquivalent);
     if (textEquivalent.IsEmpty())
       elt->GetAttribute(NS_LITERAL_STRING("title"), textEquivalent);
-    if (textEquivalent.IsEmpty())
-      elt->GetAttribute(NS_LITERAL_STRING("name"), textEquivalent);
+    if (textEquivalent.IsEmpty() && imageContent) {
+      nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mPresShell));
+      nsCOMPtr<nsIDOMNode> imageNode(do_QueryInterface(aContent));
+      if (imageNode && presShell)
+        presShell->GetImageLocation(imageNode, textEquivalent);
+    }
     if (textEquivalent.IsEmpty())
       elt->GetAttribute(NS_LITERAL_STRING("src"), textEquivalent);
+
     if (textEquivalent.IsEmpty())
       elt->GetAttribute(NS_LITERAL_STRING("data"), textEquivalent); // for <object>s with images
+    textEquivalent.CompressWhitespace();
     return AppendStringWithSpaces(aFlatString, textEquivalent);
   }
 
@@ -1309,6 +1389,25 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
 
 
 NS_IMETHODIMP nsAccessible::AppendFlatStringFromSubtree(nsIContent *aContent, nsAString *aFlatString)
+{
+  nsresult rv = AppendFlatStringFromSubtreeRecurse(aContent, aFlatString);
+  if (NS_SUCCEEDED(rv) && !aFlatString->IsEmpty()) {
+    nsAString::const_iterator start, end;
+    aFlatString->BeginReading(start);
+    aFlatString->EndReading(end);
+
+    PRInt32 spacesToTruncate = 0;
+    while (-- end != start && *end == ' ')
+      ++ spacesToTruncate;
+
+    if (spacesToTruncate > 0)
+      aFlatString->Truncate(aFlatString->Length() - spacesToTruncate);
+  }
+
+  return rv;
+}
+
+nsresult nsAccessible::AppendFlatStringFromSubtreeRecurse(nsIContent *aContent, nsAString *aFlatString)
 {
   // Depth first search for all text nodes that are decendants of content node.
   // Append all the text into one flat string
@@ -1489,20 +1588,21 @@ NS_IMETHODIMP nsAccessible::GetXULAccName(nsAString& _retval)
       //
       //   This should keep search times down and still get the relevant
       //   labels.
+      nsAutoString controlID;
+      domElement->GetAttribute(NS_LITERAL_STRING("id"), controlID);
       nsCOMPtr<nsIDOMNode> parent;
-      if (NS_SUCCEEDED(rv = mDOMNode->GetParentNode(getter_AddRefs(parent)))) {
+      if (!controlID.IsEmpty() && NS_SUCCEEDED(rv = mDOMNode->GetParentNode(getter_AddRefs(parent)))) {
         nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(parent));
         NS_ASSERTION(xulElement, "No xulElement for parent DOM Node!");
         if (xulElement) {
-          nsAutoString controlID;
-          nsCOMPtr<nsIDOMNodeList>labelList;
+          nsCOMPtr<nsIDOMNodeList> labelList;
           if (NS_SUCCEEDED(rv = xulElement->GetElementsByAttribute(NS_LITERAL_STRING("control"), controlID, getter_AddRefs(labelList))))
           {
             PRUint32 length = 0;
             if (NS_SUCCEEDED(rv = labelList->GetLength(&length)) && length > 0) {
-              for (PRUint32 i = 0; i < length; ++i) {
+              for (PRUint32 index = 0; index < length; ++index) {
                 nsCOMPtr<nsIDOMNode> child;
-                if (NS_SUCCEEDED(rv = labelList->Item(i, getter_AddRefs(child) ))) {
+                if (NS_SUCCEEDED(rv = labelList->Item(index, getter_AddRefs(child) ))) {
                   rv = AppendLabelText(child, label);
                 }
               }
