@@ -63,8 +63,6 @@
 #include "nsIDOMDocumentType.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDOMNSHTMLSelectElement.h"
-#include "nsIAccessibleSelectable.h"
-#include "nsLayoutAtoms.h"
 #include "nsString.h"
 #include "nsXPIDLString.h"
 #include "nsIAccessibilityService.h"
@@ -82,6 +80,11 @@
 #include "nsXULTreeAccessible.h"
 #include "nsITreeSelection.h"
 #include "nsAccessibilityService.h"
+#include "nsISelectionPrivate.h"
+#include "nsICaret.h"
+#include "nsIAccessibleCaret.h"
+#include "nsIDOMHTMLInputElement.h"
+#include "nsAccessibleEventData.h"
 
 NS_INTERFACE_MAP_BEGIN(nsRootAccessible)
   NS_INTERFACE_MAP_ENTRY(nsIAccessibleDocument)
@@ -89,7 +92,6 @@ NS_INTERFACE_MAP_BEGIN(nsRootAccessible)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFocusListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFormListener)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
-  NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener) 
   NS_INTERFACE_MAP_ENTRY(nsIScrollPositionListener) 
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMFormListener)
@@ -111,15 +113,17 @@ PRUint32 nsRootAccessible::gInstanceCount = 0;
 // construction 
 //-----------------------------------------------------
 nsRootAccessible::nsRootAccessible(nsIWeakReference* aShell):nsAccessible(nsnull,aShell), 
-  nsDocAccessibleMixin(aShell), mAccService(do_GetService("@mozilla.org/accessibilityService;1")),
-  mBusy(eBusyStateUninitialized), mListener(nsnull), mScrollPositionChangedTicks(0), mScrollablePresShells(nsnull)
+  nsDocAccessibleMixin(aShell), mListener(nsnull), 
+  mScrollWatchTimer(nsnull), mDocLoadTimer(nsnull), mWebProgress(nsnull),
+  mAccService(do_GetService("@mozilla.org/accessibilityService;1")),
+  mBusy(eBusyStateUninitialized), mIsNewDocument(PR_FALSE), 
+  mScrollPositionChangedTicks(0)
 {
   nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
   if (shell) {
     shell->GetDocument(getter_AddRefs(mDocument));
     mDOMNode = do_QueryInterface(mDocument);
   }
-  nsLayoutAtoms::AddRefAtoms();
   ++gInstanceCount;
 #ifdef DEBUG_LEAKS
   printf("=====> %d nsRootAccessible's %x\n", gInstanceCount, (PRUint32)this); 
@@ -137,10 +141,7 @@ nsRootAccessible::~nsRootAccessible()
   printf("======> %d nsRootAccessible's %x\n", gInstanceCount, (PRUint32)this); 
 #endif
 
-  nsLayoutAtoms::ReleaseAtoms();
   RemoveAccessibleEventListener();
-  if (mScrollablePresShells)
-    delete mScrollablePresShells;
 }
 
   /* attribute wstring accName; */
@@ -239,43 +240,36 @@ NS_IMETHODIMP nsRootAccessible::GetAccValue(nsAString& aAccValue)
   return GetURL(aAccValue);
 }
 
-void nsRootAccessible::Notify(nsITimer *timer)
+void nsRootAccessible::DocLoadCallback(nsITimer *aTimer, void *aClosure)
 {
-  // Short timer is finished 
-  if (mBusy != eBusyStateDone) {
-    mBusy = eBusyStateDone;
-    if (mListener)
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this);
-  }
+  // Doc has finished loading, fire "load finished" event
+  // This path is only used if the doc was already finished loading
+  // when the RootAccessible was created. 
+  // Otherwise, ::OnStateChange() fires the event when doc is loaded.
 
-  if (mScrollPositionChangedTicks) {
-    if (++mScrollPositionChangedTicks > 2) {
-      // Whenever scroll position changes, mScrollPositionChangeTicks gets reset to 1
-      // We only want to fire accessibilty scroll event when scrolling stops or pauses
-      // Therefore, we wait for no scroll events to occur between 2 ticks of this timer
-      // That indicates a pause in scrolling, so we fire the accessibilty scroll event
-      nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mLastScrolledPresShell));
-      if (mListener && presShell) {
-        nsCOMPtr<nsIAccessible> docAccessible;
-        if (mPresShell == mLastScrolledPresShell)
-          docAccessible = this;  // Fast way, don't need to create new accessible to fire event on the root
-        else {
-          // Need to create accessible for document that had scrolling occur in it
-          nsCOMPtr<nsIDocument> doc;
-          presShell->GetDocument(getter_AddRefs(doc));
-          nsCOMPtr<nsIDOMNode> docNode(do_QueryInterface(doc));
-          mAccService->GetAccessibleFor(docNode, getter_AddRefs(docAccessible));
-        }
+  nsRootAccessible *rootAcc = NS_REINTERPRET_CAST(nsRootAccessible*, aClosure);
+  if (rootAcc)
+    rootAcc->FireDocLoadFinished();
+}
 
-        if (docAccessible)
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SCROLLINGEND, docAccessible);
-      }
-      mScrollPositionChangedTicks = 0;
-      mLastScrolledPresShell = nsnull;
-    }
+void nsRootAccessible::ScrollTimerCallback(nsITimer *aTimer, void *aClosure)
+{
+  nsRootAccessible *rootAcc = NS_REINTERPRET_CAST(nsRootAccessible*, aClosure);
+
+  if (rootAcc && rootAcc->mScrollPositionChangedTicks && 
+      ++rootAcc->mScrollPositionChangedTicks > 2) {
+    // Whenever scroll position changes, mScrollPositionChangeTicks gets reset to 1
+    // We only want to fire accessibilty scroll event when scrolling stops or pauses
+    // Therefore, we wait for no scroll events to occur between 2 ticks of this timer
+    // That indicates a pause in scrolling, so we fire the accessibilty scroll event
+    if (rootAcc->mListener)
+      rootAcc->mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SCROLLINGEND, rootAcc, nsnull);
+    rootAcc->mScrollPositionChangedTicks = 0;
+    rootAcc->mScrollWatchTimer->Cancel();
+    rootAcc->mScrollWatchTimer = nsnull;
   }
 }
-  
+
 void nsRootAccessible::AddScrollListener(nsIPresShell *aPresShell)
 {
   nsCOMPtr<nsIViewManager> vm;
@@ -287,26 +281,8 @@ void nsRootAccessible::AddScrollListener(nsIPresShell *aPresShell)
   if (vm)
     vm->GetRootScrollableView(&scrollableView);
 
-  if (!scrollableView)
-    return;
-
-  if (!mScrollablePresShells)
-    mScrollablePresShells = new nsSupportsHashtable(SCROLL_HASH_START_SIZE, PR_TRUE);  // PR_TRUE = thread safe hash table
-
-  if (mScrollablePresShells) {
-    nsISupportsKey key(scrollableView);
-    nsCOMPtr<nsISupports> supports(dont_AddRef(NS_STATIC_CAST(nsISupports*, mScrollablePresShells->Get(&key))));
-    if (supports)  // This scroll view is already being listened to, remove it. We will re-add it w/ current pres shell
-      RemoveScrollListener(aPresShell);
-
-    // Add to hash table, so we can retrieve correct pres shell later
-    nsCOMPtr<nsIWeakReference> weakShell;
-    weakShell = do_GetWeakReference(aPresShell);
-    mScrollablePresShells->Put(&key, weakShell);
-
-    // Add listener
+  if (scrollableView)
     scrollableView->AddScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
-  }
 }
 
 void nsRootAccessible::RemoveScrollListener(nsIPresShell *aPresShell)
@@ -322,34 +298,6 @@ void nsRootAccessible::RemoveScrollListener(nsIPresShell *aPresShell)
 
   if (scrollableView)
     scrollableView->RemoveScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
-
-  if (mScrollablePresShells) {
-    nsISupportsKey key(scrollableView);
-    nsCOMPtr<nsISupports> supp;
-    mScrollablePresShells->Remove(&key, getter_AddRefs(supp));
-  }
-}
-
-PRBool PR_CALLBACK
-RemoveScrollListenerEnum(nsHashKey *aKey, void *aData, void* aClosure)
-{
-  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(NS_STATIC_CAST(nsIWeakReference*, aData)));
-
-  // If presShell gone, then our nsIScrollPositionListener will also already be gone
-  if (presShell) {
-    nsCOMPtr<nsIViewManager> vm;
-    if (presShell)
-      presShell->GetViewManager(getter_AddRefs(vm));
-
-    nsIScrollableView* scrollableView = nsnull;
-    if (vm)
-      vm->GetRootScrollableView(&scrollableView);
-
-    if (scrollableView)
-      scrollableView->RemoveScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, aClosure));
-  }
-
-  return PR_TRUE;
 }
 
 NS_IMETHODIMP nsRootAccessible::ScrollPositionWillChange(nsIScrollableView *aView, nscoord aX, nscoord aY)
@@ -360,45 +308,25 @@ NS_IMETHODIMP nsRootAccessible::ScrollPositionWillChange(nsIScrollableView *aVie
 NS_IMETHODIMP nsRootAccessible::ScrollPositionDidChange(nsIScrollableView *aScrollableView, nscoord aX, nscoord aY)
 {
   if (mListener) {
-    if (!mScrollablePresShells)
-      return NS_OK;
-    nsISupportsKey key(aScrollableView);
-    nsCOMPtr<nsIWeakReference> weakShell(dont_AddRef(NS_STATIC_CAST(nsIWeakReference*, mScrollablePresShells->Get(&key))));
-    if (!weakShell)
-      return NS_OK;
-    if (mTimer && (mBusy != eBusyStateDone || (mScrollPositionChangedTicks && mLastScrolledPresShell != weakShell))) {
-      // We need to say finish up our current time work and start a new timer
-      // either because we haven't yet fired our finished loading event, or 
-      // a scroll event from another pres shell is still waiting to be fired
-      Notify(mTimer);
-      mTimer->Cancel();
-    }
     // Start new timer, if the timer cycles at least 1 full cycle without more scroll position changes, 
     // then the ::Notify() method will fire the accessibility event for scroll position changes
-    nsresult rv;
-    mTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
-    if (NS_SUCCEEDED(rv)) {
-      const PRUint32 kScrollPosCheckWait = 50;
-      mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kScrollPosCheckWait, NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_SLACK);
+    const PRUint32 kScrollPosCheckWait = 50;
+    if (mScrollWatchTimer) {
+      mScrollWatchTimer->SetDelay(kScrollPosCheckWait);  // Create new timer, to avoid leaks
+    }
+    else {
+      mScrollWatchTimer = do_CreateInstance("@mozilla.org/timer;1");
+      if (mScrollWatchTimer) {
+        mScrollWatchTimer->InitWithFuncCallback(ScrollTimerCallback, this,
+                                                kScrollPosCheckWait, 
+                                                nsITimer::TYPE_REPEATING_SLACK);
+      }
     }
     mScrollPositionChangedTicks = 1;
-    mLastScrolledPresShell = weakShell;
   }
   return NS_OK;
 }
 
-void nsRootAccessible::StartDocReadyTimer()
-{
-  if (!mTimer) {
-    nsresult rv;
-    mTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
-    if (NS_SUCCEEDED(rv)) {
-      const PRUint32 kUpdateTimerDelay = 1;
-      mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kUpdateTimerDelay);
-    }
-  }
-}
-
 /* void addAccessibleEventListener (in nsIAccessibleEventListener aListener); */
 NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventListener *aListener)
 {
@@ -416,15 +344,15 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     // capture Form change events 
-    rv = target->AddEventListener(NS_LITERAL_STRING("change"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("select"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     // add ourself as a CheckboxStateChange listener (custom event fired in nsHTMLInputElement.cpp)
-    rv = target->AddEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     // add ourself as a RadioStateChange Listener ( custom event fired in in nsHTMLInputElement.cpp  & radio.xml)
-    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     rv = target->AddEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
@@ -439,31 +367,18 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
     rv = target->AddEventListener(NS_LITERAL_STRING("DOMMenuItemActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
     
-    // Extremely short timer, after which we announce that page is finished loading
-    // By waiting until after this short time, we know that the 3rd party accessibility software 
-    // has received it's accessible, and can handle events on it.
-    StartDocReadyTimer(); 
-
-    // Set up web progress listener - we need to know when page loading is finished
-    // That way we can send the STATE_CHANGE events for the MSAA root "pane" object (ROLE_PANE),
-    // and change the STATE_BUSY bit flag
-    nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mPresShell));
-    if (presShell) {
-      nsCOMPtr<nsIPresContext> context; 
-      presShell->GetPresContext(getter_AddRefs(context));
-      if (context) { 
-        nsCOMPtr<nsISupports> container; context->GetContainer(getter_AddRefs(container));
-        nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(container));
-        if (docShell) {
-          mWebProgress = do_GetInterface(docShell);
-          mWebProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_LOCATION | 
-                                                  nsIWebProgress::NOTIFY_STATE_DOCUMENT);
-        }
-      }
-    }
-    NS_ASSERTION(mWebProgress, "Could not get nsIWebProgress for nsRootAccessible");
+    rv = target->AddEventListener(NS_LITERAL_STRING("DOMMenuBarActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
+    
+    rv = target->AddEventListener(NS_LITERAL_STRING("DOMMenuBarInactive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
+    
+    AddContentDocListeners();
   }
 
+  if (!mCaretAccessible && mListener)
+    mAccService->CreateCaretAccessible(mDOMNode, mListener, getter_AddRefs(mCaretAccessible));
+
   return NS_OK;
 }
 
@@ -474,41 +389,61 @@ NS_IMETHODIMP nsRootAccessible::RemoveAccessibleEventListener()
     nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(mDocument));
     if (target) { 
       target->RemoveEventListener(NS_LITERAL_STRING("focus"), NS_STATIC_CAST(nsIDOMFocusListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("change"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("select"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popupshowing"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuItemActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuBarActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuBarInactive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     }
  
-    if (mTimer) {
-      mTimer->Cancel();
-      mTimer = nsnull;
+    if (mScrollWatchTimer) {
+      mScrollWatchTimer->Cancel();
+      mScrollWatchTimer = nsnull;
     }
 
-    if (mWebProgress) {
-      mWebProgress->RemoveProgressListener(this);
-      mWebProgress = nsnull;
+    if (mDocLoadTimer) {
+      mDocLoadTimer->Cancel();
+      mDocLoadTimer = nsnull;
     }
 
-    if (mScrollablePresShells)
-      mScrollablePresShells->Enumerate(RemoveScrollListenerEnum, NS_STATIC_CAST(nsIScrollPositionListener*, this));
+    RemoveContentDocListeners();
 
     mListener = nsnull;
   }
 
+  if (mCaretAccessible) {
+    mCaretAccessible->RemoveSelectionListener();
+    mCaretAccessible = nsnull;
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsRootAccessible::GetCaretAccessible(nsIAccessibleCaret **aCaretAccessible)
+{
+  *aCaretAccessible = mCaretAccessible;
+  NS_IF_ADDREF(*aCaretAccessible);
   return NS_OK;
 }
 
 void nsRootAccessible::FireAccessibleFocusEvent(nsIAccessible *focusAccessible, nsIDOMNode *focusNode)
 {
-  if (focusNode && gLastFocusedNode != focusNode) {
-    mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, focusAccessible);
+  if (focusAccessible && focusNode && gLastFocusedNode != focusNode) {
+    mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, focusAccessible, nsnull);
     NS_IF_RELEASE(gLastFocusedNode);
-    gLastFocusedNode = focusNode;
-    NS_ADDREF(gLastFocusedNode);
+    PRUint32 role = ROLE_NOTHING;
+    focusAccessible->GetAccRole(&role);
+    if (role == ROLE_MENUITEM || role == ROLE_LISTITEM)
+      gLastFocusedNode = nsnull; // This makes it report all focus events on menu and list items
+    else {
+      gLastFocusedNode = focusNode;
+      NS_IF_ADDREF(gLastFocusedNode);
+    }
+    if (mCaretAccessible)
+      mCaretAccessible->AttachNewSelectionListener(focusNode);
   }
 }
 
@@ -536,70 +471,146 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
       optionTargetNode = do_QueryInterface(selectItem);
     }
 
+    nsCOMPtr<nsIAccessible> accessible;
+    if (NS_FAILED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible))))
+      return NS_OK;
+
     // If it's a tree element, need the currently selected item
     PRInt32 treeIndex = -1;
     nsCOMPtr<nsITreeBoxObject> treeBox;
+    nsCOMPtr<nsIAccessible> treeItemAccessible;
     nsXULTreeAccessible::GetTreeBoxObject(targetNode, getter_AddRefs(treeBox));
     if (treeBox) {
       nsCOMPtr<nsITreeSelection> selection;
       treeBox->GetSelection(getter_AddRefs(selection));
-      if (selection)
+      if (selection) {
         selection->GetCurrentIndex(&treeIndex);
+        if (treeIndex >= 0) {
+          nsCOMPtr<nsIWeakReference> weakShell;
+          nsAccessibilityService::GetShellFromNode(targetNode, getter_AddRefs(weakShell));
+          treeItemAccessible = new nsXULTreeitemAccessible(accessible, targetNode, weakShell, treeIndex);
+          if (!treeItemAccessible)
+            return NS_ERROR_OUT_OF_MEMORY;
+        }
+      }
     }
 
     nsAutoString eventType;
     aEvent->GetType(eventType);
 
-    nsCOMPtr<nsIAccessible> accessible;
-
-    if (NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
-      // tree event
-      if (treeBox && treeIndex >= 0 && 
-          (eventType.EqualsIgnoreCase("DOMMenuItemActive") || eventType.EqualsIgnoreCase("select"))) {
-        nsCOMPtr<nsIWeakReference> weakShell;
-        nsAccessibilityService::GetShellFromNode(targetNode, getter_AddRefs(weakShell));
-        accessible = new nsXULTreeitemAccessible(accessible, targetNode, weakShell, treeIndex);
-        if (!accessible)
-          return NS_ERROR_OUT_OF_MEMORY;
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
-        return NS_OK;
-      }
-
-      if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) { 
-        if (optionTargetNode &&
-            NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) {
-          FireAccessibleFocusEvent(accessible, optionTargetNode);
-        }
-        else
-          FireAccessibleFocusEvent(accessible, targetNode);
-      }
-      else if (eventType.EqualsIgnoreCase("change")) {
-        if (!selectControl)   // Don't use onchange to fire EVENT_STATE_CHANGE events for selects
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
-      }
-      else if (eventType.EqualsIgnoreCase("ListitemStateChange")) {
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
-      }
-      else if (eventType.EqualsIgnoreCase("CheckboxStateChange")) { 
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
-      }
-      else if (eventType.EqualsIgnoreCase("RadioStateChange") ) {
-        // first the XUL radio buttons
-        if (targetNode &&
-            NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
-        }
-        else { // for the html radio buttons -- apparently the focus code just works. :-)
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
-        }
-      }
-      else if (eventType.EqualsIgnoreCase("popupshowing")) 
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUPOPUPSTART, accessible);
-      else if (eventType.EqualsIgnoreCase("popuphiding")) 
-        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUPOPUPEND, accessible);
+#ifndef MOZ_ACCESSIBILITY_ATK
+    // tree event
+    if (treeItemAccessible && 
+        (eventType.EqualsIgnoreCase("DOMMenuItemActive") || eventType.EqualsIgnoreCase("select"))) {
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+      return NS_OK;
     }
+
+    if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) { 
+      if (optionTargetNode &&
+          NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) {
+        FireAccessibleFocusEvent(accessible, optionTargetNode);
+      }
+      else
+        FireAccessibleFocusEvent(accessible, targetNode);
+    }
+    else if (eventType.EqualsIgnoreCase("ListitemStateChange")) {
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+      FireAccessibleFocusEvent(accessible, optionTargetNode);
+    }
+    else if (eventType.EqualsIgnoreCase("CheckboxStateChange")) { 
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+    }
+    else if (eventType.EqualsIgnoreCase("RadioStateChange") ) {
+      // first the XUL radio buttons
+      if (targetNode &&
+          NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+        FireAccessibleFocusEvent(accessible, targetNode);
+      }
+      else { // for the html radio buttons -- apparently the focus code just works. :-)
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, nsnull);
+      }
+    }
+    else if (eventType.EqualsIgnoreCase("DOMMenuBarActive")) 
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUSTART, accessible, nsnull);
+    else if (eventType.EqualsIgnoreCase("DOMMenuBarInactive")) {
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUEND, accessible, nsnull);
+      GetAccFocused(getter_AddRefs(accessible));
+      if (accessible) {
+        accessible->AccGetDOMNode(getter_AddRefs(targetNode));
+        FireAccessibleFocusEvent(accessible, targetNode);
+      }
+    }
+    else {
+      // Menu popup events
+      PRUint32 menuEvent = 0;
+      if (eventType.EqualsIgnoreCase("popupshowing"))
+        menuEvent = nsIAccessibleEventListener::EVENT_MENUPOPUPSTART;
+      else if (eventType.EqualsIgnoreCase("popuphiding"))
+        menuEvent = nsIAccessibleEventListener::EVENT_MENUPOPUPEND;
+      if (menuEvent) {
+        PRUint32 role = ROLE_NOTHING;
+        accessible->GetAccRole(&role);
+        if (role == ROLE_MENUPOPUP)
+          mListener->HandleEvent(menuEvent, accessible, nsnull);
+      }
+    }
+#else
+    AtkStateChange stateData;
+    AtkTextChange textData;
+    if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) {
+      if (treeItemAccessible) // use focused treeitem
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+      else if (optionTargetNode && // use focused option
+          NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible))))
+        FireAccessibleFocusEvent(accessible, optionTargetNode);
+      else
+        FireAccessibleFocusEvent(accessible, targetNode);
+    }
+    else if (eventType.EqualsIgnoreCase("change")) {
+      if (selectElement) // it's a HTML <select>
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_SELECTION_CHANGE, accessible, nsnull);
+      else {
+        nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(targetNode));
+        if (inputElement) { // it's a HTML <input>
+          accessible->GetAccState(&stateData.state);
+          stateData.enable = (stateData.state & STATE_CHECKED) != 0;
+          stateData.state = STATE_CHECKED;
+          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, (AccessibleEventData*)&stateData);
+        }
+      }
+    }
+    else if (eventType.EqualsIgnoreCase("select")) {
+      if (selectControl) // it's a XUL <listbox>
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_SELECTION_CHANGE, accessible, nsnull);
+      else if (treeBox && treeIndex >= 0) // it's a XUL <tree>
+        // use EVENT_FOCUS instead of EVENT_ATK_SELECTION_CHANGE
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, treeItemAccessible, nsnull);
+    }
+    else if (eventType.EqualsIgnoreCase("input")) {
+      // XXX kyle.yuan@sun.com future work, put correct values for text change data
+      textData.start = 0;
+      textData.length = 0;
+      textData.add = PR_TRUE;
+      nsAutoString accName;
+      accessible->GetAccValue(accName);
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_ATK_TEXT_CHANGE, accessible, (AccessibleEventData*)&textData);
+    }
+    else if (eventType.EqualsIgnoreCase("ListitemStateChange")) // it's a XUL <listbox>
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible, nsnull);
+    else if (eventType.EqualsIgnoreCase("CheckboxStateChange") || // it's a XUL <checkbox>
+             eventType.EqualsIgnoreCase("RadioStateChange")) { // it's a XUL <radio>
+      accessible->GetAccState(&stateData.state);
+      stateData.enable = (stateData.state & STATE_CHECKED) != 0;
+      stateData.state = STATE_CHECKED;
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible, (AccessibleEventData*)&stateData);
+    }
+    else if (eventType.EqualsIgnoreCase("popupshowing")) 
+      FireAccessibleFocusEvent(accessible, targetNode);
+    else if (eventType.EqualsIgnoreCase("popuphiding")) 
+      FireAccessibleFocusEvent(accessible, targetNode);
+#endif
   }
   return NS_OK;
 }
@@ -626,13 +637,22 @@ NS_IMETHODIMP nsRootAccessible::Focus(nsIDOMEvent* aEvent)
   return HandleEvent(aEvent);
 }
 
-NS_IMETHODIMP nsRootAccessible::Blur(nsIDOMEvent* aEvent) { return NS_OK; }
+NS_IMETHODIMP nsRootAccessible::Blur(nsIDOMEvent* aEvent) 
+{ 
+  return NS_OK; 
+}
 
 // ------- nsIDOMFormListener Methods (5) -------------
 
-NS_IMETHODIMP nsRootAccessible::Submit(nsIDOMEvent* aEvent) { return NS_OK; }
+NS_IMETHODIMP nsRootAccessible::Submit(nsIDOMEvent* aEvent) 
+{ 
+  return NS_OK; 
+}
 
-NS_IMETHODIMP nsRootAccessible::Reset(nsIDOMEvent* aEvent) { return NS_OK; }
+NS_IMETHODIMP nsRootAccessible::Reset(nsIDOMEvent* aEvent) 
+{ 
+  return NS_OK; 
+}
 
 NS_IMETHODIMP nsRootAccessible::Change(nsIDOMEvent* aEvent)
 {
@@ -641,7 +661,11 @@ NS_IMETHODIMP nsRootAccessible::Change(nsIDOMEvent* aEvent)
   // this may be the event that we have the individual Accessible objects
   //  handle themselves -- have list/combos figure out the change in selection
   //  have textareas and inputs fire a change of state etc...
+#ifdef MOZ_ACCESSIBILITY_ATK
   return HandleEvent(aEvent);
+#else
+  return NS_OK;   // Ignore form change events in MSAA
+#endif
 }
 
 // gets Select events when text is selected in a textarea or input
@@ -651,7 +675,14 @@ NS_IMETHODIMP nsRootAccessible::Select(nsIDOMEvent* aEvent)
 }
 
 // gets Input events when text is entered or deleted in a textarea or input
-NS_IMETHODIMP nsRootAccessible::Input(nsIDOMEvent* aEvent) { return NS_OK; }
+NS_IMETHODIMP nsRootAccessible::Input(nsIDOMEvent* aEvent) 
+{ 
+#ifndef MOZ_ACCESSIBILITY_ATK
+  return NS_OK; 
+#else
+  return HandleEvent(aEvent);
+#endif
+}
 
 // ------- nsIDOMXULListener Methods (8) ---------------
 
@@ -703,24 +734,109 @@ NS_IMETHODIMP nsRootAccessible::GetDocument(nsIDocument **doc)
   return nsDocAccessibleMixin::GetDocument(doc);
 }
 
+void nsRootAccessible::RemoveContentDocListeners()
+{
+  // Remove listeners associated with content documents
+
+  // Remove web progress listener
+  if (mWebProgress) {
+    mWebProgress->RemoveProgressListener(this);
+    mWebProgress = nsnull;
+  }
+
+  // Remove scroll position listener
+  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mPresShell));
+  RemoveScrollListener(presShell);
+}
+
+void nsRootAccessible::AddContentDocListeners()
+{
+  // 1) Set up scroll position listener
+  // 2) Set up web progress listener - we need to know 
+  //    when page loading is finished
+  //    That way we can send the STATE_CHANGE events for 
+  //    the MSAA root "pane" object (ROLE_PANE),
+  //    and change the STATE_BUSY bit flag
+  //    Do this only for top level content documents
+
+  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mPresShell));
+  if (!presShell)
+    return;
+
+  AddScrollListener(presShell);
+
+  nsCOMPtr<nsISupports> container;
+  mDocument->GetContainer(getter_AddRefs(container));
+
+  nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem(do_QueryInterface(container));
+  if (!docShellTreeItem)
+    return;
+
+  // Make sure we're a content docshell
+  // We don't want to listen to chrome progress
+  PRInt32 itemType;
+  docShellTreeItem->GetItemType(&itemType);
+
+  if (itemType != nsIDocShellTreeItem::typeContent)
+    return;
+
+  // Make sure we're the top content doc shell 
+  // We don't want to listen to iframe progress
+  nsCOMPtr<nsIDocShellTreeItem> topOfContentTree;
+  docShellTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(topOfContentTree));
+  if (topOfContentTree != docShellTreeItem)
+    return;
+  
+  nsCOMPtr<nsIPresContext> context; 
+  presShell->GetPresContext(getter_AddRefs(context));
+  if (!context)
+    return;
+
+  mWebProgress = do_GetInterface(docShellTreeItem);
+  if (!mWebProgress)
+    return;
+
+  mWebProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_LOCATION | 
+                                    nsIWebProgress::NOTIFY_STATE_DOCUMENT);
+
+
+  mIsNewDocument = PR_TRUE;
+  mBusy = eBusyStateLoading;
+  PRBool isLoading;
+
+  mWebProgress->GetIsLoadingDocument(&isLoading);
+  if (!isLoading) {
+    // If already loaded, fire "done loading" event after short timeout
+    // If we fired the event here, we'd get reentrancy problems
+    // Otherwise it will be fired from OnStateChange when the load is done
+    mDocLoadTimer = do_CreateInstance("@mozilla.org/timer;1");
+    if (mDocLoadTimer) {
+      mDocLoadTimer->InitWithFuncCallback(DocLoadCallback, this, 1,
+                                          nsITimer::TYPE_ONE_SHOT);
+    }
+  }
+}
+
+void nsRootAccessible::FireDocLoadFinished()
+{
+  if (mIsNewDocument) {
+    mIsNewDocument = PR_FALSE;
+
+    if (mBusy != eBusyStateDone) {
+      mBusy = eBusyStateDone;
+#ifndef MOZ_ACCESSIBILITY_ATK
+      if (mListener)
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
+#endif
+    }
+  }
+}
+
 NS_IMETHODIMP nsRootAccessible::OnStateChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
-  if ((aStateFlags & STATE_IS_DOCUMENT) && (aStateFlags & STATE_STOP)) {
-    // Set up scroll position listener
-    nsCOMPtr<nsIDOMWindow> domWin;
-    aWebProgress->GetDOMWindow(getter_AddRefs(domWin));
-    if (domWin) {
-      nsCOMPtr<nsIDOMDocument> domDoc;
-      domWin->GetDocument(getter_AddRefs(domDoc));
-      nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-      if (doc) {
-        nsCOMPtr<nsIPresShell> presShell;
-        doc->GetShellAt(0, getter_AddRefs(presShell));
-        AddScrollListener(presShell);
-      }
-    }
-  }
+  if ((aStateFlags & STATE_IS_DOCUMENT) && (aStateFlags & STATE_STOP))
+    FireDocLoadFinished();   // Doc is ready!
 
   return NS_OK;
 }
@@ -739,24 +855,26 @@ NS_IMETHODIMP nsRootAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, nsIURI *location)
 {
   // Load has been verified, it will occur, about to commence
+
+  // We won't fire a "doc finished loading" event on this nsRootAccessible 
+  // Instead we fire that on the new nsRootAccessible that is created for the new doc
+  mIsNewDocument = PR_FALSE;   // We're a doc that's going away
+
   if (mBusy != eBusyStateLoading) {
     mBusy = eBusyStateLoading; 
+    // Fire a "new doc has started to load" event
+#ifndef MOZ_ACCESSIBILITY_ATK
     if (mListener)
-      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this);
-
-    // Document is going away, remove its scroll position listener
-    nsCOMPtr<nsIDOMWindow> domWin;
-    aWebProgress->GetDOMWindow(getter_AddRefs(domWin));
-    if (domWin) {
-      nsCOMPtr<nsIDOMDocument> domDoc;
-      domWin->GetDocument(getter_AddRefs(domDoc));
-      nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-      if (doc) {
-        nsCOMPtr<nsIPresShell> presShell;
-        doc->GetShellAt(0, getter_AddRefs(presShell));
-        RemoveScrollListener(presShell);
-      }
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this, nsnull);
+#else
+    if (mListener) {
+      AtkChildrenChange childrenData;
+      childrenData.index = -1;
+      childrenData.child = 0;
+      childrenData.add = PR_FALSE;
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_REORDER , this, (AccessibleEventData*)&childrenData);
     }
+#endif
   }
 
   return NS_OK;
@@ -852,8 +970,9 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAString& aDocType)
 NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   if (mDocument) {
-    nsCOMPtr<nsINameSpaceManager> nameSpaceManager;
-    if (NS_SUCCEEDED(mDocument->GetNameSpaceManager(*getter_AddRefs(nameSpaceManager)))) 
+    nsCOMPtr<nsINameSpaceManager> nameSpaceManager =
+        do_GetService(NS_NAMESPACEMANAGER_CONTRACTID);
+    if (nameSpaceManager) 
       return nameSpaceManager->GetNameSpaceURI(aNameSpaceID, aNameSpaceURI);
   }
   return NS_ERROR_FAILURE;
@@ -889,3 +1008,10 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocShellFromPS(nsIPresShell* aPresShell,
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP nsDocAccessibleMixin::GetCaretAccessible(nsIAccessibleCaret **aCaretAccessible)
+{
+  // Caret only owned by top level window's document
+  *aCaretAccessible = nsnull;
+  return NS_OK;
+}
+
