@@ -2,9 +2,12 @@
 
 # make-jars [-f] [-v] [-l] [-d <chromeDir>] [-s <srcdir>] < <jar.mn>
 
+if ($^O ne "cygwin") {
 # we'll be pulling in some stuff from the script directory
-use FindBin;
+require FindBin;
+import FindBin;
 push @INC, $FindBin::Bin;
+}
 
 use strict;
 
@@ -21,7 +24,7 @@ import mozLock;
 
 my $objdir = getcwd;
 
-getopts("d:s:f:avl");
+getopts("d:s:f:avlD:p:");
 
 my $baseFilesDir = ".";
 if (defined($::opt_s)) {
@@ -65,6 +68,16 @@ if (defined($::opt_a)) {
     $autoreg = 0;
 }
 
+my $preprocessor = "";
+if (defined($::opt_p)) {
+    $preprocessor = $::opt_p;
+}
+
+my $defines = "";
+while (@ARGV) {
+    $defines = "$defines ".shift(@ARGV);
+}
+
 if ($verbose) {
     print "make-jars "
         . "-v -d $chromeDir "
@@ -74,6 +87,29 @@ if ($verbose) {
         . "\n";
 }
 
+my $win32 = ($^O =~ /((MS)?win32)|cygwin|os2/i) ? 1 : 0;
+my $macos = ($^O =~ /MacOS|darwin/i) ? 1 : 0;
+my $unix  = !($win32 || $macos) ? 1 : 0;
+
+sub foreignPlatformFile
+{
+   my ($jarfile) = @_;
+   
+   if (!$win32 && index($jarfile, "-win") != -1) {
+     return 1;
+   }
+   
+   if (!$unix && index($jarfile, "-unix") != -1) {
+     return 1; 
+   }
+
+   if (!$macos && index($jarfile, "-mac") != -1) {
+     return 1;
+   }
+
+   return 0;
+}
+
 sub zipErrorCheck($$)
 {
     my ($err,$lockfile) = @_;
@@ -224,7 +260,8 @@ sub RegIt
 
 sub EnsureFileInDir
 {
-    my ($destPath, $srcPath, $destFile, $srcFile, $override) = @_;
+    my ($destPath, $srcPath, $destFile, $srcFile, $override, $preproc) = @_;
+    my $objPath;
 
     #print "EnsureFileInDir($destPath, $srcPath, $destFile, $srcFile, $override)\n";
 
@@ -280,15 +317,31 @@ sub EnsureFileInDir
         if ($srcPath) {
             $file = $srcPath;
         }
+        $objPath = "$objdir/$destFile";
 
         if (!-e $file) {
-            die "error: file '$file' doesn't exist";
+            if (!-e $objPath) {
+                die "error: file '$file' doesn't exist";
+            } else {
+                $file = "$objPath";
+            }
         }
         if (!-e $dir) {
             mkpath($dir, 0, 0775) || die "can't mkpath $dir: $!";
         }
         unlink $destPath;       # in case we had a symlink on unix
-        copy($file, $destPath) || die "copy($file, $destPath) failed: $!";
+        if ($preproc) {
+	    my $preproc_file = $file;
+	    if ($^O eq 'cygwin' && $file =~ /^[a-zA-Z]:/) {
+		# convert to a cygwin path
+		$preproc_file =~ s|^([a-zA-Z]):|/cygdrive/\1|;
+	    }
+            if (system("$^X $preprocessor $defines $preproc_file > $destPath") != 0) {
+                die "Preprocessing of $file failed: ".($? >> 8);
+            }
+        } else {
+            copy($file, $destPath) || die "copy($file, $destPath) failed: $!";
+        }
 
         # fix the mod date so we don't jar everything (is this faster than just jarring everything?)
         my $mtime = stat($file)->mtime || die $!;
@@ -314,9 +367,9 @@ while (<STDIN>) {
             if (/^\s+([\w\d.\-\_\\\/\+]+)\s*(\([\w\d.\-\_\\\/]+\))?$\s*/) {
                 my $dest = $1;
                 my $srcPath = defined($2) ? substr($2, 1, -1) : $2;
-		EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 0);
+		EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 0, 0);
                 $args = "$args$dest ";
-		if ($autoreg && $dest =~ /([\w\d.\-\_\+]+)\/([\w\d.\-\_\\\/]+)contents.rdf/)
+		if (!foreignPlatformFile($jarfile)  && $autoreg && $dest =~ /([\w\d.\-\_\+]+)\/([\w\d.\-\_\\\/]+)contents.rdf/)
 		{
 		    my $chrome_type = $1;
 		    my $pkg_name = $2;
@@ -325,14 +378,26 @@ while (<STDIN>) {
             } elsif (/^\+\s+([\w\d.\-\_\\\/\+]+)\s*(\([\w\d.\-\_\\\/]+\))?$\s*/) {
                 my $dest = $1;
                 my $srcPath = defined($2) ? substr($2, 1, -1) : $2;
-                EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 1);
+                EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 1, 0);
                 $overrides = "$overrides$dest ";
-		if ($autoreg && $dest =~ /([\w\d.\-\_\+]+)\/([\w\d.\-\_\\\/]+)contents.rdf/)
+		if (!foreignPlatformFile($jarfile)  && $autoreg && $dest =~ /([\w\d.\-\_\+]+)\/([\w\d.\-\_\\\/]+)contents.rdf/)
 		{
 		    my $chrome_type = $1;
 		    my $pkg_name = $2;
 		    RegIt($chromeDir, $jarfile, $chrome_type, $pkg_name);
 		}
+	    } elsif (/^\*\s+([\w\d.\-\_\\\/\+]+)\s*(\([\w\d.\-\_\\\/]+\))?$\s*/) {
+		# preprocessed, no override
+		my $dest = $1;
+		my $srcPath = defined($2) ? substr($2, 1, -1) : $2;
+		EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 0, 1);
+		$args = "$args$dest ";
+	    } elsif (/^\*\+\s+([\w\d.\-\_\\\/\+]+)\s*(\([\w\d.\-\_\\\/]+\))?$\s*/) {
+		# preprocessed, override
+		my $dest = $1;
+		my $srcPath = defined($2) ? substr($2, 1, -1) : $2;
+		EnsureFileInDir("$chromeDir/$jarfile", $baseFilesDir, $dest, $srcPath, 1, 1);
+		$overrides = "$overrides$dest ";
             } elsif (/^\s*$/) {
                 # end with blank line
                 last;
