@@ -37,6 +37,10 @@
 
 #use diagnostics;
 require strict;
+my $dir = $0;
+$dir =~ s/[^\/]*$//;
+push(@INC, "$dir");
+require "Moz/Milestone.pm";
 use Getopt::Long;
 use Getopt::Std;
 
@@ -129,6 +133,8 @@ if ($bits eq "16") { $fileos="VOS__WINDOWS16"; }
 
 my $bufferstr="    ";
 
+my $MILESTONE_FILE = "$topsrcdir/config/milestone.txt";
+
 #Read module.ver file
 #Version file overrides for WIN32:
 #WIN32_MODULE_COMMENT
@@ -140,6 +146,7 @@ my $bufferstr="    ";
 #WIN32_MODULE_COPYRIGHT
 #WIN32_MODULE_TRADEMARKS
 #WIN32_MODULE_ORIGINAL_FILENAME
+#WIN32_MODULE_PRODUCTNAME
 
 #Override values obtained from the .ver file
 my $override_comment;
@@ -151,6 +158,7 @@ my $override_module;
 my $override_copyright;
 my $override_trademarks;
 my $override_filename;
+my $override_productname;
 if (open(VERFILE, "<$srcdir/module.ver")) 
 {
 
@@ -166,6 +174,7 @@ if (open(VERFILE, "<$srcdir/module.ver"))
 		if ($a eq "WIN32_MODULE_COPYRIGHT") { $override_copyright = $b; }
 		if ($a eq "WIN32_MODULE_TRADEMARKS") { $override_trademarks = $b; }
 		if ($a eq "WIN32_MODULE_ORIGINAL_FILENAME") { $override_filename = $b; }
+		if ($a eq "WIN32_MODULE_PRODUCTNAME") { $override_productname = $b; }
 		($a,$b) = getNextEntry();
 	}
 	close(VERFILE)
@@ -200,16 +209,16 @@ if ($official eq "1") {
 		$fileflags = "VS_FF_PRERELEASE | VS_FF_DEBUG";
 	}
 
-        # Try to grab milestone from milestone.pl
+        # Try to grab milestone.
         # I'd love to put this in the makefiles rather than here,
         # since I could run it once per build rather than once per
         # dll/program, but I can't seem to get backticks working
         # properly in the makefiles =P
         if ($milestone eq "") {
-            $milestone = `perl $topsrcdir/config/milestone.pl --topsrcdir $topsrcdir --getms`;
+            $milestone = Moz::Milestone::getOfficialMilestone($MILESTONE_FILE);
         }
 
-	if ($milestone ne "") {
+	if ($milestone ne "" && $milestone !~ /\+$/) {
 		#its a milestone build
 
 		$mpversion = $milestone;
@@ -238,7 +247,7 @@ my $fileversion = $productversion;
 my $copyright = "License: MPL 1.1/GPL 2.0/LGPL 2.1";
 my $company = "Mozilla, Netscape";
 my $trademarks = "Mozilla, Netscape";
-
+my $productname = "Mozilla";
 
 
 if (defined($override_comment)){$comment=$override_comment;}
@@ -250,6 +259,7 @@ if (defined($override_module)){$module=$override_module;}
 if (defined($override_copyright)){$copyright=$override_company;}
 if (defined($override_trademarks)){$trademarks=$override_trademarks;}
 if (defined($override_filename)){$binary=$override_filename;}
+if (defined($override_productname)){$productname=$override_productname;}
 
 
 #Override section
@@ -306,52 +316,53 @@ my $insideversion=0;
 if (open(RCINCLUDE, "<$rcinclude")) 
 {
 	print RCFILE "// From included resource $rcinclude\n";
-	my $mstring="";
+#	my $mstring="";
 	while (<RCINCLUDE>) 
 	{
-		my $instr=$_;
-		chomp($instr);
-		$mstring .= "$instr\;";
+                print RCFILE $_;
+#		my $instr=$_;
+#		chomp($instr);
+#		$mstring .= "$instr\;";
 	}
 	close(RCINCLUDE);
-	$mstring =~ s/\/\*.*\*\///g;
-	my @mlines = split(/\;/,$mstring);
-	for(@mlines)
-	{
-		my ($nocomment)=split(/\/\//,$_);
-		if (defined($nocomment) && $nocomment ne "")
-		{
-			my ($firststring,$secondstring) = split(/\s+/,$nocomment);
-			if (!defined($firststring)) {$firststring="";}
-			if (!defined($secondstring)) {$secondstring="";}
-			if ($secondstring eq "VERSIONINFO") 
-			{
-if (!$quiet || $quiet ne "1") { 
-				print "$bufferstr" . "WARNING: Included RC file ($rcinclude, $module, $binary)\n";
-				print "$bufferstr" . "WARNING: contains versioning information that will be discarded\n";
-				print "$bufferstr" . "WARNING: Remove it and use relevant overrides (in module.ver)\n";
-}
-				$versionlevel = 0;
-				$insideversion = 1; 
-			}
-			if ($firststring eq "BEGIN") { $versionlevel++; }
-			if ($secondstring eq "END") 
-			{ 
-				$versionlevel--; 
-				if ($insideversion==1 && $versionlevel==0) {$versionlevel=0;}
-			}
-			my $includecheck = $firststring . $secondstring;
-			$includecheck =~ s/<|>/"/g;
-			$includecheck = lc($includecheck);
-			if ($includecheck ne "#include\"winver.h\"")
-			{
-				if ($insideversion == 0 && $versionlevel == 0)
-				{
-					print RCFILE "$nocomment\n";	
-				}
-			}
-		}
-	}
+#	$mstring =~ s/\/\*.*\*\///g;
+#	my @mlines = split(/\;/,$mstring);
+#	for(@mlines)
+#	{
+#		my ($nocomment)=split(/\/\//,$_);
+#		if (defined($nocomment) && $nocomment ne "")
+#		{
+#			my ($firststring,$secondstring) = split(/\s+/,$nocomment);
+#			if (!defined($firststring)) {$firststring="";}
+#			if (!defined($secondstring)) {$secondstring="";}
+#			if ($secondstring eq "VERSIONINFO") 
+#			{
+#if (!$quiet || $quiet ne "1") { 
+#				print "$bufferstr" . "WARNING: Included RC file ($rcinclude, $module, $binary)\n";
+#				print "$bufferstr" . "WARNING: contains versioning information that will be discarded\n";
+#				print "$bufferstr" . "WARNING: Remove it and use relevant overrides (in module.ver)\n";
+#}
+#				$versionlevel = 0;
+#				$insideversion = 1; 
+#			}
+#			if ($firststring eq "BEGIN") { $versionlevel++; }
+#			if ($secondstring eq "END") 
+#			{ 
+#				$versionlevel--; 
+#				if ($insideversion==1 && $versionlevel==0) {$versionlevel=0;}
+#			}
+#			my $includecheck = $firststring . $secondstring;
+#			$includecheck =~ s/<|>/"/g;
+#			$includecheck = lc($includecheck);
+#			if ($includecheck ne "#include\"winver.h\"")
+#			{
+#				if ($insideversion == 0 && $versionlevel == 0)
+#				{
+#					print RCFILE "$nocomment\n";	
+#				}
+#			}
+#		}
+#	}
 	
 }
 
@@ -392,7 +403,7 @@ print RCFILE qq{
 };
 }
 print RCFILE qq{
-            VALUE "ProductName", "Mozilla"
+            VALUE "ProductName", "$productname"
         END
     END
     BLOCK "VarFileInfo"
