@@ -70,10 +70,11 @@
 #include "nsIDragService.h"
 #include "nsIDragSession.h"
 #include "nsComponentManagerUtils.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIServiceManagerUtils.h"
 #include "nsPromiseFlatString.h"
-#include "nsIIOService.h"
+#include "nsNetUtil.h"
 #include "nsIFile.h"
 #include "nsIWebNavigation.h"
 #include "nsIDragDropOverride.h"
@@ -94,6 +95,7 @@
 #include "imgIRequest.h"
 #include "gfxIImageFrame.h"
 #include "nsIInterfaceRequestor.h"
+#include "nsIDocumentEncoder.h"
 
 
 
@@ -114,7 +116,6 @@ NS_INTERFACE_MAP_END
 nsContentAreaDragDrop::nsContentAreaDragDrop ( ) 
   : mListenerInstalled(PR_FALSE), mNavigator(nsnull), mOverrideDrag(nsnull), mOverrideDrop(nsnull)
 {
-  NS_INIT_REFCNT();
 } // ctor
 
 
@@ -291,20 +292,18 @@ nsContentAreaDragDrop::ExtractURLFromData(const nsACString & inFlavor, nsISuppor
   if ( inFlavor.Equals(kUnicodeMime) ) {
     // the data is regular unicode, just go with what we get. It may be a url, it
     // may not be. *shrug*
-    nsCOMPtr<nsISupportsWString> stringData(do_QueryInterface(inDataWrapper));
+    nsCOMPtr<nsISupportsString> stringData(do_QueryInterface(inDataWrapper));
     if ( stringData ) {
-      nsXPIDLString data;
-      stringData->GetData(getter_Copies(data));
-      outURL = data;
+      stringData->GetData(outURL);
     }
   }
   else if ( inFlavor.Equals(kURLMime) ) {
     // the data is an internet shortcut of the form <url>\n<title>. Strip
     // out the url piece and return that.
-    nsCOMPtr<nsISupportsWString> stringData(do_QueryInterface(inDataWrapper));
+    nsCOMPtr<nsISupportsString> stringData(do_QueryInterface(inDataWrapper));
     if ( stringData ) {
-      nsXPIDLString data;
-      stringData->GetData(getter_Copies(data));
+      nsAutoString data;
+      stringData->GetData(data);
       PRInt32 separator = data.FindChar('\n');
       if ( separator >= 0 )
         outURL = Substring(data, 0, separator);
@@ -313,13 +312,12 @@ nsContentAreaDragDrop::ExtractURLFromData(const nsACString & inFlavor, nsISuppor
     }  
   }
   else if ( inFlavor.Equals(kFileMime) ) {
-    // the data is a file. Use the IOService to get a file:// url
+    // the data is a file. Use the necko parsing utils to get a file:// url
     // from the OS data.
-    nsCOMPtr<nsIIOService> ioService(do_GetService("@mozilla.org/network/io-service;1"));
     nsCOMPtr<nsIFile> file(do_QueryInterface(inDataWrapper));
-    if ( ioService && file ) {
+    if ( file ) {
       nsCAutoString url;
-      ioService->GetURLSpecFromFile(file, url);
+      NS_GetURLSpecFromFile(file, url);
       outURL = NS_ConvertUTF8toUCS2(url);
     } 
   }
@@ -764,6 +762,8 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
     selection->ContainsNode(draggedNode, PR_FALSE, &containsTarget);
   }
     
+  PRBool getFormattedStrings = PR_FALSE;
+
   if ( selection && !isCollapsed && containsTarget ) {
     // track down the anchor node, if any, for the url
     nsCOMPtr<nsIDOMNode> selectionStart;
@@ -779,17 +779,7 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
       }
     }
       
-    // find the title for the drag and any associated html
-    nsCOMPtr<nsISelectionPrivate> privSelection(do_QueryInterface(selection));
-    if ( privSelection ) {        
-      // the window has a selection so we should grab that rather
-      // than looking for specific elements
-      privSelection->ToStringWithFormat("text/html", 128+256, 0, getter_Copies(htmlString));
-      privSelection->ToStringWithFormat("text/plain", 0, 0, getter_Copies(titleString));
-    }
-    else 
-      selection->ToString(getter_Copies(titleString));
-
+    getFormattedStrings = PR_TRUE;
   } // if drag is within selection
   else {
     // if the alt key is down, don't start a drag if we're in an anchor because
@@ -815,6 +805,15 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
     } // area
     else {
       nsCOMPtr<nsIDOMHTMLImageElement> img(do_QueryInterface(draggedNode));
+
+      if (img && parentAnchor) {
+        // If we are dragging around an image in an anchor, then we
+        // are dragging the entire anchor!
+
+        draggedNode = parentAnchor;
+        img = nsnull;
+      }
+
       if ( img ) {
         *outIsAnchor = PR_TRUE;
         // grab the href as the url, use alt text as the title of the area if it's there.
@@ -827,14 +826,6 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
           urlString +
           NS_LITERAL_STRING("\">");
 
-        // if the image is also a link, then re-wrap htmlstring in
-        // an anchor tag
-        if ( parentAnchor ) {
-          *outIsAnchor = PR_TRUE;
-          GetAnchorURL(parentAnchor, urlString);
-          CreateLinkText(urlString, htmlString, htmlString);
-        }
-        
         // also grab the image data
         GetImageFromDOMNode(draggedNode, outImage);
         // select siblings up to and including the selected link. this
@@ -854,11 +845,11 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
           if ( linkNode ) {
             *outIsAnchor = PR_TRUE;
             GetAnchorURL(linkNode, urlString);
-            GetNodeString(linkNode, titleString);
         
             // select siblings up to and including the selected link. this
             // shouldn't be fatal, and we should still do the drag if this fails
             NormalizeSelection(linkNode, selection);
+            getFormattedStrings = PR_TRUE;
           }
           else {
             // indicate that we don't allow drags in this case
@@ -869,6 +860,22 @@ nsContentAreaDragDrop::BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outU
     }
   } // else no selection or drag outside it
 
+  if (getFormattedStrings && selection) {
+    // find the title for the drag and any associated html
+    nsCOMPtr<nsISelectionPrivate> privSelection(do_QueryInterface(selection));
+    if ( privSelection ) {        
+      // the window has a selection so we should grab that rather
+      // than looking for specific elements
+      privSelection->ToStringWithFormat("text/html",
+                                        nsIDocumentEncoder::OutputAbsoluteLinks |
+                                        nsIDocumentEncoder::OutputEncodeW3CEntities,
+                                        0, getter_Copies(htmlString));
+      privSelection->ToStringWithFormat("text/plain", 0, 0, getter_Copies(titleString));
+    }
+    else 
+      selection->ToString(getter_Copies(titleString));
+  }
+
   if ( startDrag ) {
     // default text value is the URL
     if ( titleString.IsEmpty() )
@@ -912,27 +919,27 @@ nsContentAreaDragDrop::CreateTransferable(const nsAString & inURLString, const n
     nsAutoString dragData ( inURLString );
     dragData += NS_LITERAL_STRING("\n");
     dragData += inTitleString;
-    nsCOMPtr<nsISupportsWString> urlPrimitive(do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID));
+    nsCOMPtr<nsISupportsString> urlPrimitive(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
     if ( !urlPrimitive )
       return NS_ERROR_FAILURE;
-    urlPrimitive->SetData(dragData.get());
+    urlPrimitive->SetData(dragData);
     trans->SetTransferData(kURLMime, urlPrimitive, dragData.Length() * 2);
   }
   
   // add the full html
-  nsCOMPtr<nsISupportsWString> htmlPrimitive(do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID));
+  nsCOMPtr<nsISupportsString> htmlPrimitive(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
   if ( !htmlPrimitive )
     return NS_ERROR_FAILURE;
-  htmlPrimitive->SetData(PromiseFlatString(inHTMLString).get());
+  htmlPrimitive->SetData(inHTMLString);
   trans->SetTransferData(kHTMLMime, htmlPrimitive, inHTMLString.Length() * 2);
 
   // add the plain (unicode) text. we use the url for text/unicode data if an anchor
   // is being dragged, rather than the title text of the link or the alt text for
   // an anchor image. 
-  nsCOMPtr<nsISupportsWString> textPrimitive(do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID));
+  nsCOMPtr<nsISupportsString> textPrimitive(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID));
   if ( !textPrimitive )
     return NS_ERROR_FAILURE;
-  textPrimitive->SetData(PromiseFlatString(inIsAnchor ? inURLString : inTitleString).get());
+  textPrimitive->SetData(inIsAnchor ? inURLString : inTitleString);
   trans->SetTransferData(kUnicodeMime, textPrimitive, (inIsAnchor ? inURLString.Length() : inTitleString.Length()) * 2);  
   
   // add image data, if present. For now, all we're going to do with this is turn it
@@ -1050,9 +1057,9 @@ nsContentAreaDragDrop::GetImageFrame(nsIContent* aContent, nsIDocument *aDocumen
 		if (NS_SUCCEEDED(rv) && frame) {
 			nsCOMPtr<nsIAtom> type;
 			frame->GetFrameType(getter_AddRefs(type));
-			if (type.get() == nsLayoutAtoms::imageFrame) {
+			if (type == nsLayoutAtoms::imageFrame) {
 				nsIImageFrame* imageFrame;
-				rv = frame->QueryInterface(NS_GET_IID(nsIImageFrame), (void**)&imageFrame);
+				rv = CallQueryInterface(frame, &imageFrame);
 				if (NS_FAILED(rv)) {
 					NS_WARNING("Should not happen - frame is not image frame even though type is nsLayoutAtoms::imageFrame");
 					return rv;
@@ -1110,10 +1117,11 @@ nsContentAreaDragDrop::GetImageFromDOMNode(nsIDOMNode* inNode, nsIImage**outImag
 					if (imgContainer) {
 						nsCOMPtr<gfxIImageFrame> imgFrame;
 						imgContainer->GetFrameAt(0, getter_AddRefs(imgFrame));
-						if ( imgFrame )	{
+						if (imgFrame)	{
 							nsCOMPtr<nsIInterfaceRequestor> ir = do_QueryInterface(imgFrame);
-							if ( ir )
-								rv = ir->GetInterface(NS_GET_IID(nsIImage), (void**)outImage);      // will addreff
+							if (ir) {
+								rv = CallGetInterface(ir.get(), outImage);
+							}
 						}
 					}
 				}
