@@ -59,6 +59,7 @@
 #include "nsIDOMDocumentType.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMRange.h"
+#include "nsIDOMDocument.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsHTMLAtoms.h"
@@ -68,7 +69,6 @@
 #include "nsIFrameSelection.h"
 #include "nsISupportsArray.h"
 #include "nsIParserService.h"
-#include "nsParserCIID.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptSecurityManager.h"
@@ -77,7 +77,6 @@
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID,
                      NS_ICHARSETCONVERTERMANAGER_CID);
-static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
 
 nsresult NS_NewDomSelection(nsISelection **aDomSelection);
 
@@ -159,12 +158,12 @@ protected:
   PRUint32          mEndDepth;
   PRInt32           mStartRootIndex;
   PRInt32           mEndRootIndex;
-  PRBool            mHaltRangeHint;  
   nsAutoVoidArray   mCommonAncestors;
   nsAutoVoidArray   mStartNodes;
   nsAutoVoidArray   mStartOffsets;
   nsAutoVoidArray   mEndNodes;
   nsAutoVoidArray   mEndOffsets;
+  PRPackedBool      mHaltRangeHint;  
   PRPackedBool      mIsCopying;  // Set to PR_TRUE only while copying
 };
 
@@ -183,7 +182,7 @@ NS_INTERFACE_MAP_END
 
 nsDocumentEncoder::nsDocumentEncoder()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mMimeType.Assign(NS_LITERAL_STRING("text/plain"));
 
@@ -476,7 +475,14 @@ ConvertAndWrite(const nsAString& aString,
       NS_ENSURE_SUCCESS(rv, rv);
 
       nsCAutoString entString("&#");
-      entString.AppendInt(unicodeBuf[unicodeLength - 1]);
+      if (IS_HIGH_SURROGATE(unicodeBuf[unicodeLength - 1]) && 
+          unicodeLength < startLength && IS_LOW_SURROGATE(unicodeBuf[unicodeLength]))  {
+        entString.AppendInt(SURROGATE_TO_UCS4(unicodeBuf[unicodeLength - 1],
+                                              unicodeBuf[unicodeLength]));
+        unicodeLength += 1;
+      }
+      else
+        entString.AppendInt(unicodeBuf[unicodeLength - 1]);
       entString.Append(';');
 
       rv = aStream->Write(entString.get(), entString.Length(), &written);
@@ -937,9 +943,14 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
 
       mRange = nsnull;
   } else {
-    nsCOMPtr<nsIDOMNode> doc(do_QueryInterface(mDocument));
+    nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(mDocument));
+    rv = mSerializer->AppendDocumentStart(domdoc, aOutputString);
 
-    rv = SerializeToStringRecursive(doc, aOutputString);
+    if (NS_SUCCEEDED(rv)) {
+      nsCOMPtr<nsIDOMNode> doc(do_QueryInterface(mDocument));
+
+      rv = SerializeToStringRecursive(doc, aOutputString);
+    }
   }
 
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1048,8 +1059,11 @@ protected:
   };
   
   nsresult PromoteRange(nsIDOMRange *inRange);
+  nsresult PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode, 
+                                PRInt32 *ioStartOffset, 
+                                PRInt32 *ioEndOffset);
   nsresult GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32 aOffset, 
-                                  nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset);
+                            nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset, nsIDOMNode *aCommon);
   nsCOMPtr<nsIDOMNode> GetChildAt(nsIDOMNode *aParent, PRInt32 aOffset);
   PRBool IsMozBR(nsIDOMNode* aNode);
   nsresult GetNodeLocation(nsIDOMNode *inChild, nsCOMPtr<nsIDOMNode> *outParent, PRInt32 *outOffset);
@@ -1059,7 +1073,6 @@ protected:
   PRBool IsEmptyTextContent(nsIDOMNode* aNode);
   virtual PRBool IncludeInContext(nsIDOMNode *aNode);
 
-  nsCOMPtr<nsIParserService> mParserService;
   PRBool mIsTextWidget;
 };
 
@@ -1097,9 +1110,7 @@ nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
   if (!IsScriptEnabled(mDocument))
     mFlags |= OutputNoScriptContent;
 
-  nsresult rv;
-  mParserService = do_GetService(kParserServiceCID, &rv);
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1292,6 +1303,7 @@ nsHTMLCopyEncoder::IncludeInContext(nsIDOMNode *aNode)
   if (tag.get() == nsHTMLAtoms::b        ||
       tag.get() == nsHTMLAtoms::i        ||
       tag.get() == nsHTMLAtoms::u        ||
+      tag.get() == nsHTMLAtoms::a        ||
       tag.get() == nsHTMLAtoms::tt       ||
       tag.get() == nsHTMLAtoms::s        ||
       tag.get() == nsHTMLAtoms::big      ||
@@ -1327,9 +1339,11 @@ nsHTMLCopyEncoder::PromoteRange(nsIDOMRange *inRange)
 {
   if (!inRange) return NS_ERROR_NULL_POINTER;
   nsresult rv;
-  nsCOMPtr<nsIDOMNode> startNode, endNode;
+  nsCOMPtr<nsIDOMNode> startNode, endNode, common;
   PRInt32 startOffset, endOffset;
   
+  rv = inRange->GetCommonAncestorContainer(getter_AddRefs(common));
+  NS_ENSURE_SUCCESS(rv, rv);
   rv = inRange->GetStartContainer(getter_AddRefs(startNode));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = inRange->GetStartOffset(&startOffset);
@@ -1344,19 +1358,76 @@ nsHTMLCopyEncoder::PromoteRange(nsIDOMRange *inRange)
   PRInt32 opStartOffset, opEndOffset;
   nsCOMPtr<nsIDOMRange> opRange;
   
-  rv = GetPromotedPoint( kStart, startNode, startOffset, address_of(opStartNode), &opStartOffset);
+  // examine range endpoints.  
+  rv = GetPromotedPoint( kStart, startNode, startOffset, address_of(opStartNode), &opStartOffset, common);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = GetPromotedPoint( kEnd, endNode, endOffset, address_of(opEndNode), &opEndOffset);
+  rv = GetPromotedPoint( kEnd, endNode, endOffset, address_of(opEndNode), &opEndOffset, common);
   NS_ENSURE_SUCCESS(rv, rv);
+  
+  // if both range endpoints are at the common ancestor, check for possible inclusion of ancestors
+  if ( (opStartNode == common) && (opEndNode == common) )
+  {
+    rv = PromoteAncestorChain(address_of(opStartNode), &opStartOffset, &opEndOffset);
+    NS_ENSURE_SUCCESS(rv, rv);
+    opEndNode = opStartNode;
+  }
+  
+  // set the range to the new values
   rv = inRange->SetStart(opStartNode, opStartOffset);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = inRange->SetEnd(opEndNode, opEndOffset);
   return rv;
 } 
 
+
+// PromoteAncestorChain will promote a range represented by [{*ioNode,*ioStartOffset} , {*ioNode,*ioEndOffset}]
+// The promotion is different from that found in getPromotedPoint: it will only promote one endpoint if it can
+// promote the other.  Thus, instead of having a startnode/endNode, there is just the one ioNode.
+nsresult
+nsHTMLCopyEncoder::PromoteAncestorChain(nsCOMPtr<nsIDOMNode> *ioNode, 
+                                        PRInt32 *ioStartOffset, 
+                                        PRInt32 *ioEndOffset) 
+{
+  if (!ioNode || !ioStartOffset || !ioEndOffset) return NS_ERROR_NULL_POINTER;
+
+  nsresult rv = NS_OK;
+  PRBool done = PR_FALSE;
+
+  nsCOMPtr<nsIDOMNode> frontNode, endNode, parent;
+  PRInt32 frontOffset, endOffset;
+  
+  // loop for as long as we can promote both endpoints
+  while (!done)
+  {
+    rv = (*ioNode)->GetParentNode(getter_AddRefs(parent));
+    if ((NS_FAILED(rv)) || !parent)
+      done = PR_TRUE;
+    else
+    {
+      // passing parent as last param to GetPromotedPoint() allows it to promote only one level
+      // up the heirarchy.
+      rv = GetPromotedPoint( kStart, *ioNode, *ioStartOffset, address_of(frontNode), &frontOffset, parent);
+      NS_ENSURE_SUCCESS(rv, rv);
+      // then we make the same attempt with the endpoint
+      rv = GetPromotedPoint( kEnd, *ioNode, *ioEndOffset, address_of(endNode), &endOffset, parent);
+      NS_ENSURE_SUCCESS(rv, rv);
+      // if both endpoints were promoted one level, keep looping - otherwise we are done.
+      if ( (frontNode != parent) || (endNode != parent) )
+        done = PR_TRUE;
+      else
+      {
+        *ioNode = frontNode;  
+        *ioStartOffset = frontOffset;
+        *ioEndOffset = endOffset;
+      }
+    }
+  }
+  return rv;
+}
+
 nsresult
 nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32 aOffset, 
-                                  nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset)
+                                  nsCOMPtr<nsIDOMNode> *outNode, PRInt32 *outOffset, nsIDOMNode *common)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIDOMNode> node = aNode;
@@ -1364,10 +1435,13 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
   PRInt32 offset = aOffset;
   PRBool  bResetPromotion = PR_FALSE;
   
-  // defualt values
+  // default values
   *outNode = node;
   *outOffset = offset;
 
+  if (common == node) 
+    return NS_OK;
+    
   if (aWhere == kStart)
   {
     // some special casing for text nodes
@@ -1399,12 +1473,16 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
 
     // finding the real start for this point.  look up the tree for as long as we are the 
     // first node in the container, and as long as we haven't hit the body node.
-    if (!IsRoot(node))
+    if (!IsRoot(node) && (parent != common))
     {
       rv = GetNodeLocation(node, address_of(parent), &offset);
       NS_ENSURE_SUCCESS(rv, rv);
       if (offset == -1) return NS_OK; // we hit generated content; STOP
-      while ((IsFirstNode(node)) && (!IsRoot(parent)))
+      nsIParserService *parserService =
+        nsContentUtils::GetParserServiceWeakRef();
+      if (!parserService)
+        return NS_ERROR_OUT_OF_MEMORY;
+      while ((IsFirstNode(node)) && (!IsRoot(parent)) && (parent != common))
       {
         if (bResetPromotion)
         {
@@ -1417,8 +1495,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
             PRInt32 id;
             content->GetTag(*getter_AddRefs(atom));
             atom->ToString(tag);
-            mParserService->HTMLStringTagToId(tag, &id);
-            mParserService->IsBlock(id, isBlock);
+            parserService->HTMLStringTagToId(tag, &id);
+            parserService->IsBlock(id, isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
@@ -1484,12 +1562,16 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
     
     // finding the real end for this point.  look up the tree for as long as we are the 
     // last node in the container, and as long as we haven't hit the body node.
-    if (!IsRoot(node))
+    if (!IsRoot(node) && (parent != common))
     {
       rv = GetNodeLocation(node, address_of(parent), &offset);
       NS_ENSURE_SUCCESS(rv, rv);
       if (offset == -1) return NS_OK; // we hit generated content; STOP
-      while ((IsLastNode(node)) && (!IsRoot(parent)))
+      nsIParserService *parserService =
+        nsContentUtils::GetParserServiceWeakRef();
+      if (!parserService)
+        return NS_ERROR_OUT_OF_MEMORY;
+      while ((IsLastNode(node)) && (!IsRoot(parent)) && (parent != common))
       {
         if (bResetPromotion)
         {
@@ -1502,8 +1584,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
             PRInt32 id;
             content->GetTag(*getter_AddRefs(atom));
             atom->ToString(tag);
-            mParserService->HTMLStringTagToId(tag, &id);
-            mParserService->IsBlock(id, isBlock);
+            parserService->HTMLStringTagToId(tag, &id);
+            parserService->IsBlock(id, isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
