@@ -45,7 +45,6 @@
 #include "nsString.h"
 #include "nsUnicharUtils.h"
 #include "nsXPIDLString.h"
-#include "nsParserCIID.h"
 #include "nsIServiceManager.h"
 #include "nsIDocumentEncoder.h"
 #include "nsLayoutAtoms.h"
@@ -56,8 +55,8 @@
 #include "nsITextToSubURI.h"
 #include "nsCRT.h"
 #include "nsIHTMLContent.h"
-
-static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
+#include "nsIParserService.h"
+#include "nsContentUtils.h"
 
 #define kIndentStr NS_LITERAL_STRING("  ")
 #define kLessThan NS_LITERAL_STRING("<")
@@ -100,19 +99,6 @@ nsHTMLContentSerializer::~nsHTMLContentSerializer()
   }
 }
 
-nsresult
-nsHTMLContentSerializer::GetParserService(nsIParserService** aParserService)
-{
-  if (!mParserService) {
-    nsresult rv;
-    mParserService = do_GetService(kParserServiceCID, &rv);
-    if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-  }
-
-  CallQueryInterface(mParserService.get(), aParserService);
-  return NS_OK;
-}
-
 NS_IMETHODIMP 
 nsHTMLContentSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
                               nsIAtom* aCharSet, PRBool aIsCopying)
@@ -574,8 +560,7 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   const PRUnichar* sharedName;
   name->GetUnicode(&sharedName);
 
-  nsCOMPtr<nsIParserService> parserService;
-  GetParserService(getter_AddRefs(parserService));
+  nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
 
   if (parserService && (name.get() != nsHTMLAtoms::style)) {
     nsAutoString nameStr(sharedName);
@@ -787,6 +772,9 @@ nsHTMLContentSerializer::AppendToStringWrapped(const nsASingleFragmentString& aS
   }
 }
 
+static PRUint16 kValNBSP = 160;
+static const char* kEntityNBSP = "nbsp";
+
 static PRUint16 kGTVal = 62;
 static const char* kEntities[] = {
   "", "", "", "", "", "", "", "", "", "",
@@ -824,8 +812,8 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
 
   if (aTranslateEntities && !mInCDATA) {
     if (mFlags & nsIDocumentEncoder::OutputEncodeEntities) {
-      nsCOMPtr<nsIParserService> parserService;
-      GetParserService(getter_AddRefs(parserService));
+      nsIParserService* parserService =
+        nsContentUtils::GetParserServiceWeakRef();
 
       if (!parserService) {
         NS_ERROR("Can't get parser service");
@@ -856,7 +844,11 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
         // needs to be replaced
         for (; c < fragmentEnd; c++, advanceLength++) {
           PRUnichar val = *c;
-          if ((val <= kGTVal) && (entityTable[val][0] != 0)) {
+          if (val == kValNBSP) {
+            entityText = kEntityNBSP;
+            break;
+          }
+          else if ((val <= kGTVal) && (entityTable[val][0] != 0)) {
             entityText = entityTable[val];
             break;
           } else if (mIsLatin1 && val > 127 && val < 256) {
@@ -947,8 +939,8 @@ nsHTMLContentSerializer::LineBreakBeforeOpen(nsIAtom* aName,
     return PR_TRUE;
   }
   else {
-    nsCOMPtr<nsIParserService> parserService;
-    GetParserService(getter_AddRefs(parserService));
+    nsIParserService* parserService =
+      nsContentUtils::GetParserServiceWeakRef();
     
     if (parserService) {
       nsAutoString str;
@@ -1050,8 +1042,8 @@ nsHTMLContentSerializer::LineBreakAfterClose(nsIAtom* aName,
     return PR_TRUE;
   }
   else {
-    nsCOMPtr<nsIParserService> parserService;
-    GetParserService(getter_AddRefs(parserService));
+    nsIParserService* parserService =
+      nsContentUtils::GetParserServiceWeakRef();
     
     if (parserService) {
       nsAutoString str;
