@@ -45,7 +45,6 @@
 #include "nsCOMPtr.h"
 #include "nsString.h"
 #include "nsILineBreaker.h"
-#include "nsIParserService.h"
 #include "nsIContent.h"
 #include "nsIAtom.h"
 #include "nsIHTMLToTextSink.h"
@@ -86,6 +85,9 @@ public:
                               nsAString& aStr);
   NS_IMETHOD Flush(nsAString& aStr);
 
+  NS_IMETHOD AppendDocumentStart(nsIDOMDocument *aDocument,
+                                 nsAString& aStr);
+
   // nsIContentSink
   NS_IMETHOD WillBuildModel(void) { return NS_OK; }
   NS_IMETHOD DidBuildModel(PRInt32 aQualityLevel) { return NS_OK; }
@@ -93,11 +95,11 @@ public:
   NS_IMETHOD WillResume(void) { return NS_OK; }
   NS_IMETHOD SetParser(nsIParser* aParser) { return NS_OK; }
   NS_IMETHOD OpenContainer(const nsIParserNode& aNode);
-  NS_IMETHOD CloseContainer(const nsIParserNode& aNode);
+  NS_IMETHOD CloseContainer(const nsHTMLTag aTag);
   NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
   NS_IMETHOD AddComment(const nsIParserNode& aNode) { return NS_OK; }
   NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode) { return NS_OK; }
-  NS_IMETHOD AddDocTypeDecl(const nsIParserNode& aNode, PRInt32 aMode=0) { return NS_OK; }
+  NS_IMETHOD AddDocTypeDecl(const nsIParserNode& aNode) { return NS_OK; }
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
   NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
   NS_IMETHOD NotifyTagObservers(nsIParserNode* aNode) { return NS_OK; }
@@ -105,17 +107,17 @@ public:
   // nsIHTMLContentSink
   NS_IMETHOD SetTitle(const nsString& aValue) { return NS_OK; }
   NS_IMETHOD OpenHTML(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHTML(const nsIParserNode& aNode);
+  NS_IMETHOD CloseHTML();
   NS_IMETHOD OpenHead(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHead(const nsIParserNode& aNode);
+  NS_IMETHOD CloseHead();
   NS_IMETHOD OpenBody(const nsIParserNode& aNode);
-  NS_IMETHOD CloseBody(const nsIParserNode& aNode);
+  NS_IMETHOD CloseBody();
   NS_IMETHOD OpenForm(const nsIParserNode& aNode);
-  NS_IMETHOD CloseForm(const nsIParserNode& aNode);
+  NS_IMETHOD CloseForm();
   NS_IMETHOD OpenMap(const nsIParserNode& aNode);
-  NS_IMETHOD CloseMap(const nsIParserNode& aNode);
+  NS_IMETHOD CloseMap();
   NS_IMETHOD OpenFrameset(const nsIParserNode& aNode);
-  NS_IMETHOD CloseFrameset(const nsIParserNode& aNode);
+  NS_IMETHOD CloseFrameset();
   NS_IMETHOD GetPref(PRInt32 aTag,PRBool& aPref);
   NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
 
@@ -143,9 +145,9 @@ protected:
   PRBool IsBlockLevel(PRInt32 aId);
   PRBool IsContainer(PRInt32 aId);
   PRBool IsInPre();
+  PRBool IsInOL();
   PRBool IsCurrentNodeConverted();
   nsresult GetIdForContent(nsIContent* aContent, PRInt32* aID);
-  nsresult GetParserService(nsIParserService** aParserService);
   nsresult DoOpenContainer(PRInt32 aTag);
   nsresult DoCloseContainer(PRInt32 aTag);
   nsresult DoAddLeaf(PRInt32 aTag, const nsAString& aText);
@@ -248,7 +250,6 @@ protected:
 
   nsString                     mLineBreak;
   nsCOMPtr<nsILineBreaker>     mLineBreaker;
-  nsCOMPtr<nsIParserService>   mParserService;
 
   // Conveniance constant. It would be nice to have it as a const static
   // variable, but that causes issues with OpenBSD and module unloading.
