@@ -43,7 +43,6 @@
 #include "nsIFrameLoader.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
@@ -89,6 +88,7 @@ public:
 
   // nsIFrameLoaderOwner
   NS_IMETHOD GetFrameLoader(nsIFrameLoader **aFrameLoader);
+  NS_IMETHOD SetFrameLoader(nsIFrameLoader *aFrameLoader);
 
   // nsIContent
   NS_IMETHOD SetParent(nsIContent *aParent);
@@ -309,6 +309,14 @@ nsHTMLIFrameElement::GetFrameLoader(nsIFrameLoader **aFrameLoader)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsHTMLIFrameElement::SetFrameLoader(nsIFrameLoader *aFrameLoader)
+{
+  mFrameLoader = aFrameLoader;
+
+  return NS_OK;
+}
+
 nsresult
 nsHTMLIFrameElement::LoadSrc()
 {
@@ -368,22 +376,22 @@ nsHTMLIFrameElement::StringToAttribute(nsIAtom* aAttribute,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::marginwidth) {
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::marginheight) {
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::width) {
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::height) {
-    if (ParseValueOrPercent(aValue, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseIntValue(aValue, eHTMLUnit_Pixel, PR_TRUE)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
