@@ -79,6 +79,7 @@
 #include "nsRuleNode.h"
 
 #include "nsIJSContextStack.h"
+#include "nsIView.h"
 
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 
@@ -128,7 +129,7 @@ public:
                                nsAString& aResult) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute,
                                       PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
@@ -139,9 +140,8 @@ public:
 
 protected:
   nsresult SetSrcInner(nsIURI* aBaseURL, const nsAString& aSrc);
-  static nsresult GetCallerSourceURL(nsIURI** sourceURL);
 
-  nsresult GetImageFrame(nsIImageFrame** aImageFrame);
+  void GetImageFrame(nsIImageFrame** aImageFrame);
   nsresult GetXY(PRInt32* aX, PRInt32* aY);
   nsresult GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight);
 
@@ -157,31 +157,13 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
   /*
    * nsHTMLImageElement's will be created without a nsINodeInfo passed in
    * if someone says "var img = new Image();" in JavaScript, in a case like
-   * that we request the nsINodeInfo from the anonymous nodeinfo list.
+   * that we request the nsINodeInfo from the document's nodeinfo list.
    */
+  nsresult rv;
   nsCOMPtr<nsINodeInfo> nodeInfo(aNodeInfo);
   if (!nodeInfo) {
-    nsCOMPtr<nsIJSContextStack> stack =
-      do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-
-    NS_ENSURE_TRUE(stack, NS_ERROR_NOT_AVAILABLE);
-
-    JSContext *cx = nsnull;
-
-    nsresult rv = stack->Peek(&cx);
-
-    if (NS_FAILED(rv) || !cx)
-      return NS_ERROR_UNEXPECTED;
-
-    nsCOMPtr<nsIScriptGlobalObject> globalObject;
-    nsContentUtils::GetStaticScriptGlobal(cx, ::JS_GetGlobalObject(cx),
-                                          getter_AddRefs(globalObject));;
-
-    nsCOMPtr<nsIDOMWindowInternal> win(do_QueryInterface(globalObject));
-    NS_ENSURE_TRUE(win, NS_ERROR_UNEXPECTED);
-
     nsCOMPtr<nsIDOMDocument> dom_doc;
-    win->GetDocument(getter_AddRefs(dom_doc));
+    nsContentUtils::GetDocumentFromCaller(getter_AddRefs(dom_doc));
 
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(dom_doc));
     NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
@@ -202,7 +184,7 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  nsresult rv = it->Init(nodeInfo);
+  rv = it->Init(nodeInfo);
 
   if (NS_FAILED(rv)) {
     delete it;
@@ -280,19 +262,19 @@ NS_IMPL_STRING_ATTR(nsHTMLImageElement, LongDesc, longdesc)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, UseMap, usemap)
 NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Vspace, vspace)
 
-nsresult
+void
 nsHTMLImageElement::GetImageFrame(nsIImageFrame** aImageFrame)
 {
-  NS_ENSURE_ARG_POINTER(aImageFrame);
   *aImageFrame = nsnull;
+  // If we have no parent, then we won't have a frame yet
+  if (!mParent)
+    return;
 
   nsIFrame* frame = GetPrimaryFrame(PR_TRUE);
 
   if (frame) {
     CallQueryInterface(frame, aImageFrame);
   }
-
-  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -358,54 +340,10 @@ nsHTMLImageElement::GetXY(PRInt32* aX, PRInt32* aY)
     return NS_OK;
   }
 
-  nsCOMPtr<nsIContent> docElement;
-  mDocument->GetRootContent(getter_AddRefs(docElement));
-
   nsPoint origin(0, 0);
-
-  while (frame) {
-    // Add the parent's origin to our own to get to the right
-    // coordinate system
-
-    nsPoint frameOrigin;
-    frame->GetOrigin(frameOrigin);
-    origin += frameOrigin;
-
-    nsCOMPtr<nsIContent> content;
-    frame->GetContent(getter_AddRefs(content));
-
-    if (content) {
-      // If we've hit the document element, break here
-      if (content == docElement) {
-        break;
-      }
-
-      nsCOMPtr<nsIAtom> tag;
-      content->GetTag(*getter_AddRefs(tag));
-
-      if (tag == nsHTMLAtoms::body) {
-        break;
-      }
-    }
-
-    frame->GetParent(&frame);
-  }
-
-  if (frame) {
-    // For the origin subtract out the border for the parent
-    const nsStyleBorder* border = nsnull;
-    nsStyleCoord coord;
-
-    frame->GetStyleData(eStyleStruct_Border, (const nsStyleStruct*&)border);
-    if (border) {
-      if (eStyleUnit_Coord == border->mBorder.GetLeftUnit()) {
-        origin.x -= border->mBorder.GetLeft(coord).GetCoordValue();
-      }
-      if (eStyleUnit_Coord == border->mBorder.GetTopUnit()) {
-        origin.y -= border->mBorder.GetTop(coord).GetCoordValue();
-      }
-    }
-  }
+  nsIView* parentView;
+  nsresult rv = frame->GetOffsetFromView(context, origin, &parentView);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Get the scale from that Presentation Context
   float scale;
@@ -446,7 +384,7 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
     *aWidth = 0;
   }
 
-  nsIImageFrame* imageFrame = nsnull;
+  nsIImageFrame* imageFrame;
   GetImageFrame(&imageFrame);
 
   nsIFrame* frame = nsnull;
@@ -555,7 +493,8 @@ nsHTMLImageElement::StringToAttribute(nsIAtom* aAttribute,
     return NS_CONTENT_ATTR_HAS_VALUE;
   }
   else if (aAttribute == nsHTMLAtoms::src) {
-    aResult.SetStringValue(aValue);
+    static const char* kWhitespace = " \n\r\t\b";
+    aResult.SetStringValue(nsContentUtils::TrimCharsInSet(kWhitespace, aValue));
     return NS_CONTENT_ATTR_HAS_VALUE;
   }
   else if (ParseImageAttribute(aAttribute, aValue, aResult)) {
@@ -601,7 +540,7 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
 NS_IMETHODIMP
 nsHTMLImageElement::GetMappedAttributeImpact(const nsIAtom* aAttribute,
                                              PRInt32 aModType,
-                                             PRInt32& aHint) const
+                                             nsChangeHint& aHint) const
 {
   if ((aAttribute == nsHTMLAtoms::usemap) ||
       (aAttribute == nsHTMLAtoms::ismap)  ||
@@ -652,60 +591,6 @@ nsHTMLImageElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                                   aEventStatus);
 }
 
-nsresult
-nsHTMLImageElement::GetCallerSourceURL(nsIURI** sourceURL)
-{
-  // XXX Code duplicated from nsHTMLDocument
-  // XXX Question, why does this return NS_OK on failure?
-  nsresult result = NS_OK;
-
-  // We need to use the dynamically scoped global and assume that the
-  // current JSContext is a DOM context with a nsIScriptGlobalObject so
-  // that we can get the url of the caller.
-  // XXX This will fail on non-DOM contexts :(
-
-  // Get JSContext from stack.
-
-
-
-
-  // XXX: This service should be cached.
-  nsCOMPtr<nsIJSContextStack>
-    stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", &result));
-
-  if (NS_FAILED(result))
-    return NS_ERROR_FAILURE;
-
-  JSContext *cx = nsnull;
-
-  // it's possible that there is not a JSContext on the stack.
-  // specifically this can happen when the DOM is being manipulated
-  // from native (non-JS) code.
-  if (NS_FAILED(stack->Peek(&cx)) || !cx)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIScriptGlobalObject> global;
-  nsContentUtils::GetDynamicScriptGlobal(cx, getter_AddRefs(global));
-
-  nsCOMPtr<nsIDOMWindowInternal> window(do_QueryInterface(global));
-
-  if (window) {
-    nsCOMPtr<nsIDOMDocument> domDoc;
-
-    result = window->GetDocument(getter_AddRefs(domDoc));
-    nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-
-    if (doc) {
-      result = doc->GetBaseURL(*sourceURL);
-      if (!*sourceURL) {
-        doc->GetDocumentURL(sourceURL);
-      }
-    }
-  }
-
-  return result;
-}
-
 NS_IMETHODIMP
 nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
                                PRUint32 argc, jsval *argv)
@@ -721,7 +606,7 @@ nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
   JSBool ret = JS_ValueToInt32(aContext, argv[0], &width);
   NS_ENSURE_TRUE(ret, NS_ERROR_INVALID_ARG);
 
-  nsHTMLValue widthVal((PRInt32)width, eHTMLUnit_Integer);
+  nsHTMLValue widthVal((PRInt32)width, eHTMLUnit_Pixel);
 
   nsresult rv = SetHTMLAttribute(nsHTMLAtoms::width, widthVal, PR_FALSE);
 
@@ -731,7 +616,7 @@ nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
     ret = JS_ValueToInt32(aContext, argv[1], &height);
     NS_ENSURE_TRUE(ret, NS_ERROR_INVALID_ARG);
 
-    nsHTMLValue heightVal((PRInt32)height, eHTMLUnit_Integer);
+    nsHTMLValue heightVal((PRInt32)height, eHTMLUnit_Pixel);
 
     rv = SetHTMLAttribute(nsHTMLAtoms::height, heightVal, PR_FALSE);
   }
@@ -865,8 +750,9 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
 {
   nsresult result = SetAttr(kNameSpaceID_None, nsHTMLAtoms::src, aSrc,
                             PR_TRUE);
+  NS_ENSURE_SUCCESS(result, result);
 
-  if (NS_SUCCEEDED(result) && !mDocument) {
+  if (!mDocument) {
     nsCOMPtr<nsIDocument> doc;
     mNodeInfo->GetDocument(*getter_AddRefs(doc));
 
@@ -928,15 +814,32 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
 
         nsCOMPtr<nsIDocument> doc;
         nsCOMPtr<nsILoadGroup> loadGroup;
+        nsCOMPtr<nsIURI> documentURI;
         shell->GetDocument(getter_AddRefs(doc));
         if (doc) {
           doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+
+          // Get the documment URI for the referrer.
+          doc->GetDocumentURL(getter_AddRefs(documentURI));
         }
 
-        il->LoadImage(uri, nsnull, loadGroup, this, context, nsIRequest::LOAD_NORMAL,
+        // XXX: initialDocumentURI is NULL!
+        il->LoadImage(uri, nsnull, documentURI, loadGroup, this, context, nsIRequest::LOAD_NORMAL,
                       nsnull, nsnull, getter_AddRefs(mRequest));
       }
     }
+  } else {
+    nsIImageFrame* imageFrame;
+    GetImageFrame(&imageFrame);
+    if (!imageFrame) {
+      // If we don't have an image frame, reconstruct the frame
+      // so that the new image can load.
+      nsCOMPtr<nsIPresShell> shell;
+      mDocument->GetShellAt(0, getter_AddRefs(shell));
+      if (shell) {
+        shell->RecreateFramesFor(this);
+      }
+    }
   }
 
   return result;
@@ -963,13 +866,23 @@ nsHTMLImageElement::SetSrc(const nsAString& aSrc)
   nsCOMPtr<nsIURI> baseURL;
   nsresult rv = NS_OK;
 
-  (void) GetCallerSourceURL(getter_AddRefs(baseURL));
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  nsContentUtils::GetDocumentFromCaller(getter_AddRefs(domDoc));
 
-  nsCOMPtr<nsIDocument> doc;
-  mNodeInfo->GetDocument(*getter_AddRefs(doc));
-
-  if (doc && !baseURL) {
+  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
+  if (doc) {
+    // XXX GetBaseURL should do the GetDocumentURL for us
     rv = doc->GetBaseURL(*getter_AddRefs(baseURL));
+    if (!baseURL) {
+      rv = doc->GetDocumentURL(getter_AddRefs(baseURL));
+    }
+  }
+
+  if (!baseURL) {
+    mNodeInfo->GetDocument(*getter_AddRefs(doc));
+    if (doc) {
+      rv = doc->GetBaseURL(*getter_AddRefs(baseURL));
+    }
   }
 
   if (NS_SUCCEEDED(rv)) {
@@ -996,15 +909,15 @@ nsHTMLImageElement::GetNaturalHeight(PRInt32* aNaturalHeight)
 
   *aNaturalHeight = 0;
 
-  nsIImageFrame* imageFrame = nsnull;
-  nsresult rv = GetImageFrame(&imageFrame);
+  nsIImageFrame* imageFrame;
+  GetImageFrame(&imageFrame);
 
-  if (NS_FAILED(rv) || !imageFrame)
+  if (!imageFrame)
     return NS_OK; // don't throw JS exceptions in this case
 
   PRUint32 width, height;
 
-  rv = imageFrame->GetNaturalImageSize(&width, &height);
+  nsresult rv = imageFrame->GetNaturalImageSize(&width, &height);
 
   if (NS_FAILED(rv))
     return NS_OK;
@@ -1022,14 +935,14 @@ nsHTMLImageElement::GetNaturalWidth(PRInt32* aNaturalWidth)
   *aNaturalWidth = 0;
 
   nsIImageFrame* imageFrame;
-  nsresult rv = GetImageFrame(&imageFrame);
+  GetImageFrame(&imageFrame);
 
-  if (NS_FAILED(rv) || !imageFrame)
+  if (!imageFrame)
     return NS_OK; // don't throw JS exceptions in this case
 
   PRUint32 width, height;
 
-  rv = imageFrame->GetNaturalImageSize(&width, &height);
+  nsresult rv = imageFrame->GetNaturalImageSize(&width, &height);
 
   if (NS_FAILED(rv))
     return NS_OK;
