@@ -40,7 +40,6 @@
 #include "nsIDOMNSHTMLTextAreaElement.h"
 #include "nsITextControlElement.h"
 #include "nsIControllers.h"
-#include "nsIEditorController.h"
 #include "nsContentCID.h"
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
@@ -133,7 +132,7 @@ public:
                                nsHTMLValue& aResult);
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
@@ -200,8 +199,6 @@ nsHTMLTextAreaElement::nsHTMLTextAreaElement()
 
 nsHTMLTextAreaElement::~nsHTMLTextAreaElement()
 {
-  // Null out form's pointer to us - no ref counting here!
-  SetForm(nsnull);
 }
 
 
@@ -615,13 +612,13 @@ MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
   if (!aAttributes || !aData)
     return;
 
-  nsGenericHTMLElement::MapAlignAttributeInto(aAttributes, aData);
+  nsGenericHTMLElement::MapDivAlignAttributeInto(aAttributes, aData);
   nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
 NS_IMETHODIMP
 nsHTMLTextAreaElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                                PRInt32& aHint) const
+                                                nsChangeHint& aHint) const
 {
   // XXX Bug 50280 - It is unclear why we need to do this here for 
   // rows and cols and why the AttributeChanged method in
@@ -768,12 +765,9 @@ nsHTMLTextAreaElement::DoneAddingChildren()
 NS_IMETHODIMP
 nsHTMLTextAreaElement::GetType(PRInt32* aType)
 {
-  if (aType) {
-    *aType = NS_FORM_TEXTAREA;
-    return NS_OK;
-  } else {
-    return NS_FORM_NOTOK;
-  }
+  NS_ASSERTION(aType, "Null pointer bad!");
+  *aType = NS_FORM_TEXTAREA;
+  return NS_OK;
 }
 
 nsresult
@@ -823,14 +817,6 @@ nsHTMLTextAreaElement::GetControllers(nsIControllers** aResult)
     if (NS_FAILED(rv))
       return rv;
 
-    nsCOMPtr<nsIEditorController> editorController = do_QueryInterface(controller, &rv);
-    if (NS_FAILED(rv))
-      return rv;
-
-    rv = editorController->Init(nsnull);
-    if (NS_FAILED(rv))
-      return rv;
-
     mControllers->AppendController(controller);
   }
 
