@@ -51,17 +51,18 @@
 #include "nsIRDFService.h"
 #include "pldhash.h"
 #include "nsIHttpChannel.h"
+#include "nsIHTMLCSSStyleSheet.h"
+#include "nsIHTMLStyleSheet.h"
 
 // Document.Write() related
 #include "nsIWyciwygChannel.h"
 #include "nsILoadGroup.h"
 #include "nsNetUtil.h"
 
+#include "nsICommandManager.h"
 
 class nsBaseContentList;
 class nsContentList;
-class nsIHTMLStyleSheet;
-class nsIHTMLCSSStyleSheet;
 class nsIParser;
 class nsICSSLoader;
 class nsIURI;
@@ -108,7 +109,7 @@ public:
 
   NS_IMETHOD RemoveImageMap(nsIDOMHTMLMapElement* aMap);
 
-  NS_IMETHOD GetImageMap(const nsString& aMapName,
+  NS_IMETHOD GetImageMap(const nsAString& aMapName,
                          nsIDOMHTMLMapElement** aResult);
 
   NS_IMETHOD GetAttributeStyleSheet(nsIHTMLStyleSheet** aStyleSheet);
@@ -161,7 +162,25 @@ public:
   NS_DECL_NSIDOM3NODE
 
   // nsIDOMHTMLDocument interface
-  NS_DECL_NSIDOMHTMLDOCUMENT
+  NS_IMETHOD GetTitle(nsAString & aTitle);
+  NS_IMETHOD SetTitle(const nsAString & aTitle);
+  NS_IMETHOD GetReferrer(nsAString & aReferrer);
+  NS_IMETHOD GetURL(nsAString & aURL);
+  NS_IMETHOD GetBody(nsIDOMHTMLElement * *aBody);
+  NS_IMETHOD SetBody(nsIDOMHTMLElement * aBody);
+  NS_IMETHOD GetImages(nsIDOMHTMLCollection * *aImages);
+  NS_IMETHOD GetApplets(nsIDOMHTMLCollection * *aApplets);
+  NS_IMETHOD GetLinks(nsIDOMHTMLCollection * *aLinks);
+  NS_IMETHOD GetForms(nsIDOMHTMLCollection * *aForms);
+  NS_IMETHOD GetAnchors(nsIDOMHTMLCollection * *aAnchors);
+  NS_IMETHOD GetCookie(nsAString & aCookie);
+  NS_IMETHOD SetCookie(const nsAString & aCookie);
+  NS_IMETHOD Open(void);
+  NS_IMETHOD Close(void);
+  NS_IMETHOD Write(const nsAString & text);
+  NS_IMETHOD Writeln(const nsAString & text);
+  NS_IMETHOD GetElementsByName(const nsAString & elementName,
+                               nsIDOMNodeList **_retval); 
 
   // nsIDOMNSHTMLDocument interface
   NS_DECL_NSIDOMNSHTMLDOCUMENT
@@ -203,6 +222,9 @@ protected:
                                      PRUint32 aFlags);
   virtual void InternalInsertStyleSheetAt(nsIStyleSheet* aSheet,
                                           PRInt32 aIndex);
+  virtual already_AddRefed<nsIStyleSheet> InternalGetStyleSheetAt(PRInt32 aIndex);
+  virtual PRInt32 InternalGetNumberOfStyleSheets();
+
   static PRBool MatchLinks(nsIContent *aContent, nsString* aData);
   static PRBool MatchAnchors(nsIContent *aContent, nsString* aData);
   static PRBool MatchLayers(nsIContent *aContent, nsString* aData);
@@ -228,8 +250,8 @@ protected:
 
   nsresult BaseResetToURI(nsIURI* aURI);
 
-  nsIHTMLStyleSheet*    mAttrStyleSheet;
-  nsIHTMLCSSStyleSheet* mStyleAttrStyleSheet;
+  nsCOMPtr<nsIHTMLStyleSheet> mAttrStyleSheet;
+  nsCOMPtr<nsIHTMLCSSStyleSheet> mStyleAttrStyleSheet;
   nsIURI*     mBaseURL;
   nsString*   mBaseTarget;
   nsString*   mLastModified;
@@ -252,9 +274,7 @@ protected:
   PRInt32 mNumForms;
 
 //ahmed 12-2
-#ifdef IBMBIDI
   PRInt32  mTexttype;
-#endif
   
   static nsrefcnt gRefCntRDFService;
   static nsIRDFService* gRDF;
@@ -303,6 +323,18 @@ protected:
   PLDHashTable mIdAndNameHashTable;
 
   nsCOMPtr<nsIWyciwygChannel> mWyciwygChannel;
+
+  /* Midas implementation */
+  nsresult   GetMidasCommandManager(nsICommandManager** aCommandManager);
+  PRBool     ConvertToMidasInternalCommand(const nsAString & inCommandID,
+                                           const nsAString & inParam,
+                                           nsACString& outCommandID,
+                                           nsACString& outParam,
+                                           PRBool& isBoolean,
+                                           PRBool& boolValue);
+  nsCOMPtr<nsICommandManager> mMidasCommandManager;
+  PRBool                      mEditingIsOn;
+
 };
 
 #endif /* nsHTMLDocument_h___ */
