@@ -40,7 +40,6 @@
 #include "nsIHTMLFragmentContentSink.h"
 #include "nsIParser.h"
 #include "nsIParserService.h"
-#include "nsParserCIID.h"
 #include "nsIHTMLContent.h"
 #include "nsIHTMLContentContainer.h"
 #include "nsHTMLAtoms.h"
@@ -68,8 +67,6 @@
 // at some pointe really soon!
 //
 
-static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
-
 class nsHTMLFragmentContentSink : public nsIHTMLFragmentContentSink {
 public:
   nsHTMLFragmentContentSink();
@@ -85,7 +82,7 @@ public:
   NS_IMETHOD WillResume(void);
   NS_IMETHOD SetParser(nsIParser* aParser);  
   NS_IMETHOD OpenContainer(const nsIParserNode& aNode);
-  NS_IMETHOD CloseContainer(const nsIParserNode& aNode);
+  NS_IMETHOD CloseContainer(const nsHTMLTag aTag);
   NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
   NS_IMETHOD AddComment(const nsIParserNode& aNode);
   NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode);
@@ -96,22 +93,22 @@ public:
   NS_IMETHOD EndContext(PRInt32 aID);
   NS_IMETHOD SetTitle(const nsString& aValue);
   NS_IMETHOD OpenHTML(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHTML(const nsIParserNode& aNode);
+  NS_IMETHOD CloseHTML();
   NS_IMETHOD OpenHead(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHead(const nsIParserNode& aNode);
+  NS_IMETHOD CloseHead();
   NS_IMETHOD OpenBody(const nsIParserNode& aNode);
-  NS_IMETHOD CloseBody(const nsIParserNode& aNode);
+  NS_IMETHOD CloseBody();
   NS_IMETHOD OpenForm(const nsIParserNode& aNode);
-  NS_IMETHOD CloseForm(const nsIParserNode& aNode);
+  NS_IMETHOD CloseForm();
   NS_IMETHOD OpenFrameset(const nsIParserNode& aNode);
-  NS_IMETHOD CloseFrameset(const nsIParserNode& aNode);
+  NS_IMETHOD CloseFrameset();
   NS_IMETHOD OpenNoscript(const nsIParserNode& aNode);
-  NS_IMETHOD CloseNoscript(const nsIParserNode& aNode);
+  NS_IMETHOD CloseNoscript();
   NS_IMETHOD GetPref(PRInt32 aTag,PRBool& aPref) { return NS_OK; }
   NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
   
   NS_IMETHOD OpenMap(const nsIParserNode& aNode);
-  NS_IMETHOD CloseMap(const nsIParserNode& aNode);
+  NS_IMETHOD CloseMap();
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
   NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
   NS_IMETHOD WillProcessTokens(void) { return NS_OK; }
@@ -164,6 +161,35 @@ public:
   nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
 };
 
+class nsHTMLFragmentContentSink2 : public nsHTMLFragmentContentSink 
+{
+public:
+  nsHTMLFragmentContentSink2() { mHitSentinel = PR_TRUE; mSeenBody = PR_FALSE;}
+  virtual ~nsHTMLFragmentContentSink2() {}
+  NS_IMETHODIMP OpenHead(const nsIParserNode& aNode) { return OpenContainer(aNode); }
+  NS_IMETHODIMP CloseHead() { return CloseContainer(eHTMLTag_head); }
+};
+
+nsresult
+NS_NewHTMLFragmentContentSink2(nsIHTMLFragmentContentSink** aResult)
+{
+  NS_PRECONDITION(nsnull != aResult, "null ptr");
+  if (nsnull == aResult) {
+    return NS_ERROR_NULL_POINTER;
+  }
+  nsHTMLFragmentContentSink2* it;
+  NS_NEWXPCOM(it, nsHTMLFragmentContentSink2);
+  if (nsnull == it) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  nsresult rv = it->Init();
+  if (NS_FAILED(rv)) {
+    delete it;
+    return rv;
+  }
+
+  return it->QueryInterface(NS_GET_IID(nsIHTMLFragmentContentSink), (void **)aResult);
+}
 
 nsresult
 NS_NewHTMLFragmentContentSink(nsIHTMLFragmentContentSink** aResult)
@@ -188,7 +214,7 @@ NS_NewHTMLFragmentContentSink(nsIHTMLFragmentContentSink** aResult)
 
 nsHTMLFragmentContentSink::nsHTMLFragmentContentSink()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mHitSentinel = PR_FALSE;
   mSeenBody = PR_TRUE;
   mRoot = nsnull;
@@ -249,6 +275,12 @@ nsHTMLFragmentContentSink::QueryInterface(REFNSIID aIID, void** aInstancePtr)
     NS_ADDREF_THIS();
     return NS_OK;
   } 
+  if (aIID.Equals(NS_GET_IID(nsIContentSink))) {
+    nsIContentSink* tmp = this;
+    *aInstancePtr = (void*) tmp;
+    NS_ADDREF_THIS();
+    return NS_OK;
+  } 
   if (aIID.Equals(kISupportsIID)) {
     *aInstancePtr = (void*) ((nsISupports*)this);
     NS_ADDREF_THIS();
@@ -263,11 +295,7 @@ nsresult nsHTMLFragmentContentSink::Init()
   nsresult rv = NS_NewNodeInfoManager(getter_AddRefs(mNodeInfoManager));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsINameSpaceManager> nsmgr;
-  rv = NS_NewNameSpaceManager(getter_AddRefs(nsmgr));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return mNodeInfoManager->Init(nsnull, nsmgr);
+  return mNodeInfoManager->Init(nsnull);
 }
 
 
@@ -372,7 +400,7 @@ nsHTMLFragmentContentSink::OpenHTML(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseHTML(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseHTML()
 {
   return NS_OK;
 }
@@ -380,12 +408,11 @@ nsHTMLFragmentContentSink::CloseHTML(const nsIParserNode& aNode)
 NS_IMETHODIMP 
 nsHTMLFragmentContentSink::OpenHead(const nsIParserNode& aNode)
 {
-  // XXX Not likely to get a head in the fragment
   return NS_OK;
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseHead(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseHead()
 {
   return NS_OK;
 }
@@ -405,9 +432,9 @@ nsHTMLFragmentContentSink::OpenBody(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseBody(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseBody()
 {
-  return CloseContainer(aNode);
+  return CloseContainer(eHTMLTag_body);
 }
 
 NS_IMETHODIMP 
@@ -417,9 +444,9 @@ nsHTMLFragmentContentSink::OpenForm(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseForm(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseForm()
 {
-  return CloseContainer(aNode);
+  return CloseContainer(eHTMLTag_form);
 }
 
 NS_IMETHODIMP 
@@ -429,9 +456,9 @@ nsHTMLFragmentContentSink::OpenFrameset(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseFrameset(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseFrameset()
 {
-  return CloseContainer(aNode);
+  return CloseContainer(eHTMLTag_frameset);
 }
 
 NS_IMETHODIMP 
@@ -441,9 +468,9 @@ nsHTMLFragmentContentSink::OpenMap(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseMap(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseMap()
 {
-  return CloseContainer(aNode);
+  return CloseContainer(eHTMLTag_map);
 }
 
 NS_IMETHODIMP 
@@ -453,9 +480,9 @@ nsHTMLFragmentContentSink::OpenNoscript(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseNoscript(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseNoscript()
 {
-  return CloseContainer(aNode);
+  return CloseContainer(eHTMLTag_noscript);
 }
 
 void
@@ -501,10 +528,6 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
     nsHTMLTag nodeType = nsHTMLTag(aNode.GetNodeType());
     nsIHTMLContent *content = nsnull;
 
-    nsCOMPtr<nsIParserService> parserService = 
-             do_GetService(kParserServiceCID, &result);
-    NS_ENSURE_SUCCESS(result, result);
-
     nsCOMPtr<nsINodeInfo> nodeInfo;
 
     if (nodeType == eHTMLTag_userdefined) {
@@ -513,6 +536,11 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
                                       kNameSpaceID_None,
                                       *getter_AddRefs(nodeInfo));
     } else {
+      nsIParserService* parserService =
+        nsContentUtils::GetParserServiceWeakRef();
+      if (!parserService)
+        return NS_ERROR_OUT_OF_MEMORY;
+
       const PRUnichar *name = nsnull;
 
       parserService->HTMLIdToStringTag(nodeType, &name);
@@ -557,7 +585,7 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
 }
 
 NS_IMETHODIMP 
-nsHTMLFragmentContentSink::CloseContainer(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::CloseContainer(const nsHTMLTag aTag)
 {
   if (mHitSentinel && (nsnull != GetCurrentContent())) {
     nsIContent* content;
@@ -583,10 +611,10 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
         nsCOMPtr<nsIHTMLContent> content;
         nsHTMLTag nodeType = nsHTMLTag(aNode.GetNodeType());
 
-        nsCOMPtr<nsIParserService> parserService = 
-                 do_GetService(kParserServiceCID, &result);
-
-        NS_ENSURE_SUCCESS(result, result);
+        nsIParserService* parserService =
+          nsContentUtils::GetParserServiceWeakRef();
+        if (!parserService)
+          return NS_ERROR_OUT_OF_MEMORY;
 
         nsCOMPtr<nsINodeInfo> nodeInfo;
 
@@ -828,13 +856,11 @@ nsHTMLFragmentContentSink::AddTextToContent(nsIHTMLContent* aContent,const nsStr
   
   if(aContent) {
     if (!aText.IsEmpty()) {
-      nsCOMPtr<nsIContent> text;
+      nsCOMPtr<nsITextContent> text;
       result = NS_NewTextNode(getter_AddRefs(text));
       if (NS_SUCCEEDED(result)) {
-        nsCOMPtr<nsIDOMText> tc=do_QueryInterface(text,&result);
-        if (NS_SUCCEEDED(result)) {
-          tc->SetData(aText);
-        }
+        text->SetText(aText, PR_TRUE);
+
         result=aContent->AppendChildTo(text, PR_FALSE, PR_FALSE);
       }
     }
@@ -845,33 +871,28 @@ nsHTMLFragmentContentSink::AddTextToContent(nsIHTMLContent* aContent,const nsStr
 nsresult
 nsHTMLFragmentContentSink::FlushText()
 {
-  nsresult rv = NS_OK;
-  if (0 != mTextLength) {
-    nsIContent* content;
-    rv = NS_NewTextNode(&content);
-    if (NS_OK == rv) {
-      
-      // Set the text in the text node
-      nsITextContent* text = nsnull;
-      content->QueryInterface(NS_GET_IID(nsITextContent), (void**) &text);
-      text->SetText(mText, mTextLength, PR_FALSE);
-      NS_RELEASE(text);
-
-      // Add text to its parent
-      nsIContent *parent = GetCurrentContent();
-      
-      if (nsnull == parent) {
-        parent = mRoot;
-      }
-      
-      parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
-
-      NS_RELEASE(content);
-    }
-
-    mTextLength = 0;
+  if (0 == mTextLength) {
+    return NS_OK;
   }
 
+  nsCOMPtr<nsITextContent> content;
+  nsresult rv = NS_NewTextNode(getter_AddRefs(content));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Set the text in the text node
+  content->SetText(mText, mTextLength, PR_FALSE);
+
+  // Add text to its parent
+  nsIContent *parent = GetCurrentContent();
+
+  if (!parent) {
+    parent = mRoot;
+  }
+
+  rv = parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
+
+  mTextLength = 0;
+
   return rv;
 }
 
