@@ -114,10 +114,6 @@ public:
 #define NS_CSS_GETINFO_CSS2         ((PRUint32) 0x00000004L)
 #define NS_CSS_GETINFO_CSS_FROSTING ((PRUint32) 0x00000008L)
 
-// Success code that can be returned from ParseAndAppendDeclaration()
-#define NS_CSS_PARSER_DROP_DECLARATION \
-  NS_ERROR_GENERATE_SUCCESS(NS_ERROR_MODULE_LAYOUT,1)
-
 extern NS_EXPORT nsresult
   NS_NewCSSParser(nsICSSParser** aInstancePtrResult);
 
