@@ -20,7 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   L. David Baron <dbaron@fas.harvard.edu>
+ *   L. David Baron <dbaron@dbaron.org>
  *   Daniel Glazman <glazman@netscape.com>
  *
  *
@@ -216,7 +216,7 @@ void nsCSSScanner::Init(nsIUnicharInputStream* aInput, nsIURI* aURI)
 
 #ifdef CSS_REPORT_PARSE_ERRORS
   if (aURI) {
-    aURI->GetSpec(getter_Copies(mFileName));
+    aURI->GetSpec(mFileName);
   } else {
     mFileName.Adopt(nsCRT::strdup("from DOM"));
   }
@@ -227,14 +227,19 @@ void nsCSSScanner::Init(nsIUnicharInputStream* aInput, nsIURI* aURI)
 
 #ifdef CSS_REPORT_PARSE_ERRORS
 
-void nsCSSScanner::AddToError(const nsAReadableString& aErrorText)
+// @see REPORT_UNEXPECTED_EOF in nsCSSParser.cpp
+#define REPORT_UNEXPECTED_EOF(err_) \
+  AddToError(NS_LITERAL_STRING("Unexpected end of file while searching for ") + err_ + NS_LITERAL_STRING("."))
+
+void nsCSSScanner::AddToError(const nsAString& aErrorText)
 {
-  if (!mError.Length()) {
+  if (mError.IsEmpty()) {
     mErrorLineNumber = mLineNumber;
     mErrorColNumber = mColNumber;
     mError = aErrorText;
   } else {
-    mError.Append(NS_LITERAL_STRING("  ") + aErrorText);
+    // XXX nsAutoString is workaround for string hang bug (bug 74709)!
+    mError.Append(NS_LITERAL_STRING("  ") + nsAutoString(aErrorText));
   }
 }
 
@@ -245,14 +250,12 @@ void nsCSSScanner::ClearError()
 
 void nsCSSScanner::OutputError()
 {
-  if (!mError.Length()) return;
+  if (mError.IsEmpty()) return;
  
 #ifdef DEBUG
-  printf("CSS Error (%s :%u.%u): %s\n",
-         mFileName.get(),
-         mErrorLineNumber,
-         mErrorColNumber,
-         NS_ConvertUCS2toUTF8(mError).get());
+  fprintf(stderr, "CSS Error (%s :%u.%u): %s\n",
+                  mFileName.get(), mErrorLineNumber, mErrorColNumber,
+                  NS_ConvertUCS2toUTF8(mError).get());
 #endif
 
   // Log it to the JavaScript console
@@ -277,7 +280,8 @@ void nsCSSScanner::OutputError()
   }
   ClearError();
 }
-
+#else
+#define REPORT_UNEXPECTED_EOF(err_)
 #endif // CSS_REPORT_PARSE_ERRORS
 
 PRUint32 nsCSSScanner::GetLineNumber()
@@ -498,10 +502,17 @@ PRBool nsCSSScanner::Next(PRInt32& aErrorCode, nsCSSToken& aToken)
       PRInt32 nextChar = Peek(aErrorCode);
       if (nextChar == '*') {
         (void) Read(aErrorCode);
+#if 0
+        // If we change our storage data structures such that comments are
+        // stored (for Editor), we should reenable this code, condition it
+        // on being in editor mode, and apply glazou's patch from bug
+        // 60290.
         aToken.mIdent.SetCapacity(2);
         aToken.mIdent.Assign(PRUnichar(ch));
         aToken.mIdent.Append(PRUnichar(nextChar));
         return ParseCComment(aErrorCode, aToken);
+#endif
+        return SkipCComment(aErrorCode) && Next(aErrorCode, aToken);
       }
     }
     if (ch == '<') {  // consume HTML comment tags
@@ -586,10 +597,17 @@ PRBool nsCSSScanner::NextURL(PRInt32& aErrorCode, nsCSSToken& aToken)
       PRInt32 nextChar = Peek(aErrorCode);
       if (nextChar == '*') {
         (void) Read(aErrorCode);
+#if 0
+        // If we change our storage data structures such that comments are
+        // stored (for Editor), we should reenable this code, condition it
+        // on being in editor mode, and apply glazou's patch from bug
+        // 60290.
         aToken.mIdent.SetCapacity(2);
         aToken.mIdent.Assign(PRUnichar(ch));
         aToken.mIdent.Append(PRUnichar(nextChar));
         return ParseCComment(aErrorCode, aToken);
+#endif
+        return SkipCComment(aErrorCode) && Next(aErrorCode, aToken);
       }
     }
 
@@ -859,6 +877,23 @@ PRBool nsCSSScanner::ParseNumber(PRInt32& aErrorCode, PRInt32 c,
   return PR_TRUE;
 }
 
+PRBool nsCSSScanner::SkipCComment(PRInt32& aErrorCode)
+{
+  for (;;) {
+    PRInt32 ch = Read(aErrorCode);
+    if (ch < 0) break;
+    if (ch == '*') {
+      if (LookAhead(aErrorCode, '/')) {
+        return PR_TRUE;
+      }
+    }
+  }
+
+  REPORT_UNEXPECTED_EOF(NS_LITERAL_STRING("end of comment"));
+  return PR_FALSE;
+}
+
+#if 0
 PRBool nsCSSScanner::ParseCComment(PRInt32& aErrorCode, nsCSSToken& aToken)
 {
   nsString& ident = aToken.mIdent;
@@ -879,6 +914,7 @@ PRBool nsCSSScanner::ParseCComment(PRInt32& aErrorCode, nsCSSToken& aToken)
   aToken.mType = eCSSToken_WhiteSpace;
   return PR_TRUE;
 }
+#endif
 
 #if 0
 PRBool nsCSSScanner::ParseEOLComment(PRInt32& aErrorCode, nsCSSToken& aToken)
