@@ -43,7 +43,7 @@
 #include "nsIAtom.h"
 #include "nsIURL.h"
 #include "nsISupportsArray.h"
-#include "nsHTMLAtoms.h"
+#include "nsCSSPseudoElements.h"
 #include "nsIHTMLContent.h"
 #include "nsIStyleRule.h"
 #include "nsIFrame.h"
@@ -67,10 +67,7 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aValue) const;
-  NS_IMETHOD HashValue(PRUint32& aValue) const;
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
   
   // The new mapping function.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
@@ -87,7 +84,7 @@ public:
 CSSFirstLineRule::CSSFirstLineRule(nsIHTMLCSSStyleSheet* aSheet)
   : mSheet(aSheet)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 CSSFirstLineRule::~CSSFirstLineRule()
@@ -96,20 +93,6 @@ CSSFirstLineRule::~CSSFirstLineRule()
 
 NS_IMPL_ISUPPORTS1(CSSFirstLineRule, nsIStyleRule)
 
-NS_IMETHODIMP
-CSSFirstLineRule::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  aResult = PRBool(this == aRule);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-CSSFirstLineRule::HashValue(PRUint32& aValue) const
-{
-  aValue = (PRUint32)7;         // XXX got a better suggestion?
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 CSSFirstLineRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
 {
@@ -118,14 +101,6 @@ CSSFirstLineRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
   return NS_OK;
 }
 
-// Strength is an out-of-band weighting, always 0 here
-NS_IMETHODIMP
-CSSFirstLineRule::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 CSSFirstLineRule::MapRuleInfoInto(nsRuleData* aData)
 {
@@ -211,15 +186,9 @@ CSSFirstLetterRule::CSSFirstLetterRule(nsIHTMLCSSStyleSheet* aSheet)
 class HTMLCSSStyleSheetImpl : public nsIHTMLCSSStyleSheet,
                               public nsIStyleRuleProcessor {
 public:
-  void* operator new(size_t size) CPP_THROW_NEW;
-  void* operator new(size_t size, nsIArena* aArena) CPP_THROW_NEW;
-  void operator delete(void* ptr);
-
   HTMLCSSStyleSheetImpl();
 
-  NS_IMETHOD QueryInterface(const nsIID& aIID, void** aInstancePtr);
-  NS_IMETHOD_(nsrefcnt) AddRef();
-  NS_IMETHOD_(nsrefcnt) Release();
+  NS_DECL_ISUPPORTS
 
   // basic style sheet data
   NS_IMETHOD Init(nsIURI* aURL, nsIDocument* aDocument);
@@ -231,9 +200,13 @@ public:
   NS_IMETHOD GetMediumAt(PRInt32 aIndex, nsIAtom*& aMedium) const;
   NS_IMETHOD_(PRBool) UseForMedium(nsIAtom* aMedium) const;
 
-  NS_IMETHOD GetEnabled(PRBool& aEnabled) const;
+  NS_IMETHOD GetApplicable(PRBool& aApplicable) const;
+  
   NS_IMETHOD SetEnabled(PRBool aEnabled);
 
+  NS_IMETHOD GetComplete(PRBool& aComplete) const;
+  NS_IMETHOD SetComplete();
+
   // style sheet owner info
   NS_IMETHOD GetParentSheet(nsIStyleSheet*& aParent) const;  // will be null
   NS_IMETHOD GetOwningDocument(nsIDocument*& aDocument) const;
@@ -274,10 +247,6 @@ protected:
   virtual ~HTMLCSSStyleSheetImpl();
 
 protected:
-  PRUint32 mInHeap : 1;
-  PRUint32 mRefCnt : 31;
-  NS_DECL_OWNINGTHREAD // for thread-safety checking
-
   nsIURI*         mURL;
   nsIDocument*    mDocument;
 
@@ -286,42 +255,9 @@ protected:
 };
 
 
-void* HTMLCSSStyleSheetImpl::operator new(size_t size) CPP_THROW_NEW
-{
-  HTMLCSSStyleSheetImpl* rv = (HTMLCSSStyleSheetImpl*) ::operator new(size);
-#ifdef NS_DEBUG
-  if (nsnull != rv) {
-    memset(rv, 0xEE, size);
-  }
-#endif
-  rv->mInHeap = 1;
-  return (void*) rv;
-}
-
-void* HTMLCSSStyleSheetImpl::operator new(size_t size, nsIArena* aArena) CPP_THROW_NEW
-{
-  HTMLCSSStyleSheetImpl* rv = (HTMLCSSStyleSheetImpl*) aArena->Alloc(PRInt32(size));
-#ifdef NS_DEBUG
-  if (nsnull != rv) {
-    memset(rv, 0xEE, size);
-  }
-#endif
-  rv->mInHeap = 0;
-  return (void*) rv;
-}
-
-void HTMLCSSStyleSheetImpl::operator delete(void* ptr)
-{
-  HTMLCSSStyleSheetImpl* sheet = (HTMLCSSStyleSheetImpl*) ptr;
-  if (nsnull != sheet) {
-    if (sheet->mInHeap) {
-      ::operator delete(ptr);
-    }
-  }
-}
-
 HTMLCSSStyleSheetImpl::HTMLCSSStyleSheetImpl()
   : nsIHTMLCSSStyleSheet(),
+    mRefCnt(0),
     mURL(nsnull),
     mDocument(nsnull),
     mFirstLineRule(nsnull),
@@ -363,9 +299,13 @@ HTMLCSSStyleSheetImpl::RulesMatching(ElementRuleProcessorData* aData,
 {
   nsIStyledContent *styledContent = aData->mStyledContent;
   
-  if (styledContent) 
+  if (styledContent) {
     // just get the one and only style rule from the content's STYLE attribute
-    styledContent->WalkInlineStyleRules(aData->mRuleWalker);
+    nsCOMPtr<nsIStyleRule> rule;
+    styledContent->GetInlineStyleRule(getter_AddRefs(rule));
+    if (rule)
+      aData->mRuleWalker->Forward(rule, PR_TRUE);
+  }
 
   return NS_OK;
 }
@@ -382,7 +322,7 @@ HTMLCSSStyleSheetImpl::RulesMatching(PseudoRuleProcessorData* aData,
     return NS_OK;
 
   nsIAtom* pseudoTag = aData->mPseudoTag;
-  if (pseudoTag == nsHTMLAtoms::firstLinePseudo) {
+  if (pseudoTag == nsCSSPseudoElements::firstLine) {
     if (!mFirstLineRule) {
       mFirstLineRule = new CSSFirstLineRule(this);
       if (!mFirstLineRule)
@@ -391,7 +331,7 @@ HTMLCSSStyleSheetImpl::RulesMatching(PseudoRuleProcessorData* aData,
     }
     aData->mRuleWalker->Forward(mFirstLineRule);
   }
-  else if (pseudoTag == nsHTMLAtoms::firstLetterPseudo) {
+  else if (pseudoTag == nsCSSPseudoElements::firstLetter) {
     if (!mFirstLetterRule) {
       mFirstLetterRule = new CSSFirstLetterRule(this);
       if (!mFirstLetterRule)
@@ -491,9 +431,9 @@ HTMLCSSStyleSheetImpl::UseForMedium(nsIAtom* aMedium) const
 }
 
 NS_IMETHODIMP
-HTMLCSSStyleSheetImpl::GetEnabled(PRBool& aEnabled) const
+HTMLCSSStyleSheetImpl::GetApplicable(PRBool& aApplicable) const
 {
-  aEnabled = PR_TRUE;
+  aApplicable = PR_TRUE;
   return NS_OK;
 }
 
@@ -503,6 +443,19 @@ HTMLCSSStyleSheetImpl::SetEnabled(PRBool aEnabled)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+HTMLCSSStyleSheetImpl::GetComplete(PRBool& aComplete) const
+{
+  aComplete = PR_TRUE;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+HTMLCSSStyleSheetImpl::SetComplete()
+{
+  return NS_OK;
+}
+
 // style sheet owner info
 NS_IMETHODIMP
 HTMLCSSStyleSheetImpl::GetParentSheet(nsIStyleSheet*& aParent) const
