@@ -56,6 +56,7 @@
 #include "nsLayoutCID.h"
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLElement.h"
+#include "nsCSSAnonBoxes.h"
 
 #include "nsRuleWalker.h"
 
@@ -742,9 +743,13 @@ public:
   NS_IMETHOD GetMediumAt(PRInt32 aIndex, nsIAtom*& aMedium) const;
   NS_IMETHOD_(PRBool) UseForMedium(nsIAtom* aMedium) const;
 
-  NS_IMETHOD GetEnabled(PRBool& aEnabled) const;
+  NS_IMETHOD GetApplicable(PRBool& aApplicable) const;
+  
   NS_IMETHOD SetEnabled(PRBool aEnabled);
 
+  NS_IMETHOD GetComplete(PRBool& aComplete) const;
+  NS_IMETHOD SetComplete();
+  
   // style sheet owner info
   NS_IMETHOD GetParentSheet(nsIStyleSheet*& aParent) const;  // will be null
   NS_IMETHOD GetOwningDocument(nsIDocument*& aDocument) const;
@@ -1049,7 +1054,7 @@ HTMLStyleSheetImpl::RulesMatching(PseudoRuleProcessorData* aData,
                                   nsIAtom* aMedium)
 {
   nsIAtom* pseudoTag = aData->mPseudoTag;
-  if (pseudoTag == nsHTMLAtoms::tableColPseudo) {
+  if (pseudoTag == nsCSSAnonBoxes::tableCol) {
     nsRuleWalker *ruleWalker = aData->mRuleWalker;
     if (ruleWalker) {
       ruleWalker->Forward(mTableColRule);
@@ -1104,9 +1109,9 @@ HTMLStyleSheetImpl::UseForMedium(nsIAtom* aMedium) const
 
 
 NS_IMETHODIMP
-HTMLStyleSheetImpl::GetEnabled(PRBool& aEnabled) const
+HTMLStyleSheetImpl::GetApplicable(PRBool& aApplicable) const
 {
-  aEnabled = PR_TRUE;
+  aApplicable = PR_TRUE;
   return NS_OK;
 }
 
@@ -1116,6 +1121,19 @@ HTMLStyleSheetImpl::SetEnabled(PRBool aEnabled)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+HTMLStyleSheetImpl::GetComplete(PRBool& aComplete) const
+{
+  aComplete = PR_TRUE;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+HTMLStyleSheetImpl::SetComplete()
+{
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 HTMLStyleSheetImpl::GetParentSheet(nsIStyleSheet*& aParent) const
 {
