@@ -53,7 +53,10 @@
 #include "nsNetUtil.h"
 
 #include "nsIServiceManager.h"
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranchInternal.h"
+#include "nsIObserver.h"
 #include "nsReadableUtils.h"
 
 // XXX This is here because nsCachedStyleData is accessed outside of
@@ -80,8 +83,6 @@ nsCachedStyleData::gInfo[] = {
   { 0, 0, 0 }
 };
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
 #define POSITIVE_SCALE_FACTOR 1.10 /* 10% */
 #define NEGATIVE_SCALE_FACTOR .90  /* 10% */
 
@@ -123,42 +124,72 @@ float nsStyleUtil::GetScalingFactor(PRInt32 aScaler)
 
 
 //------------------------------------------------------------------------------
-//
+// Font Algorithm Code
 //------------------------------------------------------------------------------
 
 static PRBool gNavAlgorithmPref = PR_FALSE;
 
-static int PR_CALLBACK NavAlgorithmPrefChangedCallback(const char * name, void * closure)
+class nsFontAlgorithmPrefObserver : public nsIObserver
+{
+public:
+  nsFontAlgorithmPrefObserver();
+  virtual ~nsFontAlgorithmPrefObserver();
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIOBSERVER
+};
+
+NS_IMPL_ISUPPORTS1(nsFontAlgorithmPrefObserver, nsIObserver)
+
+nsFontAlgorithmPrefObserver::nsFontAlgorithmPrefObserver()
 {
-	nsresult rv;
-	nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv));
-	if (NS_SUCCEEDED(rv) && prefs) {
-		prefs->GetBoolPref(name, &gNavAlgorithmPref);
-	}
-	return 0;
 }
 
+nsFontAlgorithmPrefObserver::~nsFontAlgorithmPrefObserver()
+{
+}
 
-//------------------------------------------------------------------------------
-//
-//------------------------------------------------------------------------------
+NS_IMETHODIMP
+nsFontAlgorithmPrefObserver::Observe(nsISupports *aSubject,
+                                     const char *aTopic,
+                                     const PRUnichar *aData)
+{
+  NS_ASSERTION(nsDependentString(aData) ==
+                 NS_LITERAL_STRING("font.size.nav4algorithm"),
+               "This is the wrong pref!");
+  NS_ASSERTION(!nsCRT::strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID),
+               "This observer only handles pref change topics");
+
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_QueryInterface(aSubject));
+  NS_ASSERTION(prefBranch, "Cannot get a pref branch");
+  prefBranch->GetBoolPref("font.size.nav4algorithm", &gNavAlgorithmPref);
+
+  return NS_OK;
+}
 
 static PRBool UseNewFontAlgorithm()
 {
-	static PRBool once = PR_TRUE;
+  static PRBool gotAlgorithm = PR_FALSE;
+  if (gotAlgorithm) {
+    gotAlgorithm = PR_TRUE;
 
-	if (once)
-	{
-		once = PR_FALSE;
+    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+    if (prefBranch) {
+      prefBranch->GetBoolPref("font.size.nav4algorithm", &gNavAlgorithmPref);
+      nsCOMPtr<nsIObserver> observer = new nsFontAlgorithmPrefObserver();
+      if (observer) {
+        nsCOMPtr<nsIPrefBranchInternal> pbi(do_QueryInterface(prefBranch));
+        if (pbi) {
+          pbi->AddObserver("font.size.nav4algorithm", observer, PR_FALSE);
+        }
+      }
+    }
+  }
 
-		nsresult rv;
-		nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv));
-		if (NS_SUCCEEDED(rv) && prefs) {
-			prefs->GetBoolPref("font.size.nav4algorithm", &gNavAlgorithmPref);
-			prefs->RegisterCallback("font.size.nav4algorithm", NavAlgorithmPrefChangedCallback, NULL);
-		}
-	}
-	return (gNavAlgorithmPref ? PR_FALSE : PR_TRUE);
+  // The pref is true if we should use the old (nav4) algorithm.
+  // Since our return is whether we should use the new algorithm,
+  // take the inverse of our cached pref value.
+  return !gNavAlgorithmPref;
 }
 
 
@@ -493,39 +524,6 @@ nsStyleUtil::ConstrainFontWeight(PRInt32 aWeight)
 }
 
 
-//------------------------------------------------------------------------------
-//
-//------------------------------------------------------------------------------
-
-const nsStyleBackground* nsStyleUtil::FindNonTransparentBackground(nsIStyleContext* aContext,
-                                                                   PRBool aStartAtParent /*= PR_FALSE*/)
-{
-  const nsStyleBackground* result = nsnull;
-  nsIStyleContext*    context;
-  if (aStartAtParent) {
-    context = aContext->GetParent();  // balance ending release
-  } else {
-    context = aContext;
-    NS_IF_ADDREF(context);  // balance ending release
-  }
-  NS_ASSERTION( context != nsnull, "Cannot find NonTransparentBackground in a null context" );
-  
-  while (nsnull != context) {
-    result = (const nsStyleBackground*)context->GetStyleData(eStyleStruct_Background);
-
-    if (0 == (result->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT)) {
-      break;
-    }
-    else {
-      nsIStyleContext* last = context;
-      context = context->GetParent();
-      NS_RELEASE(last);
-    }
-  }
-  NS_IF_RELEASE(context);
-  return result;
-}
-
 /*static*/
 PRBool nsStyleUtil::IsHTMLLink(nsIContent *aContent, nsIAtom *aTag, nsIPresContext *aPresContext, nsLinkState *aState)
 {
@@ -553,8 +551,8 @@ PRBool nsStyleUtil::IsHTMLLink(nsIContent *aContent, nsIAtom *aTag, nsIPresConte
         // if there is no link, then this anchor is not really a linkpseudo.
         // bug=23209
 
-        char* href;
-        link->GetHrefCString(href);
+        nsXPIDLCString href;
+        link->GetHrefCString(*getter_Copies(href));
 
         if (href) {
           nsILinkHandler *linkHandler = nsnull;
@@ -567,7 +565,6 @@ PRBool nsStyleUtil::IsHTMLLink(nsIContent *aContent, nsIAtom *aTag, nsIPresConte
             // no link handler?  then all links are unvisited
             linkState = eLinkState_Unvisited;
           }
-          nsCRT::free(href);
         } else {
           linkState = eLinkState_NotLink;
         }
@@ -626,10 +623,8 @@ PRBool nsStyleUtil::IsSimpleXlink(nsIContent *aContent, nsIPresContext *aPresCon
 
         // convert here, rather than twice in NS_MakeAbsoluteURI and
         // back again
-        char * href = ToNewCString(val);
-        char * absHREF = nsnull;
-        (void) NS_MakeAbsoluteURI(&absHREF, href, baseURI);
-        nsCRT::free(href);
+        nsCAutoString absHREF;
+        (void) NS_MakeAbsoluteURI(absHREF, NS_ConvertUCS2toUTF8(val), baseURI);
 
         nsILinkHandler *linkHandler = nsnull;
         aPresContext->GetLinkHandler(&linkHandler);
@@ -641,7 +636,6 @@ PRBool nsStyleUtil::IsSimpleXlink(nsIContent *aContent, nsIPresContext *aPresCon
           // no link handler?  then all links are unvisited
           *aState = eLinkState_Unvisited;
         }
-        nsCRT::free(absHREF);
 
         rv = PR_TRUE;
       }
