@@ -83,20 +83,9 @@ LAYOUT_ATOM(floaterList, "Floater-list")
 LAYOUT_ATOM(overflowList, "Overflow-list")
 LAYOUT_ATOM(popupList, "Popup-list")
 
-  // Alphabetical list of pseudo tag names for non-element content
-LAYOUT_ATOM(canvasPseudo, ":-moz-canvas")
 LAYOUT_ATOM(commentTagName, "__moz_comment")
-LAYOUT_ATOM(dummyOptionPseudo, ":-moz-dummy-option")
 LAYOUT_ATOM(textTagName, "__moz_text")
-LAYOUT_ATOM(pageBreakPseudo, ":-moz-pagebreak")
-LAYOUT_ATOM(pagePseudo, ":-moz-page")
-LAYOUT_ATOM(pageContentPseudo, ":-moz-pagecontent")
-LAYOUT_ATOM(pageSequencePseudo, ":-moz-page-sequence")
 LAYOUT_ATOM(processingInstructionTagName, "__moz_pi")
-LAYOUT_ATOM(scrolledContentPseudo, ":-moz-scrolled-content")
-LAYOUT_ATOM(viewportPseudo, ":-moz-viewport")
-LAYOUT_ATOM(viewportScrollPseudo, ":-moz-viewport-scroll")
-LAYOUT_ATOM(selectScrolledContentPseudo, ":-moz-select-scrolled-content")
 
   // Alphabetical list of frame types
 LAYOUT_ATOM(areaFrame, "AreaFrame")
@@ -105,6 +94,7 @@ LAYOUT_ATOM(blockFrame, "BlockFrame")
 LAYOUT_ATOM(boxFrame, "BoxFrame")
 LAYOUT_ATOM(brFrame, "BRFrame")
 LAYOUT_ATOM(bulletFrame, "BulletFrame")
+LAYOUT_ATOM(fieldSetFrame, "FieldSetFrame")
 LAYOUT_ATOM(gfxButtonControlFrame, "gfxButtonControlFrame")
 LAYOUT_ATOM(hrFrame, "HRFrame")
 LAYOUT_ATOM(htmlFrameInnerFrame, "htmlFrameInnerFrame")
@@ -112,6 +102,7 @@ LAYOUT_ATOM(htmlFrameOuterFrame, "htmlFrameOuterFrame")
 LAYOUT_ATOM(imageFrame, "ImageFrame")
 LAYOUT_ATOM(imageControlFrame, "ImageControlFrame")
 LAYOUT_ATOM(inlineFrame, "InlineFrame")
+LAYOUT_ATOM(legendFrame, "LegendFrame")
 LAYOUT_ATOM(letterFrame, "LetterFrame")
 LAYOUT_ATOM(lineFrame, "LineFrame")
 LAYOUT_ATOM(listControlFrame,"ListControlFrame")
@@ -145,6 +136,7 @@ LAYOUT_ATOM(maxElementSizeProperty, "MaxElementSizeProperty")  // nsSize*
 LAYOUT_ATOM(overflowAreaProperty, "OverflowArea")              // nsRect*
 LAYOUT_ATOM(overflowProperty, "OverflowProperty")              // list of nsIFrame*
 LAYOUT_ATOM(overflowLinesProperty, "OverflowLinesProperty")    // list of nsLineBox*
+LAYOUT_ATOM(overflowPlaceholdersProperty, "OverflowPlaceholdersProperty")  // nsPlaceholder*
 LAYOUT_ATOM(rowUnpaginatedHeightProperty, "RowUnpaginatedHeightProperty")  // nscoord*
 LAYOUT_ATOM(spaceManagerProperty, "SpaceManagerProperty")      // the space manager for a block
 LAYOUT_ATOM(tableBCProperty, "TableBCProperty")                // table border collapsing info (e.g. damage area, table border widths)
