@@ -249,7 +249,7 @@ nsStyleFont::Destroy(nsIPresContext* aContext) {
   aContext->FreeToShell(sizeof(nsStyleFont), this);
 }
 
-PRInt32 nsStyleFont::CalcDifference(const nsStyleFont& aOther) const
+nsChangeHint nsStyleFont::CalcDifference(const nsStyleFont& aOther) const
 {
   if (mSize == aOther.mSize) {
     return CalcFontDifference(mFont, aOther.mFont);
@@ -257,12 +257,13 @@ PRInt32 nsStyleFont::CalcDifference(const nsStyleFont& aOther) const
   return NS_STYLE_HINT_REFLOW;
 }
 
-PRInt32 nsStyleFont::CalcFontDifference(const nsFont& aFont1, const nsFont& aFont2)
+nsChangeHint nsStyleFont::CalcFontDifference(const nsFont& aFont1, const nsFont& aFont2)
 {
   if ((aFont1.size == aFont2.size) && 
       (aFont1.sizeAdjust == aFont2.sizeAdjust) && 
       (aFont1.style == aFont2.style) &&
       (aFont1.variant == aFont2.variant) &&
+      (aFont1.familyNameQuirks == aFont2.familyNameQuirks) &&
       (aFont1.weight == aFont2.weight) &&
       (aFont1.name == aFont2.name)) {
     if ((aFont1.decorations == aFont2.decorations)) {
@@ -346,7 +347,7 @@ void nsStyleMargin::RecalcData()
     mHasCachedMargin = PR_FALSE;
 }
 
-PRInt32 nsStyleMargin::CalcDifference(const nsStyleMargin& aOther) const
+nsChangeHint nsStyleMargin::CalcDifference(const nsStyleMargin& aOther) const
 {
   if (mMargin == aOther.mMargin) {
     return NS_STYLE_HINT_NONE;
@@ -404,7 +405,7 @@ void nsStylePadding::RecalcData()
     mHasCachedPadding = PR_FALSE;
 }
 
-PRInt32 nsStylePadding::CalcDifference(const nsStylePadding& aOther) const
+nsChangeHint nsStylePadding::CalcDifference(const nsStylePadding& aOther) const
 {
   if (mPadding == aOther.mPadding) {
     return NS_STYLE_HINT_NONE;
@@ -552,7 +553,7 @@ void nsStyleBorder::RecalcData()
   }
 }
 
-PRInt32 nsStyleBorder::CalcDifference(const nsStyleBorder& aOther) const
+nsChangeHint nsStyleBorder::CalcDifference(const nsStyleBorder& aOther) const
 {
   if ((mBorder == aOther.mBorder) && 
       (mFloatEdge == aOther.mFloatEdge)) {
@@ -668,8 +669,7 @@ nsStyleOutline::RecalcData(void)
     mHasCachedOutline = PR_FALSE;
 }
 
-PRInt32 
-nsStyleOutline::CalcDifference(const nsStyleOutline& aOther) const
+nsChangeHint nsStyleOutline::CalcDifference(const nsStyleOutline& aOther) const
 {
   if ((mOutlineWidth != aOther.mOutlineWidth) ||
       (mOutlineStyle != aOther.mOutlineStyle) ||
@@ -701,20 +701,17 @@ nsStyleList::nsStyleList(const nsStyleList& aSource)
   mListStyleImage = aSource.mListStyleImage;
 }
 
-PRInt32 nsStyleList::CalcDifference(const nsStyleList& aOther) const
+nsChangeHint nsStyleList::CalcDifference(const nsStyleList& aOther) const
 {
-  if (mListStylePosition == aOther.mListStylePosition)
-    if (mListStyleImage == aOther.mListStyleImage)
-      if (mListStyleType == aOther.mListStyleType) {
-        if (mImageRegion == aOther.mImageRegion)
-          return NS_STYLE_HINT_NONE;
-        if (mImageRegion.width == aOther.mImageRegion.width &&
-            mImageRegion.height == aOther.mImageRegion.height)
-          return NS_STYLE_HINT_VISUAL;
-        return NS_STYLE_HINT_REFLOW;
-      }
-      return NS_STYLE_HINT_REFLOW;
-    return NS_STYLE_HINT_REFLOW;
+  if (mListStylePosition == aOther.mListStylePosition &&
+      mListStyleImage == aOther.mListStyleImage &&
+      mListStyleType == aOther.mListStyleType) {
+    if (mImageRegion == aOther.mImageRegion)
+      return NS_STYLE_HINT_NONE;
+    if (mImageRegion.width == aOther.mImageRegion.width &&
+        mImageRegion.height == aOther.mImageRegion.height)
+      return NS_STYLE_HINT_VISUAL;
+  }
   return NS_STYLE_HINT_REFLOW;
 }
 
@@ -741,8 +738,7 @@ nsStyleXUL::nsStyleXUL(const nsStyleXUL& aSource)
   memcpy((nsStyleXUL*)this, &aSource, sizeof(nsStyleXUL));
 }
 
-PRInt32 
-nsStyleXUL::CalcDifference(const nsStyleXUL& aOther) const
+nsChangeHint nsStyleXUL::CalcDifference(const nsStyleXUL& aOther) const
 {
   if (mBoxAlign == aOther.mBoxAlign &&
       mBoxDirection == aOther.mBoxDirection &&
@@ -804,8 +800,7 @@ nsStyleSVG::nsStyleSVG(const nsStyleSVG& aSource)
   mStrokeWidth = aSource.mStrokeWidth;
 }
 
-PRInt32 
-nsStyleSVG::CalcDifference(const nsStyleSVG& aOther) const
+nsChangeHint nsStyleSVG::CalcDifference(const nsStyleSVG& aOther) const
 {
   if ( mFill.mType       != aOther.mFill.mType       ||
        mFillOpacity      != aOther.mFillOpacity      ||
@@ -860,7 +855,7 @@ nsStylePosition::nsStylePosition(const nsStylePosition& aSource)
   memcpy((nsStylePosition*)this, &aSource, sizeof(nsStylePosition));
 }
 
-PRInt32 nsStylePosition::CalcDifference(const nsStylePosition& aOther) const
+nsChangeHint nsStylePosition::CalcDifference(const nsStylePosition& aOther) const
 {
   if ((mOffset == aOther.mOffset) &&
       (mWidth == aOther.mWidth) &&
@@ -898,7 +893,7 @@ nsStyleTable::nsStyleTable(const nsStyleTable& aSource)
   memcpy((nsStyleTable*)this, &aSource, sizeof(nsStyleTable));
 }
 
-PRInt32 nsStyleTable::CalcDifference(const nsStyleTable& aOther) const
+nsChangeHint nsStyleTable::CalcDifference(const nsStyleTable& aOther) const
 {
   if ((mLayoutStrategy == aOther.mLayoutStrategy) &&
       (mFrame == aOther.mFrame) &&
@@ -936,7 +931,7 @@ nsStyleTableBorder::nsStyleTableBorder(const nsStyleTableBorder& aSource)
   memcpy((nsStyleTableBorder*)this, &aSource, sizeof(nsStyleTableBorder));
 }
 
-PRInt32 nsStyleTableBorder::CalcDifference(const nsStyleTableBorder& aOther) const
+nsChangeHint nsStyleTableBorder::CalcDifference(const nsStyleTableBorder& aOther) const
 {
   if ((mBorderCollapse == aOther.mBorderCollapse) &&
       (mCaptionSide == aOther.mCaptionSide) &&
@@ -964,7 +959,7 @@ nsStyleColor::nsStyleColor(const nsStyleColor& aSource)
   mColor = aSource.mColor;
 }
 
-PRInt32 nsStyleColor::CalcDifference(const nsStyleColor& aOther) const
+nsChangeHint nsStyleColor::CalcDifference(const nsStyleColor& aOther) const
 {
   if (mColor == aOther.mColor)
     return NS_STYLE_HINT_NONE;
@@ -979,10 +974,11 @@ nsStyleBackground::nsStyleBackground(nsIPresContext* aPresContext)
 {
   mBackgroundFlags = NS_STYLE_BG_COLOR_TRANSPARENT | NS_STYLE_BG_IMAGE_NONE;
   aPresContext->GetDefaultBackgroundColor(&mBackgroundColor);
-  aPresContext->GetDefaultBackgroundImageAttachment(&mBackgroundAttachment);
-  aPresContext->GetDefaultBackgroundImageRepeat(&mBackgroundRepeat);
-  aPresContext->GetDefaultBackgroundImageOffset(&mBackgroundXPosition, &mBackgroundYPosition);
-  aPresContext->GetDefaultBackgroundImage(mBackgroundImage);
+  mBackgroundAttachment = NS_STYLE_BG_ATTACHMENT_SCROLL;
+  mBackgroundClip = NS_STYLE_BG_CLIP_BORDER;
+  mBackgroundOrigin = NS_STYLE_BG_ORIGIN_PADDING;
+  mBackgroundRepeat = NS_STYLE_BG_REPEAT_XY;
+  mBackgroundXPosition = mBackgroundYPosition = 0;
 }
 
 nsStyleBackground::nsStyleBackground(const nsStyleBackground& aSource)
@@ -990,18 +986,19 @@ nsStyleBackground::nsStyleBackground(const nsStyleBackground& aSource)
   mBackgroundAttachment = aSource.mBackgroundAttachment;
   mBackgroundFlags = aSource.mBackgroundFlags;
   mBackgroundRepeat = aSource.mBackgroundRepeat;
-
+  mBackgroundClip = aSource.mBackgroundClip;
+  mBackgroundOrigin = aSource.mBackgroundOrigin;
   mBackgroundColor = aSource.mBackgroundColor;
   mBackgroundXPosition = aSource.mBackgroundXPosition;
   mBackgroundYPosition = aSource.mBackgroundYPosition;
   mBackgroundImage = aSource.mBackgroundImage;
 }
 
-PRInt32 nsStyleBackground::CalcDifference(const nsStyleBackground& aOther) const
+nsChangeHint nsStyleBackground::CalcDifference(const nsStyleBackground& aOther) const
 {
   if (mBackgroundAttachment != aOther.mBackgroundAttachment
-    && (NS_STYLE_BG_ATTACHMENT_FIXED == mBackgroundAttachment) ||
-      (NS_STYLE_BG_ATTACHMENT_FIXED == aOther.mBackgroundAttachment))
+    && ((NS_STYLE_BG_ATTACHMENT_FIXED == mBackgroundAttachment) ||
+        (NS_STYLE_BG_ATTACHMENT_FIXED == aOther.mBackgroundAttachment)))
     // this might require creation of a view
     return NS_STYLE_HINT_FRAMECHANGE;
 
@@ -1011,6 +1008,8 @@ PRInt32 nsStyleBackground::CalcDifference(const nsStyleBackground& aOther) const
       (mBackgroundColor == aOther.mBackgroundColor) &&
       (mBackgroundXPosition == aOther.mBackgroundXPosition) &&
       (mBackgroundYPosition == aOther.mBackgroundYPosition) &&
+      (mBackgroundClip == aOther.mBackgroundClip) &&
+      (mBackgroundOrigin == aOther.mBackgroundOrigin) &&
       (mBackgroundImage == aOther.mBackgroundImage))
     return NS_STYLE_HINT_NONE;
   return NS_STYLE_HINT_VISUAL;
@@ -1024,7 +1023,8 @@ nsStyleDisplay::nsStyleDisplay()
 {
   mAppearance = 0;
   mDisplay = NS_STYLE_DISPLAY_INLINE;
-  mPosition = NS_STYLE_POSITION_NORMAL;
+  mOriginalDisplay = NS_STYLE_DISPLAY_NONE;
+  mPosition = NS_STYLE_POSITION_STATIC;
   mFloats = NS_STYLE_FLOAT_NONE;
   mBreakType = NS_STYLE_CLEAR_NONE;
   mBreakBefore = PR_FALSE;
@@ -1038,6 +1038,7 @@ nsStyleDisplay::nsStyleDisplay(const nsStyleDisplay& aSource)
 {
   mAppearance = aSource.mAppearance;
   mDisplay = aSource.mDisplay;
+  mOriginalDisplay = aSource.mOriginalDisplay;
   mBinding = aSource.mBinding;
   mPosition = aSource.mPosition;
   mFloats = aSource.mFloats;
@@ -1049,25 +1050,30 @@ nsStyleDisplay::nsStyleDisplay(const nsStyleDisplay& aSource)
   mClip = aSource.mClip;
 }
 
-PRInt32 nsStyleDisplay::CalcDifference(const nsStyleDisplay& aOther) const
+nsChangeHint nsStyleDisplay::CalcDifference(const nsStyleDisplay& aOther) const
 {
-  if (mBinding != aOther.mBinding || mPosition != aOther.mPosition)
-    return NS_STYLE_HINT_FRAMECHANGE;
+  nsChangeHint hint = nsChangeHint_None;
 
-  if ((mDisplay == aOther.mDisplay) &&
-      (mFloats == aOther.mFloats) &&
-      (mOverflow == aOther.mOverflow)) {
-    if ((mBreakType == aOther.mBreakType) &&
-        (mBreakBefore == aOther.mBreakBefore) &&
-        (mBreakAfter == aOther.mBreakAfter) &&
-        (mClipFlags == aOther.mClipFlags) &&
-        (mClip == aOther.mClip) &&
-        (mAppearance == aOther.mAppearance)) {
-      return NS_STYLE_HINT_NONE;
-    }
-    return NS_STYLE_HINT_REFLOW;
-  }
-  return NS_STYLE_HINT_FRAMECHANGE;
+  if (mBinding != aOther.mBinding
+      || mPosition != aOther.mPosition
+      || mDisplay != aOther.mDisplay
+      || mFloats != aOther.mFloats
+      || mOverflow != aOther.mOverflow)
+    NS_UpdateHint(hint, nsChangeHint_ReconstructFrame);
+
+  // XXX the following is conservative, for now: changing float breaking shouldn't
+  // necessarily require a repaint, reflow should suffice.
+  if (mBreakType != aOther.mBreakType
+      || mBreakBefore != aOther.mBreakBefore
+      || mBreakAfter != aOther.mBreakAfter
+      || mAppearance != aOther.mAppearance)
+    NS_UpdateHint(hint, NS_CombineHint(nsChangeHint_ReflowFrame, nsChangeHint_RepaintFrame));
+
+  if (mClipFlags != aOther.mClipFlags
+      || mClip != aOther.mClip)
+    NS_UpdateHint(hint, nsChangeHint_SyncFrameView);
+
+  return hint;
 }
 
 // --------------------
@@ -1100,7 +1106,7 @@ nsStyleVisibility::nsStyleVisibility(const nsStyleVisibility& aSource)
   mOpacity = aSource.mOpacity;
 } 
 
-PRInt32 nsStyleVisibility::CalcDifference(const nsStyleVisibility& aOther) const
+nsChangeHint nsStyleVisibility::CalcDifference(const nsStyleVisibility& aOther) const
 {
   if (mOpacity != aOther.mOpacity
       && ((mOpacity < 1.0) != (aOther.mOpacity < 1.0)))
@@ -1181,8 +1187,7 @@ nsStyleContent::nsStyleContent(const nsStyleContent& aSource)
   }
 }
 
-PRInt32 
-nsStyleContent::CalcDifference(const nsStyleContent& aOther) const
+nsChangeHint nsStyleContent::CalcDifference(const nsStyleContent& aOther) const
 {
   if (mContentCount == aOther.mContentCount) {
     if ((mMarkerOffset == aOther.mMarkerOffset) &&
@@ -1235,6 +1240,8 @@ nsStyleQuotes::~nsStyleQuotes(void)
 }
 
 nsStyleQuotes::nsStyleQuotes(const nsStyleQuotes& aSource)
+  : mQuotesCount(0),
+    mQuotes(nsnull)
 {
   if (NS_SUCCEEDED(AllocateQuotes(aSource.QuotesCount()))) {
     PRUint32 count = (mQuotesCount * 2);
@@ -1244,20 +1251,21 @@ nsStyleQuotes::nsStyleQuotes(const nsStyleQuotes& aSource)
   }
 }
 
-PRInt32 
-nsStyleQuotes::CalcDifference(const nsStyleQuotes& aOther) const
+nsChangeHint nsStyleQuotes::CalcDifference(const nsStyleQuotes& aOther) const
 {
+  // If the quotes implementation is ever going to change we might not need
+  // a framechange here and a reflow should be sufficient.  See bug 35768.
   if (mQuotesCount == aOther.mQuotesCount) {
     PRUint32 ix = (mQuotesCount * 2);
     while (0 < ix--) {
       if (mQuotes[ix] != aOther.mQuotes[ix]) {
-        return NS_STYLE_HINT_REFLOW;
+        return NS_STYLE_HINT_FRAMECHANGE;
       }
     }
 
     return NS_STYLE_HINT_NONE;
   }
-  return NS_STYLE_HINT_REFLOW;
+  return NS_STYLE_HINT_FRAMECHANGE;
 }
 
 // --------------------
@@ -1280,7 +1288,7 @@ nsStyleTextReset::nsStyleTextReset(const nsStyleTextReset& aSource)
 
 nsStyleTextReset::~nsStyleTextReset(void) { }
 
-PRInt32 nsStyleTextReset::CalcDifference(const nsStyleTextReset& aOther) const
+nsChangeHint nsStyleTextReset::CalcDifference(const nsStyleTextReset& aOther) const
 {
   if (mVerticalAlign == aOther.mVerticalAlign
 #ifdef IBMBIDI
@@ -1317,7 +1325,7 @@ nsStyleText::nsStyleText(const nsStyleText& aSource)
 
 nsStyleText::~nsStyleText(void) { }
 
-PRInt32 nsStyleText::CalcDifference(const nsStyleText& aOther) const
+nsChangeHint nsStyleText::CalcDifference(const nsStyleText& aOther) const
 {
   if ((mTextAlign == aOther.mTextAlign) &&
       (mTextTransform == aOther.mTextTransform) &&
@@ -1357,7 +1365,7 @@ nsStyleUserInterface::~nsStyleUserInterface(void)
 { 
 }
 
-PRInt32 nsStyleUserInterface::CalcDifference(const nsStyleUserInterface& aOther) const
+nsChangeHint nsStyleUserInterface::CalcDifference(const nsStyleUserInterface& aOther) const
 {
   if ((mCursor != aOther.mCursor) ||
       (mCursorImage != aOther.mCursorImage))
@@ -1406,7 +1414,7 @@ nsStyleUIReset::~nsStyleUIReset(void)
 { 
 }
 
-PRInt32 nsStyleUIReset::CalcDifference(const nsStyleUIReset& aOther) const
+nsChangeHint nsStyleUIReset::CalcDifference(const nsStyleUIReset& aOther) const
 {
   if (mForceBrokenImageIcon == aOther.mForceBrokenImageIcon) {
     if (mResizer == aOther.mResizer &&
