@@ -54,6 +54,8 @@
 #include "nsXBLProtoImplProperty.h"
 #include "nsXBLProtoImplMethod.h"
 #include "nsXBLProtoImplField.h"
+#include "nsIConsoleService.h"
+#include "nsIScriptError.h"
 
 nsresult
 NS_NewXBLContentSink(nsIXMLContentSink** aResult,
@@ -85,6 +87,7 @@ nsXBLContentSink::nsXBLContentSink()
   mProperty = nsnull;
   mMethod = nsnull;
   mField = nsnull;
+  mPrettyPrintXML = PR_FALSE;
 }
 
 nsXBLContentSink::~nsXBLContentSink()
@@ -183,6 +186,40 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
   return nsXMLContentSink::FlushText(aCreateTextNode, aDidFlush);
 }
 
+NS_IMETHODIMP
+nsXBLContentSink::ReportError(const PRUnichar* aErrorText, 
+                              const PRUnichar* aSourceText)
+{
+  // XXX We should make sure the binding has no effect, but that it also
+  // gets destroyed properly without leaking.  Perhaps we should even
+  // ensure that the content that was bound is displayed with no
+  // binding.
+
+  // Report the error to the error console.
+  nsCOMPtr<nsIConsoleService> consoleService =
+    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
+  if (consoleService) {
+    // XXX It would be nice if the parser didn't pre-format it for us,
+    // because then we could create a instance of nsIScriptError and the
+    // error console would format this much more nicely for us.
+    // However, that would require duplicating even more code between
+    // the XML content sink and the XUL content sink.
+
+    consoleService->LogStringMessage(aErrorText);
+  }
+
+#ifdef DEBUG
+  // Report the error to stderr.
+  fprintf(stderr,
+          "\n%s\n%s\n\n",
+          NS_LossyConvertUCS2toASCII(aErrorText).get(),
+          NS_LossyConvertUCS2toASCII(aSourceText).get());
+#endif
+
+  // Most of what this does won't be too useful, but whatever...
+  return nsXMLContentSink::ReportError(aErrorText, aSourceText);
+}
+
 NS_IMETHODIMP 
 nsXBLContentSink::HandleStartElement(const PRUnichar *aName, 
                                      const PRUnichar **aAtts, 
@@ -683,46 +720,43 @@ nsXBLContentSink::ConstructParameter(const PRUnichar **aAtts)
 }
 
 nsresult
-nsXBLContentSink::CreateElement(const PRUnichar** aAtts, 
-                                PRUint32 aAttsCount, 
-                                PRInt32 aNameSpaceID, 
-                                nsINodeInfo* aNodeInfo, 
-                                nsIContent** aResult)
+nsXBLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
+                                nsINodeInfo* aNodeInfo, PRUint32 aLineNumber,
+                                nsIContent** aResult, PRBool* aAppendContent)
 {
-  if (aNameSpaceID == nsXULAtoms::nameSpaceID) {
-    nsXULPrototypeElement* prototype = new nsXULPrototypeElement(0);
-    if (!prototype)
-      return NS_ERROR_OUT_OF_MEMORY;
-    
-    prototype->mNodeInfo = aNodeInfo;
-
-    // Reset the refcnt to 0.  Normally XUL prototype elements get a refcnt of 1
-    // to represent ownership by the XUL prototype document.  In our case we have
-    // no prototype document, and our initial ref count of 1 will come from being
-    // wrapped by a real XUL element in the Create call below.
-    prototype->mRefCnt = 0;
-
-    AddAttributesToXULPrototype(aAtts, aAttsCount, prototype);
-
-    // Following this function call, the prototype's ref count will be 1.
-    nsresult rv = nsXULElement::Create(prototype, mDocument, PR_FALSE, aResult);
-
-    if (NS_FAILED(rv)) return rv;
-    return NS_OK;
+  if (!aNodeInfo->NamespaceEquals(kNameSpaceID_XUL)) {
+    return nsXMLContentSink::CreateElement(aAtts, aAttsCount, aNodeInfo,
+                                           aLineNumber, aResult,
+                                           aAppendContent);
   }
-  else
-    return nsXMLContentSink::CreateElement(aAtts, aAttsCount, aNameSpaceID, aNodeInfo, aResult);
+
+  *aAppendContent = PR_TRUE;
+  nsXULPrototypeElement* prototype = new nsXULPrototypeElement();
+  if (!prototype)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  prototype->mNodeInfo = aNodeInfo;
+
+  // Reset the refcnt to 0.  Normally XUL prototype elements get a refcnt of 1
+  // to represent ownership by the XUL prototype document.  In our case we have
+  // no prototype document, and our initial ref count of 1 will come from being
+  // wrapped by a real XUL element in the Create call below.
+  prototype->mRefCnt = 0;
+
+  AddAttributesToXULPrototype(aAtts, aAttsCount, prototype);
+
+  // Following this function call, the prototype's ref count will be 1.
+  return nsXULElement::Create(prototype, mDocument, PR_FALSE, aResult);
 }
 
 nsresult 
 nsXBLContentSink::AddAttributes(const PRUnichar** aAtts,
-                                nsIContent* aContent,
-                                PRBool aIsHTML)
+                                nsIContent* aContent)
 {
   if (aContent->IsContentOfType(nsIContent::eXUL))
     return NS_OK; // Nothing to do, since the proto already has the attrs.
-  else 
-    return nsXMLContentSink::AddAttributes(aAtts, aContent, aIsHTML);
+
+  return nsXMLContentSink::AddAttributes(aAtts, aContent);
 }
 
 nsresult
@@ -779,7 +813,7 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
   // XUL elements may require some additional work to compute
   // derived information.
-  if (aElement->mNodeInfo->NamespaceEquals(nsXULAtoms::nameSpaceID)) {
+  if (aElement->mNodeInfo->NamespaceEquals(kNameSpaceID_XUL)) {
     nsAutoString value;
 
     // Compute the element's class list if the element has a 'class' attribute.
