@@ -73,15 +73,7 @@
 #include "nsIURI.h"
 #include "nsIServiceManager.h"
 #include "nsICharsetAlias.h"
-#include "nsIPref.h"
-#include "nsICharsetDetector.h"
-#include "prmem.h"
-#include "prtime.h"
-#include "nsIWebShellServices.h"
-#include "nsICharsetDetectionAdaptor.h"
-#include "nsCharsetDetectionAdaptorCID.h"
 #include "nsICharsetAlias.h"
-#include "nsIParserFilter.h"
 #include "nsNetUtil.h"
 #include "nsDOMError.h"
 #include "nsScriptSecurityManager.h"
@@ -100,17 +92,15 @@
 #include "nsContentUtils.h"
 #include "nsIElementFactory.h"
 #include "nsCRT.h"
+#include "nsIWindowWatcher.h"
+#include "nsIAuthPrompt.h"
+#include "nsIScriptGlobalObjectOwner.h"
 
 static NS_DEFINE_CID(kHTMLStyleSheetCID,NS_HTMLSTYLESHEET_CID);
 
 // XXX The XML world depends on the html atoms
 #include "nsHTMLAtoms.h"
 
-#define DETECTOR_CONTRACTID_MAX 127
-static char g_detector_contractid[DETECTOR_CONTRACTID_MAX + 1];
-static PRBool gInitDetector = PR_FALSE;
-static PRBool gPlugDetector = PR_FALSE;
-
 static const char* kLoadAsData = "loadAsData";
 
 // ==================================================================
@@ -118,30 +108,6 @@ static const char* kLoadAsData = "loadAsData";
 // ==================================================================
 
 
-static int PR_CALLBACK
-MyPrefChangedCallback(const char*aPrefName, void* instance_data)
-{
-        nsresult rv;
-        nsCOMPtr<nsIPref> prefs = 
-                 do_GetService("@mozilla.org/preferences;1", &rv);
-        PRUnichar* detector_name = nsnull;
-        if(NS_SUCCEEDED(rv) && NS_SUCCEEDED(
-             rv = prefs->GetLocalizedUnicharPref("intl.charset.detector",
-                                     &detector_name)))
-        {
-			if(nsCRT::strlen(detector_name) > 0) {
-				PL_strncpy(g_detector_contractid, NS_CHARSET_DETECTOR_CONTRACTID_BASE,DETECTOR_CONTRACTID_MAX);
-				PL_strncat(g_detector_contractid, NS_ConvertUCS2toUTF8(detector_name).get(),DETECTOR_CONTRACTID_MAX);
-				gPlugDetector = PR_TRUE;
-			} else {
-				g_detector_contractid[0]=0;
-				gPlugDetector = PR_FALSE;
-			}
-            PR_FREEIF(detector_name);
-        }
-	return 0;
-}
-
 NS_EXPORT nsresult
 NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
                   const nsAString& aNamespaceURI, 
@@ -217,7 +183,8 @@ NS_NewXMLDocument(nsIDocument** aInstancePtrResult)
 
 nsXMLDocument::nsXMLDocument() 
   : mAttrStyleSheet(nsnull), mInlineStyleSheet(nsnull), 
-    mCountCatalogSheets(0), mParser(nsnull)
+    mCountCatalogSheets(0), mParser(nsnull),
+    mCrossSiteAccessEnabled(PR_FALSE), mXMLDeclarationBits(0)
 {
 }
 
@@ -280,38 +247,76 @@ nsXMLDocument::Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup)
   return result;
 }
 
+/////////////////////////////////////////////////////
+// nsIInterfaceRequestor methods:
+//
 NS_IMETHODIMP
 nsXMLDocument::GetInterface(const nsIID& aIID, void** aSink)
 {
-  // Since we implement all the interfaces that you can get with
-  // GetInterface() we can simply call QueryInterface() here and let
-  // it do all the work.
+  if (aIID.Equals(NS_GET_IID(nsIAuthPrompt))) {
+    NS_ENSURE_ARG_POINTER(aSink);
+    *aSink = nsnull;
+
+    nsresult rv;
+    nsCOMPtr<nsIWindowWatcher> ww(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
+    if (NS_FAILED(rv))
+      return rv;
+
+    nsCOMPtr<nsIAuthPrompt> prompt;
+    rv = ww->GetNewAuthPrompter(nsnull, getter_AddRefs(prompt));
+    if (NS_FAILED(rv))
+      return rv;
+
+    nsIAuthPrompt *p = prompt.get();
+    NS_ADDREF(p);
+    *aSink = p;
+    return NS_OK;
+  }
+
 
   return QueryInterface(aIID, aSink);
 }
 
-
 // nsIHttpEventSink
 NS_IMETHODIMP
 nsXMLDocument::OnRedirect(nsIHttpChannel *aHttpChannel, nsIChannel *aNewChannel)
 {
+  NS_ENSURE_ARG_POINTER(aNewChannel);
+
   nsresult rv;
 
-  nsCOMPtr<nsIScriptSecurityManager> securityManager = 
+  nsCOMPtr<nsIScriptSecurityManager> secMan = 
            do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-
-  if (NS_FAILED(rv))
-    return NS_ERROR_FAILURE;
+  if (NS_FAILED(rv)) 
+    return rv;
 
   nsCOMPtr<nsIURI> newLocation;
-  rv = aNewChannel->GetURI(getter_AddRefs(newLocation));
+  rv = aNewChannel->GetURI(getter_AddRefs(newLocation)); // The redirected URI
+  if (NS_FAILED(rv)) 
+    return rv;
 
-  if (NS_FAILED(rv))
-    return NS_ERROR_FAILURE;
+  if (mScriptContext && !mCrossSiteAccessEnabled) {
+    nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", & rv));
+    if (NS_FAILED(rv))
+      return rv;
+
+    JSContext *cx = (JSContext *)mScriptContext->GetNativeContext();
+    if (!cx)
+      return NS_ERROR_UNEXPECTED;
+
+    stack->Push(cx);
+
+    rv = secMan->CheckSameOrigin(nsnull, newLocation);
+
+    stack->Pop(&cx);
+  
+    if (NS_FAILED(rv))
+      return rv;
+  }
 
   nsCOMPtr<nsIPrincipal> newCodebase;
-  rv = securityManager->GetCodebasePrincipal(newLocation,
-                                             getter_AddRefs(newCodebase));
+  rv = secMan->GetCodebasePrincipal(newLocation,
+                                    getter_AddRefs(newCodebase));
   if (NS_FAILED(rv))
     return NS_ERROR_FAILURE;
 
@@ -352,33 +357,53 @@ nsXMLDocument::Load(const nsAString& aUrl)
 
   // Partial Reset, need to restore principal for security reasons and
   // event listener manager so that load listeners etc. will remain.
-  nsCOMPtr<nsIPrincipal> principal(dont_QueryInterface(mPrincipal));
-  nsCOMPtr<nsIEventListenerManager> elm(dont_QueryInterface(mListenerManager));
+  nsCOMPtr<nsIPrincipal> principal = mPrincipal;
+  nsCOMPtr<nsIEventListenerManager> elm = mListenerManager;
 
   Reset(nsnull, nsnull);
   
   mPrincipal = principal;
   mListenerManager = elm;
-  NS_IF_ADDREF(mPrincipal);
   NS_IF_ADDREF(mListenerManager);
   
   SetDocumentURL(uri);
   SetBaseURL(uri);
 
+  // Store script context, if any, in case we encounter redirect (because we need it there)
+  nsCOMPtr<nsIJSContextStack> stack =
+    do_GetService("@mozilla.org/js/xpc/ContextStack;1");
+  if (stack) {
+    JSContext *cx;
+    if (NS_SUCCEEDED(stack->Peek(&cx)) && cx) {
+      nsISupports *priv = (nsISupports *)::JS_GetContextPrivate(cx);
+      if (priv) {
+        priv->QueryInterface(NS_GET_IID(nsIScriptContext), getter_AddRefs(mScriptContext));        
+      }
+    }
+  }
+
+  // Find out if UniversalBrowserRead privileges are enabled - we will need this
+  // in case of a redirect
+  PRBool crossSiteAccessEnabled;
+  rv = secMan->IsCapabilityEnabled("UniversalBrowserRead", &crossSiteAccessEnabled);
+  if (NS_FAILED(rv)) return rv;
+
+  mCrossSiteAccessEnabled = crossSiteAccessEnabled;
+
   // Create a channel
   rv = NS_NewChannel(getter_AddRefs(channel), uri, nsnull, nsnull, this);
   if (NS_FAILED(rv)) return rv;
 
   // Set a principal for this document
-  NS_IF_RELEASE(mPrincipal);
+  mPrincipal = nsnull;
   nsCOMPtr<nsISupports> channelOwner;
   rv = channel->GetOwner(getter_AddRefs(channelOwner));
   if (NS_SUCCEEDED(rv) && channelOwner)
-      rv = channelOwner->QueryInterface(NS_GET_IID(nsIPrincipal), (void**)&mPrincipal);
+      mPrincipal = do_QueryInterface(channelOwner, &rv);
 
   if (NS_FAILED(rv) || !channelOwner)
   {
-    rv = secMan->GetCodebasePrincipal(uri, &mPrincipal);
+    rv = secMan->GetCodebasePrincipal(uri, getter_AddRefs(mPrincipal));
     if (!mPrincipal) return rv;
   }
 
@@ -407,6 +432,32 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
                                  PRBool aReset,
                                  nsIContentSink* aSink)
 {
+  if (nsCRT::strcmp(kLoadAsData, aCommand) == 0) {
+    // We need to disable script & style loading in this case.
+    // We leave them disabled even in EndLoad(), and let anyone
+    // who puts the document on display to worry about enabling.
+
+    // scripts
+    nsCOMPtr<nsIScriptLoader> loader;
+    nsresult rv = GetScriptLoader(getter_AddRefs(loader));
+    if (NS_FAILED(rv))
+      return rv;
+    if (loader) {
+      loader->SetEnabled(PR_FALSE); // Do not load/process scripts when loading as data
+    }
+
+    // styles
+    nsCOMPtr<nsICSSLoader> cssLoader;
+    rv = GetCSSLoader(*getter_AddRefs(cssLoader));
+    if (NS_FAILED(rv))
+      return rv;
+    if (cssLoader) {
+      cssLoader->SetEnabled(PR_FALSE); // Do not load/process styles when loading as data
+    }
+  } else if (nsCRT::strcmp("loadAsInteractiveData", aCommand) == 0) {
+    aCommand = kLoadAsData; // XBL, for example, needs scripts and styles
+  }
+
   nsresult rv = nsDocument::StartDocumentLoad(aCommand,
                                               aChannel, aLoadGroup,
                                               aContainer, 
@@ -414,26 +465,12 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
   if (NS_FAILED(rv)) return rv;
 
   nsAutoString charset(NS_LITERAL_STRING("UTF-8"));
-  PRBool bIsHTML = PR_FALSE; 
-  char* aContentType;
   PRInt32 charsetSource = kCharsetFromDocTypeDefault;
 
   nsCOMPtr<nsIURI> aUrl;
   rv = aChannel->GetURI(getter_AddRefs(aUrl));
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIMIMEService> MIMEService (do_GetService(NS_MIMESERVICE_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) return rv;
-  rv = MIMEService->GetTypeFromURI(aUrl, &aContentType);
-    
-  if (NS_SUCCEEDED(rv)) { 
-    if ( 0 == PL_strcmp(aContentType, "text/html")) {
-      bIsHTML = PR_TRUE;
-    }
-    Recycle(aContentType);
-    aContentType = nsnull;
-  }
-
   { // check channel's charset...
     nsCAutoString charsetVal;
     rv = aChannel->GetContentCharset(charsetVal);
@@ -466,159 +503,6 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
     docShell = do_QueryInterface(aContainer, &rv);
     if(NS_FAILED(rv) || !(docShell))  return rv; 
 
-    nsCOMPtr<nsIContentViewer> cv;
-    docShell->GetContentViewer(getter_AddRefs(cv));
-    if (cv) {
-      nsCOMPtr<nsIMarkupDocumentViewer> muCV = do_QueryInterface(cv);            
-      if (muCV) {
-        if(bIsHTML &&(0 == nsCRT::strcmp("view-source", aCommand))) { 
-          // only do this for view-source
-          //view source is now an XML document and we need to make provisions
-          //for the usual charset use when displaying those documents, this 
-          //code mirrors nsHTMLDocument.cpp
-          PRUnichar* requestCharset = nsnull;
-          nsIParserFilter *cdetflt = nsnull;
-          PRInt32 requestCharsetSource = kCharsetUninitialized;
-
-          //need to be able to override doc charset default on user request
-          if( kCharsetFromDocTypeDefault == charsetSource ) // it is not from HTTP header
-            charsetSource = kCharsetFromWeakDocTypeDefault;
-
-          //check hint Charset (is this needed here?)
-          PRUnichar* hintCharset = nsnull;
-          PRInt32  hintSource = kCharsetUninitialized;
-
-          rv = muCV->GetHintCharacterSet(&hintCharset); 
-          if(NS_SUCCEEDED(rv)) {
-            rv = muCV->GetHintCharacterSetSource((PRInt32 *)(&hintSource));
-            if(NS_SUCCEEDED(rv)) {
-              if(hintSource > charsetSource) {
-                charset = hintCharset;
-                Recycle(hintCharset);
-                charsetSource = hintSource;
-              }
-              if(kCharsetUninitialized != hintSource) {
-                muCV->SetHintCharacterSetSource((PRInt32)(kCharsetUninitialized));
-              }
-            }
-          }//hint Charset
-
-          // get user default charset
-          if(kCharsetFromUserDefault > charsetSource) 
-          {
-            PRUnichar* defaultCharsetFromDocShell = NULL;
-            if (muCV) {
-              rv = muCV->GetDefaultCharacterSet(&defaultCharsetFromDocShell);
-              if(NS_SUCCEEDED(rv)) {
-                charset = defaultCharsetFromDocShell;
-                Recycle(defaultCharsetFromDocShell);
-                charsetSource = kCharsetFromUserDefault;
-              }
-            }
-          }//user default
-
-          //user requested charset
-          if(NS_SUCCEEDED(rv)) 
-          {
-            if(requestCharsetSource > charsetSource) 
-            {
-              charsetSource = requestCharsetSource;
-              charset = requestCharset;
-              Recycle(requestCharset);
-            }
-          }
-
-          //charset from previous loading
-          if(kCharsetFromUserForced > charsetSource)
-          {
-            PRUnichar* forceCharsetFromDocShell = NULL;
-            if (muCV) {
-              rv = muCV->GetForceCharacterSet(&forceCharsetFromDocShell);
-            }
-            if(NS_SUCCEEDED(rv) && (nsnull != forceCharsetFromDocShell)) 
-            {
-              charset = forceCharsetFromDocShell;
-              Recycle(forceCharsetFromDocShell);
-              //TODO: we should define appropriate constant for force charset
-              charsetSource = kCharsetFromUserForced;  
-            }
-          } //previous loading
-
-          //auto-detector charset (needed here?)
-          nsresult rv_detect = NS_OK;
-          if(! gInitDetector)
-          {
-            nsCOMPtr<nsIPref> pref(do_GetService(NS_PREF_CONTRACTID));
-            if(pref)
-            {
-              PRUnichar* detector_name = nsnull;
-              if(NS_SUCCEEDED(
-                  rv_detect = pref->GetLocalizedUnicharPref("intl.charset.detector",
-		                 &detector_name)))
-              {
-                PL_strncpy(g_detector_contractid, NS_CHARSET_DETECTOR_CONTRACTID_BASE,DETECTOR_CONTRACTID_MAX);
-                PL_strncat(g_detector_contractid, NS_ConvertUCS2toUTF8(detector_name).get(),DETECTOR_CONTRACTID_MAX);
-                gPlugDetector = PR_TRUE;
-                PR_FREEIF(detector_name);
-              }
-              pref->RegisterCallback("intl.charset.detector", MyPrefChangedCallback, nsnull);
-            }
-            gInitDetector = PR_TRUE;
-          }  
-	      
-          //auto-detector charset (needed here?)
-          if((kCharsetFromAutoDetection > charsetSource )  && gPlugDetector)
-          {
-            // we could do charset detection
-            nsICharsetDetector *cdet = nsnull;
-            nsCOMPtr<nsIWebShellServices> wss;
-            nsICharsetDetectionAdaptor *adp = nsnull;
-
-            if(NS_SUCCEEDED( rv_detect = 
-       	    nsComponentManager::CreateInstance(g_detector_contractid, nsnull,
-				    NS_GET_IID(nsICharsetDetector), (void**)&cdet)))
-            {
-              if(NS_SUCCEEDED( rv_detect = 
-                  nsComponentManager::CreateInstance(
-                  NS_CHARSET_DETECTION_ADAPTOR_CONTRACTID, nsnull,
-                  NS_GET_IID(nsIParserFilter), (void**)&cdetflt)))
-              {
-                if(cdetflt && 
-                    NS_SUCCEEDED( rv_detect=
-                    cdetflt->QueryInterface(
-                    NS_GET_IID(nsICharsetDetectionAdaptor),(void**) &adp)))
-                {
-                  wss = do_QueryInterface(docShell, &rv_detect);
-
-                  if( NS_SUCCEEDED(rv_detect))
-                  {
-                    rv_detect = adp->Init(wss, cdet, (nsIDocument*)this, 
-                                     mParser, charset.get(),aCommand);													
-                    nsIParserFilter *oldFilter = nsnull;
-                    if(cdetflt)
-                      oldFilter = mParser->SetParserFilter(cdetflt);
-                    NS_IF_RELEASE(oldFilter);
-                    NS_IF_RELEASE(cdetflt);
-                  }
-                }
-              }
-            }       
-            else 
-            {
-              // IF we cannot create the detector, don't bother to 
-              // create one next time.
-              gPlugDetector = PR_FALSE;
-            }
-
-            NS_IF_RELEASE(cdet);
-            NS_IF_RELEASE(adp);
-            // NO NS_IF_RELEASE(cdetflt); here, do it after mParser->SetParserFilter
-          } //end of autodetection
-
-        } //charset selection for view source only
-      } //got document viewer
-    } //got content view
-
     nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(docShell));
     if (aSink)
       sink = do_QueryInterface(aSink);
@@ -661,6 +545,16 @@ nsXMLDocument::EndLoad()
     nsEvent event;
     event.eventStructType = NS_EVENT;
     event.message = NS_PAGE_LOAD;
+
+    nsCOMPtr<nsIScriptGlobalObject> sgo;
+    nsCOMPtr<nsIScriptGlobalObjectOwner> container =
+      do_QueryReferent(mDocumentContainer);
+    if (container) {
+      container->GetScriptGlobalObject(getter_AddRefs(sgo));
+    }
+
+    nsCxPusher pusher(sgo);
+
     HandleDOMEvent(nsnull, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
   }    
   return nsDocument::EndLoad();  
@@ -703,25 +597,32 @@ nsXMLDocument::GetInlineStyleSheet(nsIHTMLCSSStyleSheet** aResult)
 // subclass hook for sheet ordering
 void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags)
 {
+  // XXXbz this catalog stuff should be in the UA level in the cascade!
   if (aFlags & NS_STYLESHEET_FROM_CATALOG) {
     // always after other catalog sheets
-    mStyleSheets.InsertElementAt(aSheet, mCountCatalogSheets);
+    mStyleSheets.InsertObjectAt(aSheet, mCountCatalogSheets);
     ++mCountCatalogSheets;
   }
   else if (aSheet == mAttrStyleSheet) {  // always after catalog sheets
-    mStyleSheets.InsertElementAt(aSheet, mCountCatalogSheets);
+    NS_ASSERTION(mStyleSheets.Count() == 0 ||
+                 mAttrStyleSheet != mStyleSheets[0],
+                 "Adding attr sheet twice!");
+    mStyleSheets.InsertObjectAt(aSheet, mCountCatalogSheets);
   }
   else if (aSheet == mInlineStyleSheet) {  // always last
-    mStyleSheets.AppendElement(aSheet);
+    NS_ASSERTION(mStyleSheets.Count() == 0 ||
+                 mStyleSheets[mStyleSheets.Count() - 1] != mInlineStyleSheet,
+                 "Adding style attr sheet twice!");
+    mStyleSheets.AppendObject(aSheet);
   }
   else {
     PRInt32 count = mStyleSheets.Count();
-    if (count != 0 && mInlineStyleSheet == mStyleSheets.ElementAt(count - 1)) {
+    if (count != 0 && mInlineStyleSheet == mStyleSheets[count - 1]) {
       // keep attr sheet last
-      mStyleSheets.InsertElementAt(aSheet, count - 1);
+      mStyleSheets.InsertObjectAt(aSheet, count - 1);
     }
     else {
-      mStyleSheets.AppendElement(aSheet);
+      mStyleSheets.AppendObject(aSheet);
     }
   }
 }
@@ -729,8 +630,45 @@ void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags
 void
 nsXMLDocument::InternalInsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex)
 {
+  NS_ASSERTION(0 <= aIndex &&
+               aIndex <= (
+                          mStyleSheets.Count()
+                          /* Don't count Attribute stylesheet */
+                          - 1
+                          /* Don't count catalog sheets */
+                          - mCountCatalogSheets
+                          /* No insertion allowed after StyleAttr stylesheet */
+                          - (mInlineStyleSheet ? 1: 0)
+                          ),
+               "index out of bounds");
   // offset w.r.t. catalog style sheets and the attr style sheet
-  mStyleSheets.InsertElementAt(aSheet, aIndex + mCountCatalogSheets + 1);
+  mStyleSheets.InsertObjectAt(aSheet, aIndex + mCountCatalogSheets + 1);
+}
+
+already_AddRefed<nsIStyleSheet>
+nsXMLDocument::InternalGetStyleSheetAt(PRInt32 aIndex)
+{
+  PRInt32 count = InternalGetNumberOfStyleSheets();
+
+  if (aIndex >= 0 && aIndex < count) {
+    nsIStyleSheet* sheet = mStyleSheets[aIndex + mCountCatalogSheets + 1];
+    NS_ADDREF(sheet);
+    return sheet;
+  } else {
+    NS_ERROR("Index out of range");
+    return nsnull;
+  }
+}
+
+PRInt32
+nsXMLDocument::InternalGetNumberOfStyleSheets()
+{
+  PRInt32 count = mStyleSheets.Count();
+  if (count != 0 && mInlineStyleSheet == mStyleSheets[count - 1])
+    --count;
+  count -= (mCountCatalogSheets + 1); // +1 for the attr sheet
+  NS_ASSERTION(count >= 0, "Why did we end up with a negative count?");
+  return count;
 }
 
 // nsIDOMDocument interface
@@ -1024,6 +962,63 @@ nsXMLDocument::SetDefaultStylesheets(nsIURI* aUrl)
   return result;
 }
 
+NS_IMETHODIMP 
+nsXMLDocument::SetXMLDeclaration(const nsAString& aVersion,
+                                 const nsAString& aEncoding,
+                                 const nsAString& aStandalone)
+{
+  if (aVersion.IsEmpty()) {
+    mXMLDeclarationBits = 0;
+    return NS_OK;
+  }
+
+  mXMLDeclarationBits = XML_DECLARATION_BITS_DECLARATION_EXISTS;
+
+  if (!aEncoding.IsEmpty()) {
+    mXMLDeclarationBits |= XML_DECLARATION_BITS_ENCODING_EXISTS;
+  }
+
+  if (aStandalone.Equals(NS_LITERAL_STRING("yes"))) {
+    mXMLDeclarationBits |= XML_DECLARATION_BITS_STANDALONE_EXISTS |
+                           XML_DECLARATION_BITS_STANDALONE_YES;
+  } else if (aStandalone.Equals(NS_LITERAL_STRING("no"))) {
+    mXMLDeclarationBits |= XML_DECLARATION_BITS_STANDALONE_EXISTS;
+  }
+
+  return NS_OK;
+}                               
+
+NS_IMETHODIMP 
+nsXMLDocument::GetXMLDeclaration(nsAString& aVersion,
+                                 nsAString& aEncoding,
+                                 nsAString& aStandalone)
+{
+  aVersion.Truncate();
+  aEncoding.Truncate();
+  aStandalone.Truncate();
+
+  if (!(mXMLDeclarationBits & XML_DECLARATION_BITS_DECLARATION_EXISTS)) {
+    return NS_OK;
+  }
+
+  aVersion.Assign(NS_LITERAL_STRING("1.0")); // always until we start supporting 1.1 etc.
+
+  if (mXMLDeclarationBits & XML_DECLARATION_BITS_ENCODING_EXISTS) {
+    GetDocumentCharacterSet(aEncoding); // This is what we have stored, not necessarily what was written in the original
+  }
+
+  if (mXMLDeclarationBits & XML_DECLARATION_BITS_STANDALONE_EXISTS) {
+    if (mXMLDeclarationBits & XML_DECLARATION_BITS_STANDALONE_YES) {
+      aStandalone.Assign(NS_LITERAL_STRING("yes"));
+    } else {
+      aStandalone.Assign(NS_LITERAL_STRING("no"));
+    }
+  }
+
+  return NS_OK;
+}
+
+
 NS_IMETHODIMP 
 nsXMLDocument::SetBaseTarget(const nsAString &aBaseTarget)
 {
@@ -1046,7 +1041,8 @@ nsXMLDocument::GetCSSLoader(nsICSSLoader*& aLoader)
     result = NS_NewCSSLoader(this, getter_AddRefs(mCSSLoader));
     if (mCSSLoader) {
       mCSSLoader->SetCaseSensitive(PR_TRUE);
-      mCSSLoader->SetQuirkMode(PR_FALSE); // No quirks in XML
+      // No quirks in XML
+      mCSSLoader->SetCompatibilityMode(eCompatibility_FullStandards);
     }
   }
   aLoader = mCSSLoader;
@@ -1067,8 +1063,8 @@ nsXMLDocument::CreateElement(nsINodeInfo *aNodeInfo, nsIDOMElement** aResult)
   aNodeInfo->GetNamespaceID(namespaceID);
 
   nsCOMPtr<nsIElementFactory> elementFactory;
-  mNameSpaceManager->GetElementFactory(namespaceID,
-                                       getter_AddRefs(elementFactory));
+  nsContentUtils::GetNSManagerWeakRef()->GetElementFactory(namespaceID,
+                                                           getter_AddRefs(elementFactory));
 
   if (elementFactory) {
     rv = elementFactory->CreateInstanceByTag(aNodeInfo,
