@@ -58,11 +58,11 @@
 #include "nsICSSParser.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsIDOM3EventTarget.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
 #include "nsIEventListenerManager.h"
 #include "nsINameSpace.h"
-#include "nsINameSpaceManager.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFResource.h"
 #include "nsIScriptObjectOwner.h"
@@ -196,35 +196,36 @@ public:
 
     Type                     mType;
 
-    PRInt32                  mLineNo;
     PRInt32                  mRefCnt;
           
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext) = 0;
+                               nsIScriptContext* aContext,
+                               nsISupportsArray* aNodeInfos) = 0;
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
-                                 nsIURI* aDocumentURI) = 0;
+                                 nsIURI* aDocumentURI,
+                                 nsISupportsArray* aNodeInfos) = 0;
 
-    void AddRef() { mRefCnt++; };
+    void AddRef() { ++mRefCnt; };
     void Release() 
     { 
-        mRefCnt--; 
+        --mRefCnt; 
         if (mRefCnt == 0) 
             delete this; 
     };
     virtual void ReleaseSubtree() { Release(); };
 
 protected:
-    nsXULPrototypeNode(Type aType, PRInt32 aLineNo)
-        : mType(aType), mLineNo(aLineNo), mRefCnt(1) {}
+    nsXULPrototypeNode(Type aType)
+        : mType(aType), mRefCnt(1) {}
 };
 
 class nsXULPrototypeElement : public nsXULPrototypeNode
 {
 public:
-    nsXULPrototypeElement(PRInt32 aLineNo)
-        : nsXULPrototypeNode(eType_Element, aLineNo),
+    nsXULPrototypeElement()
+        : nsXULPrototypeNode(eType_Element),
           mNumChildren(0),
           mChildren(nsnull),
           mNumAttributes(0),
@@ -257,10 +258,12 @@ public:
     }
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext);
+                               nsIScriptContext* aContext,
+                               nsISupportsArray* aNodeInfos);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
-                                 nsIURI* aDocumentURI);
+                                 nsIURI* aDocumentURI,
+                                 nsISupportsArray* aNodeInfos);
 
     PRInt32                  mNumChildren;
     nsXULPrototypeNode**     mChildren;           // [OWNER]
@@ -278,25 +281,10 @@ public:
 
     static void ReleaseGlobals() 
     { 
-        NS_IF_RELEASE(sNodeInfoManager); 
         NS_IF_RELEASE(sCSSParser);
     }
 
 protected:
-    static nsINodeInfoManager* GetNodeInfoManager()
-    {
-        if (!sNodeInfoManager) {
-            CallCreateInstance(NS_NODEINFOMANAGER_CONTRACTID, &sNodeInfoManager);
-
-            nsCOMPtr<nsINameSpaceManager> nsmgr;
-            NS_NewNameSpaceManager(getter_AddRefs(nsmgr));
-
-            sNodeInfoManager->Init(nsnull, nsmgr);
-        }
-        return sNodeInfoManager;
-    }
-    static nsINodeInfoManager* sNodeInfoManager;
-
     static nsICSSParser* GetCSSParser()
     {
         if (!sCSSParser)
@@ -313,25 +301,28 @@ class nsXULDocument;
 class nsXULPrototypeScript : public nsXULPrototypeNode
 {
 public:
-    nsXULPrototypeScript(PRInt32 aLineNo, const char *aVersion);
+    nsXULPrototypeScript(PRUint16 aLineNo, const char *aVersion);
     virtual ~nsXULPrototypeScript();
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext);
+                               nsIScriptContext* aContext,
+                               nsISupportsArray* aNodeInfos);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
-                                 nsIURI* aDocumentURI);
+                                 nsIURI* aDocumentURI,
+                                 nsISupportsArray* aNodeInfos);
     virtual nsresult DeserializeOutOfLineScript(nsIObjectInputStream* aInput,
                                                 nsIScriptContext* aContext);
 
     nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
-                     nsIURI* aURI, PRInt32 aLineNo,
+                     nsIURI* aURI, PRUint16 aLineNo,
                      nsIDocument* aDocument,
                      nsIXULPrototypeDocument* aPrototypeDocument);
 
     nsCOMPtr<nsIURI>         mSrcURI;
-    PRBool                   mSrcLoading;
-    PRBool                   mOutOfLine;
+    PRUint16                 mLineNo;
+    PRPackedBool             mSrcLoading;
+    PRPackedBool             mOutOfLine;
     nsXULDocument*           mSrcLoadWaiters;   // [OWNER] but not COMPtr
     JSObject*                mJSObject;
     const char*              mLangVersion;
@@ -355,8 +346,8 @@ protected:
 class nsXULPrototypeText : public nsXULPrototypeNode
 {
 public:
-    nsXULPrototypeText(PRInt32 aLineNo)
-        : nsXULPrototypeNode(eType_Text, aLineNo)
+    nsXULPrototypeText()
+        : nsXULPrototypeNode(eType_Text)
     {
         MOZ_COUNT_CTOR(nsXULPrototypeText);
     }
@@ -367,10 +358,12 @@ public:
     }
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext);
+                               nsIScriptContext* aContext,
+                               nsISupportsArray* aNodeInfos);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
-                                 nsIURI* aDocumentURI);
+                                 nsIURI* aDocumentURI,
+                                 nsISupportsArray* aNodeInfos);
 
     nsString                 mValue;
 };
@@ -385,7 +378,6 @@ public:
 
 class nsXULElement : public nsIXULContent,
                      public nsIDOMXULElement,
-                     public nsIDOMEventReceiver,
                      public nsIScriptEventHandlerOwner,
                      public nsIChromeEventHandler
 {
@@ -401,9 +393,6 @@ protected:
     static nsrefcnt             gRefCnt;
     // pseudo-constants
     static nsIRDFService*       gRDFService;
-    static nsINameSpaceManager* gNameSpaceManager;
-    static PRInt32              kNameSpaceID_RDF;
-    static PRInt32              kNameSpaceID_XUL;
 
 public:
     static nsresult
@@ -421,6 +410,8 @@ public:
     NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep, PRBool aCompileEventHandlers);
     NS_IMETHOD GetParent(nsIContent*& aResult) const;
     NS_IMETHOD SetParent(nsIContent* aParent);
+    NS_IMETHOD_(PRBool) IsNativeAnonymous() const;
+    NS_IMETHOD_(void) SetNativeAnonymous(PRBool aAnonymous);
     NS_IMETHOD CanContainChildren(PRBool& aResult) const;
     NS_IMETHOD ChildCount(PRInt32& aResult) const;
     NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
@@ -469,6 +460,7 @@ public:
     NS_IMETHOD GetBindingParent(nsIContent** aContent);
     NS_IMETHOD SetBindingParent(nsIContent* aParent);
     NS_IMETHOD_(PRBool) IsContentOfType(PRUint32 aFlags);
+    NS_IMETHOD GetListenerManager(nsIEventListenerManager** aResult);
 
     // nsIXMLContent
     NS_IMETHOD MaybeTriggerAutoLink(nsIWebShell *aShell);
@@ -477,12 +469,12 @@ public:
     // nsIStyledContent
     NS_IMETHOD GetID(nsIAtom*& aResult) const;
     NS_IMETHOD GetClasses(nsVoidArray& aArray) const;
-    NS_IMETHOD HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
+    NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
 
     NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
-    NS_IMETHOD WalkInlineStyleRules(nsRuleWalker* aRuleWalker);
+    NS_IMETHOD GetInlineStyleRule(nsIStyleRule** aStyleRule);
     NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                        PRInt32& aHint) const;
+                                        nsChangeHint& aHint) const;
 
 
     // nsIXULContent
@@ -501,19 +493,6 @@ public:
     // nsIDOMXULElement
     NS_DECL_NSIDOMXULELEMENT
 
-    // nsIDOMEventTarget interface (from nsIDOMEventReceiver)
-    NS_IMETHOD AddEventListener(const nsAString& aType, nsIDOMEventListener* aListener, 
-                                PRBool aUseCapture);
-    NS_IMETHOD RemoveEventListener(const nsAString& aType, nsIDOMEventListener* aListener, 
-                                   PRBool aUseCapture);
-    NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool* _retval);
-
-    // nsIDOMEventReceiver
-    NS_IMETHOD AddEventListenerByIID(nsIDOMEventListener *aListener, const nsIID& aIID);
-    NS_IMETHOD RemoveEventListenerByIID(nsIDOMEventListener *aListener, const nsIID& aIID);
-    NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
-    NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
-
     // nsIScriptEventHandlerOwner
     NS_IMETHOD CompileEventHandler(nsIScriptContext* aContext,
                                    void* aTarget,
