@@ -950,9 +950,8 @@ nsXULPrototypeCache::StartFastLoad(nsIURI* aURI)
 #endif
                         rv = NS_ERROR_UNEXPECTED;
                     } else {
-                        nsXPIDLCString fileChromePath;
-                        rv = objectInput->ReadStringZ(
-                                                 getter_Copies(fileChromePath));
+                        nsCAutoString fileChromePath;
+                        rv = objectInput->ReadCString(fileChromePath);
                         if (NS_SUCCEEDED(rv) && 
                             !fileChromePath.Equals(chromePath)) { 
                             rv = NS_ERROR_UNEXPECTED;
