@@ -47,6 +47,7 @@
 */
 
 #include "nsCOMPtr.h"
+#include "nsAString.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
 #include "nsIPrincipal.h"
@@ -227,7 +228,7 @@ nsXULPrototypeDocument::nsXULPrototypeDocument()
       mGlobalObject(nsnull),
       mLoaded(PR_FALSE)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 
@@ -245,10 +246,7 @@ nsXULPrototypeDocument::Init()
     mNodeInfoManager = do_CreateInstance(NS_NODEINFOMANAGER_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsINameSpaceManager> nsmgr;
-    rv = NS_NewNameSpaceManager(getter_AddRefs(nsmgr));
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = mNodeInfoManager->Init(nsnull, nsmgr);
+    rv = mNodeInfoManager->Init(nsnull);
     NS_ENSURE_SUCCESS(rv, rv);
 
     return NS_OK;
@@ -358,7 +356,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
         return NS_ERROR_OUT_OF_MEMORY;
     rv |= mGlobalObject->SetGlobalObjectOwner(this); // does not refcount
 
-    mRoot = new nsXULPrototypeElement(-1);
+    mRoot = new nsXULPrototypeElement();
     if (! mRoot)
        return NS_ERROR_OUT_OF_MEMORY;
 
@@ -367,13 +365,30 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     NS_ASSERTION(scriptContext != nsnull,
                  "no prototype script context!");
 
+    // nsINodeInfo table
+    nsCOMPtr<nsISupportsArray> nodeInfos;
+    rv |= NS_NewISupportsArray(getter_AddRefs(nodeInfos));
+    NS_ENSURE_TRUE(nodeInfos, rv);
+
+    rv |= aStream->Read32(&referenceCount);
+    nsAutoString namespaceURI, qualifiedName;
+    for (i = 0; i < referenceCount; ++i) {
+        rv |= aStream->ReadString(namespaceURI);
+        rv |= aStream->ReadString(qualifiedName);
+
+        nsCOMPtr<nsINodeInfo> nodeInfo;
+        rv |= mNodeInfoManager->GetNodeInfo(qualifiedName, namespaceURI, *getter_AddRefs(nodeInfo));
+        rv |= nodeInfos->AppendElement(nodeInfo);
+    }
+
+    // Document contents
     PRUint32 type;
     rv |= aStream->Read32(&type);
 
     if ((nsXULPrototypeNode::Type)type != nsXULPrototypeNode::eType_Element)
         return NS_ERROR_FAILURE;
 
-    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, mNodeInfoManager);
+    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, nodeInfos);
     rv |= NotifyLoadDone();
 
     return rv;
@@ -417,6 +432,27 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     // nsIPrincipal mDocumentPrincipal
     rv |= NS_WriteOptionalObject(aStream, mDocumentPrincipal, PR_TRUE);
     
+    // nsINodeInfo table
+    nsCOMPtr<nsISupportsArray> nodeInfos;
+    rv |= mNodeInfoManager->GetNodeInfoArray(getter_AddRefs(nodeInfos));
+    NS_ENSURE_SUCCESS(rv, rv);
+    PRUint32 nodeInfoCount;
+    nodeInfos->Count(&nodeInfoCount);
+
+    rv |= aStream->Write32(nodeInfoCount);
+    for (i = 0; i < nodeInfoCount; ++i) {
+        nsCOMPtr<nsINodeInfo> nodeInfo = do_QueryElementAt(nodeInfos, i);
+        NS_ENSURE_TRUE(nodeInfo, NS_ERROR_FAILURE);
+
+        nsAutoString namespaceURI;
+        rv |= nodeInfo->GetNamespaceURI(namespaceURI);
+        rv |= aStream->WriteWStringZ(namespaceURI.get());
+
+        nsAutoString qualifiedName;
+        rv |= nodeInfo->GetQualifiedName(qualifiedName);
+        rv |= aStream->WriteWStringZ(qualifiedName.get());
+    }
+
     // Now serialize the document contents
     nsCOMPtr<nsIScriptGlobalObject> globalObject;
     rv |= GetScriptGlobalObject(getter_AddRefs(globalObject));
@@ -425,7 +461,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     rv |= globalObject->GetContext(getter_AddRefs(scriptContext));
     
     if (mRoot)
-        rv |= mRoot->Serialize(aStream, scriptContext);
+        rv |= mRoot->Serialize(aStream, scriptContext, nodeInfos);
  
     return rv;
 }
@@ -693,7 +729,7 @@ nsXULPDGlobalObject::nsXULPDGlobalObject()
     : mJSObject(nsnull),
       mGlobalObjectOwner(nsnull)
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 
