@@ -125,7 +125,6 @@
 
 //----------------------------------------------------------------------
 
-static NS_DEFINE_CID(kNameSpaceManagerCID,       NS_NAMESPACEMANAGER_CID);
 static NS_DEFINE_CID(kRDFContainerUtilsCID,      NS_RDFCONTAINERUTILS_CID);
 static NS_DEFINE_CID(kRDFServiceCID,             NS_RDFSERVICE_CID);
 
@@ -135,11 +134,8 @@ static NS_DEFINE_CID(kRDFServiceCID,             NS_RDFSERVICE_CID);
 //
 
 nsrefcnt                  nsXULTemplateBuilder::gRefCnt = 0;
-PRInt32                   nsXULTemplateBuilder::kNameSpaceID_RDF;
-PRInt32                   nsXULTemplateBuilder::kNameSpaceID_XUL;
 nsIRDFService*            nsXULTemplateBuilder::gRDFService;
 nsIRDFContainerUtils*     nsXULTemplateBuilder::gRDFContainerUtils;
-nsINameSpaceManager*      nsXULTemplateBuilder::gNameSpaceManager;
 nsIScriptSecurityManager* nsXULTemplateBuilder::gScriptSecurityManager;
 nsIPrincipal*             nsXULTemplateBuilder::gSystemPrincipal;
 
@@ -177,8 +173,6 @@ nsXULTemplateBuilder::~nsXULTemplateBuilder(void)
             gRDFContainerUtils = nsnull;
         }
 
-        NS_RELEASE(gNameSpaceManager);
-
         NS_IF_RELEASE(gSystemPrincipal);
 
         if (gScriptSecurityManager) {
@@ -195,30 +189,6 @@ nsXULTemplateBuilder::Init()
     if (gRefCnt++ == 0) {
         nsresult rv;
 
-        // Register the XUL and RDF namespaces: these'll just retrieve
-        // the IDs if they've already been registered by someone else.
-        rv = nsComponentManager::CreateInstance(kNameSpaceManagerCID,
-                                                nsnull,
-                                                NS_GET_IID(nsINameSpaceManager),
-                                                (void**) &gNameSpaceManager);
-        NS_ASSERTION(NS_SUCCEEDED(rv), "unable to create namespace manager");
-        if (NS_FAILED(rv)) return rv;
-
-        // XXX This is sure to change. Copied from mozilla/layout/xul/content/src/nsXULAtoms.cpp
-        static const char kXULNameSpaceURI[]
-            = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-
-        static const char kRDFNameSpaceURI[]
-            = RDF_NAMESPACE_URI;
-
-        rv = gNameSpaceManager->RegisterNameSpace(NS_ConvertASCIItoUCS2(kXULNameSpaceURI), kNameSpaceID_XUL);
-        NS_ASSERTION(NS_SUCCEEDED(rv), "unable to register XUL namespace");
-        if (NS_FAILED(rv)) return rv;
-
-        rv = gNameSpaceManager->RegisterNameSpace(NS_ConvertASCIItoUCS2(kRDFNameSpaceURI), kNameSpaceID_RDF);
-        NS_ASSERTION(NS_SUCCEEDED(rv), "unable to register RDF namespace");
-        if (NS_FAILED(rv)) return rv;
-
         // Initialize the global shared reference to the service
         // manager and get some shared resource objects.
         rv = nsServiceManager::GetService(kRDFServiceCID,
@@ -290,6 +260,11 @@ nsXULTemplateBuilder::CreateContents(nsIContent* aElement)
 // nsIDocumentOberver interface
 //
 
+NS_IMPL_NSIDOCUMENTOBSERVER_LOAD_STUB(nsXULTemplateBuilder)
+NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(nsXULTemplateBuilder)
+NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(nsXULTemplateBuilder)
+NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsXULTemplateBuilder)
+
 NS_IMETHODIMP
 nsXULTemplateBuilder::BeginUpdate(nsIDocument *aDocument)
 {
@@ -302,30 +277,6 @@ nsXULTemplateBuilder::EndUpdate(nsIDocument *aDocument)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXULTemplateBuilder::BeginLoad(nsIDocument *aDocument)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::EndLoad(nsIDocument *aDocument)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::BeginReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::EndReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsXULTemplateBuilder::ContentChanged(nsIDocument *aDocument,
                                      nsIContent* aContent,
@@ -334,15 +285,6 @@ nsXULTemplateBuilder::ContentChanged(nsIDocument *aDocument,
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXULTemplateBuilder::ContentStatesChanged(nsIDocument* aDocument,
-                                           nsIContent* aContent1,
-                                           nsIContent* aContent2,
-                                           PRInt32 aStateMask)
-{
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsXULTemplateBuilder::AttributeChanged(nsIDocument *aDocument,
                                        nsIContent*  aContent,
@@ -396,53 +338,6 @@ nsXULTemplateBuilder::ContentRemoved(nsIDocument *aDocument,
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleSheetAdded(nsIDocument *aDocument,
-                                      nsIStyleSheet* aStyleSheet)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleSheetRemoved(nsIDocument *aDocument,
-                                        nsIStyleSheet* aStyleSheet)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleSheetDisabledStateChanged(nsIDocument *aDocument,
-                                                     nsIStyleSheet* aStyleSheet,
-                                                     PRBool aDisabled)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleRuleChanged(nsIDocument *aDocument,
-                                       nsIStyleSheet* aStyleSheet,
-                                       nsIStyleRule* aStyleRule,
-                                       nsChangeHint aHint)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleRuleAdded(nsIDocument *aDocument,
-                                     nsIStyleSheet* aStyleSheet,
-                                     nsIStyleRule* aStyleRule)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXULTemplateBuilder::StyleRuleRemoved(nsIDocument *aDocument,
-                                       nsIStyleSheet* aStyleSheet,
-                                       nsIStyleRule* aStyleRule)
-{
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsXULTemplateBuilder::DocumentWillBeDestroyed(nsIDocument *aDocument)
 {
@@ -623,14 +518,19 @@ nsXULTemplateBuilder::Retract(nsIRDFResource* aSource,
                 ReplaceMatch(VALUE_TO_IRDFRESOURCE(memberval), match.operator->(), nsnull);
             }
         }
-
+#if 0
         // Now fire any newly revealed rules
         {
             nsTemplateMatchSet::ConstIterator last = firings.Last();
             for (nsTemplateMatchSet::ConstIterator match = firings.First(); match != last; ++match) {
                 // XXXwaterson yo. write me.
+                // The intent here is to handle any rules that might be
+                // "revealed" by the removal of an assertion from the datasource.
+                // Waterson doesn't think we support negated conditions in a rule.
+                // Nor is he sure that this is currently useful.
             }
         }
+#endif
     }
 
     return NS_OK;
@@ -1458,7 +1358,7 @@ nsXULTemplateBuilder::IsTemplateElement(nsIContent* aContent)
     PRInt32 nameSpaceID;
     aContent->GetNameSpaceID(nameSpaceID);
 
-    if (nameSpaceID == nsXULTemplateBuilder::kNameSpaceID_XUL) {
+    if (nameSpaceID == kNameSpaceID_XUL) {
         nsCOMPtr<nsIAtom> tag;
         aContent->GetTag(*getter_AddRefs(tag));
 
