@@ -24,7 +24,9 @@
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
 #include "nsEscape.h"
+#include "nsCRT.h"
 
+#include "nsIPrefService.h"
 #include "nsIPlatformCharset.h"
 #include "nsILocalFile.h"
 
@@ -38,6 +40,9 @@ nsDefaultURIFixup::nsDefaultURIFixup()
 {
   NS_INIT_ISUPPORTS();
   /* member initializers and constructor code */
+
+  // Try and get the pref service
+  mPrefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID);
 }
 
 
@@ -46,29 +51,71 @@ nsDefaultURIFixup::~nsDefaultURIFixup()
   /* destructor code */
 }
 
+/* nsIURI createExposableURI (in nsIRUI aURI); */
+NS_IMETHODIMP
+nsDefaultURIFixup::CreateExposableURI(nsIURI *aURI, nsIURI **aReturn)
+{
+    NS_ENSURE_ARG_POINTER(aURI);
+    NS_ENSURE_ARG_POINTER(aReturn);
+
+    PRBool isWyciwyg = PR_FALSE;
+    aURI->SchemeIs("wyciwyg", &isWyciwyg);
+
+    if (!isWyciwyg)
+    {
+        *aReturn = aURI;
+        NS_ADDREF(*aReturn);
+        return NS_OK;
+    }
+
+    nsCAutoString path;
+    nsresult rv = aURI->GetPath(path);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    PRUint32 pathLength = path.Length();
+    if (pathLength <= 2)
+    {
+        return NS_ERROR_FAILURE;
+    }
+
+    // Path is of the form "//123/http://foo/bar", with a variable number of digits.
+    // To figure out where the "real" URL starts, search path for a '/', starting at 
+    // the third character.
+    PRInt32 slashIndex = path.FindChar('/', 2);
+    if (slashIndex == kNotFound)
+    {
+        return NS_ERROR_FAILURE;
+    }
+
+    // Get the charset of the original URI so we can pass it to our fixed up URI.
+    nsCAutoString charset;
+    aURI->GetOriginCharset(charset);
+
+    rv = NS_NewURI(aReturn,
+                   Substring(path, slashIndex + 1, pathLength - slashIndex - 1),
+                   charset.get());
+    NS_ENSURE_SUCCESS(rv, rv);
+    return NS_OK;
+}
 
 /* nsIURI createFixupURI (in wstring aURIText, in unsigned long aFixupFlags); */
 NS_IMETHODIMP
-nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFlags, nsIURI **aURI)
+nsDefaultURIFixup::CreateFixupURI(const nsAString& aStringURI, PRUint32 aFixupFlags, nsIURI **aURI)
 {
-    NS_ENSURE_ARG_POINTER(aStringURI);
+    NS_ENSURE_ARG(!aStringURI.IsEmpty());
     NS_ENSURE_ARG_POINTER(aURI);
 
     nsresult rv;
     *aURI = nsnull;
 
-    // Try and get the prefs service
-    if (!mPrefs)
-    {
-        mPrefs = do_GetService(NS_PREF_CONTRACTID);
-    }
-
     nsAutoString uriString(aStringURI);
     uriString.Trim(" ");  // Cleanup the empty spaces that might be on each end.
 
     // Eliminate embedded newlines, which single-line text fields now allow:
     uriString.StripChars("\r\n");
 
+    NS_ENSURE_TRUE(!uriString.IsEmpty(), NS_ERROR_FAILURE);
+
     // View-source is a pseudo scheme. We're interested in fixing up the stuff
     // after it. The easiest way to do that is to call this method again with the
     // "view-source:" lopped off and then prepend it again afterwards.
@@ -78,10 +125,9 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
         nsCOMPtr<nsIURI> uri;
         PRUint32 newFixupFlags = aFixupFlags & ~FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP;
 
-        nsAutoString tempString;
-        tempString = Substring(uriString, 12, uriString.Length() - 12);
-        rv =  CreateFixupURI(tempString.get(), newFixupFlags, getter_AddRefs(uri));
-        if (NS_FAILED(rv) || !uri)
+        rv =  CreateFixupURI(Substring(uriString, 12, uriString.Length() - 12),
+                             newFixupFlags, getter_AddRefs(uri));
+        if (NS_FAILED(rv))
             return NS_ERROR_FAILURE;
         nsCAutoString spec;
         uri->GetSpec(spec);
@@ -89,18 +135,45 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
     }
     else {
         // Check for if it is a file URL
-        FileURIFixup(uriString.get(), aURI);
+        FileURIFixup(uriString, aURI);
         if(*aURI)
             return NS_OK;
 
 #ifdef XP_PC
         // Not a file URL, so translate '\' to '/' for convenience in the common protocols
+        // e.g. catch
+        //
+        //   http:\\broken.com\address
+        //   http:\\broken.com/blah
+        //   broken.com\blah
+        //
+        // Code will also do partial fix up the following urls
+        //
+        //   http:\\broken.com\address/somewhere\image.jpg (stops at first forward slash)
+        //   http:\\broken.com\blah?arg=somearg\foo.jpg (stops at question mark)
+        //   http:\\broken.com#odd\ref (stops at hash)
+        //  
         if (uriString.FindChar(':') == -1 ||
             uriString.EqualsIgnoreCase("http:", 5) ||
             uriString.EqualsIgnoreCase("https:", 6) ||
             uriString.EqualsIgnoreCase("ftp:", 4))
         {
-            uriString.ReplaceChar(PRUnichar('\\'), PRUnichar('/'));
+            // Walk the string replacing backslashes with forward slashes until
+            // the end is reached, or a question mark, or a hash, or a forward
+            // slash. The forward slash test is to stop before trampling over
+            // URIs which legitimately contain a mix of both forward and
+            // backward slashes.
+            nsAFlatString::iterator start;
+            nsAFlatString::iterator end;
+            uriString.BeginWriting(start);
+            uriString.EndWriting(end);
+            while (start != end) {
+                if (*start == '?' || *start == '#' || *start == '/')
+                    break;
+                if (*start == '\\')
+                    *start = '/';
+                ++start;
+            }
         }
 #endif
     }
@@ -116,12 +189,23 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
                         uriString.EqualsIgnoreCase("ftp:", 4) ||
                         uriString.EqualsIgnoreCase("file:", 5));
 
-    // Just try to create an URL out of it
-    rv = NS_NewURI(aURI, uriString, bUseNonDefaultCharsetForURI ? GetCharsetForUrlBar() : nsnull);
-    if (rv == NS_ERROR_UNKNOWN_PROTOCOL)
-    {
-        return rv;
+    // Check the scheme...
+    NS_LossyConvertUCS2toASCII asciiURI(uriString);
+    nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsCAutoString scheme;
+    ioService->ExtractScheme(asciiURI, scheme);
+    nsCOMPtr<nsIProtocolHandler> ourHandler, extHandler;
+    
+    ioService->GetProtocolHandler(scheme.get(), getter_AddRefs(ourHandler));
+    extHandler = do_GetService(NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX"default");
+
+    if (ourHandler != extHandler || !PossiblyHostPortUrl(uriString)) {
+        // Just try to create an URL out of it
+        rv = NS_NewURI(aURI, uriString,
+                       bUseNonDefaultCharsetForURI ? GetCharsetForUrlBar() : nsnull);
     }
+
     if (*aURI) {
         if (aFixupFlags & FIXUP_FLAGS_MAKE_ALTERNATE_URI)
             MakeAlternateURI(*aURI);
@@ -132,13 +216,13 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
     // Test whether keywords need to be fixed up
     if (aFixupFlags & FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP) {
         PRBool fixupKeywords = PR_FALSE;
-        if (mPrefs)
+        if (mPrefBranch)
         {
-            NS_ENSURE_SUCCESS(mPrefs->GetBoolPref("keyword.enabled", &fixupKeywords), NS_ERROR_FAILURE);
+            NS_ENSURE_SUCCESS(mPrefBranch->GetBoolPref("keyword.enabled", &fixupKeywords), NS_ERROR_FAILURE);
         }
         if (fixupKeywords)
         {
-            KeywordURIFixup(uriString.get(), aURI);
+            KeywordURIFixup(uriString, aURI);
             if(*aURI)
                 return NS_OK;
         }
@@ -200,7 +284,7 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
     // Did the caller want us to try an alternative URI?
     // If so, attempt to fixup http://foo into http://www.foo.com
 
-    if (aURI && aFixupFlags & FIXUP_FLAGS_MAKE_ALTERNATE_URI) {
+    if (*aURI && aFixupFlags & FIXUP_FLAGS_MAKE_ALTERNATE_URI) {
         MakeAlternateURI(*aURI);
     }
 
@@ -210,12 +294,12 @@ nsDefaultURIFixup::CreateFixupURI(const PRUnichar *aStringURI, PRUint32 aFixupFl
 
 PRBool nsDefaultURIFixup::MakeAlternateURI(nsIURI *aURI)
 {
-    PRBool makeAlternate = PR_TRUE;
-    if (!mPrefs)
+    if (!mPrefBranch)
     {
         return PR_FALSE;
     }
-    mPrefs->GetBoolPref("browser.fixup.alternate.enabled", &makeAlternate);
+    PRBool makeAlternate = PR_TRUE;
+    mPrefBranch->GetBoolPref("browser.fixup.alternate.enabled", &makeAlternate);
     if (!makeAlternate)
     {
         return PR_FALSE;
@@ -259,7 +343,7 @@ PRBool nsDefaultURIFixup::MakeAlternateURI(nsIURI *aURI)
 
     nsCAutoString prefix("www.");
     nsXPIDLCString prefPrefix;
-    rv = mPrefs->GetCharPref("browser.fixup.alternate.prefix", getter_Copies(prefPrefix));
+    rv = mPrefBranch->GetCharPref("browser.fixup.alternate.prefix", getter_Copies(prefPrefix));
     if (NS_SUCCEEDED(rv))
     {
         prefix.Assign(prefPrefix);
@@ -267,7 +351,7 @@ PRBool nsDefaultURIFixup::MakeAlternateURI(nsIURI *aURI)
 
     nsCAutoString suffix(".com");
     nsXPIDLCString prefSuffix;
-    rv = mPrefs->GetCharPref("browser.fixup.alternate.suffix", getter_Copies(prefSuffix));
+    rv = mPrefBranch->GetCharPref("browser.fixup.alternate.suffix", getter_Copies(prefSuffix));
     if (NS_SUCCEEDED(rv))
     {
         suffix.Assign(prefSuffix);
@@ -311,13 +395,12 @@ PRBool nsDefaultURIFixup::MakeAlternateURI(nsIURI *aURI)
     return PR_TRUE;
 }
 
-nsresult nsDefaultURIFixup::FileURIFixup(const PRUnichar* aStringURI, 
+nsresult nsDefaultURIFixup::FileURIFixup(const nsAString& aStringURI, 
                                          nsIURI** aURI)
 {
-    nsAutoString uriSpecIn(aStringURI);
     nsCAutoString uriSpecOut;
 
-    nsresult rv = ConvertFileToStringURI(uriSpecIn, uriSpecOut);
+    nsresult rv = ConvertFileToStringURI(aStringURI, uriSpecOut);
     if (NS_SUCCEEDED(rv))
     {
         // if this is file url, uriSpecOut is already in FS charset
@@ -327,7 +410,7 @@ nsresult nsDefaultURIFixup::FileURIFixup(const PRUnichar* aStringURI,
     return NS_ERROR_FAILURE;
 }
 
-nsresult nsDefaultURIFixup::ConvertFileToStringURI(nsString& aIn,
+nsresult nsDefaultURIFixup::ConvertFileToStringURI(const nsAString& aIn,
                                                    nsCString& aOut)
 {
     PRBool attemptFixup = PR_FALSE;
@@ -339,9 +422,8 @@ nsresult nsDefaultURIFixup::ConvertFileToStringURI(nsString& aIn,
         attemptFixup = PR_TRUE;
     }
 #elif XP_UNIX
-    // Check if it starts with / or \ (UNIX)
-    const PRUnichar * up = aIn.get();
-    if((PRUnichar('/') == *up) || (PRUnichar('\\') == *up))
+    // Check if it starts with / (UNIX)
+    if(aIn.First() == PRUnichar('/'))
     {
         attemptFixup = PR_TRUE;
     }
@@ -399,20 +481,125 @@ nsresult nsDefaultURIFixup::ConvertFileToStringURI(nsString& aIn,
     return NS_ERROR_FAILURE;
 }
 
-PRBool nsDefaultURIFixup::PossiblyByteExpandedFileName(nsString& aIn)
+PRBool nsDefaultURIFixup::PossiblyHostPortUrl(const nsAString &aUrl)
 {
-  // XXXXX HACK XXXXX : please don't copy this code.
-  // There are cases where aIn contains the locale byte chars padded to short
-  // (thus the name "ByteExpanded"); whereas other cases 
-  // have proper Unicode code points.
-  // This is a temporary fix.  Please refer to 58866, 86948
-  const PRUnichar* uniChar = aIn.get();
-  for (PRUint32 i = 0; i < aIn.Length(); i++)  {
-    if ((uniChar[i] >= 0x0080) && (uniChar[i] <= 0x00FF))  {
-      return PR_TRUE;
+    // Oh dear, the protocol is invalid. Test if the protocol might
+    // actually be a url without a protocol:
+    //
+    //   http://www.faqs.org/rfcs/rfc1738.html
+    //   http://www.faqs.org/rfcs/rfc2396.html
+    //
+    // e.g. Anything of the form:
+    //
+    //   <hostname>:<port> or
+    //   <hostname>:<port>/
+    //
+    // Where <hostname> is a string of alphanumeric characters and dashes
+    // seperated by dots.
+    // and <port> is a 5 or less digits. This actually breaks the rfc2396
+    // definition of a scheme which allows dots in schemes.
+    //
+    // Note:
+    //   People expecting this to work with
+    //   <user>:<password>@<host>:<port>/<url-path> will be disappointed!
+    //
+    // Note: Parser could be a lot tighter, tossing out silly hostnames
+    //       such as those containing consecutive dots and so on.
+
+    // Read the hostname which should of the form
+    // [a-zA-Z0-9\-]+(\.[a-zA-Z0-9\-]+)*:
+
+    nsReadingIterator<PRUnichar> iterBegin;
+    nsReadingIterator<PRUnichar> iterEnd;
+    aUrl.BeginReading(iterBegin);
+    aUrl.EndReading(iterEnd);
+    nsReadingIterator<PRUnichar> iter = iterBegin;
+
+    while (iter != iterEnd)
+    {
+        PRUint32 chunkSize = 0;
+        // Parse a chunk of the address
+        while (iter != iterEnd &&
+               (*iter == PRUnichar('-') ||
+                nsCRT::IsAsciiAlpha(*iter) ||
+                nsCRT::IsAsciiDigit(*iter)))
+        {
+            ++chunkSize;
+            ++iter;
+        }
+        if (chunkSize == 0 || iter == iterEnd)
+        {
+            return PR_FALSE;
+        }
+        if (*iter == PRUnichar(':'))
+        {
+            // Go onto checking the for the digits
+            break;
+        }
+        if (*iter != PRUnichar('.'))
+        {
+            // Whatever it is, it ain't a hostname!
+            return PR_FALSE;
+        }
+        ++iter;
     }
-  }
-  return PR_FALSE;
+    if (iter == iterEnd)
+    {
+        // No point continuing since there is no colon
+        return PR_FALSE;
+    }
+    ++iter;
+
+    // Count the number of digits after the colon and before the
+    // next forward slash (or end of string)
+
+    PRUint32 digitCount = 0;
+    while (iter != iterEnd && digitCount <= 5)
+    {
+        if (nsCRT::IsAsciiDigit(*iter))
+        {
+            digitCount++;
+        }
+        else if (*iter == PRUnichar('/'))
+        {
+            break;
+        }
+        else
+        {
+            // Whatever it is, it ain't a port!
+            return PR_FALSE;
+        }
+        ++iter;
+    }
+    if (digitCount == 0 || digitCount > 5)
+    {
+        // No digits or more digits than a port would have.
+        return PR_FALSE;
+    }
+
+    // Yes, it's possibly a host:port url
+    return PR_TRUE;
+}
+
+PRBool nsDefaultURIFixup::PossiblyByteExpandedFileName(const nsAString& aIn)
+{
+    // XXXXX HACK XXXXX : please don't copy this code.
+    // There are cases where aIn contains the locale byte chars padded to short
+    // (thus the name "ByteExpanded"); whereas other cases 
+    // have proper Unicode code points.
+    // This is a temporary fix.  Please refer to 58866, 86948
+
+    nsReadingIterator<PRUnichar> iter;
+    nsReadingIterator<PRUnichar> iterEnd;
+    aIn.BeginReading(iter);
+    aIn.EndReading(iterEnd);
+    while (iter != iterEnd)
+    {
+        if (*iter >= 0x0080 && *iter <= 0x00FF)
+            return PR_TRUE;
+        ++iter;
+    }
+    return PR_FALSE;
 }
 
 const char * nsDefaultURIFixup::GetFileSystemCharset()
@@ -450,7 +637,7 @@ const char * nsDefaultURIFixup::GetCharsetForUrlBar()
   return charset;
 }
 
-nsresult nsDefaultURIFixup::KeywordURIFixup(const PRUnichar* aStringURI, 
+nsresult nsDefaultURIFixup::KeywordURIFixup(const nsAString & aURIString, 
                                             nsIURI** aURI)
 {
     // These are keyword formatted strings
@@ -466,11 +653,10 @@ nsresult nsDefaultURIFixup::KeywordURIFixup(const PRUnichar* aStringURI,
     // "nonQualifiedHost?args"
     // "nonQualifiedHost?some args"
 
-    nsAutoString uriString(aStringURI);
-    if(uriString.FindChar('.') == -1 && uriString.FindChar(':') == -1)
+    if(aURIString.FindChar('.') == -1 && aURIString.FindChar(':') == -1)
     {
-        PRInt32 qMarkLoc = uriString.FindChar('?');
-        PRInt32 spaceLoc = uriString.FindChar(' ');
+        PRInt32 qMarkLoc = aURIString.FindChar('?');
+        PRInt32 spaceLoc = aURIString.FindChar(' ');
 
         PRBool keyword = PR_FALSE;
         if(qMarkLoc == 0)
@@ -481,7 +667,7 @@ nsresult nsDefaultURIFixup::KeywordURIFixup(const PRUnichar* aStringURI,
         if(keyword)
         {
             nsCAutoString keywordSpec("keyword:");
-            char *utf8Spec = ToNewUTF8String(uriString);
+            char *utf8Spec = ToNewUTF8String(aURIString);
             if(utf8Spec)
             {
                 char* escapedUTF8Spec = nsEscape(utf8Spec, url_Path);
