@@ -32,6 +32,7 @@
 #include "nsIPref.h"
 #include "nsVoidArray.h"
 #include "nsIScriptContext.h"
+#include "nsITimer.h"
 
 #include "nsCDocShell.h"
 #include "nsIDocShellTreeOwner.h"
@@ -69,7 +70,6 @@
 #include "nsISHistory.h"
 #include "nsIStringBundle.h"
 #include "nsISupportsArray.h"
-#include "nsITimerCallback.h"
 #include "nsIWebNavigation.h"
 #include "nsIWebPageDescriptor.h"
 #include "nsIWebProgressListener.h"
@@ -116,9 +116,8 @@ public:
     nsRefreshTimer();
 
     NS_DECL_ISUPPORTS
- 
-    // nsITimerCallback interface
-    NS_IMETHOD_(void) Notify(nsITimer * timer);
+    NS_DECL_NSITIMERCALLBACK
+
     PRInt32 GetDelay() { return mDelay ;}
 
     nsCOMPtr<nsIDocShell> mDocShell;
@@ -201,7 +200,6 @@ protected:
     NS_IMETHOD SetupNewViewer(nsIContentViewer * aNewViewer);
 
     NS_IMETHOD GetEldestPresContext(nsIPresContext** aPresContext);
-    NS_IMETHOD CreateFixupURI(const PRUnichar * aStringURI, nsIURI ** aURI);
     NS_IMETHOD GetCurrentDocumentOwner(nsISupports ** aOwner);
     virtual nsresult DoURILoad(nsIURI * aURI,
                                nsIURI * aReferrer,
@@ -236,9 +234,8 @@ protected:
     PRBool ShouldDiscardLayoutState(nsIHttpChannel * aChannel);
     
     // Global History
-    NS_IMETHOD ShouldAddToGlobalHistory(nsIURI * aURI, PRBool * aShouldAdd);
-    NS_IMETHOD AddToGlobalHistory(nsIURI * aURI);
-    NS_IMETHOD UpdateCurrentGlobalHistory();
+    nsresult ShouldAddToGlobalHistory(nsIURI * aURI, PRBool * aShouldAdd);
+    nsresult AddToGlobalHistory(nsIChannel* aChannel, nsIURI * aURI, PRBool aHidden);
 
     // Helper Routines
     NS_IMETHOD GetPromptAndStringBundle(nsIPrompt ** aPrompt,
@@ -253,6 +250,9 @@ protected:
     NS_IMETHOD RefreshURIFromQueue();
     NS_IMETHOD DisplayLoadError(nsresult aError, nsIURI *aURI, const PRUnichar *aURL);
     NS_IMETHOD LoadErrorPage(nsIURI *aURI, const PRUnichar *aURL, const PRUnichar *aPage, const PRUnichar *aDescription);
+    PRBool IsPrintingOrPP(PRBool aDisplayErrorDialog = PR_TRUE);
+
+    nsresult SetBaseUrlForWyciwyg(nsIContentViewer * aContentViewer);
 
     static  inline  PRUint32
     PRTimeToSeconds(PRTime t_usec)
@@ -364,6 +364,9 @@ protected:
     nsIDocShellTreeOwner *     mTreeOwner; // Weak Reference
     nsIChromeEventHandler *    mChromeEventHandler; //Weak Reference
 
+    // Indivates that a DocShell in this "docshell tree" is printing
+    PRBool mIsPrintingOrPP;
+
 public:
     class InterfaceRequestorProxy : public nsIInterfaceRequestor {
     public:
