@@ -117,7 +117,8 @@ typedef unsigned long HMTX;
 #include "nsIFileStream.h"
 #include "nsISHistoryInternal.h"
 
-#include "nsIHttpChannel.h" // add this to the ick include list...we need it to QI for post data interface
+#include "nsIHttpChannel.h"
+#include "nsIUploadChannel.h"
 
 #include "nsILocaleService.h"
 #include "nsIStringBundle.h"
@@ -164,6 +165,18 @@ static NS_DEFINE_CID(kCDOMRangeCID,           NS_RANGE_CID);
 
 //----------------------------------------------------------------------
 
+static PRBool
+IsOffline()
+{
+    PRBool offline = PR_TRUE;
+    nsCOMPtr<nsIIOService> ios(do_GetIOService());
+    if (ios)
+        ios->GetOffline(&offline);
+    return offline;
+}
+
+//----------------------------------------------------------------------
+
 // Note: operator new zeros our memory
 nsWebShell::nsWebShell() : nsDocShell()
 {
@@ -175,7 +188,7 @@ nsWebShell::nsWebShell() : nsDocShell()
     printf("WEBSHELL+ = %ld\n", gNumberOfWebShells);
 #endif
 
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mThread = nsnull;
   InitFrameData();
   mItemType = typeContent;
@@ -187,11 +200,11 @@ nsWebShell::~nsWebShell()
    Destroy();
 
   // Stop any pending document loads and destroy the loader...
-  if (nsnull != mDocLoader) {
+  if (mDocLoader) {
     mDocLoader->Stop();
     mDocLoader->SetContainer(nsnull);
     mDocLoader->Destroy();
-    NS_RELEASE(mDocLoader);
+    mDocLoader = nsnull;
   }
   // Cancel any timers that were set for this loader.
   CancelRefreshURITimers();
@@ -342,8 +355,10 @@ NS_IMETHODIMP
 nsWebShell::GetDocumentLoader(nsIDocumentLoader*& aResult)
 {
   aResult = mDocLoader;
-  NS_IF_ADDREF(mDocLoader);
-  return (nsnull != mDocLoader) ? NS_OK : NS_ERROR_FAILURE;
+  if (!mDocLoader)
+    return NS_ERROR_FAILURE;
+  NS_ADDREF(aResult);
+  return NS_OK;
 }
 
 //----------------------------------------------------------------------
@@ -449,25 +464,25 @@ nsWebShell::SetRendering(PRBool aRender)
 
 struct OnLinkClickEvent : public PLEvent {
   OnLinkClickEvent(nsWebShell* aHandler, nsIContent* aContent,
-                   nsLinkVerb aVerb, const PRUnichar* aURLSpec,
+                   nsLinkVerb aVerb, nsIURI* aURI,
                    const PRUnichar* aTargetSpec, nsIInputStream* aPostDataStream = 0, 
                    nsIInputStream* aHeadersDataStream = 0);
   ~OnLinkClickEvent();
 
   void HandleEvent() {
-    mHandler->OnLinkClickSync(mContent, mVerb, mURLSpec.get(),
+    mHandler->OnLinkClickSync(mContent, mVerb, mURI,
                               mTargetSpec.get(), mPostDataStream,
                               mHeadersDataStream,
                               nsnull, nsnull);
   }
 
-  nsWebShell*     mHandler;
-  nsString       mURLSpec;
-  nsString       mTargetSpec;
+  nsWebShell*              mHandler;
+  nsCOMPtr<nsIURI>         mURI;
+  nsString                 mTargetSpec;
   nsCOMPtr<nsIInputStream> mPostDataStream;
   nsCOMPtr<nsIInputStream> mHeadersDataStream;
   nsCOMPtr<nsIContent>     mContent;
-  nsLinkVerb      mVerb;
+  nsLinkVerb               mVerb;
 };
 
 static void PR_CALLBACK HandlePLEvent(OnLinkClickEvent* aEvent)
@@ -483,14 +498,14 @@ static void PR_CALLBACK DestroyPLEvent(OnLinkClickEvent* aEvent)
 OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
                                    nsIContent *aContent,
                                    nsLinkVerb aVerb,
-                                   const PRUnichar* aURLSpec,
+                                   nsIURI* aURI,
                                    const PRUnichar* aTargetSpec,
                                    nsIInputStream* aPostDataStream,
                                    nsIInputStream* aHeadersDataStream)
 {
   mHandler = aHandler;
   NS_ADDREF(aHandler);
-  mURLSpec.Assign(aURLSpec);
+  mURI = aURI;
   mTargetSpec.Assign(aTargetSpec);
   mPostDataStream = aPostDataStream;
   mHeadersDataStream = aHeadersDataStream;
@@ -518,7 +533,7 @@ OnLinkClickEvent::~OnLinkClickEvent()
 NS_IMETHODIMP
 nsWebShell::OnLinkClick(nsIContent* aContent,
                         nsLinkVerb aVerb,
-                        const PRUnichar* aURLSpec,
+                        nsIURI* aURI,
                         const PRUnichar* aTargetSpec,
                         nsIInputStream* aPostDataStream,
                         nsIInputStream* aHeadersDataStream)
@@ -526,7 +541,7 @@ nsWebShell::OnLinkClick(nsIContent* aContent,
   OnLinkClickEvent* ev;
   nsresult rv = NS_OK;
 
-  ev = new OnLinkClickEvent(this, aContent, aVerb, aURLSpec,
+  ev = new OnLinkClickEvent(this, aContent, aVerb, aURI,
                             aTargetSpec, aPostDataStream, aHeadersDataStream);
   if (nsnull == ev) {
     rv = NS_ERROR_OUT_OF_MEMORY;
@@ -549,7 +564,7 @@ nsWebShell::GetEventQueue(nsIEventQueue **aQueue)
 NS_IMETHODIMP
 nsWebShell::OnLinkClickSync(nsIContent *aContent,
                             nsLinkVerb aVerb,
-                            const PRUnichar* aURLSpec,
+                            nsIURI* aURI,
                             const PRUnichar* aTargetSpec,
                             nsIInputStream* aPostDataStream,
                             nsIInputStream* aHeadersDataStream,
@@ -575,36 +590,7 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
       // Fall through, this seems like the most reasonable action
     case eLinkVerb_Replace:
       {
-        // for now, just hack the verb to be view-link-clicked
-        // and down in the load document code we'll detect this and
-        // set the correct uri loader command
-        nsCOMPtr<nsIURI> uri;
-        NS_NewURI(getter_AddRefs(uri), nsDependentString(aURLSpec), nsnull);
-
-        // No URI object? This may indicate the URLspec is for an
-        // unrecognized protocol. Embedders might still be interested
-        // in handling the click, so we fire a notification before
-        // throwing the click away.
-        if (!uri && NS_SUCCEEDED(EnsureContentListener()))
-        {
-            NS_ConvertUCS2toUTF8 spec(aURLSpec);
-            PRBool abort = PR_FALSE;
-            uri = do_CreateInstance(kSimpleURICID, &rv);
-            NS_ASSERTION(NS_SUCCEEDED(rv), "can't create simple uri");
-            if (NS_SUCCEEDED(rv))
-            {
-                rv = uri->SetSpec(spec);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "spec is invalid");
-                if (NS_SUCCEEDED(rv))
-                {
-                    mContentListener->OnStartURIOpen(uri, &abort);
-                }
-            }
-            // We didn't load the URI, so we failed
-            return NS_ERROR_FAILURE;
-        }
-
-        return InternalLoad(uri,                // New URI
+        return InternalLoad(aURI,               // New URI
                             mCurrentURI,        // Referer URI
                             nsnull,             // No onwer
                             PR_TRUE,            // Inherit owner from document
@@ -629,64 +615,132 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
 
 NS_IMETHODIMP
 nsWebShell::OnOverLink(nsIContent* aContent,
-                       const PRUnichar* aURLSpec,
+                       nsIURI* aURI,
                        const PRUnichar* aTargetSpec)
 {
-    nsCOMPtr<nsIWebBrowserChrome> browserChrome(do_GetInterface(mTreeOwner));
+  nsCOMPtr<nsIWebBrowserChrome> browserChrome(do_GetInterface(mTreeOwner));
   nsresult rv = NS_ERROR_FAILURE;
 
-  if(browserChrome)  {
+  if (browserChrome)  {
     nsCOMPtr<nsITextToSubURI> textToSubURI = do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
 
-    PRUnichar *uStr;
-    nsString aSpec(aURLSpec);
-    PRInt32 pos = aSpec.FindChar(':');
-    nsAutoString scheme;
-    static const char kMailToURI[] = "mailto";
-    // If the scheme is mailtourl then should not convert from a document charset
-    if ((pos == (PRInt32)(sizeof kMailToURI - 1)) && (aSpec.Left(scheme, pos) != -1) &&
-         scheme.EqualsIgnoreCase(kMailToURI)) {
-      // use UTF-8 instead and apply URL escape.
-      rv = textToSubURI->UnEscapeAndConvert("UTF-8",NS_ConvertUCS2toUTF8(aURLSpec).get(), &uStr);
-    }
-    else  {
-      // use doc charset instead and apply URL escape.
-      nsCOMPtr<nsIPresShell> presShell;
-      nsCOMPtr<nsIDocument> doc;
-      nsDocShell::GetPresShell(getter_AddRefs(presShell));
-      NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
-      presShell->GetDocument(getter_AddRefs(doc));
-      NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
-      nsAutoString charset;
-      NS_ENSURE_SUCCESS(doc->GetDocumentCharacterSet(charset), NS_ERROR_FAILURE);
-      rv = textToSubURI->UnEscapeAndConvert(NS_ConvertUCS2toUTF8(charset.get()).get(),
-             NS_ConvertUCS2toUTF8(aURLSpec).get(), &uStr);    
-    }
+    // use url origin charset to unescape the URL
+    nsCAutoString charset;
+    rv = aURI->GetOriginCharset(charset);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    if (NS_SUCCEEDED(rv))   {
-      rv = browserChrome->SetStatus(nsIWebBrowserChrome::STATUS_LINK, uStr);
-      nsMemory::Free(uStr);
-    }
+    nsCAutoString spec;
+    rv = aURI->GetSpec(spec);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsAutoString uStr;
+    rv = textToSubURI->UnEscapeURIForUI(charset, spec, uStr);    
+
+    if (NS_SUCCEEDED(rv))
+      rv = browserChrome->SetStatus(nsIWebBrowserChrome::STATUS_LINK, uStr.get());
   }
   return rv;
 }
 
 NS_IMETHODIMP
-nsWebShell::GetLinkState(const char* aLinkURI, nsLinkState& aState)
+nsWebShell::OnLeaveLink()
+{
+  nsCOMPtr<nsIWebBrowserChrome> browserChrome(do_GetInterface(mTreeOwner));
+  nsresult rv = NS_ERROR_FAILURE;
+
+  if (browserChrome)  {
+      rv = browserChrome->SetStatus(nsIWebBrowserChrome::STATUS_LINK,
+                                    NS_LITERAL_STRING("").get());
+  }
+  return rv;
+}
+
+nsresult
+nsWebShell::NormalizeURI(nsACString& aURLSpec)
+{
+  nsIURI *uri = nsnull;
+  nsCAutoString scheme;
+  nsresult rv = mIOService->ExtractScheme(aURLSpec, scheme);
+  if (NS_FAILED(rv)) return rv;
+
+  // keep tempUri up here to keep it in scope
+  nsCOMPtr<nsIURI> tempUri;
+
+  // used to avoid extra work later
+  PRBool clearUri(PR_TRUE);
+  
+  if (scheme.Equals(NS_LITERAL_CSTRING("http"))) {
+    if (mCachedHttpUrl)
+      rv = mCachedHttpUrl->SetSpec(aURLSpec);
+    else
+      rv = NS_NewURI(getter_AddRefs(mCachedHttpUrl), aURLSpec);
+    
+    uri = mCachedHttpUrl;
+  }
+
+  else if (scheme.Equals(NS_LITERAL_CSTRING("https"))) {
+    if (mCachedHttpsUrl)
+      rv = mCachedHttpsUrl->SetSpec(aURLSpec);
+    else
+      rv = NS_NewURI(getter_AddRefs(mCachedHttpsUrl), aURLSpec);
+
+    uri = mCachedHttpsUrl;
+  }
+
+  else if (scheme.Equals(NS_LITERAL_CSTRING("ftp"))) {
+    if (mCachedFtpUrl)
+      rv = mCachedFtpUrl->SetSpec(aURLSpec);
+    else      
+      rv = NS_NewURI(getter_AddRefs(mCachedFtpUrl), aURLSpec);
+    
+    uri = mCachedFtpUrl;
+  } else {
+    rv = NS_NewURI(getter_AddRefs(tempUri), aURLSpec);
+    uri = tempUri;
+    clearUri = PR_FALSE;
+  }
+
+  // covers all above failures
+  if (NS_FAILED(rv)) return rv;
+  
+  rv = uri->GetSpec(aURLSpec);
+
+  // clear out the old spec, for security reasons - old data should
+  // not be floating around in cached URIs!
+  // (but avoid doing extra work if we're just destroying the uri)
+  if (clearUri)
+    uri->SetSpec(NS_LITERAL_CSTRING(""));
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsWebShell::GetLinkState(const nsACString& aLinkURI, nsLinkState& aState)
 {
   aState = eLinkState_Unvisited;
 
-   if(mGlobalHistory)
-      {
-        PRBool isVisited;
-        NS_ENSURE_SUCCESS(mGlobalHistory->IsVisited(aLinkURI, &isVisited),
-                          NS_ERROR_FAILURE);
-        if (isVisited)
-          aState = eLinkState_Visited;
-      }
+  // no history, leave state unchanged
+  if (!mGlobalHistory)
+    return NS_OK;
+  
+  // default to the given URI
+  nsCAutoString resolvedPath(aLinkURI);
+  nsresult rv;
 
-   return NS_OK;
+  // get the cached IO service
+  if (!mIOService)
+    mIOService = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
+        
+  NormalizeURI(resolvedPath);
+
+  PRBool isVisited;
+  NS_ENSURE_SUCCESS(mGlobalHistory->IsVisited(resolvedPath.get(), &isVisited),
+                    NS_ERROR_FAILURE);
+  if (isVisited)
+    aState = eLinkState_Visited;
+  
+  return NS_OK;
 }
 
 //----------------------------------------------------------------------
@@ -757,7 +811,6 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 
       nsCAutoString oldSpec;
       url->GetSpec(oldSpec);
-      NS_ConvertUTF8toUCS2 oldSpecW(oldSpec);
       
       //
       // First try keyword fixup
@@ -838,7 +891,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
         if (doCreateAlternate)
         {
           newURI = nsnull;
-          mURIFixup->CreateFixupURI(oldSpecW.get(),
+          mURIFixup->CreateFixupURI(NS_ConvertUTF8toUCS2(oldSpec),
               nsIURIFixup::FIXUP_FLAGS_MAKE_ALTERNATE_URI, getter_AddRefs(newURI));
         }
       }
@@ -878,7 +931,9 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 
     // Errors to be shown only on top-level frames
     if ((aStatus == NS_ERROR_UNKNOWN_HOST || 
-         aStatus == NS_ERROR_CONNECTION_REFUSED) &&
+         aStatus == NS_ERROR_CONNECTION_REFUSED ||
+         aStatus == NS_ERROR_UNKNOWN_PROXY_HOST || 
+         aStatus == NS_ERROR_PROXY_CONNECTION_REFUSED) &&
             (isTopFrame || mUseErrorPages)) {
       DisplayLoadError(aStatus, url, nsnull);
     }
@@ -886,6 +941,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
     else if (aStatus == NS_ERROR_NET_TIMEOUT ||
              aStatus == NS_ERROR_REDIRECT_LOOP ||
              aStatus == NS_ERROR_UNKNOWN_SOCKET_TYPE ||
+             aStatus == NS_ERROR_NET_INTERRUPT ||
              aStatus == NS_ERROR_NET_RESET) {
       DisplayLoadError(aStatus, url, nsnull);
     }
@@ -894,86 +950,99 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
        * the cache is not in cache. May be this is one of those 
        * postdata results. Throw a  dialog to the user,
        * saying that the page has expired from cache and ask if 
-       * they wish to refetch the page from the net.
+       * they wish to refetch the page from the net. Do this only
+       * if the request is a form post.
        */
-      nsCOMPtr<nsIPrompt> prompter;
-      PRBool repost;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-      GetPromptAndStringBundle(getter_AddRefs(prompter), 
-                                getter_AddRefs(stringBundle));
- 
-      if (stringBundle && prompter) {
-        nsXPIDLString messageStr;
-        nsresult rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("repost").get(), 
-                                                      getter_Copies(messageStr));
-          
-        if (NS_SUCCEEDED(rv) && messageStr) {
-          prompter->Confirm(nsnull, messageStr, &repost);
-          /* If the user pressed cancel in the dialog, 
-           * return failure. Don't try to load the page with out 
-           * the post data. 
-           */
-          if (!repost)
-            return NS_OK;
+      nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
+      nsCAutoString method;
+      if (httpChannel)
+        httpChannel->GetRequestMethod(method);
+      if (method.Equals("POST") && !IsOffline()) {
+        nsCOMPtr<nsIPrompt> prompter;
+        PRBool repost;
+        nsCOMPtr<nsIStringBundle> stringBundle;
+        GetPromptAndStringBundle(getter_AddRefs(prompter), 
+                                  getter_AddRefs(stringBundle));
+   
+        if (stringBundle && prompter) {
+          nsXPIDLString messageStr;
+          nsresult rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("repost").get(), 
+                                                        getter_Copies(messageStr));
+            
+          if (NS_SUCCEEDED(rv) && messageStr) {
+            prompter->Confirm(nsnull, messageStr, &repost);
+            /* If the user pressed cancel in the dialog, 
+             * return failure. Don't try to load the page with out 
+             * the post data. 
+             */
+            if (!repost)
+              return NS_OK;
 
-          // The user wants to repost the data to the server. 
-          // If the page was loaded due to a back/forward/go
-          // operation, update the session history index.
-          // This is similar to the updating done in 
-          // nsDocShell::OnNewURI() for regular pages          
-          nsCOMPtr<nsISHistory> rootSH=mSessionHistory;
-          if (!mSessionHistory) {
-            nsCOMPtr<nsIDocShellTreeItem> root;
-            //Get the root docshell
-            GetSameTypeRootTreeItem(getter_AddRefs(root));
-            if (root) {
-              // QI root to nsIWebNavigation
-              nsCOMPtr<nsIWebNavigation> rootAsWebnav(do_QueryInterface(root));
-              if (rootAsWebnav) {
-               // Get the handle to SH from the root docshell          
-               rootAsWebnav->GetSessionHistory(getter_AddRefs(rootSH));             
+            // The user wants to repost the data to the server. 
+            // If the page was loaded due to a back/forward/go
+            // operation, update the session history index.
+            // This is similar to the updating done in 
+            // nsDocShell::OnNewURI() for regular pages          
+            nsCOMPtr<nsISHistory> rootSH=mSessionHistory;
+            if (!mSessionHistory) {
+              nsCOMPtr<nsIDocShellTreeItem> root;
+              //Get the root docshell
+              GetSameTypeRootTreeItem(getter_AddRefs(root));
+              if (root) {
+                // QI root to nsIWebNavigation
+                nsCOMPtr<nsIWebNavigation> rootAsWebnav(do_QueryInterface(root));
+                if (rootAsWebnav) {
+                 // Get the handle to SH from the root docshell          
+                 rootAsWebnav->GetSessionHistory(getter_AddRefs(rootSH));             
+                }
+              }
+            }  // mSessionHistory
+
+            if (rootSH && (mLoadType & LOAD_CMD_HISTORY)) {
+              nsCOMPtr<nsISHistoryInternal> shInternal(do_QueryInterface(rootSH));
+              if (shInternal)
+               shInternal->UpdateIndex();
+            }
+            /* The user does want to repost the data to the server.
+             * Initiate a new load again.
+             */
+            /* Get the postdata if any from the channel */
+            nsCOMPtr<nsIInputStream> inputStream;
+            nsCOMPtr<nsIURI> referrer;
+            if (channel) {
+              nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
+   
+              if(httpChannel) {
+                httpChannel->GetReferrer(getter_AddRefs(referrer));
+                nsCOMPtr<nsIUploadChannel> uploadChannel(do_QueryInterface(channel));
+                if (uploadChannel) {
+                  uploadChannel->GetUploadStream(getter_AddRefs(inputStream));
+                }
               }
             }
-          }  // mSessionHistory
-
-          if (rootSH && (mLoadType & LOAD_CMD_HISTORY)) {
-            nsCOMPtr<nsISHistoryInternal> shInternal(do_QueryInterface(rootSH));
-            if (shInternal)
-             shInternal->UpdateIndex();
-          }
-          /* The user does want to repost the data to the server.
-           * Initiate a new load again.
-           */
-          /* Get the postdata if any from the channel */
-          nsCOMPtr<nsIInputStream> inputStream;
-          nsCOMPtr<nsIURI> referrer;
-          if (channel) {
-            nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
- 
-            if(httpChannel) {
-              httpChannel->GetReferrer(getter_AddRefs(referrer));
-              httpChannel->GetUploadStream(getter_AddRefs(inputStream));
+            nsCOMPtr<nsISeekableStream> postDataSeekable(do_QueryInterface(inputStream));
+            if (postDataSeekable)
+            {
+               postDataSeekable->Seek(nsISeekableStream::NS_SEEK_SET, 0);
             }
-          }
-          nsCOMPtr<nsISeekableStream> postDataSeekable(do_QueryInterface(inputStream));
-          if (postDataSeekable)
-          {
-             postDataSeekable->Seek(nsISeekableStream::NS_SEEK_SET, 0);
-          }
-          InternalLoad(url,                               // URI
-                       referrer,                          // Refering URI
-                       nsnull,                            // Owner
-                       PR_TRUE,                           // Inherit owner
-                       nsnull,                            // No window target
-                       inputStream,                       // Post data stream
-                       nsnull,                            // No headers stream
-                       LOAD_RELOAD_BYPASS_PROXY_AND_CACHE,// Load type
-                       nsnull,                            // No SHEntry
-                       PR_TRUE,                           // first party site
-                       nsnull,                            // No nsIDocShell
-                       nsnull);                           // No nsIRequest
+            InternalLoad(url,                               // URI
+                         referrer,                          // Refering URI
+                         nsnull,                            // Owner
+                         PR_TRUE,                           // Inherit owner
+                         nsnull,                            // No window target
+                         inputStream,                       // Post data stream
+                         nsnull,                            // No headers stream
+                         LOAD_RELOAD_BYPASS_PROXY_AND_CACHE,// Load type
+                         nsnull,                            // No SHEntry
+                         PR_TRUE,                           // first party site
+                         nsnull,                            // No nsIDocShell
+                         nsnull);                           // No nsIRequest
           }
         }
+      }
+      else {
+        DisplayLoadError(aStatus, url, nsnull);
+      }
     }
   } // if we have a host
 
@@ -1159,7 +1228,8 @@ NS_IMETHODIMP nsWebShell::Create()
   // doc loader and store it...as more of the docshell lands, we'll be able to get rid
   // of this hack...
   nsCOMPtr<nsIURILoader> uriLoader = do_GetService(NS_URI_LOADER_CONTRACTID);
-  uriLoader->GetDocumentLoaderForContext(NS_STATIC_CAST( nsISupports*, (nsIWebShell *) this), &mDocLoader);
+  uriLoader->GetDocumentLoaderForContext(NS_STATIC_CAST(nsIWebShell*, this),
+                                         getter_AddRefs(mDocLoader));
 
   nsCOMPtr<nsIContentViewerContainer> shellAsContainer = do_QueryInterface(NS_STATIC_CAST(nsIWebShell*, this));
   // Set the webshell as the default IContentViewerContainer for the loader...
