@@ -40,95 +40,16 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsIComponentManager.h"
+#include "nsIControllerCommandManager.h"
 #include "nsComposerController.h"
-#include "nsIEditorShell.h"
-
-#if 0
-#include "nsIEditor.h"
-#include "nsIEditorMailSupport.h"
-#include "nsIFormControlFrame.h"
-#include "nsISelection.h"
-#include "nsIHTMLEditor.h"
-#include "nsISupportsPrimitives.h"
-#include "nsXPIDLString.h"
-
-#include "nsISelectionController.h"
-#include "nsIDocument.h"
-#include "nsIPresShell.h"
-
-#include "nsEditorCommands.h"
-#endif
-
 #include "nsComposerCommands.h"
 
-NS_IMPL_ADDREF(nsComposerController)
-NS_IMPL_RELEASE(nsComposerController)
-
-NS_INTERFACE_MAP_BEGIN(nsComposerController)
-	NS_INTERFACE_MAP_ENTRY(nsIController)
-       NS_INTERFACE_MAP_ENTRY(nsICommandController)
-	NS_INTERFACE_MAP_ENTRY(nsIEditorController)
-	NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
-	NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIEditorController)
-NS_INTERFACE_MAP_END
-
-
-nsComposerController::nsComposerController()
-{
-  NS_INIT_REFCNT();  
-}
-
-nsComposerController::~nsComposerController()
-{
-}
-
-NS_IMETHODIMP nsComposerController::Init(nsISupports *aCommandRefCon)
-{
-  nsresult  rv;
- 
-  // get our ref to the singleton command manager
-  rv = GetComposerCommandManager(getter_AddRefs(mCommandManager));  
-  if (NS_FAILED(rv)) return rv;  
-
-  mCommandRefCon = aCommandRefCon;     // no addref  
-  
-  mCommandManager = do_CreateInstance(NS_CONTROLLERCOMMANDMANAGER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-
-  // register the commands.
-  rv = nsComposerController::RegisterComposerCommands(mCommandManager);
-  if (NS_FAILED(rv)) return rv;
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsComposerController::SetCommandRefCon(nsISupports *aCommandRefCon)
-{
-  mCommandRefCon = aCommandRefCon;     // no addref  
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsComposerController::GetInterface(const nsIID & aIID, void * *result)
-{
-  NS_ENSURE_ARG_POINTER(result);
-
-  if (NS_SUCCEEDED(QueryInterface(aIID, result)))
-    return NS_OK;
-  
-  if (mCommandManager && aIID.Equals(NS_GET_IID(nsIControllerCommandManager)))
-    return mCommandManager->QueryInterface(aIID, result);
-    
-  return NS_NOINTERFACE;
-}
-
 #define NS_REGISTER_ONE_COMMAND(_cmdClass, _cmdName)                    \
   {                                                                     \
     _cmdClass* theCmd;                                                  \
     NS_NEWXPCOM(theCmd, _cmdClass);                                     \
     if (!theCmd) return NS_ERROR_OUT_OF_MEMORY;                         \
-    rv = inCommandManager->RegisterCommand(NS_LITERAL_STRING(_cmdName), \
+    rv = inCommandManager->RegisterCommand(_cmdName, \
                        NS_STATIC_CAST(nsIControllerCommand *, theCmd)); \
   }
 
@@ -137,15 +58,15 @@ nsComposerController::GetInterface(const nsIID & aIID, void * *result)
     _cmdClass* theCmd;                                                  \
     NS_NEWXPCOM(theCmd, _cmdClass);                                     \
     if (!theCmd) return NS_ERROR_OUT_OF_MEMORY;                         \
-    rv = inCommandManager->RegisterCommand(NS_LITERAL_STRING(_cmdName), \
+    rv = inCommandManager->RegisterCommand(_cmdName, \
                        NS_STATIC_CAST(nsIControllerCommand *, theCmd));
 
 #define NS_REGISTER_NEXT_COMMAND(_cmdClass, _cmdName)                   \
-    rv = inCommandManager->RegisterCommand(NS_LITERAL_STRING(_cmdName), \
+    rv = inCommandManager->RegisterCommand(_cmdName, \
                         NS_STATIC_CAST(nsIControllerCommand *, theCmd));
 
 #define NS_REGISTER_LAST_COMMAND(_cmdClass, _cmdName)                   \
-    rv = inCommandManager->RegisterCommand(NS_LITERAL_STRING(_cmdName), \
+    rv = inCommandManager->RegisterCommand(_cmdName, \
                        NS_STATIC_CAST(nsIControllerCommand *, theCmd)); \
   }
 
@@ -153,18 +74,45 @@ nsComposerController::GetInterface(const nsIID & aIID, void * *result)
   {                                                                     \
     _cmdClass* theCmd = new _cmdClass(_styleTag);                       \
     if (!theCmd) return NS_ERROR_OUT_OF_MEMORY;                         \
-    rv = inCommandManager->RegisterCommand(NS_LITERAL_STRING(_cmdName), \
+    rv = inCommandManager->RegisterCommand(_cmdName, \
+                       NS_STATIC_CAST(nsIControllerCommand *, theCmd)); \
+  }
+  
+#define NS_REGISTER_TAG_COMMAND(_cmdClass, _cmdName, _tagName)          \
+  {                                                                     \
+    _cmdClass* theCmd = new _cmdClass(_tagName);                        \
+    if (!theCmd) return NS_ERROR_OUT_OF_MEMORY;                         \
+    rv = inCommandManager->RegisterCommand(_cmdName,                    \
                        NS_STATIC_CAST(nsIControllerCommand *, theCmd)); \
   }
   
 
 // static
-nsresult nsComposerController::RegisterComposerCommands(nsIControllerCommandManager *inCommandManager)
+nsresult
+nsComposerController::RegisterEditorDocStateCommands(
+                        nsIControllerCommandManager *inCommandManager)
+{
+  nsresult rv;
+
+  // observer commands for document state
+  NS_REGISTER_ONE_COMMAND(nsDocumentStateCommand, "obs_documentCreated")
+  NS_REGISTER_ONE_COMMAND(nsDocumentStateCommand, "obs_documentWillBeDestroyed")
+  NS_REGISTER_ONE_COMMAND(nsDocumentStateCommand, "obs_documentLocationChanged")
+
+  // commands that may get or change state
+  NS_REGISTER_ONE_COMMAND(nsSetDocumentStateCommand, "cmd_setDocumentModified")
+  return NS_OK;
+}
+
+// static
+nsresult
+nsComposerController::RegisterHTMLEditorCommands(
+                        nsIControllerCommandManager *inCommandManager)
 {
   nsresult rv;
   
   // Edit menu
-  NS_REGISTER_ONE_COMMAND(nsPasteQuotationCommand, "cmd_pasteQuote");
+  NS_REGISTER_ONE_COMMAND(nsPasteNoFormattingCommand, "cmd_pasteNoFormatting");
 
   // indent/outdent
   NS_REGISTER_ONE_COMMAND(nsIndentCommand, "cmd_indent");
@@ -189,7 +137,7 @@ nsresult nsComposerController::RegisterComposerCommands(nsIControllerCommandMana
   NS_REGISTER_STYLE_COMMAND(nsStyleUpdatingCommand, "cmd_code", "code");
   NS_REGISTER_STYLE_COMMAND(nsStyleUpdatingCommand, "cmd_samp", "samp");
   NS_REGISTER_STYLE_COMMAND(nsStyleUpdatingCommand, "cmd_var", "var");
-  
+
   // lists
   NS_REGISTER_STYLE_COMMAND(nsListCommand,     "cmd_ol", "ol");
   NS_REGISTER_STYLE_COMMAND(nsListCommand,     "cmd_ul", "ul");
@@ -200,6 +148,7 @@ nsresult nsComposerController::RegisterComposerCommands(nsIControllerCommandMana
   // format stuff
   NS_REGISTER_ONE_COMMAND(nsParagraphStateCommand,       "cmd_paragraphState");
   NS_REGISTER_ONE_COMMAND(nsFontFaceStateCommand,        "cmd_fontFace");
+  NS_REGISTER_ONE_COMMAND(nsFontSizeStateCommand,        "cmd_fontSize");
   NS_REGISTER_ONE_COMMAND(nsFontColorStateCommand,       "cmd_fontColor");
   NS_REGISTER_ONE_COMMAND(nsBackgroundColorStateCommand, "cmd_backgroundColor");
   NS_REGISTER_ONE_COMMAND(nsHighlightColorStateCommand,  "cmd_highlight");
@@ -210,89 +159,10 @@ nsresult nsComposerController::RegisterComposerCommands(nsIControllerCommandMana
   NS_REGISTER_ONE_COMMAND(nsIncreaseFontSizeCommand, "cmd_increaseFont");
   NS_REGISTER_ONE_COMMAND(nsDecreaseFontSizeCommand, "cmd_decreaseFont");
 
+  // Insert content
+  NS_REGISTER_ONE_COMMAND(nsInsertHTMLCommand, "cmd_insertHTML");
+  NS_REGISTER_TAG_COMMAND(nsInsertTagCommand, "cmd_insertLinkNoUI", "a");
+  NS_REGISTER_TAG_COMMAND(nsInsertTagCommand, "cmd_insertImageNoUI", "img");
+
   return NS_OK;
 }
-
-/* =======================================================================
- * nsIController
- * ======================================================================= */
-
-NS_IMETHODIMP
-nsComposerController::IsCommandEnabled(const nsAString & aCommand,
-                                       PRBool *aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-  return mCommandManager->IsCommandEnabled(aCommand, mCommandRefCon, aResult);
-}
-
-NS_IMETHODIMP
-nsComposerController::SupportsCommand(const nsAString & aCommand,
-                                      PRBool *aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-  return mCommandManager->SupportsCommand(aCommand, mCommandRefCon, aResult);
-}
-
-NS_IMETHODIMP
-nsComposerController::DoCommand(const nsAString & aCommand)
-{
-  return mCommandManager->DoCommand(aCommand, mCommandRefCon);
-}
-
-NS_IMETHODIMP
-nsComposerController::OnEvent(const nsAString & aEventName)
-{
-  return NS_OK;
-}
-
-nsWeakPtr nsComposerController::sComposerCommandManager = NULL;
-
-// static
-nsresult nsComposerController::GetComposerCommandManager(nsIControllerCommandManager* *outCommandManager)
-{
-  NS_ENSURE_ARG_POINTER(outCommandManager);
-
-  nsCOMPtr<nsIControllerCommandManager> cmdManager = do_QueryReferent(sComposerCommandManager);
-  if (!cmdManager)
-  {
-    nsresult rv;
-    cmdManager = do_CreateInstance(NS_CONTROLLERCOMMANDMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-
-    // register the commands. This just happens once per instance
-    rv = nsComposerController::RegisterComposerCommands(cmdManager);
-    if (NS_FAILED(rv)) return rv;
-
-    // save the singleton in our static weak reference
-    sComposerCommandManager = getter_AddRefs(NS_GetWeakReference(cmdManager, &rv));
-    if (NS_FAILED(rv))  return rv;
-  }
-
-  NS_ADDREF(*outCommandManager = cmdManager);
-  return NS_OK;
-}
-
-
-//GetCommandState
-/*
-cmd_bold,cmd_italic,cmd_underline ->state commands
-state_start : true,false
-state_end   : true,false
-state_all   : true,false
-state_mixed : true,false
-*/
-/* void getCommandState (in DOMString aCommandName, inout nsICommandParams aCommandParams); */
-NS_IMETHODIMP nsComposerController::GetCommandState(nsICommandParams *aCommandParams)
-{
-  if (!mCommandRefCon || !mCommandManager)
-    return NS_ERROR_NOT_INITIALIZED;
-  return mCommandManager->GetCommandState(aCommandParams,mCommandRefCon);
-}
-
-/* void doCommand (in DOMString aCommandName, in nsICommandParams aCommandParams); */
-NS_IMETHODIMP nsComposerController::DoCommand(nsICommandParams *aCommandParams)
-{
-  if (!mCommandRefCon || !mCommandManager)
-    return NS_ERROR_NOT_INITIALIZED;
-  return mCommandManager->DoCommandParams(aCommandParams,mCommandRefCon);
-}
