@@ -40,7 +40,8 @@
 #include "nsHTMLEditor.h"
 #include "nsCOMPtr.h"
 #include "nsHTMLEditUtils.h"
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #include "nsIServiceManager.h"
 #include "nsIEditProperty.h"
 #include "ChangeCSSInlineStyleTxn.h"
@@ -54,8 +55,6 @@
 #include "nsUnicharUtils.h"
 #include "nsHTMLCSSUtils.h"
 
-static NS_DEFINE_CID(kPrefServiceCID,       NS_PREF_CID);
-
 static
 void ProcessBValue(const nsAString * aInputString, nsAString & aOutputString,
                    const char * aDefaultValueString,
@@ -200,105 +199,105 @@ void ProcessMarginRightValue(const nsAString * aInputString, nsAString & aOutput
 }
 
 const nsHTMLCSSUtils::CSSEquivTable boldEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_font_weight, ProcessBValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_font_weight, ProcessBValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable italicEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_font_style, ProcessDefaultValue, "italic", nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_font_style, ProcessDefaultValue, "italic", nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable underlineEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue, "underline", nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue, "underline", nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable strikeEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue, "line-through", nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_text_decoration, ProcessDefaultValue, "line-through", nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable ttEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_font_family, ProcessDefaultValue, "monospace", nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_font_family, ProcessDefaultValue, "monospace", nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable fontColorEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable fontFaceEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_font_family, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_font_family, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable bgcolorEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_background_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_background_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable backgroundImageEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_background_image, ProcessExtendedValue, nsnull, "url(", ")", PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_background_image, ProcessExtendedValue, nsnull, "url(", ")", PR_TRUE, PR_TRUE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable textColorEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_color, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable borderEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_border, ProcessExtendedValue, nsnull, nsnull, "px solid", PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_border, ProcessExtendedValue, nsnull, nsnull, "px solid", PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable textAlignEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_text_align, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_text_align, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable captionAlignEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_caption_side, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_caption_side, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable verticalAlignEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_vertical_align, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_vertical_align, ProcessSameValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable nowrapEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_whitespace, ProcessDefaultValue, "nowrap", nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_whitespace, ProcessDefaultValue, "nowrap", nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable widthEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_width, ProcessLengthValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_width, ProcessLengthValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable heightEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_height, ProcessLengthValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_height, ProcessLengthValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable listStyleTypeEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_list_style_type, ProcessListStyleTypeValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_list_style_type, ProcessListStyleTypeValue, nsnull, nsnull, nsnull, PR_TRUE, PR_TRUE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable tableAlignEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_text_align, ProcessDefaultValue, "left", nsnull, nsnull, PR_FALSE },
-  { nsHTMLCSSUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue, nsnull, nsnull, nsnull, PR_TRUE },
-  { nsHTMLCSSUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_text_align, ProcessDefaultValue, "left", nsnull, nsnull, PR_FALSE, PR_FALSE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
 const nsHTMLCSSUtils::CSSEquivTable hrAlignEquivTable[] = {
-  { nsHTMLCSSUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue, nsnull, nsnull, nsnull, PR_TRUE },
-  { nsHTMLCSSUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue, nsnull, nsnull, nsnull, PR_TRUE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_margin_left, ProcessMarginLeftValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
+  { nsHTMLCSSUtils::eCSSEditableProperty_margin_right, ProcessMarginRightValue, nsnull, nsnull, nsnull, PR_TRUE, PR_FALSE },
   { nsHTMLCSSUtils::eCSSEditableProperty_NONE, 0 }
 };
 
@@ -307,6 +306,10 @@ nsHTMLCSSUtils::nsHTMLCSSUtils()
 {
 }
 
+nsHTMLCSSUtils::~nsHTMLCSSUtils()
+{
+}
+
 nsresult
 nsHTMLCSSUtils::Init(nsHTMLEditor *aEditor)
 {
@@ -314,9 +317,10 @@ nsHTMLCSSUtils::Init(nsHTMLEditor *aEditor)
   mHTMLEditor = NS_STATIC_CAST(nsHTMLEditor*, aEditor);
 
   // let's retrieve the value of the "CSS editing" pref
-  nsCOMPtr<nsIPref> prefService = do_GetService(kPrefServiceCID, &result);
-  if (NS_SUCCEEDED(result) && prefService) {
-    result = prefService->GetBoolPref("editor.use_css", &mIsCSSPrefChecked);
+  nsCOMPtr<nsIPrefBranch> prefBranch =
+    do_GetService(NS_PREFSERVICE_CONTRACTID, &result);
+  if (NS_SUCCEEDED(result) && prefBranch) {
+    result = prefBranch->GetBoolPref("editor.use_css", &mIsCSSPrefChecked);
     if (NS_FAILED(result)) return result;
   }
   return result;
@@ -371,6 +375,18 @@ nsHTMLCSSUtils::IsCSSEditableProperty(nsIDOMNode * aNode,
     return PR_TRUE;
   }
 
+  if (aAttribute && (aAttribute->Equals(NS_LITERAL_STRING("valign"))) &&
+      (nsIEditProperty::col == tagName
+       || nsIEditProperty::colgroup   == tagName
+       || nsIEditProperty::tbody  == tagName
+       || nsIEditProperty::td  == tagName
+       || nsIEditProperty::th  == tagName
+       || nsIEditProperty::tfoot  == tagName
+       || nsIEditProperty::thead  == tagName
+       || nsIEditProperty::tr  == tagName)) {
+    return PR_TRUE;
+  }
+
   // attributes TEXT, BACKGROUND and BGCOLOR on BODY
   if (aAttribute && (nsIEditProperty::body == tagName) &&
       (aAttribute->Equals(NS_LITERAL_STRING("text"))
@@ -443,12 +459,18 @@ nsHTMLCSSUtils::IsCSSEditableProperty(nsIDOMNode * aNode,
 // the lowest level above the transaction; adds the css declaration "aProperty : aValue" to
 // the inline styles carried by aElement
 nsresult
-nsHTMLCSSUtils::SetCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, const nsAString & aValue)
+nsHTMLCSSUtils::SetCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, const nsAString & aValue,
+                               PRBool aSuppressTransaction)
 {
   ChangeCSSInlineStyleTxn *txn;
   nsresult result = CreateCSSPropertyTxn(aElement, aProperty, aValue, &txn, PR_FALSE);
   if (NS_SUCCEEDED(result))  {
-    result = mHTMLEditor->Do(txn);
+    if (aSuppressTransaction) {
+      result = txn->DoTransaction();
+    }
+    else {
+      result = mHTMLEditor->Do(txn);
+    }
   }
   // The transaction system (if any) has taken ownwership of txn
   NS_IF_RELEASE(txn);
@@ -459,12 +481,18 @@ nsHTMLCSSUtils::SetCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, con
 // specified for the CSS property aProperty, or totally remove the declaration if this
 // property accepts only one value
 nsresult
-nsHTMLCSSUtils::RemoveCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, const nsAString & aValue)
+nsHTMLCSSUtils::RemoveCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, const nsAString & aValue,
+                                  PRBool aSuppressTransaction)
 {
   ChangeCSSInlineStyleTxn *txn;
   nsresult result = CreateCSSPropertyTxn(aElement, aProperty, aValue, &txn, PR_TRUE);
   if (NS_SUCCEEDED(result))  {
-    result = mHTMLEditor->Do(txn);
+    if (aSuppressTransaction) {
+      result = txn->DoTransaction();
+    }
+    else {
+      result = mHTMLEditor->Do(txn);
+    }
   }
   // The transaction system (if any) has taken ownwership of txn
   NS_IF_RELEASE(txn);
@@ -605,7 +633,7 @@ nsHTMLCSSUtils::RemoveCSSInlineStyle(nsIDOMNode *aNode, nsIAtom *aProperty, cons
   nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(aNode);
 
   // remove the property from the style attribute
-  nsresult res = RemoveCSSProperty(elem, aProperty, aPropertyValue);
+  nsresult res = RemoveCSSProperty(elem, aProperty, aPropertyValue, PR_FALSE);
   if (NS_FAILED(res)) return res;
 
   if (mHTMLEditor->NodeIsType(aNode, nsIEditProperty::span)) {
@@ -649,26 +677,27 @@ nsresult
 nsHTMLCSSUtils::GetDefaultBackgroundColor(nsAString & aColor)
 {
   nsresult result;
-  nsCOMPtr<nsIPref> prefService = do_GetService(kPrefServiceCID, &result);
+  nsCOMPtr<nsIPrefBranch> prefBranch =
+    do_GetService(NS_PREFSERVICE_CONTRACTID, &result);
   if (NS_FAILED(result)) return result;
   aColor.Assign(NS_LITERAL_STRING("#ffffff"));
   nsXPIDLCString returnColor;
-  if (prefService) {
+  if (prefBranch) {
     PRBool useCustomColors;
-    result = prefService->GetBoolPref("editor.use_custom_colors", &useCustomColors);
+    result = prefBranch->GetBoolPref("editor.use_custom_colors", &useCustomColors);
     if (NS_FAILED(result)) return result;
     if (useCustomColors) {
-      result = prefService->CopyCharPref("editor.background_color",
-                                         getter_Copies(returnColor));
+      result = prefBranch->GetCharPref("editor.background_color",
+                                       getter_Copies(returnColor));
       if (NS_FAILED(result)) return result;
     }
     else {
       PRBool useSystemColors;
-      result = prefService->GetBoolPref("browser.display.use_system_colors", &useSystemColors);
+      result = prefBranch->GetBoolPref("browser.display.use_system_colors", &useSystemColors);
       if (NS_FAILED(result)) return result;
       if (!useSystemColors) {
-        result = prefService->CopyCharPref("browser.display.background_color",
-                                           getter_Copies(returnColor));
+        result = prefBranch->GetCharPref("browser.display.background_color",
+                                         getter_Copies(returnColor));
         if (NS_FAILED(result)) return result;
       }
     }
@@ -684,13 +713,14 @@ nsresult
 nsHTMLCSSUtils::GetDefaultLengthUnit(nsAString & aLengthUnit)
 {
   nsresult result;
-  nsCOMPtr<nsIPref> prefService = do_GetService(kPrefServiceCID, &result);
+  nsCOMPtr<nsIPrefBranch> prefBranch =
+    do_GetService(NS_PREFSERVICE_CONTRACTID, &result);
   if (NS_FAILED(result)) return result;
   aLengthUnit.Assign(NS_LITERAL_STRING("px"));
-  if (NS_SUCCEEDED(result) && prefService) {
+  if (NS_SUCCEEDED(result) && prefBranch) {
     nsXPIDLCString returnLengthUnit;
-    result = prefService->CopyCharPref("editor.css.default_length_unit",
-                                       getter_Copies(returnLengthUnit));
+    result = prefBranch->GetCharPref("editor.css.default_length_unit",
+                                     getter_Copies(returnLengthUnit));
     if (NS_FAILED(result)) return result;
     if (returnLengthUnit) {
       aLengthUnit.Assign(NS_ConvertASCIItoUCS2(returnLengthUnit));
@@ -826,10 +856,11 @@ nsHTMLCSSUtils::BuildCSSDeclarations(nsVoidArray & aPropertyArray,
   aValueArray.Clear();
 
   // if we have an input value, let's use it
-  nsAutoString value;
+  nsAutoString value, lowerCasedValue;
   if (aValue) {
     value.Assign(*aValue);
-    ToLowerCase(value);
+    lowerCasedValue.Assign(*aValue);
+    ToLowerCase(lowerCasedValue);
   }
 
   PRInt8 index = 0;
@@ -840,7 +871,8 @@ nsHTMLCSSUtils::BuildCSSDeclarations(nsVoidArray & aPropertyArray,
       nsIAtom * cssPropertyAtom;
       // find the equivalent css value for the index-th property in
       // the equivalence table
-      (*aEquivTable[index].processValueFunctor) (&value, cssValue,
+      (*aEquivTable[index].processValueFunctor) (aEquivTable[index].caseSensitiveValue ? &value : &lowerCasedValue,
+                                                 cssValue,
                                                  aEquivTable[index].defaultValue,
                                                  aEquivTable[index].prependValue,
                                                  aEquivTable[index].appendValue);
@@ -954,7 +986,8 @@ nsHTMLCSSUtils::SetCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
                                             nsIAtom *aHTMLProperty,
                                             const nsAString *aAttribute,
                                             const nsAString *aValue,
-                                            PRInt32 * aCount)
+                                            PRInt32 * aCount,
+                                            PRBool aSuppressTransaction)
 {
   nsCOMPtr<nsIDOMElement> theElement = do_QueryInterface(aNode);
   nsresult res = NS_OK;
@@ -977,7 +1010,7 @@ nsHTMLCSSUtils::SetCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
       cssValueArray.StringAt(index, valueString);
       nsCOMPtr<nsIDOMElement> theElement = do_QueryInterface(aNode);
       res = SetCSSProperty(theElement, (nsIAtom *)cssPropertyArray.ElementAt(index),
-                           valueString);
+                           valueString, aSuppressTransaction);
       if (NS_FAILED(res)) return res;
     }
   }
@@ -987,9 +1020,10 @@ nsHTMLCSSUtils::SetCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
 // Remove from aNode the CSS inline style equivalent to HTMLProperty/aAttribute/aValue for the node
 nsresult
 nsHTMLCSSUtils::RemoveCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
-                                            nsIAtom *aHTMLProperty,
-                                            const nsAString *aAttribute,
-                                            const nsAString *aValue)
+                                               nsIAtom *aHTMLProperty,
+                                               const nsAString *aAttribute,
+                                               const nsAString *aValue,
+                                               PRBool aSuppressTransaction)
 {
   nsCOMPtr<nsIDOMElement> theElement = do_QueryInterface(aNode);
   nsresult res = NS_OK;
@@ -1011,7 +1045,8 @@ nsHTMLCSSUtils::RemoveCSSEquivalentToHTMLStyle(nsIDOMNode * aNode,
       nsAutoString valueString;
       cssValueArray.StringAt(index, valueString);
       nsCOMPtr<nsIDOMElement> theElement = do_QueryInterface(aNode);
-      res = RemoveCSSProperty(theElement, (nsIAtom *)cssPropertyArray.ElementAt(index), valueString);
+      res = RemoveCSSProperty(theElement, (nsIAtom *)cssPropertyArray.ElementAt(index), valueString,
+                              aSuppressTransaction);
       if (NS_FAILED(res)) return res;
     }
   }
@@ -1053,6 +1088,7 @@ nsHTMLCSSUtils::GetCSSEquivalentToHTMLInlineStyleSet(nsIDOMNode * aNode,
   nsresult res = GetElementContainerOrSelf(aNode, getter_AddRefs(theElement));
   if (NS_FAILED(res)) return res;
   nsCOMPtr<nsIDOMNode> theNode = do_QueryInterface(theElement);
+  aValueString.Truncate();
 
   if (theElement && IsCSSEditableProperty(theNode, aHTMLProperty, aAttribute)) {
     // Yes, the requested HTML style has a CSS equivalence in this implementation
