@@ -86,6 +86,7 @@ public:
   NS_DECL_NSIPLAINTEXTEDITOR
 
   /* ------------ nsIEditorMailSupport overrides -------------- */
+  NS_IMETHOD InsertTextWithQuotations(const nsAString &aStringToInsert);
   NS_IMETHOD PasteAsQuotation(PRInt32 aSelectionType);
   NS_IMETHOD InsertAsQuotation(const nsAString& aQuotedText,
                                nsIDOMNode** aNodeInserted);
@@ -109,14 +110,17 @@ public:
   NS_IMETHOD BeginComposition(nsTextEventReply* aReply);
   NS_IMETHOD SetAttributeOrEquivalent(nsIDOMElement * aElement,
                                       const nsAString & aAttribute,
-                                      const nsAString & aValue);
+                                      const nsAString & aValue,
+                                      PRBool aSuppressTransaction);
   NS_IMETHOD RemoveAttributeOrEquivalent(nsIDOMElement * aElement,
-                                         const nsAString & aAttribute);
+                                         const nsAString & aAttribute,
+                                         PRBool aSuppressTransaction);
 
   /** prepare the editor for use */
   NS_IMETHOD Init(nsIDOMDocument *aDoc, nsIPresShell *aPresShell,  nsIContent *aRoot, nsISelectionController *aSelCon, PRUint32 aFlags);
   
   NS_IMETHOD GetDocumentIsEmpty(PRBool *aDocumentIsEmpty);
+  NS_IMETHOD GetIsDocumentEditable(PRBool *aIsDocumentEditable);
 
   NS_IMETHOD DeleteSelection(EDirection aAction);
 
