@@ -22,6 +22,7 @@
 #include <nsIDocShell.h>
 #include <nsIWebProgress.h>
 #include "nsIWidget.h"
+#include "nsCRT.h"
 
 // for NS_APPSHELL_CID
 #include <nsWidgetsCID.h>
@@ -48,7 +49,7 @@
 #include <nsIFocusController.h>
 
 // for profiles
-#include <nsMPFileLocProvider.h>
+#include <nsProfileDirServiceProvider.h>
 
 // all of our local includes
 #include "EmbedPrivate.h"
@@ -59,9 +60,13 @@
 #include "EmbedWindowCreator.h"
 #include "EmbedStream.h"
 
-static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
+#ifdef _BUILD_STATIC_BIN
+#include "nsStaticComponent.h"
+nsresult PR_CALLBACK
+gtk_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
+#endif
 
-static const char sWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
+static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
 PRUint32     EmbedPrivate::sWidgetCount = 0;
 char        *EmbedPrivate::sCompPath    = nsnull;
@@ -156,7 +161,7 @@ EmbedPrivate::Init(GtkMozEmbed *aOwningWidget)
     windowCreator = NS_STATIC_CAST(nsIWindowCreator *, creator);
 
     // Attach it via the watcher service
-    nsCOMPtr<nsIWindowWatcher> watcher = do_GetService(sWatcherContractID);
+    nsCOMPtr<nsIWindowWatcher> watcher = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
     if (watcher)
       watcher->SetWindowCreator(windowCreator);
   }
@@ -347,11 +352,16 @@ EmbedPrivate::PushStartup(void)
     nsCOMPtr<nsILocalFile> binDir;
     
     if (sCompPath) {
-      rv = NS_NewLocalFile(sCompPath, 1, getter_AddRefs(binDir));
+      rv = NS_NewNativeLocalFile(nsDependentCString(sCompPath), 1, getter_AddRefs(binDir));
       if (NS_FAILED(rv))
 	return;
     }
 
+#ifdef _BUILD_STATIC_BIN
+    // Initialize XPCOM's module info table
+    NSGetStaticModuleInfo = gtk_getModuleInfo;
+#endif
+
     rv = NS_InitEmbedding(binDir, sAppFileLocProvider);
     if (NS_FAILED(rv))
       return;
@@ -774,33 +784,28 @@ EmbedPrivate::StartupProfile(void)
   if (sProfileDir && sProfileName) {
     nsresult rv;
     nsCOMPtr<nsILocalFile> profileDir;
-    PRBool exists = PR_FALSE;
-    PRBool isDir = PR_FALSE;
-    profileDir = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-    rv = profileDir->InitWithPath(sProfileDir);
+    NS_NewNativeLocalFile(nsDependentCString(sProfileDir), PR_TRUE,
+                          getter_AddRefs(profileDir));
+    if (!profileDir)
+      return NS_ERROR_FAILURE;
+    rv = profileDir->AppendNative(nsDependentCString(sProfileName));
     if (NS_FAILED(rv))
       return NS_ERROR_FAILURE;
-    profileDir->Exists(&exists);
-    profileDir->IsDirectory(&isDir);
-    // if it exists and it isn't a directory then give up now.
-    if (!exists) {
-      rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
-      if NS_FAILED(rv) {
-	return NS_ERROR_FAILURE;
-      }
-    }
-    else if (exists && !isDir) {
+
+    nsCOMPtr<nsProfileDirServiceProvider> locProvider;
+    NS_NewProfileDirServiceProvider(PR_TRUE, getter_AddRefs(locProvider));
+    if (!locProvider)
       return NS_ERROR_FAILURE;
-    }
-    // actually create the loc provider and initialize prefs.
-    nsMPFileLocProvider *locProvider;
-    // Set up the loc provider.  This has a really strange ownership
-    // model.  When I initialize it it will register itself with the
-    // directory service.  The directory service becomes the owning
-    // reference.  So, when that service is shut down this object will
-    // be destroyed.  It's not leaking here.
-    locProvider = new nsMPFileLocProvider;
-    rv = locProvider->Initialize(profileDir, sProfileName);
+    
+    // Directory service holds an strong reference to any
+    // provider that is registered with it. Let it hold the
+    // only ref. locProvider won't die when we leave this scope.
+    rv = locProvider->Register();
+    if (NS_FAILED(rv))
+      return rv;
+    rv = locProvider->SetProfileDir(profileDir);
+    if (NS_FAILED(rv))
+      return rv;
 
     // get prefs
     nsCOMPtr<nsIPref> pref;
@@ -809,8 +814,6 @@ EmbedPrivate::StartupProfile(void)
       return NS_ERROR_FAILURE;
     sPrefs = pref.get();
     NS_ADDREF(sPrefs);
-    sPrefs->ResetPrefs();
-    sPrefs->ReadUserPrefs(nsnull);
   }
   return NS_OK;
 }
