@@ -63,8 +63,6 @@
 // for the focus hacking we need to do
 #include <nsIFocusController.h>
 
-// for profiles
-#include <nsMPFileLocProvider.h>
 #include "nsIWebBrowserPrint.h"
 
 // all of our local includes
@@ -81,8 +79,6 @@
 
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
-static const char sWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
-
 nsIAppShell *EmbedPrivate::sAppShell    = nsnull;
 nsIPref     *EmbedPrivate::sPrefs       = nsnull;
 nsVoidArray *EmbedPrivate::sWindowList  = nsnull;
@@ -173,7 +169,7 @@ EmbedPrivate::Init(PtWidget_t *aOwningWidget)
 		windowCreator = NS_STATIC_CAST(nsIWindowCreator *, creator);
 
 		// Attach it via the watcher service
-		nsCOMPtr<nsIWindowWatcher> watcher = do_GetService(sWatcherContractID);
+		nsCOMPtr<nsIWindowWatcher> watcher = do_GetService(NS_WINDOWWATCHER_CONTRACTID);
 		if (watcher)
       		watcher->SetWindowCreator(windowCreator);
   }
