@@ -36,7 +36,6 @@
 #include "nsIHistoryEntry.h"
 
 #include "nsIWebBrowserPrint.h"
-#include "nsIPrintOptions.h"
 
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOMEventListener.h"
@@ -45,7 +44,7 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsIDOMWindow.h"
 #include "nsNetUtil.h"
-#include "nsMPFileLocProvider.h"
+#include "nsProfileDirServiceProvider.h"
 #include "nsIFocusController.h"
 #include <nsIWebBrowserFind.h>
 
@@ -308,21 +307,20 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt )
 					moz->EmbedRef->Back();
 				else 
 				{
-				PhDim_t dim;
+					PhDim_t dim;
 
-				PtWidgetDim(widget, &dim);
-				dim.w = (argt->value * dim.w)/100;
-				dim.h = (argt->value * dim.h)/100;
-				printf("Scroll: V: %d P: (%d, %d) \n", argt->value, dim.w, dim.h);
+					PtWidgetDim(widget, &dim);
+					dim.w = (argt->value * dim.w)/100;
+					dim.h = (argt->value * dim.h)/100;
 
-				if (argt->value == WWW_DIRECTION_UP)
-					moz->EmbedRef->ScrollUp(dim.h);
-				else if (argt->value ==	WWW_DIRECTION_DOWN)
-					moz->EmbedRef->ScrollDown(dim.h);
-				else if (argt->value ==	WWW_DIRECTION_LEFT)
-					moz->EmbedRef->ScrollLeft(dim.w);
-				else if (argt->value == WWW_DIRECTION_RIGHT)
-					moz->EmbedRef->ScrollRight(dim.w);
+					if (argt->value == WWW_DIRECTION_UP)
+						moz->EmbedRef->ScrollUp(dim.h);
+					else if (argt->value ==	WWW_DIRECTION_DOWN)
+						moz->EmbedRef->ScrollDown(dim.h);
+					else if (argt->value ==	WWW_DIRECTION_LEFT)
+						moz->EmbedRef->ScrollLeft(dim.w);
+					else if (argt->value == WWW_DIRECTION_RIGHT)
+						moz->EmbedRef->ScrollRight(dim.w);
 				}
 			}
 			break;
@@ -501,7 +499,6 @@ mozilla_get_info(PtWidget_t *widget, PtArg_t *argt)
 {
 	PtMozillaWidget_t *moz = (PtMozillaWidget_t *) widget;
 	nsIPref *pref = moz->EmbedRef->GetPrefs();
-	int retval;
 
 	switch (argt->type)
 	{
@@ -556,9 +553,9 @@ mozilla_get_info(PtWidget_t *widget, PtArg_t *argt)
 					strncpy( HistoryReplyBuf[j].title, ToNewCString(stitle), 127 );
 					HistoryReplyBuf[j].title[127] = '\0';
 
-					char *urlspec;
-					url->GetSpec( &urlspec );
-					strcpy( HistoryReplyBuf[j].url, urlspec );
+					nsCAutoString specString;
+					url->GetSpec(specString);
+					strcpy( HistoryReplyBuf[j].url, (char *) specString.get() );
 				}
 			}
 			break;
@@ -961,39 +958,34 @@ static void mozilla_get_pref( PtWidget_t *widget, char *option, char *value ) {
 static int
 StartupProfile(char *sProfileDir, char *sProfileName)
 {
-  	// initialize profiles
-  	if (sProfileDir && sProfileName) 
+  // initialize profiles
+  if (sProfileDir && sProfileName) 
 	{
-		nsresult rv;
-		nsCOMPtr<nsILocalFile> profileDir;
-		PRBool exists = PR_FALSE;
-		PRBool isDir = PR_FALSE;
-		profileDir = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-		rv = profileDir->InitWithPath(sProfileDir);
-		if (NS_FAILED(rv))
-		  	return NS_ERROR_FAILURE;
-		profileDir->Exists(&exists);
-		profileDir->IsDirectory(&isDir);
-		// if it exists and it isn't a directory then give up now.
-		if (!exists) 
-		{
-		  	rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
-		  	if NS_FAILED(rv)
-				return NS_ERROR_FAILURE;
-		}
-		else if (exists && !isDir)
-		  	return NS_ERROR_FAILURE;
+    nsresult rv;
+    nsCOMPtr<nsILocalFile> profileDir;
+    NS_NewNativeLocalFile(nsDependentCString(sProfileDir), PR_TRUE,
+                          getter_AddRefs(profileDir));
+    if (!profileDir)
+      return NS_ERROR_FAILURE;
+    rv = profileDir->AppendNative(nsDependentCString(sProfileName));
+    if (NS_FAILED(rv))
+      return NS_ERROR_FAILURE;
 
-		// actually create the loc provider and initialize prefs.
-		nsMPFileLocProvider *locProvider;
-		// Set up the loc provider.  This has a really strange ownership
-		// model.  When I initialize it it will register itself with the
-		// directory service.  The directory service becomes the owning
-		// reference.  So, when that service is shut down this object will
-		// be destroyed.  It's not leaking here.
-		locProvider = new nsMPFileLocProvider;
-		rv = locProvider->Initialize(profileDir, sProfileName);
-  	}
+    nsCOMPtr<nsProfileDirServiceProvider> locProvider;
+    NS_NewProfileDirServiceProvider(PR_TRUE, getter_AddRefs(locProvider));
+    if (!locProvider)
+      return NS_ERROR_FAILURE;
+    
+    // Directory service holds an strong reference to any
+    // provider that is registered with it. Let it hold the
+    // only ref. locProvider won't die when we leave this scope.
+    rv = locProvider->Register();
+    if (NS_FAILED(rv))
+      return rv;
+    rv = locProvider->SetProfileDir(profileDir);
+    if (NS_FAILED(rv))
+      return rv;
+  }
   
 	return NS_OK;
 }
@@ -1098,7 +1090,7 @@ PtWidgetClass_t *PtCreateMozillaClass( void )
 	if (StartupEmbedding() == -1)
 		return (NULL);
 
-	Init_nsUnknownContentTypeHandler_Factory( );
+//	Init_nsUnknownContentTypeHandler_Factory( );
 
 	nsCOMPtr<nsIFactory> promptFactory;
 	NS_NewPromptServiceFactory(getter_AddRefs(promptFactory));
