@@ -43,7 +43,6 @@
 #include "nsVoidArray.h"
 
 #include "nsCWebBrowserPersist.h"
-#include "nsDOMWalker.h"
 
 class nsEncoderNodeFixup;
 class nsIStorageStream;
@@ -53,7 +52,6 @@ struct URIData;
 class nsWebBrowserPersist : public nsIInterfaceRequestor,
                             public nsIWebBrowserPersist,
                             public nsIStreamListener,
-                            public nsDOMWalkerCallback,
                             public nsIProgressEventSink,
                             public nsSupportsWeakReference
 {
@@ -69,7 +67,7 @@ public:
     NS_DECL_NSIREQUESTOBSERVER
     NS_DECL_NSISTREAMLISTENER
     NS_DECL_NSIPROGRESSEVENTSINK
-    
+
 // Protected members
 protected:    
     virtual ~nsWebBrowserPersist();
@@ -103,28 +101,41 @@ private:
     nsresult MakeOutputStreamFromURI(nsIURI *aURI, nsIOutputStream  **aOutStream);
     nsresult CreateChannelFromURI(nsIURI *aURI, nsIChannel **aChannel);
     nsresult StartUpload(nsIStorageStream *aOutStream, nsIURI *aDestinationURI,
-        const char *aContentType);
+        const nsACString &aContentType);
     nsresult CalculateAndAppendFileExt(nsIURI *aURI, nsIChannel *aChannel,
         nsIURI *aOriginalURIWithExtension);
+    nsresult CalculateUniqueFilename(nsIURI *aURI);
     nsresult MakeFilenameFromURI(
         nsIURI *aURI, nsString &aFilename);
+    nsresult StoreURI(
+        const char *aURI,
+        PRBool aNeedsPersisting = PR_TRUE,
+        URIData **aData = nsnull);
     nsresult StoreURIAttribute(
         nsIDOMNode *aNode, const char *aAttribute,
         PRBool aNeedsPersisting = PR_TRUE,
         URIData **aData = nsnull);
+    PRBool GetQuotedAttributeValue(
+    const nsAString &aSource, const nsAString &aAttribute, nsAString &aValue);
+
     nsresult GetNodeToFixup(nsIDOMNode *aNodeIn, nsIDOMNode **aNodeOut);
+    nsresult FixupURI(nsAString &aURI);
     nsresult FixupNodeAttribute(nsIDOMNode *aNode, const char *aAttribute);
     nsresult FixupAnchor(nsIDOMNode *aNode);
+    nsresult FixupXMLStyleSheetLink(nsIDOMProcessingInstruction *aPI, const nsAString &aHref);
+    nsresult GetXMLStyleSheetLink(nsIDOMProcessingInstruction *aPI, nsAString &aHref);
+
     nsresult StoreAndFixupStyleSheet(nsIStyleSheet *aStyleSheet);
     nsresult SaveDocumentWithFixup(
         nsIDocument *pDocument, nsIDocumentEncoderNodeFixup *pFixup,
-        nsIURI *aFile, PRBool aReplaceExisting, const char *aFormatType,
+        nsIURI *aFile, PRBool aReplaceExisting, const nsACString &aFormatType,
         const nsString &aSaveCharset, PRUint32  aFlags);
     nsresult SaveSubframeContent(
         nsIDOMDocument *aFrameContent, URIData *aData);
     nsresult SetDocumentBase(nsIDOMDocument *aDocument, nsIURI *aBaseURI);
     nsresult SendErrorStatusChange(
         PRBool aIsReadError, nsresult aResult, nsIRequest *aRequest, nsIURI *aURI);
+    nsresult OnWalkDOMNode(nsIDOMNode *aNode);
 
     nsresult FixRedirectedChannelEntry(nsIChannel *aNewChannel);
 
@@ -133,9 +144,6 @@ private:
     PRBool SerializeNextFile();
     void CalcTotalProgress();
 
-    // nsDOMWalkerCallback method
-    nsresult OnWalkDOMNode(nsIDOMNode *aNode, PRBool *aAbort);
-
     // Hash table enumerators
     static PRBool PR_CALLBACK EnumPersistURIs(
         nsHashKey *aKey, void *aData, void* closure);
@@ -153,8 +161,6 @@ private:
         nsHashKey *aKey, void *aData, void* closure);
     static PRBool PR_CALLBACK EnumCountURIsToPersist(
         nsHashKey *aKey, void *aData, void* closure);
-    static PRBool PR_CALLBACK EnumCheckForDuplicateFileNames(
-        nsHashKey *aKey, void *aData, void* closure);
 
     nsCOMPtr<nsIURI>          mCurrentDataPath;
     PRBool                    mCurrentDataPathIsRelative;
@@ -170,6 +176,7 @@ private:
     nsHashtable               mUploadList;
     nsHashtable               mURIMap;
     nsVoidArray               mDocList;
+    nsCStringArray            mFilenameList;
     PRPackedBool              mFirstAndOnlyUse;
     PRPackedBool              mCancel;
     PRPackedBool              mJustStartedLoading;
