@@ -463,7 +463,9 @@ NS_IMETHODIMP CBrowserImpl::SetVisibility(PRBool aVisibility)
 }
 
 //*****************************************************************************
-// CBrowserImpl::nsIStreamListener (used for nsIRequest & UriContentListener)
+// CBrowserImpl::nsIStreamListener 
+// (used for nsIRequest, nsIChannel, and UriContentListener)
+//*****************************************************************************   
 
 NS_IMETHODIMP CBrowserImpl::OnDataAvailable(nsIRequest *request,
 				nsISupports *ctxt, nsIInputStream *input,
@@ -489,17 +491,32 @@ NS_IMETHODIMP CBrowserImpl::OnDataAvailable(nsIRequest *request,
 	FormatAndPrintOutput("OnDataAvailable() offset = ", offset, 1);
 	FormatAndPrintOutput("OnDataAvailable() count = ", count, 1);
 
+	if (!ctxt)
+		QAOutput("OnDataAvailable():We didn't get the nsISupports object.", 1);
+	else
+		QAOutput("OnDataAvailable():We got the nsISupports object.", 1);
+
 	return NS_OK;
 }
 
+//*****************************************************************************
+// CBrowserImpl::nsIRequestObserver 
+// (used for nsIRequest, nsIChannel, and UriContentListener)
+//*****************************************************************************   
+
 NS_IMETHODIMP CBrowserImpl::OnStartRequest(nsIRequest *request,
 				nsISupports *ctxt)
 {
 	nsCString stringMsg;
 
-	QAOutput("##### BEGIN: nsIStreamListener::OnStartRequest() #####");
+	QAOutput("##### BEGIN: nsIRequestObserver::OnStartRequest() #####");
 	RequestName(request, stringMsg, 1);
 
+	if (!ctxt)
+		QAOutput("OnStartRequest():We didn't get the nsISupports object.", 1);
+	else
+		QAOutput("OnStartRequest():We got the nsISupports object.", 1);
+
 	return NS_OK;
 }
 
@@ -508,15 +525,24 @@ NS_IMETHODIMP CBrowserImpl::OnStopRequest(nsIRequest *request,
 {
 	nsCString stringMsg;
 
-	RvTestResult(rv, "nsIStreamListener::OnStopRequest rv input", 1);
+	RvTestResult(rv, "nsIRequestObserver::OnStopRequest rv input", 1);
 	RequestName(request, stringMsg, 1);
+
+	if (!ctxt)
+		QAOutput("OnStopRequest():We didn't get the nsISupports object.", 1);
+	else
+		QAOutput("OnStopRequest():We got the nsISupports object.", 1);
+
+	RvTestResult(rv, "OnStopRequest() rv test", 1);
+
 	QAOutput("##### END: nsIStreamListener::OnStopRequest() #####");
 		
 	return NS_OK;
 }
 
 //*****************************************************************************   
-//  Tool Tip Listener
+//  CBrowserImpl::Tool Tip Listener
+//*****************************************************************************   
 
 NS_IMETHODIMP CBrowserImpl::OnShowTooltip(PRInt32 aXCoords, PRInt32 aYCoords,
 										  const PRUnichar *aTipText)
@@ -536,7 +562,8 @@ NS_IMETHODIMP CBrowserImpl::OnHideTooltip()
 
 
 //*****************************************************************************   
-//  UriContentListener
+//  CBrowserImpl::UriContentListener
+//*****************************************************************************   
 
 NS_IMETHODIMP CBrowserImpl::OnStartURIOpen(nsIURI *aURI, PRBool *_retval)
 {
