@@ -59,6 +59,18 @@
 #include <io.h>
 #include <fcntl.h>
 
+
+#ifdef _BUILD_STATIC_BIN
+#include "nsStaticComponent.h"
+nsresult PR_CALLBACK
+app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
+#endif
+
+
+#ifdef NS_TRACE_MALLOC
+#include "nsTraceMalloc.h"
+#endif
+
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #undef THIS_FILE
@@ -84,16 +96,106 @@ static NS_DEFINE_CID(kPrintingPromptServiceCID, NS_PRINTINGPROMPTSERVICE_CID);
     {0xf68578eb, 0x6ec2, 0x4169, {0xae, 0x19, 0x8c, 0x62, 0x43, 0xf0, 0xab, 0xe1}}
 static NS_DEFINE_CID(kHelperAppLauncherDialogCID, NS_HELPERAPPLAUNCHERDIALOG_CID);
 
+class CMfcEmbedCommandLine : public CCommandLineInfo
+{
+public:
+
+    CMfcEmbedCommandLine(CMfcEmbedApp& app) : CCommandLineInfo(),
+                                              mApp(app)
+    {
+    }
+
+    // generic parser which bundles up flags and their parameters, to
+    // pass to HandleFlag() or HandleNakedParameter()
+    // if you're adding new parameters, please don't touch this
+    // function and instead add your own handler below
+    virtual void ParseParam(LPCTSTR szParam, BOOL bFlag, BOOL bLast)
+    {
+        CCommandLineInfo::ParseParam(szParam, bFlag, bLast);
+        if (bFlag) {
+            // advance past extra stuff like --foo
+            while (*szParam && *szParam == '-')
+                szParam++;
+
+            // previous argument was a flag too, so process that first
+            if (!mLastFlag.IsEmpty())
+                HandleFlag(mLastFlag);
+            
+            mLastFlag = szParam;
+
+            // oops, no more arguments coming, so handle this now
+            if (bLast)
+                HandleFlag(mLastFlag);
+            
+        } else {
+            if (!mLastFlag.IsEmpty())
+                HandleFlag(mLastFlag, szParam);
+                
+            mLastFlag.Truncate();
+        }
+    }
+
+    // handle flag-based parameters
+    void HandleFlag(const nsACString& flag, const char* param=nsnull)
+    {
+        if (flag.Equals("console"))
+            DoConsole();
+        else if (flag.Equals("chrome"))
+            DoChrome();
+#ifdef NS_TRACE_MALLOC
+        else if (flag.Equals("trace-malloc"))
+            DoTraceMalloc(flag, param);
+#endif
+        // add new flag handlers here (please add a DoFoo() method below!)
+    }
+
+    void HandleNakedParameter(const char* flag) {
+        // handle non-flag arguments here
+    }
+
+    // add your specific handlers here
+    void DoConsole() {
+        mApp.ShowDebugConsole();
+    }
+
+    void DoChrome() {
+        mApp.m_bChrome = TRUE;
+    }
+
+#ifdef NS_TRACE_MALLOC
+    void DoTraceMalloc(const nsACString& flag, const char* param)
+    {
+        if (!param) {
+            NS_WARNING("--trace-malloc needs a filename as a parameter");
+            return;
+        }
+
+        // build up fake argv/argc arguments for tracemalloc stuff
+        char* argv[] = { "mfcembed", "--trace-malloc",
+                         NS_CONST_CAST(char*, param) };
+        
+        NS_TraceMallocStartupArgs(3, argv);
+    }
+#endif
+    
+private:
+    // autostring is fine, this is a stack based object anyway
+    nsCAutoString mLastFlag;
+
+    CMfcEmbedApp& mApp;
+};
+
+
 BEGIN_MESSAGE_MAP(CMfcEmbedApp, CWinApp)
-	//{{AFX_MSG_MAP(CMfcEmbedApp)
-	ON_COMMAND(ID_NEW_BROWSER, OnNewBrowser)
-	ON_COMMAND(ID_NEW_EDITORWINDOW, OnNewEditor)
-	ON_COMMAND(ID_MANAGE_PROFILES, OnManageProfiles)
-	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
+    //{{AFX_MSG_MAP(CMfcEmbedApp)
+    ON_COMMAND(ID_NEW_BROWSER, OnNewBrowser)
+    ON_COMMAND(ID_NEW_EDITORWINDOW, OnNewEditor)
+    ON_COMMAND(ID_MANAGE_PROFILES, OnManageProfiles)
+    ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
     ON_COMMAND(ID_EDIT_PREFERENCES, OnEditPreferences)
-	// NOTE - the ClassWizard will add and remove mapping macros here.
-	//    DO NOT EDIT what you see in these blocks of generated code!
-	//}}AFX_MSG_MAP
+    // NOTE - the ClassWizard will add and remove mapping macros here.
+    //    DO NOT EDIT what you see in these blocks of generated code!
+    //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
 CMfcEmbedApp::CMfcEmbedApp() :
@@ -110,48 +212,6 @@ CMfcEmbedApp::CMfcEmbedApp() :
 
 CMfcEmbedApp theApp;
 
-BOOL CMfcEmbedApp::IsCmdLineSwitch(const char *pSwitch, BOOL bRemove)
-{
-    //  Search for the switch in the command line.
-    //  Don't take it out of m_lpCmdLine by default
-    char *pFound = PL_strcasestr(m_lpCmdLine, pSwitch);
-    if(pFound == NULL ||
-        // Switch must be at beginning of command line
-        // or have a space in front of it to avoid
-        // mangling filenames
-        ( (pFound != m_lpCmdLine) &&
-          *(pFound-1) != ' ' ) ) 
-    {
-        return(FALSE);
-    }
-
-    if (bRemove) 
-    {
-        // remove the flag from the command line
-        char *pTravEnd = pFound + strlen(pSwitch);
-        char *pTraverse = pFound;
-
-        *pTraverse = *pTravEnd;
-        while(*pTraverse != '\0')   
-        {
-            pTraverse++;
-            pTravEnd++;
-            *pTraverse = *pTravEnd;
-        }
-    }
-
-    return(TRUE);
-}
-
-void CMfcEmbedApp::ParseCmdLine()
-{
-    // Show Debug Console?
-    if(IsCmdLineSwitch("-console"))
-        ShowDebugConsole();
-    if(IsCmdLineSwitch("-chrome"))
-        m_bChrome = TRUE;
-}
-
 /* Some Gecko interfaces are implemented as components, automatically
    registered at application initialization. nsIPrompt is an example:
    the default implementation uses XUL, not native windows. Embedding
@@ -283,6 +343,7 @@ void CMfcEmbedApp::ShowDebugConsole()
 #endif
 }
 
+
 // Initialize our MFC application and also init
 // the Gecko embedding APIs
 // Note that we're also init'ng the profile switching
@@ -292,29 +353,36 @@ void CMfcEmbedApp::ShowDebugConsole()
 //
 BOOL CMfcEmbedApp::InitInstance()
 {
-    ParseCmdLine();
+#ifdef _BUILD_STATIC_BIN
+    // Initialize XPCOM's module info table
+    NSGetStaticModuleInfo = app_getModuleInfo;
+#endif
 
-	Enable3dControls();
+    
+    CMfcEmbedCommandLine cmdLine(*this);
+    ParseCommandLine(cmdLine);
+    
+    Enable3dControls();
 
-	//
-	// 1. Determine the name of the dir from which the MRE based app is being run
-	// from [It's OK to do this even if you're not running in an MRE env]
-	//
-	// 2. Create an nsILocalFile out of it which will passed in to NS_InitEmbedding()
-	//
-	// Please see http://www.mozilla.org/projects/embedding/MRE.html
-	// for more info. on MRE
+    //
+    // 1. Determine the name of the dir from which the GRE based app is being run
+    // from [It's OK to do this even if you're not running in an GRE env]
+    //
+    // 2. Create an nsILocalFile out of it which will passed in to NS_InitEmbedding()
+    //
+    // Please see http://www.mozilla.org/projects/embedding/MRE.html
+    // for more info. on GRE
 
-	char curDir[_MAX_PATH+1];
-	::GetCurrentDirectory(_MAX_PATH, curDir);
-	nsresult rv;
-	nsCOMPtr<nsILocalFile> mreAppDir;
-	rv = NS_NewNativeLocalFile(nsDependentCString(curDir), TRUE, getter_AddRefs(mreAppDir));
-	NS_ASSERTION(NS_SUCCEEDED(rv), "failed to create mreAppDir localfile");
+    char curDir[_MAX_PATH+1];
+    ::GetCurrentDirectory(_MAX_PATH, curDir);
+    nsresult rv;
+    nsCOMPtr<nsILocalFile> mreAppDir;
+    rv = NS_NewNativeLocalFile(nsDependentCString(curDir), TRUE, getter_AddRefs(mreAppDir));
+    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to create mreAppDir localfile");
 
-	// Take a look at 
-	// http://www.mozilla.org/projects/xpcom/file_locations.html
-	// for more info on File Locations
+    // Take a look at 
+    // http://www.mozilla.org/projects/xpcom/file_locations.html
+    // for more info on File Locations
 
     winEmbedFileLocProvider *provider = new winEmbedFileLocProvider("MfcEmbed");
     if(!provider)
@@ -345,81 +413,81 @@ BOOL CMfcEmbedApp::InitInstance()
         return FALSE;
     }
 
-	if(!InitializeProfiles())
-	{
+    if(!InitializeProfiles())
+    {
         ASSERT(FALSE);
         NS_TermEmbedding();
-		return FALSE;
-	}
+        return FALSE;
+    }
 
 
     if(!CreateHiddenWindow())
-	{
+    {
         ASSERT(FALSE);
         NS_TermEmbedding();
-		return FALSE;
-	}
+        return FALSE;
+    }
 
-	// Create the first browser frame window
-	OnNewBrowser();
+    // Create the first browser frame window
+    OnNewBrowser();
 
-	return TRUE;
+    return TRUE;
 }
 
 CBrowserFrame* CMfcEmbedApp::CreateNewBrowserFrame(PRUint32 chromeMask,
-												   PRInt32 x, PRInt32 y,
-												   PRInt32 cx, PRInt32 cy,
-												   PRBool bShowWindow,
-												   PRBool bIsEditor
-												   )
+                                                   PRInt32 x, PRInt32 y,
+                                                   PRInt32 cx, PRInt32 cy,
+                                                   PRBool bShowWindow,
+                                                   PRBool bIsEditor
+                                                   )
 {
-	UINT resId = bIsEditor ? IDR_EDITOR : IDR_MAINFRAME;
+    UINT resId = bIsEditor ? IDR_EDITOR : IDR_MAINFRAME;
 
-	// Setup a CRect with the requested window dimensions
-	CRect winSize(x, y, cx, cy);
+    // Setup a CRect with the requested window dimensions
+    CRect winSize(x, y, cx, cy);
 
-	// Use the Windows default if all are specified as -1
-	if(x == -1 && y == -1 && cx == -1 && cy == -1)
-		winSize = CFrameWnd::rectDefault;
+    // Use the Windows default if all are specified as -1
+    if(x == -1 && y == -1 && cx == -1 && cy == -1)
+        winSize = CFrameWnd::rectDefault;
 
-	// Load the window title from the string resource table
-	CString strTitle;
-	strTitle.LoadString(IDR_MAINFRAME);
+    // Load the window title from the string resource table
+    CString strTitle;
+    strTitle.LoadString(IDR_MAINFRAME);
 
-	// Now, create the browser frame
-	CBrowserFrame* pFrame = bIsEditor ? ( new  CEditorFrame(chromeMask) ) :
-					    ( new  CBrowserFrame(chromeMask) );
-	pFrame->SetEditable(bIsEditor);
+    // Now, create the browser frame
+    CBrowserFrame* pFrame = bIsEditor ? ( new  CEditorFrame(chromeMask) ) :
+                        ( new  CBrowserFrame(chromeMask) );
+    pFrame->SetEditable(bIsEditor);
 
-	if (!pFrame->Create(NULL, strTitle, WS_OVERLAPPEDWINDOW, 
-					winSize, NULL, MAKEINTRESOURCE(resId), 0L, NULL))
-	{
-		return NULL;
-	}
+    if (!pFrame->Create(NULL, strTitle, WS_OVERLAPPEDWINDOW, 
+                    winSize, NULL, MAKEINTRESOURCE(resId), 0L, NULL))
+    {
+        return NULL;
+    }
 
-	// load accelerator resource
-	pFrame->LoadAccelTable(MAKEINTRESOURCE(IDR_MAINFRAME));
+    // load accelerator resource
+    pFrame->LoadAccelTable(MAKEINTRESOURCE(IDR_MAINFRAME));
 
-	// Show the window...
-	if(bShowWindow)
-	{
-		pFrame->ShowWindow(SW_SHOW);
-		pFrame->UpdateWindow();
-	}
+    // Show the window...
+    if(bShowWindow)
+    {
+        pFrame->ShowWindow(SW_SHOW);
+        pFrame->UpdateWindow();
+    }
 
-	// Add to the list of BrowserFrame windows
-	m_FrameWndLst.AddHead(pFrame);
+    // Add to the list of BrowserFrame windows
+    m_FrameWndLst.AddHead(pFrame);
 
-	return pFrame;
+    return pFrame;
 }
 
 void CMfcEmbedApp::OnNewBrowser()
 {
-	CBrowserFrame *pBrowserFrame = CreateNewBrowserFrame();
+    CBrowserFrame *pBrowserFrame = CreateNewBrowserFrame();
 
-	//Load the HomePage into the browser view
-	if(pBrowserFrame && (GetStartupPageMode() == 1))
-		pBrowserFrame->m_wndBrowserView.LoadHomePage();
+    //Load the HomePage into the browser view
+    if(pBrowserFrame && (GetStartupPageMode() == 1))
+        pBrowserFrame->m_wndBrowserView.LoadHomePage();
 }
 
 void CMfcEmbedApp::OnNewEditor() 
@@ -445,55 +513,55 @@ void CMfcEmbedApp::OnNewEditor()
 //
 void CMfcEmbedApp::RemoveFrameFromList(CBrowserFrame* pFrm, BOOL bCloseAppOnLastFrame/*= TRUE*/)
 {
-	POSITION pos = m_FrameWndLst.Find(pFrm);
-	m_FrameWndLst.RemoveAt(pos);
+    POSITION pos = m_FrameWndLst.Find(pFrm);
+    m_FrameWndLst.RemoveAt(pos);
 
-	// Send a WM_QUIT msg. to the hidden window if we've
-	// just closed the last browserframe window and
-	// if the bCloseAppOnLastFrame is TRUE. This be FALSE
-	// only in the case we're switching profiles
-	// Without this the hidden window will stick around
-	// i.e. the app will never die even after all the 
-	// visible windows are gone.
-	if(m_FrameWndLst.GetCount() == 0 && bCloseAppOnLastFrame)
-		m_pMainWnd->PostMessage(WM_QUIT);
+    // Send a WM_QUIT msg. to the hidden window if we've
+    // just closed the last browserframe window and
+    // if the bCloseAppOnLastFrame is TRUE. This be FALSE
+    // only in the case we're switching profiles
+    // Without this the hidden window will stick around
+    // i.e. the app will never die even after all the 
+    // visible windows are gone.
+    if(m_FrameWndLst.GetCount() == 0 && bCloseAppOnLastFrame)
+        m_pMainWnd->PostMessage(WM_QUIT);
 }
 
 int CMfcEmbedApp::ExitInstance()
 {
-	// When File/Exit is chosen and if the user
-	// has opened multiple browser windows shut all
-	// of them down properly before exiting the app
+    // When File/Exit is chosen and if the user
+    // has opened multiple browser windows shut all
+    // of them down properly before exiting the app
 
-	CBrowserFrame* pBrowserFrame = NULL;
+    CBrowserFrame* pBrowserFrame = NULL;
 
-	POSITION pos = m_FrameWndLst.GetHeadPosition();
-	while( pos != NULL )
-	{
-		pBrowserFrame = (CBrowserFrame *) m_FrameWndLst.GetNext(pos);
-		if(pBrowserFrame)
-		{
-			pBrowserFrame->ShowWindow(false);
-			pBrowserFrame->DestroyWindow();
-		}
-	}
-	m_FrameWndLst.RemoveAll();
+    POSITION pos = m_FrameWndLst.GetHeadPosition();
+    while( pos != NULL )
+    {
+        pBrowserFrame = (CBrowserFrame *) m_FrameWndLst.GetNext(pos);
+        if(pBrowserFrame)
+        {
+            pBrowserFrame->ShowWindow(false);
+            pBrowserFrame->DestroyWindow();
+        }
+    }
+    m_FrameWndLst.RemoveAll();
 
     if (m_pMainWnd)
         m_pMainWnd->DestroyWindow();
 
     delete m_ProfileMgr;
 
-	NS_TermEmbedding();
+    NS_TermEmbedding();
 
-	return 1;
+    return 1;
 }
 
 BOOL CMfcEmbedApp::OnIdle(LONG lCount)
 {
-	CWinApp::OnIdle(lCount);
+    CWinApp::OnIdle(lCount);
 
-	return FALSE;
+    return FALSE;
 }
 
 void CMfcEmbedApp::OnManageProfiles()
@@ -525,7 +593,7 @@ void CMfcEmbedApp::OnEditPreferences()
                 rv = prefs->SavePrefFile(nsnull);
         }
         else
-		    NS_ASSERTION(PR_FALSE, "Could not get preferences service");
+            NS_ASSERTION(PR_FALSE, "Could not get preferences service");
     }
 }
 
@@ -535,7 +603,7 @@ BOOL CMfcEmbedApp::InitializeProfiles()
     if (!m_ProfileMgr)
         return FALSE;
 
-	  nsresult rv;
+      nsresult rv;
     nsCOMPtr<nsIObserverService> observerService = 
              do_GetService("@mozilla.org/observer-service;1", &rv);
     observerService->AddObserver(this, "profile-approve-change", PR_TRUE);
@@ -544,7 +612,7 @@ BOOL CMfcEmbedApp::InitializeProfiles()
 
     m_ProfileMgr->StartUp();
 
-	return TRUE;
+    return TRUE;
 }
 
 // When the profile switch happens, all open browser windows need to be 
@@ -554,31 +622,31 @@ BOOL CMfcEmbedApp::InitializeProfiles()
 // switches
 BOOL CMfcEmbedApp::CreateHiddenWindow()
 {
-	CFrameWnd *hiddenWnd = new CFrameWnd;
-	if(!hiddenWnd)
-		return FALSE;
+    CFrameWnd *hiddenWnd = new CFrameWnd;
+    if(!hiddenWnd)
+        return FALSE;
 
     RECT bounds = { -10010, -10010, -10000, -10000 };
     hiddenWnd->Create(NULL, "main", WS_DISABLED, bounds, NULL, NULL, 0, NULL);
     m_pMainWnd = hiddenWnd;
 
-	return TRUE;
+    return TRUE;
 }
 
 nsresult CMfcEmbedApp::InitializePrefs()
 {
    nsresult rv;
    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
-   if (NS_SUCCEEDED(rv)) {	  
+   if (NS_SUCCEEDED(rv)) {      
 
-		// We are using the default prefs from mozilla. If you were
-		// disributing your own, this would be done simply by editing
-		// the default pref files.
-		
-		PRBool inited;
-		rv = prefs->GetBoolPref("mfcbrowser.prefs_inited", &inited);
-		if (NS_FAILED(rv) || !inited)
-		{
+        // We are using the default prefs from mozilla. If you were
+        // disributing your own, this would be done simply by editing
+        // the default pref files.
+        
+        PRBool inited;
+        rv = prefs->GetBoolPref("mfcbrowser.prefs_inited", &inited);
+        if (NS_FAILED(rv) || !inited)
+        {
             m_iStartupPage = 1;
             m_strHomePage = "http://www.mozilla.org/projects/embedding";
 
@@ -603,10 +671,10 @@ nsresult CMfcEmbedApp::InitializePrefs()
             if(pBuf)
                 m_strHomePage = pBuf;
         }       
-	}
-	else
-		NS_ASSERTION(PR_FALSE, "Could not get preferences service");
-		
+    }
+    else
+        NS_ASSERTION(PR_FALSE, "Could not get preferences service");
+        
     return rv;
 }
 
@@ -622,7 +690,7 @@ nsresult CMfcEmbedApp::InitializeWindowCreator()
   // give an nsIWindowCreator to the WindowWatcher service
   nsCOMPtr<nsIWindowCreator> windowCreator(NS_STATIC_CAST(nsIWindowCreator *, this));
   if (windowCreator) {
-    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
     if (wwatch) {
       wwatch->SetWindowCreator(windowCreator);
       return NS_OK;
@@ -663,21 +731,21 @@ NS_IMETHODIMP CMfcEmbedApp::Observe(nsISupports *aSubject, const char *aTopic, c
         // Close all open windows. Alternatively, we could just call CBrowserWindow::Stop()
         // on each. Either way, we have to stop all network activity on this phase.
         
-	    POSITION pos = m_FrameWndLst.GetHeadPosition();
-	    while( pos != NULL )
-	    {
-		    CBrowserFrame *pBrowserFrame = (CBrowserFrame *) m_FrameWndLst.GetNext(pos);
-		    if(pBrowserFrame)
-		    {
-			    pBrowserFrame->ShowWindow(false);
+        POSITION pos = m_FrameWndLst.GetHeadPosition();
+        while( pos != NULL )
+        {
+            CBrowserFrame *pBrowserFrame = (CBrowserFrame *) m_FrameWndLst.GetNext(pos);
+            if(pBrowserFrame)
+            {
+                pBrowserFrame->ShowWindow(false);
 
-				// Passing in FALSE below so that we do not
-				// kill the main app during a profile switch
-				RemoveFrameFromList(pBrowserFrame, FALSE);
+                // Passing in FALSE below so that we do not
+                // kill the main app during a profile switch
+                RemoveFrameFromList(pBrowserFrame, FALSE);
 
-				pBrowserFrame->DestroyWindow();
-		    }
-	    }
+                pBrowserFrame->DestroyWindow();
+            }
+        }
     }
     else if (nsCRT::strcmp(aTopic, "profile-after-change") == 0)
     {
@@ -715,15 +783,15 @@ NS_IMETHODIMP CMfcEmbedApp::CreateChromeWindow(nsIWebBrowserChrome *parent,
 class CAboutDlg : public CDialog
 {
 public:
-	CAboutDlg();
+    CAboutDlg();
 
-	enum { IDD = IDD_ABOUTBOX };
+    enum { IDD = IDD_ABOUTBOX };
 
 protected:
-	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
+    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
 
 protected:
-	DECLARE_MESSAGE_MAP()
+    DECLARE_MESSAGE_MAP()
 };
 
 CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
@@ -732,7 +800,7 @@ CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
 
 void CAboutDlg::DoDataExchange(CDataExchange* pDX)
 {
-	CDialog::DoDataExchange(pDX);
+    CDialog::DoDataExchange(pDX);
 }
 
 BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
@@ -741,6 +809,6 @@ END_MESSAGE_MAP()
 // Show the AboutDlg
 void CMfcEmbedApp::OnAppAbout()
 {
-	CAboutDlg aboutDlg;
-	aboutDlg.DoModal();
+    CAboutDlg aboutDlg;
+    aboutDlg.DoModal();
 }
