@@ -36,6 +36,8 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "plstr.h"
+#include "prmem.h"
 #include "nsIServiceManager.h"
 #include "nsCookieManager.h"
 #include "nsCRT.h"
@@ -55,7 +57,6 @@ class nsCookieEnumerator : public nsISimpleEnumerator
 
         nsCookieEnumerator() : mCookieCount(0) 
         {
-            NS_INIT_REFCNT();
         }
 
         NS_IMETHOD HasMoreElements(PRBool *result) 
@@ -66,17 +67,17 @@ class nsCookieEnumerator : public nsISimpleEnumerator
 
         NS_IMETHOD GetNext(nsISupports **result) 
         {
-          char *name;
-          char *value;
+          nsCAutoString name;
+          nsCAutoString value;
           PRBool isDomain;
-          char * host;
-          char * path;
+          nsCAutoString host;
+          nsCAutoString path;
           PRBool isSecure;
           PRUint64 expires;
           nsCookieStatus status;
           nsCookiePolicy policy;
           nsresult rv = COOKIE_Enumerate
-            (mCookieCount++, &name, &value, &isDomain, &host, &path, &isSecure, &expires,
+            (mCookieCount++, name, value, &isDomain, host, path, &isSecure, &expires,
               &status, &policy);
           if (NS_SUCCEEDED(rv)) {
             nsICookie *cookie =
@@ -104,11 +105,10 @@ NS_IMPL_ISUPPORTS1(nsCookieEnumerator, nsISimpleEnumerator);
 ////////////////////////////////////////////////////////////////////////////////
 // nsCookieManager Implementation
 
-NS_IMPL_ISUPPORTS2(nsCookieManager, nsICookieManager, nsISupportsWeakReference);
+NS_IMPL_ISUPPORTS3(nsCookieManager, nsICookieManager, nsICookieManager2, nsISupportsWeakReference);
 
 nsCookieManager::nsCookieManager()
 {
-  NS_INIT_REFCNT();
 }
 
 nsCookieManager::~nsCookieManager(void)
@@ -123,7 +123,7 @@ nsresult nsCookieManager::Init()
 
 NS_IMETHODIMP nsCookieManager::RemoveAll(void) {
   ::COOKIE_RemoveAll();
-  ::COOKIE_Write();
+  ::COOKIE_Write(nsnull);
   return NS_OK;
 }
 
@@ -139,6 +139,33 @@ NS_IMETHODIMP nsCookieManager::GetEnumerator(nsISimpleEnumerator * *entries)
     return NS_OK;
 }
 
+NS_IMETHODIMP nsCookieManager::Add(const nsACString &aDomain,
+                                   const nsACString &aPath,
+                                   const nsACString &aName,
+                                   const nsACString &aValue,
+                                   PRBool aSecure,
+                                   PRInt32 aExpires)
+{
+  // nasty COOKIE method requires caller to hand off ownership of strings.
+  /* nulls aren't allowed (cookie code is full of checks as if they were
+     but see COOKIE_Write) */
+  char *domainCopy = PL_strdup(PromiseFlatCString(aDomain).get()),
+       *pathCopy = PL_strdup(PromiseFlatCString(aPath).get()),
+       *nameCopy = PL_strdup(PromiseFlatCString(aName).get()),
+       *valueCopy = PL_strdup(PromiseFlatCString(aValue).get());
+
+  if (domainCopy && pathCopy && nameCopy && valueCopy)
+    return ::COOKIE_AddCookie(domainCopy, pathCopy, nameCopy, valueCopy,
+                aSecure, PR_TRUE, aExpires,
+                nsICookie::STATUS_UNKNOWN, nsICookie::POLICY_UNKNOWN);
+
+  if (domainCopy) PL_strfree(domainCopy);
+  if (pathCopy) PL_strfree(pathCopy);
+  if (nameCopy) PL_strfree(nameCopy);
+  if (valueCopy) PL_strfree(valueCopy);
+  return NS_ERROR_OUT_OF_MEMORY;
+}
+
 NS_IMETHODIMP nsCookieManager::Remove
   (const nsACString& host, const nsACString& name, const nsACString& path, PRBool blocked) {
 //  (const nsAUTF8String& host, const nsACString& name, const nsAUTF8String& path, PRBool blocked) {
