@@ -66,17 +66,22 @@ extern void COOKIE_DeletePersistentUserData(void);
 extern PRInt32 COOKIE_Count();
 extern nsresult COOKIE_Enumerate
     (PRInt32 count,
-     char **name,
-     char **value,
-     PRBool *isDomain,
-     char ** host,
-     char ** path,
+     nsACString & name,
+     nsACString & value,
+     PRBool * isDomain,
+     nsACString & host,
+     nsACString & path,
      PRBool * isSecure,
      PRUint64 * expires,
      nsCookieStatus * status,
      nsCookiePolicy * policy);
 extern void COOKIE_Remove
   (const char* host, const char* name, const char* path, const PRBool blocked);
+extern nsresult COOKIE_AddCookie(char *aDomain, char *aPath,
+                  char *aName, char *aValue,
+                  PRBool aSecure, PRBool aIsDomain,
+                  time_t aExpires,
+                  nsCookieStatus aStatus, nsCookiePolicy aPolicy);
 
 typedef struct _cookie_CookieStruct {
   char * path;
