@@ -51,18 +51,6 @@
 
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 
-static PRBool ShouldBlockImageByScheme(nsIURI * aURI)
-{
-    nsCAutoString scheme;
-
-    if (NS_FAILED(aURI->GetScheme(scheme)))
-      return PR_FALSE;
-
-    return scheme.Equals(NS_LITERAL_CSTRING("http")) || scheme.Equals(NS_LITERAL_CSTRING("https")) ||
-           scheme.Equals(NS_LITERAL_CSTRING("imap")) || scheme.Equals(NS_LITERAL_CSTRING("mailbox")) ||
-           scheme.Equals(NS_LITERAL_CSTRING("news")) || scheme.Equals(NS_LITERAL_CSTRING("snews"));
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -76,7 +64,7 @@ NS_IMPL_ISUPPORTS2(nsImgManager,
 
 nsImgManager::nsImgManager()
 {
-    NS_INIT_REFCNT();
+    NS_INIT_ISUPPORTS();
 }
 
 nsImgManager::~nsImgManager(void)
@@ -92,9 +80,9 @@ nsresult nsImgManager::Init()
 }
 
 
-NS_IMETHODIMP nsImgManager::Block(const char * imageURL)
+NS_IMETHODIMP nsImgManager::Block(nsIURI* imageURI)
 {
-    ::IMAGE_Block(imageURL,mIOService);
+    ::IMAGE_Block(imageURI);
     return NS_OK;
 }
 
@@ -136,9 +124,6 @@ NS_IMETHODIMP nsImgManager::ShouldLoad(PRInt32 aContentType,
                 rv = doc->GetBaseURL(*getter_AddRefs(baseURI));
                 if (NS_FAILED(rv) || !baseURI) return rv;
 
-                if (!ShouldBlockImageByScheme(baseURI)) //JFD: Why do we care about the base url anyway?
-                  return NS_OK;
-
                 // Let check if we are running a mail window, doesn't matter if mail images are allowed
                 if (IMAGE_BlockedInMail()) {
                     nsCOMPtr<nsIDocShell> docshell;
