@@ -52,18 +52,17 @@
 #include "nsIViewManager.h"
 #include "nsIWidget.h"
 #include "nsIPresContext.h"
-#include "nsIStyleContext.h" 
 #include "nsXULAtoms.h"
 #include "nsHTMLAtoms.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 
 nsIDOMWindowInternal*
-inLayoutUtils::GetWindowFor(nsIDOMElement* aElement)
+inLayoutUtils::GetWindowFor(nsIDOMNode* aNode)
 {
   nsCOMPtr<nsIDOMDocument> doc1;
-  aElement->GetOwnerDocument(getter_AddRefs(doc1));
-  return GetWindowFor(doc1);
+  aNode->GetOwnerDocument(getter_AddRefs(doc1));
+  return GetWindowFor(doc1.get());
 }
 
 nsIDOMWindowInternal*
@@ -143,16 +142,18 @@ inLayoutUtils::GetEventStateManagerFor(nsIDOMElement *aElement)
 }
 
 nsPoint
-inLayoutUtils::GetClientOrigin(nsIFrame* aFrame)
+inLayoutUtils::GetClientOrigin(nsIPresContext* aPresContext,
+                               nsIFrame* aFrame)
 {
   nsPoint result(0,0);
-  nsIFrame* parent = aFrame;
-  while (parent) {
-    nsPoint origin;
-    parent->GetOrigin(origin);
-    result.x += origin.x;
-    result.y += origin.y;
-    parent->GetParent(&parent);
+  nsIView* view;
+  aFrame->GetOffsetFromView(aPresContext, result, &view);
+  while (view) {
+    nscoord x, y;
+    view->GetPosition(&x, &y);
+    result.x += x;
+    result.y += y;
+    view->GetParent(view);
   }
   return result;
 }
@@ -268,39 +269,13 @@ inLayoutUtils::GetSubDocumentFor(nsIDOMNode* aNode)
 nsIDOMNode*
 inLayoutUtils::GetContainerFor(nsIDOMDocument* aDoc)
 {
-  nsCOMPtr<nsIDOMNode> container;
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(aDoc));
 
-  // get the doc shell for this document and look for the parent doc shell
   nsCOMPtr<nsIDOMWindowInternal> win = inLayoutUtils::GetWindowFor(aDoc);
-  nsCOMPtr<nsIScriptGlobalObject> so = do_QueryInterface(win);
+  nsCOMPtr<nsIDOMElement> elem;
+  win->GetFrameElement(getter_AddRefs(elem));
 
-  nsCOMPtr<nsIDocShell> docShell;
-  so->GetDocShell(getter_AddRefs(docShell));
-  nsCOMPtr<nsIDocShellTreeItem> treeItem = do_QueryInterface(docShell);
-
-  nsCOMPtr<nsIDocShellTreeItem> parentItem;
-  treeItem->GetParent(getter_AddRefs(parentItem));
-  if (!parentItem) return nsnull;
-  nsCOMPtr<nsIDocShell> parentDocShell = do_QueryInterface(parentItem);
-
-  // find the content node (browser, iframe, etc..) that contains this document
-  nsCOMPtr<nsIPresShell> presShell;
-  parentDocShell->GetPresShell(getter_AddRefs(presShell));
-
-  nsCOMPtr<nsIDocument> parent_doc;
-  presShell->GetDocument(getter_AddRefs(parent_doc));
-
-  nsCOMPtr<nsIDOMNode> node;
-
-  if (parent_doc) {
-    nsCOMPtr<nsIContent> content;
-    parent_doc->FindContentForSubDocument(doc, getter_AddRefs(content));
-
-    node = do_QueryInterface(content);
-  }
-
-  return node;
+  return elem;
 }
 
 PRBool
@@ -313,7 +288,7 @@ inLayoutUtils::IsDocumentElement(nsIDOMNode* aNode)
   if (parent) {
     PRUint16 nodeType;
     parent->GetNodeType(&nodeType);
-    if (nodeType == 9)
+    if (nodeType == nsIDOMNode::DOCUMENT_NODE)
       result = PR_TRUE;
   }
 
