@@ -35,24 +35,20 @@
 #ifndef TRANSFRMX_EXPR_H
 #define TRANSFRMX_EXPR_H
 
-#include "txError.h"
-#include "TxString.h"
-#include "ErrorObserver.h"
-#include "NodeSet.h"
-#include "Stack.h"
-#include "ExprResult.h"
 #include "baseutils.h"
+#include "dom.h"
+#include "List.h"
+#include "txAtom.h"
 #include "TxObject.h"
-#include "primitives.h"
+#include "TxString.h"
 
 /*
   XPath class definitions.
   Much of this code was ported from XSL:P.
 */
 
-/*
- * necessary prototypes
- */
+class ExprResult;
+class NodeSet;
 class txIParseContext;
 class txIMatchContext;
 class txIEvalContext;
@@ -99,6 +95,9 @@ public:
     TX_DECL_EVALUATE; \
     void toString(String& aDest)
 
+#define TX_DECL_FUNCTION \
+    TX_DECL_EVALUATE; \
+    nsresult getNameAtom(txAtom** aAtom)
 
 /**
  * This class represents a FunctionCall as defined by the XPath 1.0
@@ -117,7 +116,7 @@ public:
      * Virtual methods from Expr 
     **/
     virtual ExprResult* evaluate(txIEvalContext* aContext) = 0;
-    virtual void toString(String& dest);
+    void toString(String& aDest);
 
     /**
      * Adds the given parameter to this FunctionCall's parameter list
@@ -135,10 +134,9 @@ public:
 
 protected:
 
-    List params;
+    txList params;
 
     FunctionCall();
-    FunctionCall(const String& name);
 
     /*
      * Evaluates the given Expression and converts its result to a String.
@@ -163,7 +161,10 @@ protected:
      */
     NodeSet* evaluateToNodeSet(Expr* aExpr, txIEvalContext* aContext);
 
-    String name;
+    /*
+     * Returns the name of the function as an atom.
+     */
+    virtual nsresult getNameAtom(txAtom** aAtom) = 0;
 }; //-- FunctionCall
 
 
