@@ -29,45 +29,68 @@
 #define TRANSFRMX_PROCESSORSTATE_H
 
 #include "NodeSet.h"
-#include "NodeStack.h"
-#include "Stack.h"
 #include "ErrorObserver.h"
-#include "NamedMap.h"
 #include "txPatternParser.h"
 #include "Expr.h"
-#include "StringList.h"
 #include "txOutputFormat.h"
 #include "Map.h"
 #include "txIXPathContext.h"
+#include "txExpandedNameMap.h"
+#include "txXMLEventHandler.h"
+#include "XSLTFunctions.h"
+#include "txError.h"
+#include "nsVoidArray.h"
+#include "nsDoubleHashtable.h"
 
+class txVariableMap;
 class txXSLKey;
-class txDecimalFormat;
+
+class txLoadedDocumentEntry : public PLDHashStringEntry {
+public:
+    txLoadedDocumentEntry(const void* aKey) : PLDHashStringEntry(aKey)
+    {
+    }
+    ~txLoadedDocumentEntry()
+    {
+        delete mDocument;
+    }
+    Document* mDocument;
+};
+
+DECL_DHASH_WRAPPER(txLoadedDocumentsBase, txLoadedDocumentEntry, nsAString&)
+
+class txLoadedDocumentsHash : public txLoadedDocumentsBase
+{
+public:
+    ~txLoadedDocumentsHash();
+    nsresult init(Document* aSourceDocument, Document* aStyleDocument);
+    void Add(Document* aDocument);
+    Document* Get(const nsAString& aURI);
+
+private:
+    friend class ProcessorState;
+    Document* mSourceDocument;
+    Document* mStyleDocument;
+};
 
 /**
  * Class used for keeping the current state of the XSL Processor
-**/
+ */
 class ProcessorState : public txIMatchContext {
 
 public:
-    /**
-     * Creates a new ProcessorState
-    **/
-    ProcessorState();
-
     /**
      * Creates a new ProcessorState for the given XSL document
      * And result Document
-    **/
-    ProcessorState(Document* aSourceDocument,
-                   Document* aXslDocument,
-                   Document* aResultDocument);
+     */
+    ProcessorState(Node* aSourceNode, Document* aXslDocument);
 
     /**
      * Destroys this ProcessorState
-    **/
+     */
     ~ProcessorState();
 
-    /*
+    /**
      * Contain information that is import precedence dependant.
      */
     class ImportFrame {
@@ -76,17 +99,17 @@ public:
         ~ImportFrame();
     
         // Map of named templates
-        NamedMap mNamedTemplates;
+        txExpandedNameMap mNamedTemplates;
 
         // Map of template modes, each item in the map is a list
         // of templates
-        NamedMap mMatchableTemplates;
+        txExpandedNameMap mMatchableTemplates;
 
         // List of whitespace preserving and stripping nametests
         txList mWhiteNameTests;
 
         // Map of named attribute sets
-        NamedMap mNamedAttributeSets;
+        txExpandedNameMap mNamedAttributeSets;
 
         // Output format, as specified by the xsl:output elements
         txOutputFormat mOutputFormat;
@@ -94,15 +117,17 @@ public:
         // ImportFrame which is the first one *not* imported by this frame
         ImportFrame* mFirstNotImported;
 
+        // Map of top-level variables/parameters
+        txExpandedNameMap mVariables;
+
         // The following stuff is missing here:
 
         // Namespace aliases (xsl:namespace-alias)
-        // Toplevel variables/parameters
     };
     // To be able to do some cleaning up in destructor
     friend class ImportFrame;
 
-    /*
+    /**
      * Adds the given attribute set to the list of available named attribute
      * sets
      * @param aAttributeSet the Element to add as a named attribute set
@@ -112,17 +137,17 @@ public:
 
     /**
      * Registers the given ErrorObserver with this ProcessorState
-    **/
+     */
     void addErrorObserver(ErrorObserver& errorObserver);
 
     /**
      * Adds the given template to the list of templates to process
      * @param aXslTemplate  The Element to add as a template
      * @param aImportFrame  ImportFrame to add the template to
-    **/
+     */
     void addTemplate(Element* aXslTemplate, ImportFrame* aImportFrame);
 
-    /*
+    /**
      * Adds the given LRE Stylesheet to the list of templates to process
      * @param aStylesheet  The Stylesheet to add as a template
      * @param importFrame  ImportFrame to add the template to
@@ -132,31 +157,42 @@ public:
     /**
      * Returns the AttributeSet associated with the given name
      * or null if no AttributeSet is found
-    **/
-    NodeSet* getAttributeSet(const String& aName);
+     */
+    NodeSet* getAttributeSet(const txExpandedName& aName);
 
     /**
-     * Returns the source node currently being processed
-    **/
-    Node* getCurrentNode();
-
-    /*
      * Returns the template associated with the given name, or
      * null if not template is found
      */
-    Element* getNamedTemplate(String& aName);
+    Element* getNamedTemplate(const txExpandedName& aName);
 
     /**
      * Returns the OutputFormat which contains information on how
-     * to serialize the output. I will be removing this soon, when
-     * change to an event based printer, so that I can serialize
-     * as I go
-    **/
+     * to serialize the output.
+     */
     txOutputFormat* getOutputFormat();
 
+    /**
+     * Add a global variable
+     */
+    nsresult addGlobalVariable(const txExpandedName& aVarName,
+                               Element* aVarElem,
+                               ImportFrame* aImportFrame,
+                               ExprResult* aDefaultValue);
 
-    Stack* getVariableSetStack();
+    /**
+     * Returns map on top of the stack of local variable-bindings
+     */
+    txVariableMap* getLocalVariables();
+    
+    /**
+     * Sets top map of the local variable-bindings stack
+     */
+    void setLocalVariables(txVariableMap* aMap);
 
+    /**
+     * Enums for the getExpr and getPattern functions
+     */
     enum ExprAttr {
         SelectAttr = 0,
         TestAttr,
@@ -171,11 +207,6 @@ public:
     Expr* getExpr(Element* aElem, ExprAttr aAttr);
     txPattern* getPattern(Element* aElem, PatternAttr aAttr);
 
-    /**
-     * Returns a pointer to the result document
-    **/
-    Document* getResultDocument();
-
     /**
      * Retrieve the document designated by the URI uri, using baseUri as base URI.
      * Parses it as an XML document, and returns it. If a fragment identifier is
@@ -186,20 +217,23 @@ public:
      * @param baseUri the base URI used to resolve the URI if uri is relative
      * @return loaded document or element pointed to by fragment identifier. If
      *         loading or parsing fails NULL will be returned.
-    **/
-    Node* retrieveDocument(const String& uri, const String& baseUri);
+     */
+    Node* retrieveDocument(const nsAString& uri, const nsAString& baseUri);
 
-    /*
+    /**
      * Return stack of urls of currently entered stylesheets
      */
-    Stack* getEnteredStylesheets();
+    nsStringArray& getEnteredStylesheets()
+    {
+        return mEnteredStylesheets;
+    }
 
     /**
      * Return list of import containers
-    **/
+     */
     List* getImportFrames();
 
-    /*
+    /**
      * Find template in specified mode matching the supplied node
      * @param aNode        node to find matching template for
      * @param aMode        mode of the template
@@ -208,10 +242,13 @@ public:
      * @return             root-node of found template, null if none is found
      */
     Node* findTemplate(Node* aNode,
-                       const String& aMode,
-                       ImportFrame** aImportFrame);
+                       const txExpandedName& aMode,
+                       ImportFrame** aImportFrame)
+    {
+        return findTemplate(aNode, aMode, 0, aImportFrame);
+    }
 
-    /*
+    /**
      * Find template in specified mode matching the supplied node. Only search
      * templates imported by a specific ImportFrame
      * @param aNode        node to find matching template for
@@ -223,42 +260,37 @@ public:
      * @return             root-node of found template, null if none is found
      */
     Node* findTemplate(Node* aNode,
-                       const String& aMode,
+                       const txExpandedName& aMode,
                        ImportFrame* aImportedBy,
                        ImportFrame** aImportFrame);
 
-    /*
+    /**
      * Struct holding information about a current template rule
      */
     struct TemplateRule {
         ImportFrame* mFrame;
-        const String* mMode;
-        NamedMap* mParams;
+        const txExpandedName* mMode;
+        txVariableMap* mParams;
     };
 
-    /*
+    /**
      * Gets current template rule
      */
     TemplateRule* getCurrentTemplateRule();
 
-    /*
+    /**
      * Sets current template rule
      */
     void setCurrentTemplateRule(TemplateRule* aTemplateRule);
 
-    /**
-     * Determines if the given XSL node allows Whitespace stripping
-    **/
-    MBool isXSLStripSpaceAllowed(Node* node);
-
     /**
      * Adds the set of names to the Whitespace preserving element set
-    **/
-    void preserveSpace(String& names);
+     */
+    void preserveSpace(nsAString& names);
 
-    void processAttrValueTemplate(const String& aAttValue,
+    void processAttrValueTemplate(const nsAFlatString& aAttValue,
                                   Element* aContext,
-                                  String& aResult);
+                                  nsAString& aResult);
 
     /**
      * Adds the set of names to the Whitespace stripping handling list.
@@ -266,22 +298,22 @@ public:
      * with MB_FALSE
      * aElement is used to resolve QNames
      */
-    void shouldStripSpace(String& aNames, Element* aElement,
+    void shouldStripSpace(const nsAString& aNames, Element* aElement,
                           MBool aShouldStrip,
                           ImportFrame* aImportFrame);
 
     /**
      * Adds the supplied xsl:key to the set of keys
-    **/
+     */
     MBool addKey(Element* aKeyElem);
 
     /**
      * Returns the key with the supplied name
      * returns NULL if no such key exists
-    **/
-    txXSLKey* getKey(String& keyName);
+     */
+    txXSLKey* getKey(txExpandedName& keyName);
 
-    /*
+    /**
      * Adds a decimal format. Returns false if the format already exists
      * but dosn't contain the exact same parametervalues
      */
@@ -289,16 +321,31 @@ public:
 
     /**
      * Returns a decimal format or NULL if no such format exists.
-    **/
-    txDecimalFormat* getDecimalFormat(String& name);
+     */
+    txDecimalFormat* getDecimalFormat(const txExpandedName& name);
 
-    /*
+    /**
+     * Returns a pointer to a document that can be used to create RTFs
+     */
+    Document* getRTFDocument();
+
+    /**
+     * Sets a new document to be used for creating RTFs
+     */
+    void setRTFDocument(Document* aDoc);
+
+    /**
+     * Returns the stylesheet document
+     */
+    Document* getStylesheetDocument();
+
+    /**
      * Virtual methods from txIEvalContext
      */
 
     TX_DECL_MATCH_CONTEXT;
 
-    /*
+    /**
      * Set the current txIEvalContext and get the prior one
      */
     txIEvalContext* setEvalContext(txIEvalContext* aEContext)
@@ -308,7 +355,7 @@ public:
         return tmp;
     }
 
-    /*
+    /**
      * Get the current txIEvalContext
      */
     txIEvalContext* getEvalContext()
@@ -317,10 +364,10 @@ public:
     }
 
 
-    /*
+    /**
      * More other functions
      */
-    void receiveError(String& errorMessage)
+    void receiveError(const nsAString& errorMessage)
     {
         receiveError(errorMessage, NS_ERROR_FAILURE);
     }
@@ -328,13 +375,20 @@ public:
     /**
      * Returns a FunctionCall which has the given name.
      * @return the FunctionCall for the function with the given name.
-    **/
-    nsresult resolveFunctionCall(txAtom* aName, PRInt32 aID,
+     */
+    nsresult resolveFunctionCall(nsIAtom* aName, PRInt32 aID,
                                  Element* aElem, FunctionCall*& aFunction);
 
-private:
+#ifdef TX_EXE
+    txIOutputXMLEventHandler* mOutputHandler;
+#else
+    nsCOMPtr<txIOutputXMLEventHandler> mOutputHandler;
+#endif
+    txXMLEventHandler* mResultHandler;
+    
+    txIOutputHandlerFactory* mOutputHandlerFactory;
 
-    enum XMLSpaceMode {STRIP = 0, DEFAULT, PRESERVE};
+private:
 
     class MatchableTemplate {
     public:
@@ -347,95 +401,116 @@ private:
         txPattern* mMatch;
         double mPriority;
     };
+    
+    class GlobalVariableValue : public TxObject {
+    public:
+        GlobalVariableValue(ExprResult* aValue = 0)
+            : mValue(aValue), mFlags(nonOwned)
+        {
+        }
+        virtual ~GlobalVariableValue();
+      
+        ExprResult* mValue;
+        char mFlags;
+        enum _flags 
+        {
+            nonOwned = 0,
+            evaluating,
+            owned
+        };
+    };
 
     /**
      * The list of ErrorObservers registered with this ProcessorState
-    **/
+     */
     List errorObservers;
 
     /**
      * Stack of URIs for currently entered stylesheets
-    **/
-    Stack enteredStylesheets;
+     */
+    nsStringArray mEnteredStylesheets;
 
     /**
      * List of import containers. Sorted by ascending import precedence
-    **/
+     */
     txList mImportFrames;
 
     /**
      * The output format used when serializing the result
-    **/
+     */
     txOutputFormat mOutputFormat;
 
     /**
      * The set of loaded documents. This includes both document() loaded
      * documents and xsl:include/xsl:import'ed documents.
-    **/
-    NamedMap loadedDocuments;
+     */
+    txLoadedDocumentsHash mLoadedDocuments;
     
     /**
      * The set of all available keys
-    **/
-    NamedMap xslKeys;
-
-    /*
-     * A list of all avalible decimalformats
      */
-    NamedMap decimalFormats;
+    txExpandedNameMap mXslKeys;
+
+    /**
+     * The set of all avalible decimalformats
+     */
+    txExpandedNameMap mDecimalFormats;
     
-    /*
-     * bool indicating if the default decimal format has been explicitly set
-     * by the stylesheet
+    /**
+     * Default decimal-format
      */
-    MBool          defaultDecimalFormatSet;
+    txDecimalFormat mDefaultDecimalFormat;
 
-    /*
+    /**
      * List of hashes with parsed expression. Every listitem holds the
      * expressions for an attribute name
      */
     Map            mExprHashes[3];
 
-    /*
+    /**
      * List of hashes with parsed patterns. Every listitem holds the
      * patterns for an attribute name
      */
     Map            mPatternHashes[2];
 
-    /*
+    /**
      * Current txIEvalContext*
      */
     txIEvalContext* mEvalContext;
 
-    /*
+    /**
      * Current template rule
      */
     TemplateRule*  mCurrentTemplateRule;
 
-    Document*      mSourceDocument;
-    Document*      xslDocument;
-    Document*      resultDocument;
-    Stack          variableSets;
+    /**
+     * Top of stack of local variable-bindings
+     */
+    txVariableMap* mLocalVariables;
+    
+    /**
+     * Map of values of global variables
+     */
+    txExpandedNameMap mGlobalVariableValues;
 
     /**
-     * Returns the closest xml:space value for the given node
-    **/
-    XMLSpaceMode getXMLSpaceMode(Node* aNode);
-
+     * Document used to create RTFs
+     */
+    Document* mRTFDocument;
+    
     /**
-     * Initializes the ProcessorState
-    **/
-    void initialize();
-
-}; //-- ProcessorState
+     * Source-node where processing started
+     */
+    Node* mSourceNode;
+};
 
 /**
  * txNameTestItem holds both an ElementExpr and a bool for use in
  * whitespace stripping.
-**/
+ */
 class txNameTestItem {
 public:
-    txNameTestItem(txAtom* aPrefix, txAtom* aLocalName, PRUint32 aNSID,
+    txNameTestItem(nsIAtom* aPrefix, nsIAtom* aLocalName, PRInt32 aNSID,
                    MBool stripSpace)
         : mNameTest(aPrefix, aLocalName, aNSID, Node::ELEMENT_NODE),
           mStrips(stripSpace)
@@ -459,7 +534,7 @@ protected:
     MBool mStrips;
 };
 
-/*
+/**
  * txPSParseContext
  * a txIParseContext forwarding all but resolveNamespacePrefix
  * to a ProcessorState
@@ -475,10 +550,10 @@ public:
     {
     }
 
-    nsresult resolveNamespacePrefix(txAtom* aPrefix, PRInt32& aID);
-    nsresult resolveFunctionCall(txAtom* aName, PRInt32 aID,
+    nsresult resolveNamespacePrefix(nsIAtom* aPrefix, PRInt32& aID);
+    nsresult resolveFunctionCall(nsIAtom* aName, PRInt32 aID,
                                  FunctionCall*& aFunction);
-    void receiveError(const String& aMsg, nsresult aRes);
+    void receiveError(const nsAString& aMsg, nsresult aRes);
 
 protected:
     ProcessorState* mPS;
