@@ -59,7 +59,12 @@
 #include "nsNetUtil.h"
 #include "nsUnicharUtils.h"
 #include "txAtoms.h"
+#include "TxLog.h"
+#include "nsIConsoleService.h"
 #include "nsIDOMDocumentFragment.h"
+#include "nsINameSpaceManager.h"
+
+extern nsINameSpaceManager* gTxNameSpaceManager;
 
 static NS_DEFINE_CID(kXMLDocumentCID, NS_XMLDOCUMENT_CID);
 static NS_DEFINE_CID(kHTMLDocumentCID, NS_HTMLDOCUMENT_CID);
@@ -77,7 +82,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(const String& aRootName,
                                        nsIDOMDocument* aSourceDocument,
                                        nsIDOMDocument* aResultDocument,
                                        nsITransformObserver* aObserver)
-    : mStyleSheetCount(0),
+    : mBadChildLevel(0),
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
@@ -95,7 +100,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(const String& aRootName,
 
 txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
                                        nsIDOMDocumentFragment* aFragment)
-    : mStyleSheetCount(0),
+    : mBadChildLevel(0),
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
@@ -107,10 +112,6 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
     mOutputFormat.setFromDefaults();
 
     aFragment->GetOwnerDocument(getter_AddRefs(mDocument));
-    nsCOMPtr<nsIDocument> doc = do_QueryInterface(mDocument);
-    NS_ASSERTION(doc, "can't QI to nsIDocument");
-    doc->GetNameSpaceManager(*getter_AddRefs(mNameSpaceManager));
-    NS_ASSERTION(mNameSpaceManager, "Can't get namespace manager.");
 
     mCurrentNode = aFragment;
 }
@@ -131,6 +132,10 @@ void txMozillaXMLOutput::attribute(const String& aName,
         // XXX Signal this? (can't add attributes after element closed)
         return;
 
+    if (mBadChildLevel) {
+        return;
+    }
+
     nsCOMPtr<nsIDOMElement> element = do_QueryInterface(mCurrentNode);
     NS_ASSERTION(element, "No element to add the attribute to.");
     if (!element)
@@ -146,7 +151,7 @@ void txMozillaXMLOutput::attribute(const String& aName,
     }
     else {
         nsAutoString nsURI;
-        mNameSpaceManager->GetNameSpaceURI(aNsID, nsURI);
+        gTxNameSpaceManager->GetNameSpaceURI(aNsID, nsURI);
         element->SetAttributeNS(nsURI, aName, aValue);
     }
 }
@@ -155,6 +160,10 @@ void txMozillaXMLOutput::characters(const String& aData)
 {
     closePrevious(eCloseElement);
 
+    if (mBadChildLevel) {
+        return;
+    }
+
     mText.Append(aData);
 }
 
@@ -162,6 +171,10 @@ void txMozillaXMLOutput::comment(const String& aData)
 {
     closePrevious(eCloseElement | eFlushText);
 
+    if (mBadChildLevel) {
+        return;
+    }
+
     TX_ENSURE_CURRENTNODE;
 
     nsCOMPtr<nsIDOMComment> comment;
@@ -207,6 +220,13 @@ void txMozillaXMLOutput::endDocument()
 void txMozillaXMLOutput::endElement(const String& aName, const PRInt32 aNsID)
 {
     TX_ENSURE_CURRENTNODE;
+
+    if (mBadChildLevel) {
+        --mBadChildLevel;
+        PR_LOG(txLog::xslt, PR_LOG_DEBUG,
+               ("endElement, mBadChildLevel = %d\n", mBadChildLevel));
+        return;
+    }
     
 #ifdef DEBUG
     nsAutoString nodeName;
@@ -292,10 +312,7 @@ void txMozillaXMLOutput::processingInstruction(const String& aTarget, const Stri
 
     if (ssle) {
         ssle->SetEnableUpdates(PR_TRUE);
-        rv = ssle->UpdateStyleSheet(nsnull, mStyleSheetCount);
-        if (NS_SUCCEEDED(rv) || (rv == NS_ERROR_HTMLPARSER_BLOCK)) {
-            mStyleSheetCount++;
-        }
+        ssle->UpdateStyleSheet(nsnull);
     }
 }
 
@@ -318,8 +335,23 @@ void txMozillaXMLOutput::startElement(const String& aName,
 {
     TX_ENSURE_CURRENTNODE;
 
+    if (mBadChildLevel) {
+        ++mBadChildLevel;
+        PR_LOG(txLog::xslt, PR_LOG_DEBUG,
+               ("startElement, mBadChildLevel = %d\n", mBadChildLevel));
+        return;
+    }
+
     closePrevious(eCloseElement | eFlushText);
 
+    if (mBadChildLevel) {
+        // eCloseElement couldn't add the parent, we fail as well
+        ++mBadChildLevel;
+        PR_LOG(txLog::xslt, PR_LOG_DEBUG,
+               ("startElement, mBadChildLevel = %d\n", mBadChildLevel));
+        return;
+    }
+
     nsresult rv;
 
     nsCOMPtr<nsIDOMElement> element;
@@ -336,7 +368,7 @@ void txMozillaXMLOutput::startElement(const String& aName,
     }
     else {
         nsAutoString nsURI;
-        mNameSpaceManager->GetNameSpaceURI(aNsID, nsURI);
+        gTxNameSpaceManager->GetNameSpaceURI(aNsID, nsURI);
         rv = mDocument->CreateElementNS(nsURI, aName,
                                         getter_AddRefs(element));
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create element");
@@ -411,7 +443,20 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
                 nsCOMPtr<nsIDOMNode> resultNode;
 
                 rv = mParentNode->AppendChild(mCurrentNode, getter_AddRefs(resultNode));
-                NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append node");
+                if (NS_FAILED(rv)) {
+                    mBadChildLevel = 1;
+                    mCurrentNode = mParentNode;
+                    PR_LOG(txLog::xslt, PR_LOG_DEBUG,
+                           ("closePrevious, mBadChildLevel = %d\n",
+                            mBadChildLevel));
+                    // warning to the console
+                    nsCOMPtr<nsIConsoleService> consoleSvc = 
+                        do_GetService("@mozilla.org/consoleservice;1", &rv);
+                    if (consoleSvc) {
+                        consoleSvc->LogStringMessage(
+                            NS_LITERAL_STRING("failed to create XSLT content").get());
+                    }
+                }
             }
         }
         mParentNode = nsnull;
@@ -565,9 +610,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             do_QueryInterface(aElement);
         if (ssle) {
             ssle->SetEnableUpdates(PR_TRUE);
-            rv = ssle->UpdateStyleSheet(nsnull, mStyleSheetCount);
-            if (NS_SUCCEEDED(rv) || (rv == NS_ERROR_HTMLPARSER_BLOCK))
-              mStyleSheetCount++;
+            ssle->UpdateStyleSheet(nsnull);
         }
     }
 }
@@ -628,8 +671,6 @@ txMozillaXMLOutput::createResultDocument(const String& aName, PRInt32 aNsID,
         doc = do_QueryInterface(aResultDocument);
     }
     mCurrentNode = mDocument;
-    doc->GetNameSpaceManager(*getter_AddRefs(mNameSpaceManager));
-    NS_ASSERTION(mNameSpaceManager, "Can't get namespace manager.");
 
     // Reset and set up the document
     nsCOMPtr<nsILoadGroup> loadGroup;
