@@ -49,6 +49,11 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 
+#include "nsIJSContextStack.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsICodebasePrincipal.h"
+#include "nsIURI.h"
+
 #include "nsLayoutCID.h" // XXX Need range CID
 static NS_DEFINE_CID(kRangeCID,NS_RANGE_CID);
 
@@ -128,16 +133,79 @@ static nsresult SetUpEncoder(nsIDOMNode *aRoot, const char* aCharset, nsIDocumen
   return rv;
 }
 
-NS_IMETHODIMP
-nsDOMSerializer::SerializeToString(nsIDOMNode *root, PRUnichar **_retval)
+nsresult CheckSameOrigin(nsIDOMNode *aRoot)
 {
-  NS_ENSURE_ARG_POINTER(root);
+  // Get JSContext from stack.
+  nsCOMPtr<nsIJSContextStack> stack =
+    do_GetService("@mozilla.org/js/xpc/ContextStack;1");
+
+  JSContext *cx = nsnull;
+  nsresult rv = NS_OK;
+
+  if (stack) {
+    rv = stack->Peek(&cx);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  if (cx) {
+    // We're called from script, make sure the caller and the root are
+    // from the same origin...
+
+    nsCOMPtr<nsIDOMDocument> owner_doc(do_QueryInterface(aRoot));
+
+    if (!owner_doc) {
+      aRoot->GetOwnerDocument(getter_AddRefs(owner_doc));
+    }
+
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(owner_doc));
+
+    if (doc) {
+      nsCOMPtr<nsIPrincipal> principal;
+      nsCOMPtr<nsIURI> root_uri;
+
+      doc->GetPrincipal(getter_AddRefs(principal));
+
+      nsCOMPtr<nsICodebasePrincipal> codebase_principal =
+        do_QueryInterface(principal);
+
+      if (codebase_principal) {
+        codebase_principal->GetURI(getter_AddRefs(root_uri));
+      }
+
+      if (root_uri) {
+        nsCOMPtr<nsIScriptSecurityManager> secMan = 
+          do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = secMan->CheckSameOrigin(cx, root_uri);
+
+        if (NS_FAILED(rv)) {
+          // The node that's being serialized comes from a different
+          // origin than the calling script comes from...
+
+          return rv;
+        }
+      }
+    }      
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMSerializer::SerializeToString(nsIDOMNode *aRoot, PRUnichar **_retval)
+{
+  NS_ENSURE_ARG_POINTER(aRoot);
   NS_ENSURE_ARG_POINTER(_retval);  
   
   *_retval = nsnull;
 
+  nsresult rv = CheckSameOrigin(aRoot);
+  if (NS_FAILED(rv))
+    return rv;
+
   nsCOMPtr<nsIDocumentEncoder> encoder;
-  nsresult rv = SetUpEncoder(root,nsnull,getter_AddRefs(encoder));
+  rv = SetUpEncoder(aRoot, nsnull, getter_AddRefs(encoder));
   if (NS_FAILED(rv))
     return rv;
 
@@ -154,19 +222,23 @@ nsDOMSerializer::SerializeToString(nsIDOMNode *root, PRUnichar **_retval)
 }
 
 NS_IMETHODIMP
-nsDOMSerializer::SerializeToStream(nsIDOMNode *root, 
-                                   nsIOutputStream *stream, 
-                                   const char *charset)
+nsDOMSerializer::SerializeToStream(nsIDOMNode *aRoot, 
+                                   nsIOutputStream *aStream, 
+                                   const char *aCharset)
 {
-  NS_ENSURE_ARG_POINTER(root);
-  NS_ENSURE_ARG_POINTER(stream);
+  NS_ENSURE_ARG_POINTER(aRoot);
+  NS_ENSURE_ARG_POINTER(aStream);
   // The charset arg can be null, in which case we get the document's
   // charset and use that when serializing.
-  
-  nsCOMPtr<nsIDocumentEncoder> encoder;
-  nsresult rv = SetUpEncoder(root,charset,getter_AddRefs(encoder));
+
+  nsresult rv = CheckSameOrigin(aRoot);
   if (NS_FAILED(rv))
     return rv;
 
-  return encoder->EncodeToStream(stream);
+  nsCOMPtr<nsIDocumentEncoder> encoder;
+  rv = SetUpEncoder(aRoot, aCharset, getter_AddRefs(encoder));
+  if (NS_FAILED(rv))
+    return rv;
+
+  return encoder->EncodeToStream(aStream);
 }
