@@ -46,7 +46,6 @@
 #include "nsCRT.h"
 
 #include "nsDeviceContextGTK.h"
-#include "nsFontMetricsGTK.h"
 #include "nsGfxCIID.h"
 
 #ifdef USE_POSTSCRIPT
@@ -58,6 +57,8 @@
 #include "nsIDeviceContextXPrint.h"
 #endif /* USE_XPRINT */
 
+#include "nsFontMetricsUtils.h"
+
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
@@ -65,14 +66,24 @@
 #include "gdksuperwin.h"
 #endif /* MOZ_WIDGET_GTK */
 
+#ifdef MOZ_WIDGET_GTK2
+#include <pango/pango.h>
+#endif
+
+#ifdef MOZ_ENABLE_XFT
+#include "nsFontMetricsUtils.h"
+#include <X11/Xlib.h>
+#include <X11/Xdefs.h>
+#include <X11/Xft/Xft.h>
+
+static PRInt32 GetXftDPI(void);
+#endif
+
 #include <X11/Xatom.h>
 
 #include "nsDeviceContextSpecG.h"
 
-#define NS_TO_GDK_RGB(ns) (ns & 0xff) << 16 | (ns & 0xff00) | ((ns >> 16) & 0xff)
-
-#define GDK_COLOR_TO_NS_RGB(c) \
-  ((nscolor) NS_RGB(c.red, c.green, c.blue))
+static PRInt32 GetOSDPI(void);
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
@@ -99,7 +110,7 @@ class nsSystemFontsGTK {
     const nsFont& GetButtonFont() { return mButtonFont; }
 
   private:
-    nsresult GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont,
+    nsresult GetSystemFontInfo(GtkWidget *aWidget, nsFont* aFont,
                                float aPixelsToTwips) const;
 
     /*
@@ -353,6 +364,18 @@ NS_IMETHODIMP nsDeviceContextGTK::CreateRenderingContext(nsIRenderingContext *&a
   return rv;
 }
 
+NS_IMETHODIMP nsDeviceContextGTK::CreateRenderingContextInstance(nsIRenderingContext *&aContext)
+{
+  nsCOMPtr<nsIRenderingContext> renderingContext = new nsRenderingContextGTK();
+  if (!renderingContext)
+    return NS_ERROR_OUT_OF_MEMORY;
+         
+  aContext = renderingContext;
+  NS_ADDREF(aContext);
+  
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDeviceContextGTK::SupportsNativeWidgets(PRBool &aSupportsWidgets)
 {
   //XXX it is very critical that this not lie!! MMP
@@ -433,7 +456,7 @@ NS_IMETHODIMP nsDeviceContextGTK::ConvertPixel(nscolor aColor,
 
 NS_IMETHODIMP nsDeviceContextGTK::CheckFontExistence(const nsString& aFontName)
 {
-  return nsFontMetricsGTK::FamilyExists(this, aFontName);
+  return NS_FontMetricsFamilyExists(this, aFontName);
 }
 
 NS_IMETHODIMP nsDeviceContextGTK::GetDeviceSurfaceDimensions(PRInt32 &aWidth, PRInt32 &aHeight)
@@ -596,9 +619,7 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDepth(PRUint32& aDepth)
 nsresult
 nsDeviceContextGTK::SetDPI(PRInt32 aPrefDPI)
 {
-  // Set OSVal to what the operating system thinks the logical resolution is.
-  float screenWidthIn = float(::gdk_screen_width_mm()) / 25.4f;
-  PRInt32 OSVal = NSToCoordRound(float(::gdk_screen_width()) / screenWidthIn);
+  PRInt32 OSVal = GetOSDPI();
 
   if (aPrefDPI > 0) {
     // If there's a valid pref value for the logical resolution,
@@ -681,8 +702,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(parent);
   gtk_widget_realize(label);
 
-  GtkStyle *style = gtk_widget_get_style(label);
-  GetSystemFontInfo(style, &mDefaultFont, aPixelsToTwips);
+  GetSystemFontInfo(label, &mDefaultFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -697,8 +717,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_set_rc_style(entry);
   gtk_widget_realize(entry);
 
-  style = gtk_widget_get_style(entry);
-  GetSystemFontInfo(style, &mFieldFont, aPixelsToTwips);
+  GetSystemFontInfo(entry, &mFieldFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -717,8 +736,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(menu);
   gtk_widget_realize(accel_label);
 
-  style = gtk_widget_get_style(accel_label);
-  GetSystemFontInfo(style, &mMenuFont, aPixelsToTwips);
+  GetSystemFontInfo(accel_label, &mMenuFont, aPixelsToTwips);
 
   gtk_widget_unref(menu);
 
@@ -738,8 +756,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(button);
   gtk_widget_realize(label);
 
-  style = gtk_widget_get_style(label);
-  GetSystemFontInfo(style, &mButtonFont, aPixelsToTwips);
+  GetSystemFontInfo(label, &mButtonFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -776,7 +793,9 @@ ListFontProps(XFontStruct *aFont, Display *aDisplay)
    if (pos < 0) \
      return ; \
   }  
- 
+
+#ifdef MOZ_WIDGET_GTK
+
 static void
 AppendFontFFREName(nsString& aString, const char* aXLFDName)
 {
@@ -811,14 +830,8 @@ AppendFontName(XFontStruct* aFontStruct, nsString& aString, Display *aDisplay)
 {
   unsigned long pr = 0;
   // we first append the FFRE name to reconstruct font more faithfully
-#ifdef MOZ_WIDGET_GTK
   unsigned long font_atom = gdk_atom_intern("FONT", FALSE);
-#endif
-#ifdef MOZ_WIDGET_GTK2
-  unsigned long font_atom = gdk_x11_get_xatom_by_name("FONT");
-#endif
-  ::XGetFontProperty(aFontStruct, font_atom, &pr);
-  if(pr) {
+  if (::XGetFontProperty(aFontStruct, font_atom, &pr) && pr) {
     char* xlfdName = ::XGetAtomName(aDisplay, pr);
     AppendFontFFREName(aString, xlfdName);
     ::XFree(xlfdName);
@@ -827,10 +840,9 @@ AppendFontName(XFontStruct* aFontStruct, nsString& aString, Display *aDisplay)
   aString.Append(PRUnichar(','));
 
   // next, we need to append family name to cover more encodings.
-  ::XGetFontProperty(aFontStruct, XA_FAMILY_NAME, &pr);
-  if (!pr)
-    ::XGetFontProperty(aFontStruct, XA_FULL_NAME, &pr);
-  if (pr) {
+  if ((::XGetFontProperty(aFontStruct, XA_FAMILY_NAME, &pr) ||
+       ::XGetFontProperty(aFontStruct, XA_FULL_NAME, &pr)) &&
+      pr) {
     char *fontName = ::XGetAtomName(aDisplay, pr);
     aString.AppendWithConversion(fontName);
     ::XFree(fontName);
@@ -847,8 +859,7 @@ GetFontWeight(XFontStruct* aFontStruct, Display *aDisplay)
   unsigned long pr = 0;
   Atom weightName = ::XInternAtom(aDisplay, "WEIGHT_NAME", True);
   if (weightName != None) {
-    ::XGetFontProperty(aFontStruct, weightName, &pr);
-    if (pr) {
+    if (::XGetFontProperty(aFontStruct, weightName, &pr) && pr) {
       char *weightString = ::XGetAtomName(aDisplay, pr);
       if (nsCRT::strcasecmp(weightString, "bold") == 0)
         weight = NS_FONT_WEIGHT_BOLD;
@@ -857,8 +868,7 @@ GetFontWeight(XFontStruct* aFontStruct, Display *aDisplay)
   }
 
   pr = 0;
-  ::XGetFontProperty(aFontStruct, XA_WEIGHT, &pr);
-  if ( pr > 10 )
+  if (::XGetFontProperty(aFontStruct, XA_WEIGHT, &pr) && pr > 10 )
     weight = NS_FONT_WEIGHT_BOLD;
 
   return weight;
@@ -869,22 +879,18 @@ GetFontSize(XFontStruct *aFontStruct, float aPixelsToTwips)
 {
   unsigned long pr = 0;
   Atom pixelSizeAtom = ::XInternAtom(GDK_DISPLAY(), "PIXEL_SIZE", 0);
-  ::XGetFontProperty(aFontStruct, pixelSizeAtom, &pr);
-  if (!pr)
+  if (!::XGetFontProperty(aFontStruct, pixelSizeAtom, &pr) || !pr)
     return DEFAULT_TWIP_FONT_SIZE;
   return NSIntPixelsToTwips(pr, aPixelsToTwips);
 }
 
 nsresult
-nsSystemFontsGTK::GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont, float aPixelsToTwips) const
+nsSystemFontsGTK::GetSystemFontInfo(GtkWidget *aWidget, nsFont* aFont,
+                                    float aPixelsToTwips) const
 {
-#ifdef MOZ_WIDGET_GTK
-  GdkFont *theFont = aStyle->font;
-#endif /* MOZ_WIDGET_GTK */
+  GtkStyle *style = gtk_widget_get_style(aWidget);
 
-#ifdef MOZ_WIDGET_GTK2
-  GdkFont *theFont = gtk_style_get_font(aStyle);
-#endif /* MOZ_WIDGET_GTK2 */
+  GdkFont *theFont = style->font;
 
   aFont->style       = NS_FONT_STYLE_NORMAL;
   aFont->weight      = NS_FONT_WEIGHT_NORMAL;
@@ -893,10 +899,8 @@ nsSystemFontsGTK::GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont, float aPixe
   // do we have the default_font defined by GTK/GDK then
   // we use it, if not then we load helvetica, if not then
   // we load fixed font else we error out.
-#ifdef MOZ_WIDGET_GTK
   if (!theFont)
     theFont = default_font; // GTK default font
-#endif
 
   if (!theFont)
     theFont = ::gdk_font_load( GDK_DEFAULT_FONT1 );
@@ -945,3 +949,113 @@ nsSystemFontsGTK::GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont, float aPixe
   }
   return NS_OK;
 }
+#endif /* MOZ_WIDGET_GTK */
+
+#ifdef MOZ_WIDGET_GTK2
+nsresult
+nsSystemFontsGTK::GetSystemFontInfo(GtkWidget *aWidget, nsFont* aFont,
+                                    float aPixelsToTwips) const
+{
+  GtkSettings *settings = gtk_widget_get_settings(aWidget);
+
+  aFont->style       = NS_FONT_STYLE_NORMAL;
+  aFont->decorations = NS_FONT_DECORATION_NONE;
+
+  gchar *fontname;
+  g_object_get(settings, "gtk-font-name", &fontname, NULL);
+
+  PangoFontDescription *desc;
+  desc = pango_font_description_from_string(fontname);
+
+  aFont->name.Truncate();
+  aFont->name.AppendWithConversion(pango_font_description_get_family(desc));
+  aFont->weight = pango_font_description_get_weight(desc);
+
+  gint size;
+  size = pango_font_description_get_size(desc);
+
+  aFont->size = NSIntPointsToTwips(size / PANGO_SCALE);
+
+  return NS_OK;
+}
+#endif /* MOZ_WIDGET_GTK2 */
+
+#ifdef MOZ_WIDGET_GTK
+/* static */
+PRInt32
+GetOSDPI(void)
+{
+
+#ifdef MOZ_ENABLE_XFT
+  // try to get it from xft
+  if (NS_IsXftEnabled()) {
+    PRInt32 xftdpi = GetXftDPI();
+    if (xftdpi)
+      return xftdpi;
+  }
+#endif /* MOZ_ENABLE_XFT */
+
+  // Set OSVal to what the operating system thinks the logical resolution is.
+  float screenWidthIn = float(::gdk_screen_width_mm()) / 25.4f;
+  return NSToCoordRound(float(::gdk_screen_width()) / screenWidthIn);
+}
+#endif /* MOZ_WIDGET_GTK */
+
+#ifdef MOZ_WIDGET_GTK2
+/* static */
+PRInt32
+GetOSDPI(void)
+{
+  GtkSettings *settings = gtk_settings_get_default();
+
+  // first try to get the gtk2 dpi
+  gint dpi = 0;
+
+  // See if there's a gtk-xft-dpi object on the settings object - note
+  // that we don't have to free the spec since it isn't addrefed
+  // before being returned.  It's just part of an internal object.
+  // The gtk-xft-dpi setting is included in rh8 and might be included
+  // in later versions of gtk, so we conditionally check for it.
+  GParamSpec *spec;
+  spec = g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(settings)),
+                                      "gtk-xft-dpi");
+  if (spec) {
+    g_object_get(G_OBJECT(settings),
+                 "gtk-xft-dpi", &dpi,
+                 NULL);
+  }
+
+  if (dpi > 0)
+    return NSToCoordRound(dpi / 1024.0);
+
+#ifdef MOZ_ENABLE_XFT
+  // try to get it from xft
+  PRInt32 xftdpi = GetXftDPI();
+
+  if (xftdpi)
+    return xftdpi;
+#endif /* MOZ_ENABLE_XFT */
+  
+  // fall back to the physical resolution
+  float screenWidthIn = float(::gdk_screen_width_mm()) / 25.4f;
+  return NSToCoordRound(float(::gdk_screen_width()) / screenWidthIn);
+}
+#endif /* MOZ_WIDGET_GTK2 */
+
+#ifdef MOZ_ENABLE_XFT
+/* static */
+PRInt32
+GetXftDPI(void)
+{
+  char *val = XGetDefault(GDK_DISPLAY(), "Xft", "dpi");
+  if (val) {
+    char *e;
+    double d = strtod(val, &e);
+
+    if (e != val)
+      return NSToCoordRound(d);
+  }
+
+  return 0;
+}
+#endif /* MOZ_ENABLE_XFT */
