@@ -44,7 +44,6 @@
 #include "nsGfxCIID.h"
 
 #include "nsBlender.h"
-#include "nsFontMetricsGTK.h"
 #include "nsRenderingContextGTK.h"
 #include "nsDeviceContextGTK.h"
 // aka    nsDeviceContextSpecGTK.h
@@ -62,11 +61,20 @@
 #ifdef NATIVE_THEME_SUPPORT
 #include "nsNativeThemeGTK.h"
 #endif
+#ifdef MOZ_ENABLE_XFT
+#include "nsFontMetricsXft.h"
+#endif
+#ifdef MOZ_ENABLE_COREXFONTS
+#include "nsFontMetricsGTK.h"
+#endif
+#include "nsFontMetricsUtils.h"
 #include "nsPrintSession.h"
+#include "gfxImageFrame.h"
+#include "x11shared/nsFT2FontCatalog.h"
+#include "nsFreeType.h"
 
 // objects that just require generic constructors
 
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsFontMetricsGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsDeviceContextGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsRenderingContextGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsImageGTK)
@@ -74,7 +82,6 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsBlender)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsRegionGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsDeviceContextSpecGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsDeviceContextSpecFactoryGTK)
-NS_GENERIC_FACTORY_CONSTRUCTOR(nsFontEnumeratorGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsFontList);
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsScreenManagerGtk)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsPrintOptionsGTK)
@@ -83,9 +90,86 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsPrinterEnumeratorGTK)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsNativeThemeGTK)
 #endif
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsPrintSession, Init)
+NS_GENERIC_FACTORY_CONSTRUCTOR(gfxImageFrame)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsFT2FontCatalog)
+#ifdef MOZ_ENABLE_FREETYPE2
+NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsFreeType2, Init)
+#endif
 
 // our custom constructors
 
+static nsresult
+nsFontMetricsConstructor(nsISupports *aOuter, REFNSIID aIID, void **aResult)
+{
+  nsIFontMetrics *result;
+
+  if (!aResult)
+    return NS_ERROR_NULL_POINTER;
+
+  *aResult = nsnull;
+
+  if (aOuter)
+    return NS_ERROR_NO_AGGREGATION;
+
+#ifdef MOZ_ENABLE_XFT
+  if (NS_IsXftEnabled()) {
+    result = new nsFontMetricsXft();
+    if (!result)
+      return NS_ERROR_OUT_OF_MEMORY;
+  } else {
+#endif
+#ifdef MOZ_ENABLE_COREXFONTS
+    result = new nsFontMetricsGTK();
+    if (!result)
+      return NS_ERROR_OUT_OF_MEMORY;
+#endif
+#ifdef MOZ_ENABLE_XFT
+  }
+#endif
+
+  NS_ADDREF(result);
+  nsresult rv = result->QueryInterface(aIID, aResult);
+  NS_RELEASE(result);
+
+  return rv;
+}
+
+static nsresult
+nsFontEnumeratorConstructor(nsISupports *aOuter, REFNSIID aIID, void **aResult)
+{
+  nsIFontEnumerator *result;
+
+  if (!aResult)
+    return NS_ERROR_NULL_POINTER;
+
+  *aResult = nsnull;
+
+  if (aOuter)
+    return NS_ERROR_NO_AGGREGATION;
+
+#ifdef MOZ_ENABLE_XFT
+  if (NS_IsXftEnabled()) {
+    result = new nsFontEnumeratorXft();
+    if (!result)
+      return NS_ERROR_OUT_OF_MEMORY;
+  } else {
+#endif
+#ifdef MOZ_ENABLE_COREXFONTS
+    result = new nsFontEnumeratorGTK();
+    if (!result)
+      return NS_ERROR_OUT_OF_MEMORY;
+#endif
+#ifdef MOZ_ENABLE_XFT
+  }
+#endif
+
+  NS_ADDREF(result);
+  nsresult rv = result->QueryInterface(aIID, aResult);
+  NS_RELEASE(result);
+
+  return rv;
+}
+
 static NS_IMETHODIMP nsScriptableRegionConstructor(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 {
   nsresult rv;
@@ -133,7 +217,7 @@ static const nsModuleComponentInfo components[] =
     NS_FONT_METRICS_CID,
     //    "@mozilla.org/gfx/font_metrics/gtk;1",
     "@mozilla.org/gfx/fontmetrics;1",
-    nsFontMetricsGTKConstructor },
+    nsFontMetricsConstructor },
   { "Gtk Device Context",
     NS_DEVICE_CONTEXT_CID,
     //    "@mozilla.org/gfx/device_context/gtk;1",
@@ -182,7 +266,7 @@ static const nsModuleComponentInfo components[] =
     NS_FONT_ENUMERATOR_CID,
     //    "@mozilla.org/gfx/font_enumerator/gtk;1",
     "@mozilla.org/gfx/fontenumerator;1",
-    nsFontEnumeratorGTKConstructor },
+    nsFontEnumeratorConstructor },
   { "Font List",  
     NS_FONTLIST_CID,
     //    "@mozilla.org/gfx/fontlist;1"
@@ -198,12 +282,26 @@ static const nsModuleComponentInfo components[] =
     //    "@mozilla.org/gfx/printer_enumerator/gtk;1",
     "@mozilla.org/gfx/printerenumerator;1",
     nsPrinterEnumeratorGTKConstructor },
+  { "windows image frame",
+    GFX_IMAGEFRAME_CID,
+    "@mozilla.org/gfx/image/frame;2",
+    gfxImageFrameConstructor, },
   { "Print Session",
     NS_PRINTSESSION_CID,
     "@mozilla.org/gfx/printsession;1",
-    nsPrintSessionConstructor }
+    nsPrintSessionConstructor },
+  { "TrueType Font Catalog Service",
+    NS_FONTCATALOGSERVICE_CID,
+    "@mozilla.org/gfx/xfontcatalogservice;1",
+    nsFT2FontCatalogConstructor },
+#ifdef MOZ_ENABLE_FREETYPE2
+  { "FreeType2 routines",
+    NS_FREETYPE2_CID,
+    NS_FREETYPE2_CONTRACTID,
+    nsFreeType2Constructor },
+#endif
 #ifdef NATIVE_THEME_SUPPORT
-  ,{ "Native Theme Renderer",
+   { "Native Theme Renderer",
     NS_THEMERENDERER_CID,
     "@mozilla.org/chrome/chrome-native-theme;1",
     nsNativeThemeGTKConstructor }
@@ -215,6 +313,7 @@ nsGfxGTKModuleDtor(nsIModule *self)
 {
   nsRenderingContextGTK::Shutdown();
   nsDeviceContextGTK::Shutdown();
+  nsImageGTK::Shutdown();
   nsGCCache::Shutdown();
 #ifdef MOZ_WIDGET_GTK
   nsRegionGTK::Shutdown();
