@@ -32,6 +32,8 @@
 #include "nsIServiceManager.h"
 #include "nsUnicharUtils.h"
 
+#include "nsOS2Uni.h"
+
 PRINTDLG nsDeviceContextSpecOS2::PrnDlg;
 
 //----------------------------------------------------------------------------------
@@ -74,7 +76,7 @@ int            GlobalPrinters::mGlobalNumPrinters = 0;
  */
 nsDeviceContextSpecOS2 :: nsDeviceContextSpecOS2()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mQueue = nsnull;
 }
 
@@ -379,7 +381,7 @@ nsresult nsDeviceContextSpecOS2::GetPRTQUEUE( PRTQUEUE *&p)
 //  Printer Enumerator
 nsPrinterEnumeratorOS2::nsPrinterEnumeratorOS2()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 NS_IMPL_ISUPPORTS1(nsPrinterEnumeratorOS2, nsIPrinterEnumerator)
@@ -463,6 +465,7 @@ NS_IMETHODIMP nsPrinterEnumeratorOS2::InitPrintSettingsFromPrinter(const PRUnich
 
   // Free them, we won't need them for a while
   GlobalPrinters::GetInstance()->FreeGlobalPrinters();
+  aPrintSettings->SetIsInitializedFromPrinter(PR_TRUE);
   return NS_OK;
 }
 
@@ -507,10 +510,14 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
   for (int i = 0; i < mGlobalNumPrinters; i++) {
     nsXPIDLCString printer;
     nsDeviceContextSpecOS2::PrnDlg.GetPrinter(i, getter_Copies(printer));
+    PRUnichar* printerName = new PRUnichar[strlen(printer)+1];
+    MultiByteToWideChar(0, printer, strlen(printer)+1, printerName, strlen(printer)+1);
+    nsAutoString nativePrinterName(printerName);
+    delete [] printerName;
     if ( defaultPrinter == i ) 
-       mGlobalPrinterList->InsertStringAt(NS_ConvertASCIItoUCS2(printer), 0);
+       mGlobalPrinterList->InsertStringAt(nativePrinterName, 0);
     else 
-       mGlobalPrinterList->AppendString(NS_ConvertASCIItoUCS2(printer));
+       mGlobalPrinterList->AppendString(nativePrinterName);
   } 
   return NS_OK;
 }
@@ -525,8 +532,12 @@ void GlobalPrinters::GetDefaultPrinterName(PRUnichar*& aDefaultPrinterName)
   int defaultPrinter = nsDeviceContextSpecOS2::PrnDlg.GetDefaultPrinter();
   nsXPIDLCString printer;
   nsDeviceContextSpecOS2::PrnDlg.GetPrinter(defaultPrinter, getter_Copies(printer));
+  PRUnichar* printerName = new PRUnichar[strlen(printer)+1];
+  MultiByteToWideChar(0, printer, strlen(printer)+1, printerName, strlen(printer)+1);
+  nsAutoString nativePrinterName(printerName);
+  delete [] printerName; 
 
-  aDefaultPrinterName = ToNewUnicode(NS_ConvertASCIItoUCS2(printer));
+  aDefaultPrinterName = ToNewUnicode(nativePrinterName);
 }
 
 void GlobalPrinters::FreeGlobalPrinters()
@@ -826,7 +837,7 @@ HDC PrnOpenDC( PRTQUEUE *pInfo, PSZ pszApplicationName, int copies, int destinat
       else
          dcType = OD_QUEUED;
    } else {
-      if (file && strlen(file) != 0) 
+      if (file && *file)
          pszLogAddress = (PSZ) file;
       else    
          pszLogAddress = "FILE";
