@@ -44,6 +44,8 @@
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 
+#include "nsOS2Uni.h"
+
 #ifdef MOZ_MATHML
   #include <math.h>
 #endif
@@ -68,7 +70,6 @@ static NS_DEFINE_CID(kLocaleServiceCID, NS_LOCALESERVICE_CID);
 nsVoidArray  *nsFontMetricsOS2::gGlobalFonts = nsnull;
 PRBool        nsFontMetricsOS2::gSubstituteVectorFonts = PR_TRUE;
 PLHashTable  *nsFontMetricsOS2::gFamilyNames = nsnull;
-long          nsFontMetricsOS2::gSystemRes = 0;
 int           nsFontMetricsOS2::gCachedIndex = 0;
 nsICollation *nsFontMetricsOS2::gCollation = nsnull;
 
@@ -198,7 +199,7 @@ nsFontOS2::GetWidth( HPS aPS, const PRUnichar* aString,
   PRInt32 srcLength = aLength, destLength = aLength;
   mConverter->Convert(aString, &srcLength, pstr, &destLength);
 #else
-  int convertedLength = WideCharToMultiByte( mFattrs.usCodePage, aString,
+  int convertedLength = WideCharToMultiByte( mConvertCodePage, aString,
                                              aLength, pstr, destLength );
 #endif
 
@@ -230,7 +231,7 @@ nsFontOS2::DrawString( HPS aPS, nsDrawingSurfaceOS2* aSurface,
   PRInt32 srcLength = aLength, destLength = aLength;
   mConverter->Convert(aString, &srcLength, pstr, &destLength);
 #else
-  int convertedLength = WideCharToMultiByte( mFattrs.usCodePage, aString,
+  int convertedLength = WideCharToMultiByte( mConvertCodePage, aString,
                                              aLength, pstr, destLength );
 #endif
 
@@ -387,12 +388,6 @@ InitGlobals(void)
 
   ulSystemCodePage = WinQueryCp(HMQ_CURRENT);
 
-   // find system screen resolution. used here and in RealizeFont
-  HPS ps = ::WinGetScreenPS(HWND_DESKTOP);
-  HDC hdc = GFX (::GpiQueryDevice (ps), HDC_ERROR);
-  GFX (::DevQueryCaps(hdc, CAPS_HORIZONTAL_FONT_RES, 1, &nsFontMetricsOS2::gSystemRes), FALSE);
-  ::WinReleasePS(ps);
-
   if( !nsFontMetricsOS2::gGlobalFonts )
   {
     nsresult res = nsFontMetricsOS2::InitializeGlobalFonts();
@@ -541,11 +536,12 @@ nsFontMetricsOS2::SetFontHandle( HPS aPS, nsFontOS2* aFont )
     {
       long lFonts = 0;
       FONTMETRICS* pMetrics = getMetrics( lFonts, fattrs->szFacename, aPS);
+      int browserRes = mDeviceContext->GetDPI();
 
       int curPoints = 0;
       for( int i = 0; i < lFonts; i++)
       {
-        if( pMetrics[i].sYDeviceRes == gSystemRes )
+        if( pMetrics[i].sYDeviceRes == browserRes )
         {
           if (pMetrics[i].sNominalPointSize / 10 == points)
           {
@@ -599,15 +595,24 @@ nsFontMetricsOS2::SetFontHandle( HPS aPS, nsFontOS2* aFont )
     {
       if (!strcmp(name.get(), gCharsetInfo[j].mLangGroup))
       {
-        fattrs->usCodePage = gCharsetInfo[j].mCodePage;
-        mCodePage = gCharsetInfo[j].mCodePage;
+        mConvertCodePage = gCharsetInfo[j].mCodePage;
         break;
       }
     }
   }
 
-   // set up the charbox;  set for image fonts also, in case we need to
-   //  substitute a vector font later on (for UTF-8, etc)
+  // Symbols fonts must be created with codepage 65400,
+  // so use 65400 for the fattrs codepage. We still do
+  // conversions with the charset codepage
+  if (strcmp(fattrs->szFacename, "Webdings") == 0) {
+    fattrs->usCodePage = 65400;
+  } else {
+    fattrs->usCodePage = mConvertCodePage;
+  }
+
+
+  // set up the charbox;  set for image fonts also, in case we need to
+  //  substitute a vector font later on (for UTF-8, etc)
   float app2dev, fHeight;
   mDeviceContext->GetAppUnitsToDevUnits( app2dev );
 
@@ -1110,8 +1115,8 @@ FontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
   nsFontMetricsOS2* metrics = (nsFontMetricsOS2*) aData;
   /* Hack for Truetype on OS/2 - if it's Arial and not 1252 or 0, just get another font */
   if (aFamily.Find("Arial", IGNORE_CASE) != -1) {
-     if (metrics->mCodePage != 1252) {
-        if ((metrics->mCodePage == 0) &&
+     if (metrics->mConvertCodePage != 1252) {
+        if ((metrics->mConvertCodePage == 0) &&
             (ulSystemCodePage != 850) &&
             (ulSystemCodePage != 437)) {
            return PR_TRUE; // don't stop
@@ -1287,6 +1292,8 @@ nsFontMetricsOS2::RealizeFont()
     return NS_ERROR_FAILURE;
   }
 
+  font->mConvertCodePage = mConvertCodePage;
+
    // 9) Record font handle & record various font metrics to cache
   mFontHandle = font;
   GFX (::GpiCreateLogFont (ps, 0, 1, &(font->mFattrs)), GPI_ERROR);
@@ -1523,6 +1530,7 @@ nsFontMetricsOS2::GetUnicodeFont( HPS aPS )
   if( fh )
   {
     fh->mFattrs.usCodePage = 1208;
+    fh->mConvertCodePage = 1208;
     return fh;
   }
     
@@ -1545,6 +1553,7 @@ nsFontMetricsOS2::GetUnicodeFont( HPS aPS )
   if (context.mFont) // a suitable font was found
   {
     context.mFont->mFattrs.usCodePage = 1208;
+    context.mFont->mConvertCodePage = 1208;
     return context.mFont;
   }
   
@@ -1568,7 +1577,7 @@ nsFontMetricsOS2::ResolveForwards(HPS                  aPS,
   fontSwitch.mFont = 0;
   nsFontOS2* fh = nsnull;
   
-  if( mCodePage == 1252 )
+  if( mConvertCodePage == 1252 )
   {
     while( running && firstChar < lastChar )
     {
@@ -1614,6 +1623,7 @@ nsFontMetricsOS2::ResolveForwards(HPS                  aPS,
         fh = new nsFontOS2();
         *fh = *mFontHandle;
         fh->mFattrs.usCodePage = 1252;
+        fh->mConvertCodePage = 1252;
         fontSwitch.mFont = fh;
         while( ++currChar < lastChar )
         {
@@ -1662,7 +1672,7 @@ nsFontMetricsOS2::ResolveBackwards(HPS                  aPS,
   fontSwitch.mFont = 0;
   nsFontOS2* fh = nsnull;
   
-  if( mCodePage == 1252 )
+  if( mConvertCodePage == 1252 )
   {
     while( running && firstChar > lastChar )
     {
@@ -1708,6 +1718,7 @@ nsFontMetricsOS2::ResolveBackwards(HPS                  aPS,
         fh = new nsFontOS2();
         *fh = *mFontHandle;
         fh->mFattrs.usCodePage = 1252;
+        fh->mConvertCodePage = 1252;
         fontSwitch.mFont = fh;
         while( --currChar > lastChar )
         {
@@ -1846,11 +1857,34 @@ nsFontMetricsOS2::InitializeGlobalFonts()
     if( pFontMetrics[i].usWeightClass > 5 )
       font->metrics.fsSelection |= FM_SEL_BOLD;
 
-     /* Set the key values for Collation sort */
-    char* fontptr;
-    if( font->metrics.fsType & FM_TYPE_DBCS )
+     // Problem:  OS/2 has many non-standard fonts that do not follow the
+     //           normal Family-name/Face-name conventions (i.e. 'foo',
+     //           'foo bold', 'foo italic', 'foo bold italic').  This is
+     //           especially true for DBCS fonts (i.e. the 'WarpSans' family
+     //           can contain the 'WarpSans', 'WarpSans Bold', and 'WarpSans
+     //           Combined' faces).
+     // Solution: Unfortunately, there is no perfect way to handle this.  After
+     //           many attempts, we will attempt to remedy the situation by
+     //           searching the Facename for certain indicators ('bold',
+     //           'italic', 'oblique', 'regular').  If the Facename contains
+     //           one of these indicators, then we will create the sort key
+     //           based on the Familyname.  Otherwise, use the Facename.
+    char* fontptr = nsnull;
+    if (PL_strcasestr(font->metrics.szFacename, "bold") != nsnull ||
+        PL_strcasestr(font->metrics.szFacename, "italic") != nsnull ||
+        PL_strcasestr(font->metrics.szFacename, "oblique") != nsnull ||
+        PL_strcasestr(font->metrics.szFacename, "regular") != nsnull ||
+        PL_strcasestr(font->metrics.szFacename, "-normal") != nsnull)
     {
+      fontptr = font->metrics.szFamilyname;
+    } else {
       fontptr = font->metrics.szFacename;
+    }
+    
+     // The fonts in gBadDBCSFontMapping do not display well in non-Chinese
+     //   systems.  Map them to a more intelligible name.
+    if (font->metrics.fsType & FM_TYPE_DBCS)
+    {
       if ((ulSystemCodePage != 1386) &&
           (ulSystemCodePage != 1381) &&
           (ulSystemCodePage != 950)) {
@@ -1863,41 +1897,33 @@ nsFontMetricsOS2::InitializeGlobalFonts()
            j++;
         }
       }
-        
-       /* Make vertical fonts (start with '@') come right after the
-          non-vertical font with the same name                      */
-      convertedLength = MultiByteToWideChar(0, fontptr, strlen(fontptr), str, FACESIZE);
-      if( fontptr[0] == '@' )
-      {
-        font->name.Assign( str + 1, convertedLength-1 );
-        font->name.Append( NS_LITERAL_STRING(" ") );
-      }
-      else
-        font->name.Assign( str, convertedLength);
     }
-    else
+
+     // For vertical DBCS fonts (those that start with '@'), we want them to
+     //   appear directly below the non-vertical font of the same family.
+    convertedLength = MultiByteToWideChar(0, fontptr, strlen(fontptr), str, FACESIZE);
+    if (fontptr[0] == '@')
     {
-      convertedLength = MultiByteToWideChar(0, font->metrics.szFamilyname, 
-                          strlen(font->metrics.szFamilyname), str, FACESIZE);
+      font->name.Assign( str + 1, convertedLength-1 );
+      font->name.Append( NS_LITERAL_STRING(" ") );
+    } else {
       font->name.Assign( str, convertedLength );
     }
-      
+
+     // Create collation sort key
     res = gCollation->GetSortKeyLen(kCollationCaseInSensitive, 
                                    font->name, &font->len);
 
-    if( NS_SUCCEEDED(res) )
+    if (NS_SUCCEEDED(res))
     {
       font->key = (PRUint8*) nsMemory::Alloc(font->len * sizeof(PRUint8));
       res = gCollation->CreateRawSortKey( kCollationCaseInSensitive, 
                                          font->name, font->key,
                                          &font->len );
 
-       /* reset DBCS name to actual value */
-      if( font->metrics.fsType & FM_TYPE_DBCS && fontptr[0] == '@' )
-      {
-        convertedLength = MultiByteToWideChar(0, fontptr, strlen(fontptr), str, FACESIZE);
-        font->name.Assign( str, convertedLength );
-      }
+       // reset DBCS name to actual value
+      if (fontptr[0] == '@')
+        font->name.Insert( NS_LITERAL_STRING("@"), 0 );
     }
     
     gGlobalFonts->AppendElement(font);
@@ -1905,8 +1931,8 @@ nsFontMetricsOS2::InitializeGlobalFonts()
 
   gGlobalFonts->Sort(CompareFontFamilyNames, gCollation);
 
-   /* Set nextFamily in the array, such that we can skip from family to
-    * family in order to speed up searches.                             */
+   // Set nextFamily in the array, such that we can skip from family to
+   //   family in order to speed up searches.
   int prevIndex = 0;
   nsGlobalFont* font = (nsGlobalFont*)gGlobalFonts->ElementAt(0);
   PRUint8* lastkey = font->key;
@@ -1958,6 +1984,12 @@ nsFontMetricsOS2::InitializeGlobalFonts()
     else
       printf( " :       " );
 
+    if( fm->fsType & FM_TYPE_DBCS ||
+        fm->fsType & FM_TYPE_MBCS )
+      printf( " : M/DBCS" );
+    else
+      printf( " :       " );
+
     if( font->nextFamily )
       printf( " : %d", font->nextFamily );
       
