@@ -67,13 +67,6 @@ public:
   NS_IMETHOD  GetScrollBarDimensions(float &aWidth, float &aHeight) const;
   NS_IMETHOD  GetSystemFont(nsSystemFontID anID, nsFont *aFont) const;
 
-  //get a low level drawing surface for rendering. the rendering context
-  //that is passed in is used to create the drawing surface if there isn't
-  //already one in the device context. the drawing surface is then cached
-  //in the device context for re-use.
-
-  NS_IMETHOD  GetDrawingSurface(nsIRenderingContext &aContext, nsDrawingSurface &aSurface);
-
   NS_IMETHOD  ConvertPixel(nscolor aColor, PRUint32 & aPixel);
   NS_IMETHOD  CheckFontExistence(const nsString& aFontName);
 
