@@ -39,7 +39,9 @@
 #include "nsPrintOptionsWin.h"
 #include "nsPrintSettingsWin.h"
 
-
+#include "nsGfxCIID.h"
+#include "nsIServiceManager.h"
+const char kPrinterEnumeratorContractID[] = "@mozilla.org/gfx/printerenumerator;1";
 
 /** ---------------------------------------------------
  *  See documentation in nsPrintOptionsWin.h
@@ -59,16 +61,12 @@ nsPrintOptionsWin::~nsPrintOptionsWin()
 }
 
 /* nsIPrintSettings CreatePrintSettings (); */
-NS_IMETHODIMP nsPrintOptionsWin::CreatePrintSettings(nsIPrintSettings **_retval)
+nsresult nsPrintOptionsWin::_CreatePrintSettings(nsIPrintSettings **_retval)
 {
   nsresult rv = NS_OK;
   nsPrintSettingsWin* printSettings = new nsPrintSettingsWin(); // does not initially ref count
   NS_ASSERTION(printSettings, "Can't be NULL!");
 
-  rv = printSettings->QueryInterface(NS_GET_IID(nsIPrintSettings), (void**)_retval); // ref counts
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  InitPrintSettingsFromPrefs(*_retval, PR_FALSE, 0); // ignore return value
-
-  return rv;
+  return printSettings->QueryInterface(NS_GET_IID(nsIPrintSettings), (void**)_retval); // ref counts
 }
+
