@@ -20,7 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * Contributor(s): rickg@netscape.com 
+ *          rickg@netscape.com 
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -42,7 +42,8 @@
      
           
 #include "nsDebug.h"      
-#include "COtherDTD.h" 
+#include "nsIAtom.h"
+#include "COtherDTD.h"
 #include "nsHTMLTokens.h"
 #include "nsCRT.h"     
 #include "nsParser.h"  
@@ -145,7 +146,7 @@ NS_IMPL_RELEASE(COtherDTD)
  *  @return   
  */ 
 COtherDTD::COtherDTD() : nsIDTD() {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
   mSink = 0; 
   mParser=0;        
   mLineNumber=1;  
@@ -442,37 +443,40 @@ nsresult COtherDTD::DidBuildModel(nsresult anErrorCode,PRBool aNotifySink,nsIPar
 
           PRInt32 theIndex=mBodyContext->GetCount()-1;
           eHTMLTags theChild = mBodyContext->TagAt(theIndex); 
-          while(theIndex>0) {             
-            eHTMLTags theParent= mBodyContext->TagAt(--theIndex);
-            CElement *theElement=gElementTable->mElements[theParent];
-            nsCParserNode *theNode=mBodyContext->PeekNode();
+          while (theIndex>0) {             
+            eHTMLTags theParent = mBodyContext->TagAt(--theIndex);
+            CElement *theElement = gElementTable->mElements[theParent];
+            nsCParserNode *theNode = mBodyContext->PeekNode();
             theElement->HandleEndToken(theNode,theChild,mBodyContext,mSink);
-            theChild=theParent;
+            theChild = theParent;
           }
 
-          nsEntryStack *theChildStyles=0;
-          nsCParserNode* theNode=(nsCParserNode*)mBodyContext->Pop(theChildStyles);   
-          if(theNode) {
-            mSink->CloseHTML(*theNode);
+          nsEntryStack*  theChildStyles = 0;
+          nsCParserNode* theNode = (nsCParserNode*)mBodyContext->Pop(theChildStyles);   
+          if (theNode) {
+            mSink->CloseHTML();
           }
-
+          NS_ASSERTION(!theChildStyles, "there should no residual style information in this dtd");
+          IF_DELETE(theChildStyles, mNodeAllocator);
         }       
         else {
           //If you're here, then an error occured, but we still have nodes on the stack.
           //At a minimum, we should grab the nodes and recycle them.
           //Just to be correct, we'll also recycle the nodes. 
   
-          while(mBodyContext->GetCount() > 0) {  
+          while (mBodyContext->GetCount() > 0) {  
  
-            nsEntryStack *theChildStyles=0;
-            nsCParserNode* theNode=(nsCParserNode*)mBodyContext->Pop(theChildStyles);
-            if(theNode) {
-              theNode->mUseCount=0;
-              if(theChildStyles) {
+            nsEntryStack *theChildStyles = 0;
+            nsCParserNode* theNode = (nsCParserNode*)mBodyContext->Pop(theChildStyles);
+            if (theNode) {
+              theNode->mUseCount = 0;
+              if (theChildStyles) {
                 delete theChildStyles;
               } 
               IF_FREE(theNode, mNodeAllocator);
             }
+            NS_ASSERTION(!theChildStyles, "there should no residual style information in this dtd");
+            IF_DELETE(theChildStyles, mNodeAllocator);
           }    
         }    
   
@@ -480,7 +484,7 @@ nsresult COtherDTD::DidBuildModel(nsresult anErrorCode,PRBool aNotifySink,nsIPar
     } //if aparser  
 
       //No matter what, you need to call did build model.
-    result=aSink->DidBuildModel(0); 
+    result = aSink->DidBuildModel(0); 
 
   } //if asink
   return result; 
@@ -580,7 +584,7 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
         if(theNextToken)  {
           eHTMLTokenTypes theType=eHTMLTokenTypes(theNextToken->GetTokenType());
           if(eToken_newline==theType){
-            mLineNumber++;
+            ++mLineNumber;
             mTokenizer->PopToken();  //skip 1st newline inside PRE and LISTING
           }//if
         }//if
@@ -598,7 +602,7 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
 
         if(theCount) {
           PRInt32 theIndex=0;
-          for(theIndex=0;theIndex<theCount;theIndex++){
+          for(theIndex=0;theIndex<theCount;++theIndex){
             const nsAString& theKey = aNode.GetKeyAt(theIndex);
             if(theKey.Equals(NS_LITERAL_STRING("ENTITY"), nsCaseInsensitiveStringComparator())) {
               const nsAString& theName=aNode.GetValueAt(theIndex);
@@ -661,7 +665,7 @@ nsresult COtherDTD::WillHandleStartTag(CToken* aToken,eHTMLTags aTag,nsIParserNo
 
     switch(aTag) {
       case eHTMLTag_newline:
-        mLineNumber++;
+        ++mLineNumber;
         break;
       default:
         break; 
@@ -720,7 +724,7 @@ nsresult COtherDTD::HandleStartToken(CToken* aToken) {
           case eHTMLTag_html:  
             if(!mBodyContext->HasOpenContainer(theChildTag)){
               mSink->OpenHTML(*theNode);
-              mBodyContext->Push(theNode,0);
+              mBodyContext->Push(theNode, 0, PR_FALSE);
             } 
             theTagWasHandled=PR_TRUE;   
             break;         
@@ -814,7 +818,7 @@ nsresult COtherDTD::CollectAttributes(nsIParserNode& aNode,eHTMLTags aTag,PRInt3
   if(aCount<=theAvailTokenCount) {
     //gElementTable->mElements[aTag]->GetSkipTarget();
     CToken* theToken=0; 
-    for(attr=0;attr<aCount;attr++){  
+    for(attr=0;attr<aCount;++attr){  
       theToken=mTokenizer->PopToken(); 
       if(theToken)  {
         // Sanitize the key for it might contain some non-alpha-non-digit characters
