@@ -99,8 +99,6 @@ Stopwatch vsTimer;
 
 static NS_DEFINE_IID(kClassIID,     NS_VIEWSOURCE_HTML_IID);
 
-static int gErrorThreshold = 10;
-
 // Define this to dump the viewsource stuff to a file
 //#define DUMP_TO_FILE
 #ifdef DUMP_TO_FILE
@@ -204,7 +202,6 @@ public:
     mEndNode(),
     mStartNode(),
     mTokenNode(),
-    mErrorNode(),
     mITextToken(),
     mErrorToken(NS_LITERAL_STRING("error")) {
   }
@@ -218,9 +215,8 @@ public:
   }
 
   nsCParserNode       mEndNode;
-  nsCParserNode       mStartNode;
-  nsCParserNode       mTokenNode;
-  nsCParserNode       mErrorNode;
+  nsCParserStartNode  mStartNode;
+  nsCParserStartNode  mTokenNode;
   CIndirectTextToken  mITextToken;
   CTextToken          mErrorToken;
 };
@@ -331,7 +327,7 @@ static const char* const kDumpFileAfterText[] = {
  *  @return  
  */
 CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mStartTag = VIEW_SOURCE_START_TAG;
   mEndTag = VIEW_SOURCE_END_TAG;
@@ -350,8 +346,13 @@ CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
   mWrapLongLines = PR_FALSE;
   nsCOMPtr<nsIPref> thePrefsService(do_GetService(NS_PREF_CONTRACTID));
   if (thePrefsService) {
-    thePrefsService->GetBoolPref("view_source.syntax_highlight", &mSyntaxHighlight);
-    thePrefsService->GetBoolPref("view_source.wrap_long_lines", &mWrapLongLines);
+    PRBool temp;
+    nsresult rv;
+    rv = thePrefsService->GetBoolPref("view_source.syntax_highlight", &temp);
+    mSyntaxHighlight = NS_SUCCEEDED(rv) ? temp : PR_TRUE;
+
+    rv = thePrefsService->GetBoolPref("view_source.wrap_long_lines", &temp);
+    mWrapLongLines = NS_SUCCEEDED(rv) ? temp : PR_FALSE;
   }
 
   mParser=0;
@@ -486,18 +487,17 @@ nsresult CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
 
 #ifdef DUMP_TO_FILE
     if (gDumpFile) {
-      nsCAutoString filename;
-      filename.AssignWithConversion(mFilename);
 
       fprintf(gDumpFile, "<html>\n");
       fprintf(gDumpFile, "<head>\n");
       fprintf(gDumpFile, "<title>");
       fprintf(gDumpFile, "Source of: ");
-      fprintf(gDumpFile, filename);
+      fputs(NS_ConvertUCS2toUTF8(mFilename).get(), gDumpFile);
       fprintf(gDumpFile, "</title>\n");
       fprintf(gDumpFile, "<link rel=\"stylesheet\" type=\"text/css\" href=\"resource:/res/viewsource.css\">\n");
+      fprintf(gDumpFile, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
       fprintf(gDumpFile, "</head>\n");
-      fprintf(gDumpFile, "<body>\n");
+      fprintf(gDumpFile, "<body id=\"viewsource\">\n");
       fprintf(gDumpFile, "<pre>\n");
     }
 #endif //DUMP_TO_FILE
@@ -548,15 +548,22 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
       nsCParserNode headNode(&headToken, 0/*stack token*/);
       mSink->OpenHead(headNode);
 
-      // Note that XUL with automatically add the prefix "Source of: "
-      mSink->SetTitle(mFilename);
+      // Note that XUL will automatically add the prefix "Source of: "
+      if (Substring(mFilename, 0, 5).Equals(NS_LITERAL_STRING("data:")) &&
+          mFilename.Length() > 50) {
+        nsAutoString dataFilename(Substring(mFilename, 0, 50));
+        dataFilename.Append(NS_LITERAL_STRING("..."));
+        mSink->SetTitle(dataFilename);
+      } else {
+        mSink->SetTitle(mFilename);
+      }
 
       if (theAllocator) {
         tag.Assign(NS_LITERAL_STRING("LINK"));
         CStartToken* theToken=NS_STATIC_CAST(CStartToken*,theAllocator->CreateTokenOfType(eToken_start,eHTMLTag_link,tag));
         if(theToken) {
           CAttributeToken *theAttr;
-          nsCParserNode theNode(theToken, theAllocator);
+          nsCParserStartNode theNode(theToken, theAllocator);
 
           theAttr=(CAttributeToken*)theAllocator->CreateTokenOfType(eToken_attribute,eHTMLTag_unknown,NS_LITERAL_STRING("stylesheet"));
           theAttr->SetKey(NS_LITERAL_STRING("rel"));
@@ -577,7 +584,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
 
       CEndToken endHeadToken(eHTMLTag_head);
       nsCParserNode endHeadNode(&endHeadToken, 0/*stack token*/);
-      result = mSink->CloseHead(endHeadNode);
+      result = mSink->CloseHead();
       if(NS_SUCCEEDED(result)) {
         mHasOpenRoot = PR_TRUE;
         if (didBlock) {
@@ -590,7 +597,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
         tag.Assign(NS_LITERAL_STRING("BODY"));
         CStartToken* bodyToken=NS_STATIC_CAST(CStartToken*,theAllocator->CreateTokenOfType(eToken_start, eHTMLTag_body, tag));
         if (bodyToken) {
-          nsCParserNode bodyNode(bodyToken, theAllocator);
+          nsCParserStartNode bodyNode(bodyToken, theAllocator);
           CAttributeToken *theAttr=nsnull;
           theAttr=(CAttributeToken*)theAllocator->CreateTokenOfType(eToken_attribute,eHTMLTag_unknown,NS_ConvertASCIItoUCS2(kBodyId));
           theAttr->SetKey(NS_LITERAL_STRING("id"));
@@ -670,7 +677,7 @@ nsresult  CViewSourceHTML::GenerateSummary() {
 void CViewSourceHTML::StartNewPreBlock(void){
   CEndToken endToken(eHTMLTag_pre);
   nsCParserNode endNode(&endToken, 0/*stack token*/);
-  mSink->CloseContainer(endNode);
+  mSink->CloseContainer(eHTMLTag_pre);
 
   CStartToken startToken(eHTMLTag_pre);
   nsCParserNode startNode(&startToken, 0/*stack token*/);
@@ -718,15 +725,15 @@ NS_IMETHODIMP CViewSourceHTML::DidBuildModel(nsresult anErrorCode,PRBool aNotify
       if(ePlainText!=mDocType) {
         CEndToken theToken(eHTMLTag_pre);
         nsCParserNode preNode(&theToken, 0/*stack token*/);
-        mSink->CloseContainer(preNode);
+        mSink->CloseContainer(eHTMLTag_pre);
         
         CEndToken bodyToken(eHTMLTag_body);
         nsCParserNode bodyNode(&bodyToken, 0/*stack token*/);
-        mSink->CloseBody(bodyNode);
+        mSink->CloseBody();
         
         CEndToken htmlToken(eHTMLTag_html);
         nsCParserNode htmlNode(&htmlToken, 0/*stack token*/);
-        mSink->CloseHTML(htmlNode);
+        mSink->CloseHTML();
       }
       result = mSink->DidBuildModel(1);
     }
@@ -895,7 +902,7 @@ nsresult CViewSourceHTML::WriteAttributes(PRInt32 attrCount) {
     CSharedVSContext& theContext=CSharedVSContext::GetSharedContext();
 
     int attr=0;
-    for(attr=0;attr<attrCount;attr++){
+    for(attr=0;attr<attrCount;++attr){
       CToken* theToken=mTokenizer->PeekToken();
       if(theToken)  {
         eHTMLTokenTypes theType=eHTMLTokenTypes(theToken->GetTokenType());
@@ -977,9 +984,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
   mSink->AddLeaf(theNode);
 #ifdef DUMP_TO_FILE
   if (gDumpFile) {
-    nsCAutoString cstr;
-    cstr.AssignWithConversion(aText);
-    fprintf(gDumpFile, cstr);
+    fputs(NS_ConvertUCS2toUTF8(aText).get(), gDumpFile);
   }
 #endif // DUMP_TO_FILE
 
@@ -987,7 +992,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
     theContext.mStartNode.ReleaseAll(); 
     CEndToken theEndToken(eHTMLTag_span);
     theContext.mEndNode.Init(&theEndToken, 0/*stack token*/);
-    mSink->CloseContainer(theContext.mEndNode);  //emit </starttag>...
+    mSink->CloseContainer(eHTMLTag_span);  //emit </starttag>...
 #ifdef DUMP_TO_FILE
     if (gDumpFile)
       fprintf(gDumpFile, "</span>");
@@ -1040,7 +1045,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
     
     case eToken_start:
       {
-        mTagCount++;
+        ++mTagCount;
 
         const nsAString& startValue = aToken->GetStringValue();
         result=WriteTag(mStartTag,startValue,aToken->GetAttributeCount(),PR_TRUE);
@@ -1100,9 +1105,9 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
     case eToken_newline:
       {
         const nsAString& newlineValue = aToken->GetStringValue();
-        mLineNumber++; 
+        ++mLineNumber; 
         result=WriteTag(mText,newlineValue,0,PR_FALSE);
-        mTokenCount++;
+        ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
             mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK)
           StartNewPreBlock();
@@ -1113,7 +1118,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
       {
         const nsAString& wsValue = aToken->GetStringValue();
         result=WriteTag(mText,wsValue,0,PR_FALSE);
-        mTokenCount++;
+        ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
             mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK &&
             !wsValue.IsEmpty()) {
@@ -1128,7 +1133,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
       {
         const nsAString& str = aToken->GetStringValue();         
         result=WriteTag(mText,str,aToken->GetAttributeCount(),PR_TRUE);
-        mTokenCount++;
+        ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
             mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK && !str.IsEmpty()) {
           PRUnichar ch = str.Last();
