@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -44,7 +44,7 @@
  */
 #include "jsstddef.h"
 #include <stdlib.h>     /* for free, called by JS_ARENA_DESTROY */
-#include <string.h>	/* for memset, called by jsarena.h macros if DEBUG */
+#include <string.h>     /* for memset, called by jsarena.h macros if DEBUG */
 #include "jstypes.h"
 #include "jsarena.h" /* Added by JSIFY */
 #include "jsutil.h" /* Added by JSIFY */
@@ -82,7 +82,7 @@
 #define GC_ARENA_SIZE   (GC_THINGS_SIZE + GC_FLAGS_SIZE)
 
 /*
- * The private JSGCThing struct, which describes a gcFreelist element.
+ * The private JSGCThing struct, which describes a gcFreeList element.
  */
 struct JSGCThing {
     JSGCThing   *next;
@@ -90,7 +90,7 @@ struct JSGCThing {
 };
 
 /*
- * A GC arena contains one flag byte for every thing in its heap, and supports
+ * A GC arena contains one flag byte for each thing in its heap, and supports
  * O(1) lookup of a flag given its thing's address.
  *
  * To implement this, we take advantage of the thing/flags numerology: given
@@ -145,7 +145,7 @@ struct JSGCThing {
  * no discontinuities over the split created by the thing pages.  So if, for a
  * given JSGCPageInfo *pi, we find that
  *
- *  pi->flags + ((jsuword)thing % 1023) / sizeof(JSGCThing) >= pi->split
+ *  pi->flags + ((jsuword)thing % 1024) / sizeof(JSGCThing) >= pi->split
  *
  * then we must add GC_THINGS_SIZE to the nominal flags pointer to jump over
  * all the thing pages that split the flags into two discontiguous spans.
@@ -263,18 +263,19 @@ js_InitGC(JSRuntime *rt, uint32 maxbytes)
     JS_ASSERT(sizeof(JSStackHeader) >= 2 * sizeof(jsval));
 
     if (!gc_finalizers[GCX_OBJECT]) {
-	gc_finalizers[GCX_OBJECT] = (GCFinalizeOp)js_FinalizeObject;
-	gc_finalizers[GCX_STRING] = (GCFinalizeOp)js_FinalizeString;
+        gc_finalizers[GCX_OBJECT] = (GCFinalizeOp)js_FinalizeObject;
+        gc_finalizers[GCX_STRING] = (GCFinalizeOp)js_FinalizeString;
 #ifdef DEBUG
-	gc_finalizers[GCX_DOUBLE] = (GCFinalizeOp)js_FinalizeDouble;
+        gc_finalizers[GCX_DOUBLE] = (GCFinalizeOp)js_FinalizeDouble;
 #endif
-	gc_finalizers[GCX_MUTABLE_STRING] = (GCFinalizeOp)js_FinalizeString;
+        gc_finalizers[GCX_MUTABLE_STRING] = (GCFinalizeOp)js_FinalizeString;
     }
 
     JS_InitArenaPool(&rt->gcArenaPool, "gc-arena", GC_ARENA_SIZE,
-		     sizeof(JSGCThing));
+                     sizeof(JSGCThing));
     if (!JS_DHashTableInit(&rt->gcRootsHash, JS_DHashGetStubOps(), NULL,
                            sizeof(JSGCRootHashEntry), GC_ROOTS_SIZE)) {
+        rt->gcRootsHash.ops = NULL;
         return JS_FALSE;
     }
     rt->gcLocksHash = NULL;     /* create lazily */
@@ -339,8 +340,8 @@ js_FinishGC(JSRuntime *rt)
     JS_FinishArenaPool(&rt->gcArenaPool);
     JS_ArenaFinish();
 
+    if (rt->gcRootsHash.ops) {
 #ifdef DEBUG
-    {
         uint32 leakedroots = 0;
 
         /* Warn (but don't assert) debug builds of any remaining roots. */
@@ -360,10 +361,11 @@ js_FinishGC(JSRuntime *rt)
                         (unsigned long) leakedroots);
             }
         }
-    }
 #endif
 
-    JS_DHashTableFinish(&rt->gcRootsHash);
+        JS_DHashTableFinish(&rt->gcRootsHash);
+        rt->gcRootsHash.ops = NULL;
+    }
     if (rt->gcLocksHash) {
         JS_DHashTableDestroy(rt->gcLocksHash);
         rt->gcLocksHash = NULL;
@@ -376,7 +378,7 @@ js_AddRoot(JSContext *cx, void *rp, const char *name)
 {
     JSBool ok = js_AddRootRT(cx->runtime, rp, name);
     if (!ok)
-	JS_ReportOutOfMemory(cx);
+        JS_ReportOutOfMemory(cx);
     return ok;
 }
 
@@ -470,10 +472,10 @@ js_AllocGCThing(JSContext *cx, uintN flags)
 retry:
     thing = rt->gcFreeList;
     if (thing) {
-	rt->gcFreeList = thing->next;
-	flagp = thing->flagp;
-	METER(rt->gcStats.freelen--);
-	METER(rt->gcStats.recycle++);
+        rt->gcFreeList = thing->next;
+        flagp = thing->flagp;
+        METER(rt->gcStats.freelen--);
+        METER(rt->gcStats.recycle++);
     } else {
         if (rt->gcBytes < rt->gcMaxBytes &&
             (tried_gc || rt->gcMallocBytes < rt->gcMaxBytes))
@@ -505,13 +507,21 @@ retry:
             JS_ArenaCountAllocation(pool, nb);
         }
 
-        /* Consider doing a "last ditch" GC if thing couldn't be allocated. */
+        /*
+         * Consider doing a "last ditch" GC if thing couldn't be allocated.
+         *
+         * Keep rt->gcLock across the call into js_GC so we don't starve and
+         * lose to racing threads who deplete the heap just after js_GC has
+         * replenished it (or has synchronized with a racing GC that collected
+         * a bunch of garbage).  This unfair scheduling can happen on certain
+         * operating systems.  For the gory details, see Mozilla bug 162779
+         * (http://bugzilla.mozilla.org/show_bug.cgi?id=162779).
+         */
         if (!thing) {
             if (!tried_gc) {
-                JS_UNLOCK_GC(rt);
-                js_GC(cx, GC_KEEP_ATOMS);
+                rt->gcPoke = JS_TRUE;
+                js_GC(cx, GC_KEEP_ATOMS | GC_ALREADY_LOCKED);
                 tried_gc = JS_TRUE;
-                JS_LOCK_GC(rt);
                 METER(rt->gcStats.retry++);
                 goto retry;
             }
@@ -541,18 +551,25 @@ retry:
 JSBool
 js_LockGCThing(JSContext *cx, void *thing)
 {
-    JSRuntime *rt;
+    JSBool ok = js_LockGCThingRT(cx->runtime, thing);
+    if (!ok)
+        JS_ReportOutOfMemory(cx);
+    return ok;
+}
+
+JSBool
+js_LockGCThingRT(JSRuntime *rt, void *thing)
+{
     uint8 *flagp, flags, lockbits;
     JSBool ok;
     JSGCLockHashEntry *lhe;
 
     if (!thing)
-	return JS_TRUE;
+        return JS_TRUE;
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
 
-    ok = JS_TRUE;
-    rt = cx->runtime;
+    ok = JS_FALSE;
     JS_LOCK_GC(rt);
     lockbits = (flags & GCF_LOCKMASK);
 
@@ -566,7 +583,7 @@ js_LockGCThing(JSContext *cx, void *thing)
                                          sizeof(JSGCLockHashEntry),
                                          GC_ROOTS_SIZE);
                     if (!rt->gcLocksHash)
-                        goto outofmem;
+                        goto error;
                 } else {
 #ifdef DEBUG
                     JSDHashEntryHdr *hdr = 
@@ -578,7 +595,7 @@ js_LockGCThing(JSContext *cx, void *thing)
                 lhe = (JSGCLockHashEntry *)
                     JS_DHashTableOperate(rt->gcLocksHash, thing, JS_DHASH_ADD);
                 if (!lhe)
-                    goto outofmem;
+                    goto error;
                 lhe->thing = thing;
                 lhe->count = 1;
                 *flagp = (uint8)(flags + GCF_LOCK);
@@ -597,33 +614,27 @@ js_LockGCThing(JSContext *cx, void *thing)
             *flagp = (uint8)(flags + GCF_LOCK);
         }
     } else {
-	METER(rt->gcStats.stuck++);
+        METER(rt->gcStats.stuck++);
     }
 
     METER(rt->gcStats.lock++);
-out:
+    ok = JS_TRUE;
+error:
     JS_UNLOCK_GC(rt);
     return ok;
-
-outofmem:
-    JS_ReportOutOfMemory(cx);
-    ok = JS_FALSE;
-    goto out;
 }
 
 JSBool
-js_UnlockGCThing(JSContext *cx, void *thing)
+js_UnlockGCThingRT(JSRuntime *rt, void *thing)
 {
-    JSRuntime *rt;
     uint8 *flagp, flags, lockbits;
     JSGCLockHashEntry *lhe;
 
     if (!thing)
-	return JS_TRUE;
+        return JS_TRUE;
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
 
-    rt = cx->runtime;
     JS_LOCK_GC(rt);
     lockbits = (flags & GCF_LOCKMASK);
 
@@ -647,7 +658,7 @@ js_UnlockGCThing(JSContext *cx, void *thing)
             *flagp = (uint8)(flags - GCF_LOCK);
         }
     } else {
-	METER(rt->gcStats.unstuck++);
+        METER(rt->gcStats.unstuck++);
     }
 
     rt->gcPoke = JS_TRUE;
@@ -674,24 +685,46 @@ gc_object_class_name(void* thing)
 {
     uint8 *flagp = js_GetGCThingFlags(thing);
     const char *className = "";
+    static char depbuf[32];
 
-    if (flagp && ((*flagp & GCF_TYPEMASK) == GCX_OBJECT)) {
+    switch (*flagp & GCF_TYPEMASK) {
+      case GCX_OBJECT: {
         JSObject  *obj = (JSObject *)thing;
         JSClass   *clasp = JSVAL_TO_PRIVATE(obj->slots[JSSLOT_CLASS]);
         className = clasp->name;
 #ifdef HAVE_XPCONNECT
-        if ((clasp->flags & JSCLASS_PRIVATE_IS_NSISUPPORTS) &&
-            (clasp->flags & JSCLASS_HAS_PRIVATE)) {
+        if (clasp->flags & JSCLASS_PRIVATE_IS_NSISUPPORTS) {
             jsval privateValue = obj->slots[JSSLOT_PRIVATE];
-            void  *privateThing = JSVAL_IS_VOID(privateValue)
-                                  ? NULL
-                                  : JSVAL_TO_PRIVATE(privateValue);
 
-            const char *xpcClassName = GetXPCObjectClassName(privateThing);
-            if (xpcClassName)
-                className = xpcClassName;
+            JS_ASSERT(clasp->flags & JSCLASS_HAS_PRIVATE);
+            if (!JSVAL_IS_VOID(privateValue)) {
+                void  *privateThing = JSVAL_TO_PRIVATE(privateValue);
+                const char *xpcClassName = GetXPCObjectClassName(privateThing);
+
+                if (xpcClassName)
+                    className = xpcClassName;
+            }
         }
 #endif
+        break;
+      }
+
+      case GCX_STRING:
+      case GCX_MUTABLE_STRING: {
+        JSString *str = (JSString *)thing;
+        if (JSSTRING_IS_DEPENDENT(str)) {
+            JS_snprintf(depbuf, sizeof depbuf, "start:%u, length:%u",
+                        JSSTRDEP_START(str), JSSTRDEP_LENGTH(str));
+            className = depbuf;
+        } else {
+            className = "string";
+        }
+        break;
+      }
+
+      case GCX_DOUBLE:
+        className = "double";
+        break;
     }
 
     return className;
@@ -756,21 +789,21 @@ js_MarkAtom(JSContext *cx, JSAtom *atom, void *arg)
     jsval key;
 
     if (atom->flags & ATOM_MARK)
-	return;
+        return;
     atom->flags |= ATOM_MARK;
     key = ATOM_KEY(atom);
     if (JSVAL_IS_GCTHING(key)) {
 #ifdef GC_MARK_DEBUG
-	char name[32];
+        char name[32];
 
-	if (JSVAL_IS_STRING(key)) {
-	    JS_snprintf(name, sizeof name, "'%s'",
-			JS_GetStringBytes(JSVAL_TO_STRING(key)));
-	} else {
-	    JS_snprintf(name, sizeof name, "<%x>", key);
-	}
+        if (JSVAL_IS_STRING(key)) {
+            JS_snprintf(name, sizeof name, "'%s'",
+                        JS_GetStringBytes(JSVAL_TO_STRING(key)));
+        } else {
+            JS_snprintf(name, sizeof name, "<%x>", key);
+        }
 #endif
-	GC_MARK(cx, JSVAL_TO_GCTHING(key), name, arg);
+        GC_MARK(cx, JSVAL_TO_GCTHING(key), name, arg);
     }
 }
 
@@ -789,7 +822,7 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 #endif
 
     if (!thing)
-	return;
+        return;
 
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
@@ -800,12 +833,12 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 #endif
 
     if (flags & GCF_MARK)
-	return;
+        return;
 
     *flagp |= GCF_MARK;
     rt = cx->runtime;
     METER(if (++rt->gcStats.depth > rt->gcStats.maxdepth)
-	      rt->gcStats.maxdepth = rt->gcStats.depth);
+              rt->gcStats.maxdepth = rt->gcStats.depth);
 
 #ifdef GC_MARK_DEBUG
     if (js_DumpGCHeap)
@@ -814,8 +847,8 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 
     switch (flags & GCF_TYPEMASK) {
       case GCX_OBJECT:
-	obj = (JSObject *) thing;
-	vp = obj->slots;
+        obj = (JSObject *) thing;
+        vp = obj->slots;
         if (!vp) {
             /* If obj->slots is null, obj must be a newborn. */
             JS_ASSERT(!obj->map);
@@ -838,7 +871,8 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
                     jsval nval;
 
                     slot = vp - obj->slots;
-                    for (sprop = scope->props; ; sprop = sprop->next) {
+                    for (sprop = SCOPE_LAST_PROP(scope); ;
+                         sprop = sprop->parent) {
                         if (!sprop) {
                             switch (slot) {
                               case JSSLOT_PROTO:
@@ -859,9 +893,7 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
                             break;
                         }
                         if (sprop->slot == slot) {
-                            nval = sprop->symbols
-                                   ? js_IdToValue(sym_id(sprop->symbols))
-                                   : sprop->id;
+                            nval = ID_TO_VALUE(sprop->id);
                             if (JSVAL_IS_INT(nval)) {
                                 JS_snprintf(name, sizeof name, "%ld",
                                             (long)JSVAL_TO_INT(nval));
@@ -893,7 +925,7 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
       case GCX_MUTABLE_STRING:
         str = (JSString *)thing;
         if (JSSTRING_IS_DEPENDENT(str))
-            js_MarkGCThing(cx, JSSTRDEP_BASE(str), arg);
+            GC_MARK(cx, JSSTRDEP_BASE(str), "base", arg);
         break;
     }
 
@@ -915,11 +947,11 @@ gc_root_marker(JSDHashTable *table, JSDHashEntryHdr *hdr, uint32 num, void *arg)
         JSArena *a;
         jsuword firstpage;
         JSBool root_points_to_gcArenaPool = JS_FALSE;
-	void *thing = JSVAL_TO_GCTHING(v);
+        void *thing = JSVAL_TO_GCTHING(v);
 
         for (a = cx->runtime->gcArenaPool.first.next; a; a = a->next) {
             firstpage = FIRST_THING_PAGE(a);
-	    if (JS_UPTRDIFF(thing, firstpage) < a->avail - firstpage) {
+            if (JS_UPTRDIFF(thing, firstpage) < a->avail - firstpage) {
                 root_points_to_gcArenaPool = JS_TRUE;
                 break;
             }
@@ -950,15 +982,15 @@ gc_lock_marker(JSDHashTable *table, JSDHashEntryHdr *hdr, uint32 num, void *arg)
     return JS_DHASH_NEXT;
 }
 
-JS_FRIEND_API(void)
-js_ForceGC(JSContext *cx)
+void
+js_ForceGC(JSContext *cx, uintN gcflags)
 {
     uintN i;
 
     for (i = 0; i < GCX_NTYPES; i++)
         cx->newborn[i] = NULL;
     cx->runtime->gcPoke = JS_TRUE;
-    js_GC(cx, 0);
+    js_GC(cx, gcflags);
     JS_ArenaFinish();
 }
 
@@ -979,7 +1011,7 @@ js_GC(JSContext *cx, uintN gcflags)
     JSRuntime *rt;
     JSContext *iter, *acx;
     JSStackFrame *fp, *chain;
-    uintN i, depth, nslots;
+    uintN i, depth, nslots, type;
     JSStackHeader *sh;
     JSArena *a, **ap;
     uint8 flags, *flagp, *split;
@@ -997,13 +1029,20 @@ js_GC(JSContext *cx, uintN gcflags)
     JS_ASSERT(!JS_IS_RUNTIME_LOCKED(rt));
 #endif
 
-    /* Don't run gc if it is disabled (unless this is the last context). */
-    if (rt->gcDisabled && !(gcflags & GC_LAST_CONTEXT))
+    /*
+     * Don't collect garbage if the runtime is down or if GC is disabled, and
+     * we're not the last context in the runtime.  The last context must force
+     * a GC, and nothing should disable that final collection or there may be
+     * shutdown leaks, or runtime bloat until the next context is created.
+     */
+    if ((rt->state != JSRTS_UP || rt->gcDisabled) &&
+        !(gcflags & GC_LAST_CONTEXT)) {
         return;
+    }
 
     /*
      * Let the API user decide to defer a GC if it wants to (unless this
-     * is the last context). Call the callback regardless.
+     * is the last context).  Invoke the callback regardless.
      */
     if (rt->gcCallback) {
         if (!rt->gcCallback(cx, JSGC_BEGIN) && !(gcflags & GC_LAST_CONTEXT))
@@ -1011,26 +1050,28 @@ js_GC(JSContext *cx, uintN gcflags)
     }
 
     /* Lock out other GC allocator and collector invocations. */
-    JS_LOCK_GC(rt);
+    if (!(gcflags & GC_ALREADY_LOCKED))
+        JS_LOCK_GC(rt);
 
     /* Do nothing if no assignment has executed since the last GC. */
     if (!rt->gcPoke) {
-	METER(rt->gcStats.nopoke++);
-	JS_UNLOCK_GC(rt);
-	return;
+        METER(rt->gcStats.nopoke++);
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
+        return;
     }
-    rt->gcPoke = JS_FALSE;
     METER(rt->gcStats.poke++);
 
 #ifdef JS_THREADSAFE
     /* Bump gcLevel and return rather than nest on this thread. */
     currentThread = js_CurrentThreadId();
     if (rt->gcThread == currentThread) {
-	JS_ASSERT(rt->gcLevel > 0);
-	rt->gcLevel++;
-	METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-		  rt->gcStats.maxlevel = rt->gcLevel);
-        JS_UNLOCK_GC(rt);
+        JS_ASSERT(rt->gcLevel > 0);
+        rt->gcLevel++;
+        METER(if (rt->gcLevel > rt->gcStats.maxlevel)
+                  rt->gcStats.maxlevel = rt->gcLevel);
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
         return;
     }
 
@@ -1047,7 +1088,7 @@ js_GC(JSContext *cx, uintN gcflags)
          * keep a sub-list of contexts having the same id?
          */
         iter = NULL;
-        while ((acx = js_ContextIterator(rt, &iter)) != NULL) {
+        while ((acx = js_ContextIterator(rt, JS_FALSE, &iter)) != NULL) {
             if (acx->thread == cx->thread && acx->requestDepth)
                 requestDebit++;
         }
@@ -1073,17 +1114,18 @@ js_GC(JSContext *cx, uintN gcflags)
     /* If another thread is already in GC, don't attempt GC; wait instead. */
     if (rt->gcLevel > 0) {
         /* Bump gcLevel to restart the current GC, so it finds new garbage. */
-	rt->gcLevel++;
-	METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-		  rt->gcStats.maxlevel = rt->gcLevel);
+        rt->gcLevel++;
+        METER(if (rt->gcLevel > rt->gcStats.maxlevel)
+                  rt->gcStats.maxlevel = rt->gcLevel);
 
         /* Wait for the other thread to finish, then resume our request. */
         while (rt->gcLevel > 0)
             JS_AWAIT_GC_DONE(rt);
         if (requestDebit)
             rt->requestCount += requestDebit;
-	JS_UNLOCK_GC(rt);
-	return;
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
+        return;
     }
 
     /* No other thread is in GC, so indicate that we're now in GC. */
@@ -1092,16 +1134,16 @@ js_GC(JSContext *cx, uintN gcflags)
 
     /* Wait for all other requests to finish. */
     while (rt->requestCount > 0)
-	JS_AWAIT_REQUEST_DONE(rt);
+        JS_AWAIT_REQUEST_DONE(rt);
 
 #else  /* !JS_THREADSAFE */
 
     /* Bump gcLevel and return rather than nest; the outer gc will restart. */
     rt->gcLevel++;
     METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-	      rt->gcStats.maxlevel = rt->gcLevel);
+              rt->gcStats.maxlevel = rt->gcLevel);
     if (rt->gcLevel > 1)
-	return;
+        return;
 
 #endif /* !JS_THREADSAFE */
 
@@ -1122,6 +1164,12 @@ js_GC(JSContext *cx, uintN gcflags)
     /* Drop atoms held by the property cache, and clear property weak links. */
     js_DisablePropertyCache(cx);
     js_FlushPropertyCache(cx);
+#ifdef DEBUG_brendan
+  { extern void js_DumpScopeMeters(JSRuntime *rt);
+    js_DumpScopeMeters(rt);
+  }
+#endif
+
 restart:
     rt->gcNumber++;
 
@@ -1133,22 +1181,22 @@ restart:
         JS_DHashTableEnumerate(rt->gcLocksHash, gc_lock_marker, cx);
     js_MarkAtomState(&rt->atomState, gcflags, gc_mark_atom_key_thing, cx);
     iter = NULL;
-    while ((acx = js_ContextIterator(rt, &iter)) != NULL) {
-	/*
-	 * Iterate frame chain and dormant chains. Temporarily tack current
-	 * frame onto the head of the dormant list to ease iteration.
-	 *
-	 * (NB: see comment on this whole "dormant" thing in js_Execute.)
-	 */
-	chain = acx->fp;
-	if (chain) {
-	    JS_ASSERT(!chain->dormantNext);
-	    chain->dormantNext = acx->dormantFrameChain;
-	} else {
-	    chain = acx->dormantFrameChain;
-	}
+    while ((acx = js_ContextIterator(rt, JS_TRUE, &iter)) != NULL) {
+        /*
+         * Iterate frame chain and dormant chains. Temporarily tack current
+         * frame onto the head of the dormant list to ease iteration.
+         *
+         * (NB: see comment on this whole "dormant" thing in js_Execute.)
+         */
+        chain = acx->fp;
+        if (chain) {
+            JS_ASSERT(!chain->dormantNext);
+            chain->dormantNext = acx->dormantFrameChain;
+        } else {
+            chain = acx->dormantFrameChain;
+        }
 
-	for (fp = chain; fp; fp = chain = chain->dormantNext) {
+        for (fp = chain; fp; fp = chain = chain->dormantNext) {
             do {
                 if (fp->callobj)
                     GC_MARK(cx, fp->callobj, "call object", NULL);
@@ -1185,25 +1233,37 @@ restart:
                 GC_MARK(cx, fp->scopeChain, "scope chain", NULL);
                 if (fp->sharpArray)
                     GC_MARK(cx, fp->sharpArray, "sharp array", NULL);
-            } while ((fp = fp->down) != NULL);
-	}
 
-	/* Cleanup temporary "dormant" linkage. */
-	if (acx->fp)
-	    acx->fp->dormantNext = NULL;
+                if (fp->objAtomMap) {
+                    JSAtom **vector, *atom;
+
+                    nslots = fp->objAtomMap->length;
+                    vector = fp->objAtomMap->vector;
+                    for (i = 0; i < nslots; i++) {
+                        atom = vector[i];
+                        if (atom)
+                            GC_MARK_ATOM(cx, atom, NULL);
+                    }
+                }
+            } while ((fp = fp->down) != NULL);
+        }
+
+        /* Cleanup temporary "dormant" linkage. */
+        if (acx->fp)
+            acx->fp->dormantNext = NULL;
 
         /* Mark other roots-by-definition in acx. */
-	GC_MARK(cx, acx->globalObject, "global object", NULL);
-	GC_MARK(cx, acx->newborn[GCX_OBJECT], "newborn object", NULL);
-	GC_MARK(cx, acx->newborn[GCX_STRING], "newborn string", NULL);
-	GC_MARK(cx, acx->newborn[GCX_DOUBLE], "newborn double", NULL);
+        GC_MARK(cx, acx->globalObject, "global object", NULL);
+        GC_MARK(cx, acx->newborn[GCX_OBJECT], "newborn object", NULL);
+        GC_MARK(cx, acx->newborn[GCX_STRING], "newborn string", NULL);
+        GC_MARK(cx, acx->newborn[GCX_DOUBLE], "newborn double", NULL);
         GC_MARK(cx, acx->newborn[GCX_MUTABLE_STRING], "newborn mutable string",
                 NULL);
-	for (i = GCX_EXTERNAL_STRING; i < GCX_NTYPES; i++)
+        for (i = GCX_EXTERNAL_STRING; i < GCX_NTYPES; i++)
             GC_MARK(cx, acx->newborn[i], "newborn external string", NULL);
 #if JS_HAS_EXCEPTIONS
-	if (acx->throwing && JSVAL_IS_GCTHING(acx->exception))
-	    GC_MARK(cx, JSVAL_TO_GCTHING(acx->exception), "exception", NULL);
+        if (acx->throwing && JSVAL_IS_GCTHING(acx->exception))
+            GC_MARK(cx, JSVAL_TO_GCTHING(acx->exception), "exception", NULL);
 #endif
 #if JS_HAS_LVALUE_RETURN
         if (acx->rval2set && JSVAL_IS_GCTHING(acx->rval2))
@@ -1227,6 +1287,7 @@ restart:
      * rather than nest badly and leave the unmarked newborn to be swept.
      */
     js_SweepAtomState(&rt->atomState);
+    js_SweepScopeProperties(rt);
     for (a = rt->gcArenaPool.first.next; a; a = a->next) {
         flagp = (uint8 *) a->base;
         split = (uint8 *) FIRST_THING_PAGE(a);
@@ -1241,9 +1302,12 @@ restart:
                 *flagp &= ~GCF_MARK;
             } else if (!(flags & (GCF_LOCKMASK | GCF_FINAL))) {
                 /* Call the finalizer with GCF_FINAL ORed into flags. */
-                finalizer = gc_finalizers[flags & GCF_TYPEMASK];
+                type = flags & GCF_TYPEMASK;
+                finalizer = gc_finalizers[type];
                 if (finalizer) {
                     *flagp = (uint8)(flags | GCF_FINAL);
+                    if (type >= GCX_EXTERNAL_STRING)
+                        js_PurgeDeflatedStringCache((JSString *)thing);
                     finalizer(cx, thing);
                 }
 
@@ -1265,7 +1329,7 @@ restart:
     ap = &rt->gcArenaPool.first.next;
     a = *ap;
     if (!a)
-	goto out;
+        goto out;
     all_clear = JS_TRUE;
     flp = oflp = &rt->gcFreeList;
     *flp = NULL;
@@ -1319,15 +1383,16 @@ out:
     js_EnablePropertyCache(cx);
     rt->gcLevel = 0;
     rt->gcLastBytes = rt->gcBytes;
-    rt->gcRunning = JS_FALSE;
+    rt->gcPoke = rt->gcRunning = JS_FALSE;
 
 #ifdef JS_THREADSAFE
     /* If we were invoked during a request, pay back the temporary debit. */
     if (requestDebit)
-	rt->requestCount += requestDebit;
+        rt->requestCount += requestDebit;
     rt->gcThread = 0;
     JS_NOTIFY_GC_DONE(rt);
-    JS_UNLOCK_GC(rt);
+    if (!(gcflags & GC_ALREADY_LOCKED))
+        JS_UNLOCK_GC(rt);
 #endif
 
     if (rt->gcCallback)
