@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -68,7 +68,9 @@ nsXPConnect::nsXPConnect()
     // then we'll set this up later as needed.
     CreateRuntime();
 
-    mInterfaceInfoManager = XPTI_GetInterfaceInfoManager();
+    nsCOMPtr<nsIInterfaceInfoManager> iim = 
+        dont_AddRef(XPTI_GetInterfaceInfoManager());
+    CallQueryInterface(iim, &mInterfaceInfoManager);
 
     nsServiceManager::GetService(XPC_CONTEXT_STACK_CONTRACTID,
                                  NS_GET_IID(nsIThreadJSContextStack),
@@ -76,13 +78,12 @@ nsXPConnect::nsXPConnect()
 
 #ifdef XPC_TOOLS_SUPPORT
   {
-    const char* filename = PR_GetEnv("MOZILLA_JS_PROFILER_OUTPUT");
-    if(filename)
+    char* filename = PR_GetEnv("MOZILLA_JS_PROFILER_OUTPUT");
+    if(filename && *filename)
     {
-
         mProfilerOutputFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
         if(mProfilerOutputFile &&
-           NS_SUCCEEDED(mProfilerOutputFile->InitWithPath(filename)))
+           NS_SUCCEEDED(mProfilerOutputFile->InitWithNativePath(nsDependentCString(filename))))
         {
             mProfiler = do_GetService(XPCTOOLS_PROFILER_CONTRACTID);
             if(mProfiler)
@@ -219,7 +220,7 @@ nsXPConnect::ReleaseXPConnectSingleton()
         {
             FILE* oldFileHandle = js_DumpGCHeap;
             js_DumpGCHeap = stdout;
-            js_ForceGC(ccx);
+            JS_GC(ccx);
             js_DumpGCHeap = oldFileHandle;
         }
 #endif
@@ -241,16 +242,14 @@ nsXPConnect::ReleaseXPConnectSingleton()
 
 // static
 nsresult
-nsXPConnect::GetInterfaceInfoManager(nsIInterfaceInfoManager** iim,
+nsXPConnect::GetInterfaceInfoManager(nsIInterfaceInfoSuperManager** iim,
                                      nsXPConnect* xpc /*= nsnull*/)
 {
-    nsIInterfaceInfoManager* temp;
-
     if(!xpc && !(xpc = GetXPConnect()))
         return NS_ERROR_FAILURE;
 
-    *iim = temp = xpc->mInterfaceInfoManager;
-    NS_IF_ADDREF(temp);
+    *iim = xpc->mInterfaceInfoManager;
+    NS_IF_ADDREF(*iim);
     return NS_OK;
 }
 
@@ -342,6 +341,67 @@ nsXPConnect::FindMainThread()
     return gMainThread;
 }
 
+/***************************************************************************/
+
+typedef PRBool (*InfoTester)(nsIInterfaceInfoManager* manager, const void* data,
+                             nsIInterfaceInfo** info);
+
+static PRBool IIDTester(nsIInterfaceInfoManager* manager, const void* data,
+                        nsIInterfaceInfo** info)
+{
+    return NS_SUCCEEDED(manager->GetInfoForIID((const nsIID *) data, info)) &&
+           *info;
+}
+
+static PRBool NameTester(nsIInterfaceInfoManager* manager, const void* data,
+                      nsIInterfaceInfo** info)
+{
+    return NS_SUCCEEDED(manager->GetInfoForName((const char *) data, info)) &&
+           *info;
+}
+
+static nsresult FindInfo(InfoTester tester, const void* data, 
+                         nsIInterfaceInfoSuperManager* iism,
+                         nsIInterfaceInfo** info)
+{
+    if(tester(iism, data, info))
+        return NS_OK;
+    
+    // If not found, then let's ask additional managers.
+
+    PRBool yes;
+    nsCOMPtr<nsISimpleEnumerator> list;
+
+    if(NS_SUCCEEDED(iism->HasAdditionalManagers(&yes)) && yes &&
+       NS_SUCCEEDED(iism->EnumerateAdditionalManagers(getter_AddRefs(list))) &&
+       list)
+    {
+        PRBool more;
+        nsCOMPtr<nsIInterfaceInfoManager> current;
+
+        while(NS_SUCCEEDED(list->HasMoreElements(&more)) && more &&
+              NS_SUCCEEDED(list->GetNext(getter_AddRefs(current))) && current)
+        {
+            if(tester(current, data, info))
+                return NS_OK;
+        }
+    }
+    
+    return NS_ERROR_NO_INTERFACE;
+}    
+
+nsresult
+nsXPConnect::GetInfoForIID(const nsIID * aIID, nsIInterfaceInfo** info)
+{
+    return FindInfo(IIDTester, aIID, mInterfaceInfoManager, info);
+}
+
+nsresult
+nsXPConnect::GetInfoForName(const char * name, nsIInterfaceInfo** info)
+{
+    return FindInfo(NameTester, name, mInterfaceInfoManager, info);
+}
+
 /***************************************************************************/
 /***************************************************************************/
 // nsIXPConnect interface methods...
@@ -380,6 +440,10 @@ nsXPConnect::InitClasses(JSContext * aJSContext, JSObject * aGlobalJSObj)
     if(!nsXPCComponents::AttachNewComponentsObject(ccx, scope, aGlobalJSObj))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
+#ifdef XPC_IDISPATCH_SUPPORT
+    // Initialize any properties IDispatch needs on the global object
+    XPCIDispatchExtension::Initialize(ccx, aGlobalJSObj);
+#endif
     return NS_OK;
 }
 
@@ -1025,7 +1089,7 @@ nsXPConnect::DebugDump(PRInt16 depth)
 {
 #ifdef DEBUG
     depth-- ;
-    XPC_LOG_ALWAYS(("nsXPConnect @ %x with mRefCnt = %d", this, mRefCnt));
+    XPC_LOG_ALWAYS(("nsXPConnect @ %x with mRefCnt = %d", this, mRefCnt.get()));
     XPC_LOG_INDENT();
         XPC_LOG_ALWAYS(("gSelf @ %x", gSelf));
         XPC_LOG_ALWAYS(("gOnceAliveNowDead is %d", (int)gOnceAliveNowDead));
