@@ -41,6 +41,7 @@
 #include "nsCoord.h"
 #include "nsEvent.h"
 #include "nsReflowType.h"
+#include "nsCompatibility.h"
 
 class nsIAtom;
 class nsIContent;
@@ -120,13 +121,13 @@ enum nsAttributeChangeType {
  */
 class nsIPresShell : public nsISupports {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IPRESSHELL_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IPRESSHELL_IID)
 
   NS_IMETHOD Init(nsIDocument* aDocument,
                   nsIPresContext* aPresContext,
                   nsIViewManager* aViewManager,
                   nsIStyleSet* aStyleSet,
-                  PRBool aIsQuirksMode) = 0;
+                  nsCompatibility aCompatMode) = 0;
 
   /**
    * All callers are responsible for calling |Destroy| after calling
@@ -205,6 +206,13 @@ public:
   // Make shell stop being a document observer
   NS_IMETHOD EndObservingDocument() = 0;
 
+  /**
+   * Determine if InitialReflow() was previously called.
+   * @param aDidInitialReflow PR_TRUE if InitalReflow() was previously called,
+   * PR_FALSE otherwise.
+   */
+  NS_IMETHOD GetDidInitialReflow(PRBool *aDidInitialReflow) = 0;
+
   /**
    * Perform the initial reflow. Constructs the frame for the root content
    * object and then reflows the frame model into the specified width and
@@ -280,6 +288,10 @@ public:
   NS_IMETHOD CancelReflowCommand(nsIFrame* aTargetFrame, nsReflowType* aCmdType) = 0;
   NS_IMETHOD CancelAllReflowCommands() = 0;
 
+  /**
+   * Recreates the frames for a node
+   */
+  NS_IMETHOD RecreateFramesFor(nsIContent* aContent) = 0;
 
   /**
    * Determine if it is safe to flush all pending notifications
@@ -340,10 +352,14 @@ public:
 
 
   /**
-   * Scrolls the view of the document so that the anchor with the specified
-   * name is displayed at the top of the window
+   * Informs the pres shell that the document is now at the anchor with
+   * the given name.  If |aScroll| is true, scrolls the view of the
+   * document so that the anchor with the specified name is displayed at
+   * the top of the window.  If |aAnchorName| is empty, then this informs
+   * the pres shell that there is no current target, and |aScroll| must
+   * be false.
    */
-  NS_IMETHOD GoToAnchor(const nsAString& aAnchorName) = 0;
+  NS_IMETHOD GoToAnchor(const nsAString& aAnchorName, PRBool aScroll) = 0;
 
   /**
    * Scrolls the view of the document so that the frame is displayed at the 
@@ -401,14 +417,18 @@ public:
   NS_IMETHOD DoCopy() = 0;
 
   /**
-   * Copy link location.
+   * Get link location.
    */
-  NS_IMETHOD DoCopyLinkLocation(nsIDOMNode* aNode) = 0;
+  NS_IMETHOD GetLinkLocation(nsIDOMNode* aNode, nsAString& aLocation) = 0;
 
   /**
-   * Copy image methods.
+   * Get image location.
+   */
+  NS_IMETHOD GetImageLocation(nsIDOMNode* aNode, nsAString& aLocation) = 0;
+
+  /**
+   * Copy image.
    */
-  NS_IMETHOD DoCopyImageLocation(nsIDOMNode* aNode) = 0;
   NS_IMETHOD DoCopyImageContents(nsIDOMNode* aNode) = 0;
 
   /**
@@ -524,6 +544,16 @@ public:
    */
   NS_IMETHOD UnsuppressPainting() = 0;
 
+  /**
+   * Called to disable nsITheme support in a specific presshell.
+   */
+  NS_IMETHOD DisableThemeSupport() = 0;
+
+  /**
+   * Indicates whether theme support is enabled.
+   */
+  virtual PRBool IsThemeSupportEnabled() = 0;
+
   /**
    * See if reflow verification is enabled. To enable reflow verification add
    * "verifyreflow:1" to your NSPR_LOG_MODULES environment variable
