@@ -63,6 +63,7 @@
 #include "nsIDOMWindow.h"
 #include "nsNetUtil.h"
 #include "nsXPIDLString.h"
+#include "nsIWeakReferenceUtils.h"
 
 #include "prprf.h"
 #include "nsContentPolicyUtils.h"
@@ -77,6 +78,9 @@
 #include "nsIDOMHTMLImageElement.h"
 #include "nsIImageFrame.h"
 
+//needed for resetting of image service color
+#include "nsLayoutCID.h"
+#include "nsISelectionImageService.h"
 
 static nscolor
 MakeColorPref(const char *colstr)
@@ -130,6 +134,8 @@ PR_STATIC_CALLBACK(PRBool) destroy_loads(nsHashKey *aKey, void *aData, void* clo
 static NS_DEFINE_CID(kLookAndFeelCID,  NS_LOOKANDFEEL_CID);
 #include "nsContentCID.h"
 static NS_DEFINE_CID(kEventStateManagerCID, NS_EVENTSTATEMANAGER_CID);
+static NS_DEFINE_CID(kSelectionImageService, NS_SELECTIONIMAGESERVICE_CID);
+
 
 nsPresContext::nsPresContext()
   : mDefaultVariableFont("serif", NS_FONT_STYLE_NORMAL, NS_FONT_VARIANT_NORMAL,
@@ -148,8 +154,8 @@ nsPresContext::nsPresContext()
       NS_FONT_WEIGHT_NORMAL, 0, NSIntPointsToTwips(12)),
     mNoTheme(PR_FALSE)
 {
-  NS_INIT_REFCNT();
-  mCompatibilityMode = eCompatibility_Standard;
+  NS_INIT_ISUPPORTS();
+  mCompatibilityMode = eCompatibility_FullStandards;
   mWidgetRenderingMode = eWidgetRendering_Gfx; 
   mImageAnimationMode = imgIContainer::kNormalAnimMode;
   mImageAnimationModePref = imgIContainer::kNormalAnimMode;
@@ -161,6 +167,9 @@ nsPresContext::nsPresContext()
   mStopChrome = PR_TRUE;
 
   mShell = nsnull;
+  mLinkHandler = nsnull;
+  mContainer = nsnull;
+
 
   mDefaultColor = NS_RGB(0x00, 0x00, 0x00);
   mDefaultBackgroundColor = NS_RGB(0xFF, 0xFF, 0xFF);
@@ -187,9 +196,6 @@ nsPresContext::nsPresContext()
   mFocusRingWidth = 1;
   mFocusRingOnAnything = PR_FALSE;
 
-  mDefaultBackgroundImageAttachment = NS_STYLE_BG_ATTACHMENT_SCROLL;
-  mDefaultBackgroundImageRepeat = NS_STYLE_BG_REPEAT_XY;
-  mDefaultBackgroundImageOffsetX = mDefaultBackgroundImageOffsetY = 0;
   mLanguageSpecificTransformType = eLanguageSpecificTransformType_Unknown;
   mIsRenderingOnlySelection = PR_FALSE;
 #ifdef IBMBIDI
@@ -407,7 +413,7 @@ nsPresContext::GetDocumentColorPreferences()
   PRBool usePrefColors = PR_TRUE;
   PRBool boolPref;
   nsXPIDLCString colorStr;
-  nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryInterface(mContainer));
+  nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryReferent(mContainer));
   if (docShell) {
     PRInt32 docShellType;
     docShell->GetItemType(&docShellType);
@@ -598,7 +604,7 @@ nsPresContext::ClearStyleDataAndReflow()
 void
 nsPresContext::PreferenceChanged(const char* aPrefName)
 {
-  nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryInterface(mContainer));
+  nsCOMPtr<nsIDocShellTreeItem> docShell(do_QueryReferent(mContainer));
   if (docShell) {
     PRInt32 docShellType;
     docShell->GetItemType(&docShellType);
@@ -769,7 +775,7 @@ nsPresContext::SetCompatibilityMode(nsCompatibility aMode)
   nsCOMPtr<nsIStyleSet> set;
   mShell->GetStyleSet(getter_AddRefs(set));
   if (set) {
-    set->EnableQuirkStyleSheet(mCompatibilityMode != eCompatibility_Standard);
+    set->EnableQuirkStyleSheet(mCompatibilityMode == eCompatibility_NavQuirks);
   }
   return NS_OK;
 }
@@ -883,24 +889,42 @@ nsPresContext::SetImageAnimationMode(PRUint16 aMode)
   return NS_OK;
 }
 
-/* This function has now been deprecated.  It is no longer necesary to
- * hold on to presContext just to get a nsLookAndFeel.  nsLookAndFeel is
- * now a service provided by ServiceManager.
+/*
+ * It is no longer necesary to hold on to presContext just to get a
+ * nsILookAndFeel, which can now be obtained through the service
+ * manager.  However, this cached copy can be used when a pres context
+ * is available, for faster performance.
  */
 NS_IMETHODIMP
 nsPresContext::GetLookAndFeel(nsILookAndFeel** aLookAndFeel)
 {
   NS_PRECONDITION(aLookAndFeel, "null out param");
-  nsresult result = NS_OK;
   if (! mLookAndFeel) {
-    mLookAndFeel = do_GetService(kLookAndFeelCID,&result);
+    nsresult rv;
+    mLookAndFeel = do_GetService(kLookAndFeelCID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
   *aLookAndFeel = mLookAndFeel;
-  NS_IF_ADDREF(*aLookAndFeel);
-  return result;
+  NS_ADDREF(*aLookAndFeel);
+  return NS_OK;
 }
 
-
+/*
+ * Get the cached IO service, faster than the service manager could.
+ */
+NS_IMETHODIMP
+nsPresContext::GetIOService(nsIIOService** aIOService)
+{
+  NS_PRECONDITION(aIOService, "null out param");
+  if (! mIOService) {
+    nsresult rv;
+    mIOService = do_GetIOService(&rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  *aIOService = mIOService;
+  NS_ADDREF(*aIOService);
+  return NS_OK;
+}
 
 NS_IMETHODIMP
 nsPresContext::GetBaseURL(nsIURI** aResult)
@@ -1164,38 +1188,6 @@ nsPresContext::GetDefaultBackgroundColor(nscolor* aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsPresContext::GetDefaultBackgroundImage(nsString& aImage)
-{
-  aImage = mDefaultBackgroundImage;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::GetDefaultBackgroundImageRepeat(PRUint8* aRepeat)
-{
-  NS_PRECONDITION(aRepeat, "null out param");
-  *aRepeat = mDefaultBackgroundImageRepeat;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::GetDefaultBackgroundImageOffset(nscoord* aX, nscoord* aY)
-{
-  NS_PRECONDITION(aX && aY, "null out param");
-  *aX = mDefaultBackgroundImageOffsetX;
-  *aY = mDefaultBackgroundImageOffsetY;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::GetDefaultBackgroundImageAttachment(PRUint8* aAttachment)
-{
-  NS_PRECONDITION(aAttachment, "null out param");
-  *aAttachment = mDefaultBackgroundImageAttachment;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsPresContext::GetDefaultLinkColor(nscolor* aColor)
 {
@@ -1269,35 +1261,6 @@ nsPresContext::SetDefaultBackgroundColor(nscolor aColor)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsPresContext::SetDefaultBackgroundImage(const nsString& aImage)
-{
-  mDefaultBackgroundImage = aImage;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::SetDefaultBackgroundImageRepeat(PRUint8 aRepeat)
-{
-  mDefaultBackgroundImageRepeat = aRepeat;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::SetDefaultBackgroundImageOffset(nscoord aX, nscoord aY)
-{
-  mDefaultBackgroundImageOffsetX = aX;
-  mDefaultBackgroundImageOffsetY = aY;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPresContext::SetDefaultBackgroundImageAttachment(PRUint8 aAttachment)
-{
-  mDefaultBackgroundImageAttachment = aAttachment;
-  return NS_OK;
-}
-
 NS_IMETHODIMP 
 nsPresContext::SetDefaultLinkColor(nscolor aColor)
 {
@@ -1414,6 +1377,19 @@ NS_IMETHODIMP
 nsPresContext::GetImageLoadFlags(nsLoadFlags& aLoadFlags)
 {
   aLoadFlags = nsIRequest::LOAD_NORMAL;
+
+  nsCOMPtr<nsIDocument> doc;
+  (void) mShell->GetDocument(getter_AddRefs(doc));
+
+  if (doc) {
+    nsCOMPtr<nsILoadGroup> loadGroup;
+    (void) doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+
+    if (loadGroup) {
+      loadGroup->GetLoadFlags(&aLoadFlags);
+    }
+  }
+
   return NS_OK;
 }
 
@@ -1541,7 +1517,7 @@ nsPresContext::GetLinkHandler(nsILinkHandler** aResult)
 NS_IMETHODIMP
 nsPresContext::SetContainer(nsISupports* aHandler)
 {
-  mContainer = aHandler;
+  mContainer = do_GetWeakReference(aHandler);
   if (mContainer) {
     GetDocumentColorPreferences();
   }
@@ -1552,18 +1528,18 @@ NS_IMETHODIMP
 nsPresContext::GetContainer(nsISupports** aResult)
 {
   NS_PRECONDITION(aResult, "null out param");
-  *aResult = mContainer;
-  NS_IF_ADDREF(mContainer);
-  return NS_OK;
+  if (!mContainer) {
+    *aResult = nsnull;
+    return NS_OK;
+  }
+
+  return CallQueryReferent(mContainer.get(), aResult);
 }
 
 NS_IMETHODIMP
 nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
 {
-  NS_PRECONDITION(nsnull != aManager, "null ptr");
-  if (nsnull == aManager) {
-    return NS_ERROR_NULL_POINTER;
-  }
+  NS_PRECONDITION(aManager, "null ptr");
 
   if (!mEventManager) {
     nsresult rv;
@@ -1571,10 +1547,10 @@ nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
     if (NS_FAILED(rv)) {
       return rv;
     }
-  }
 
-  //Not refcnted, set null in destructor
-  mEventManager->SetPresContext(this);
+    //Not refcnted, set null in destructor
+    mEventManager->SetPresContext(this);
+  }
 
   *aManager = mEventManager;
   NS_IF_ADDREF(*aManager);
@@ -1584,7 +1560,7 @@ nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
 #ifdef IBMBIDI
 //ahmed
 NS_IMETHODIMP
-nsPresContext::IsArabicEncoding(PRBool& aResult)
+nsPresContext::IsArabicEncoding(PRBool& aResult) const
 {
   aResult=PR_FALSE;
   if ( (mCharset.EqualsIgnoreCase("ibm864") )||(mCharset.EqualsIgnoreCase("ibm864i") )||(mCharset.EqualsIgnoreCase("windows-1256") )||(mCharset.EqualsIgnoreCase("iso-8859-6") ))
@@ -1593,7 +1569,7 @@ nsPresContext::IsArabicEncoding(PRBool& aResult)
 }
 
 NS_IMETHODIMP
-nsPresContext::IsVisRTL(PRBool& aResult)
+nsPresContext::IsVisRTL(PRBool& aResult) const
 {
   aResult=PR_FALSE;
   if ( (mIsVisual)&&(GET_BIDI_OPTION_DIRECTION(mBidi) == IBMBIDI_TEXTDIRECTION_RTL) )
@@ -1681,7 +1657,7 @@ NS_IMETHODIMP   nsPresContext::SetBidi(PRUint32 aSource, PRBool aForceReflow)
   }
   return NS_OK;
 }
-NS_IMETHODIMP   nsPresContext::GetBidi(PRUint32* aDest)
+NS_IMETHODIMP   nsPresContext::GetBidi(PRUint32* aDest) const
 {
   if (aDest)
     *aDest = mBidi;
@@ -1704,7 +1680,7 @@ nsPresContext::GetIsBidiSystem(PRBool& aResult) const
 }
 
 NS_IMETHODIMP
-nsPresContext::GetBidiCharset(nsAString &aCharSet)
+nsPresContext::GetBidiCharset(nsAString &aCharSet) const
 {
   aCharSet = mCharset;
   return NS_OK;
@@ -1795,6 +1771,13 @@ nsPresContext::SysColorChanged()
     mShell->GetStyleSet(getter_AddRefs(set));
     set->ClearStyleData(this, nsnull, nsnull);
   }
+  nsCOMPtr<nsISelectionImageService> imageService;
+  nsresult result;
+  imageService = do_GetService(kSelectionImageService, &result);
+  if (NS_SUCCEEDED(result) && imageService)
+  {
+    imageService->Reset();
+  }
   return NS_OK;
 }
 
