@@ -38,7 +38,6 @@
 #include "nsAbsoluteContainingBlock.h"
 #include "nsContainerFrame.h"
 #include "nsReflowPath.h"
-#include "nsIStyleContext.h"
 #include "nsIViewManager.h"
 #include "nsLayoutAtoms.h"
 #include "nsIPresShell.h"
@@ -46,12 +45,17 @@
 #include "nsIPresContext.h"
 #include "nsIFrameManager.h"
 
+#ifdef DEBUG
+#include "nsBlockFrame.h"
+#endif
+
+
 nsresult
 nsAbsoluteContainingBlock::FirstChild(const nsIFrame* aDelegatingFrame,
                                       nsIAtom*        aListName,
                                       nsIFrame**      aFirstChild) const
 {
-  NS_PRECONDITION(nsLayoutAtoms::absoluteList == aListName, "unexpected child list name");
+  NS_PRECONDITION(GetChildListName() == aListName, "unexpected child list name");
   *aFirstChild = mAbsoluteFrames.FirstChild();
   return NS_OK;
 }
@@ -62,7 +66,7 @@ nsAbsoluteContainingBlock::SetInitialChildList(nsIFrame*       aDelegatingFrame,
                                                nsIAtom*        aListName,
                                                nsIFrame*       aChildList)
 {
-  NS_PRECONDITION(nsLayoutAtoms::absoluteList == aListName, "unexpected child list name");
+  NS_PRECONDITION(GetChildListName() == aListName, "unexpected child list name");
 #ifdef NS_DEBUG
   nsFrame::VerifyDirtyBitSet(aChildList);
 #endif
@@ -89,7 +93,7 @@ nsAbsoluteContainingBlock::AppendFrames(nsIFrame*       aDelegatingFrame,
   nsHTMLReflowCommand* reflowCmd;
   rv = NS_NewHTMLReflowCommand(&reflowCmd, aDelegatingFrame, eReflowType_ReflowDirty);
   if (NS_SUCCEEDED(rv)) {
-    reflowCmd->SetChildListName(nsLayoutAtoms::absoluteList);
+    reflowCmd->SetChildListName(GetChildListName());
     aPresShell.AppendReflowCommand(reflowCmd);
   }
 
@@ -116,7 +120,7 @@ nsAbsoluteContainingBlock::InsertFrames(nsIFrame*       aDelegatingFrame,
   nsHTMLReflowCommand* reflowCmd;
   rv = NS_NewHTMLReflowCommand(&reflowCmd, aDelegatingFrame, eReflowType_ReflowDirty);
   if (NS_SUCCEEDED(rv)) {
-    reflowCmd->SetChildListName(nsLayoutAtoms::absoluteList);
+    reflowCmd->SetChildListName(GetChildListName());
     aPresShell.AppendReflowCommand(reflowCmd);
   }
 
@@ -138,6 +142,21 @@ nsAbsoluteContainingBlock::RemoveFrame(nsIFrame*       aDelegatingFrame,
   return result ? NS_OK : NS_ERROR_FAILURE;
 }
 
+nsresult
+nsAbsoluteContainingBlock::ReplaceFrame(nsIFrame*       aDelegatingFrame,
+                                        nsIPresContext* aPresContext,
+                                        nsIPresShell&   aPresShell,
+                                        nsIAtom*        aListName,
+                                        nsIFrame*       aOldFrame,
+                                        nsIFrame*       aNewFrame)
+{
+  PRBool result = mAbsoluteFrames.ReplaceFrame(aPresContext, aDelegatingFrame,
+                                               aOldFrame, aNewFrame, PR_TRUE);
+  NS_ASSERTION(result, "Problems replacing a frame");
+  return result ? NS_OK : NS_ERROR_FAILURE;
+}
+
+
 // Destructor function for the collapse offset frame property
 static void
 DestroyRectFunc(nsIPresContext* aPresContext,
@@ -189,16 +208,19 @@ nsAbsoluteContainingBlock::Reflow(nsIFrame*                aDelegatingFrame,
                                   const nsHTMLReflowState& aReflowState,
                                   nscoord                  aContainingBlockWidth,
                                   nscoord                  aContainingBlockHeight,
-                                  nsRect&                  aChildBounds)
+                                  nsRect*                  aChildBounds)
 {
   // Initialize OUT parameter
-  aChildBounds.SetRect(0, 0, 0, 0);
+  if (aChildBounds)
+    aChildBounds->SetRect(0, 0, 0, 0);
 
   // Make a copy of the reflow state.  If the reason is
   // eReflowReason_Incremental (which should mean either that the target
   // is the frame for which this is the absolute container or that the
   // container changed size due to incremental reflow of its children),
   // then change it to eReflowReason_Resize.
+  // XXXldb If the target is this frame, shouldn't we be setting it
+  // appropriately (which might mean to StyleChanged)?
   nsHTMLReflowState reflowState(aReflowState);
   if (eReflowReason_Incremental == reflowState.reason) {
     reflowState.reason = eReflowReason_Resize;
@@ -224,25 +246,27 @@ nsAbsoluteContainingBlock::Reflow(nsIFrame*                aDelegatingFrame,
     ReflowAbsoluteFrame(aDelegatingFrame, aPresContext, reflowState, aContainingBlockWidth,
                         aContainingBlockHeight, kidFrame, reason, kidStatus);
 
-    // Add in the child's bounds
-    nsRect  kidBounds;
-    kidFrame->GetRect(kidBounds);
-    aChildBounds.UnionRect(aChildBounds, kidBounds);
+    if (aChildBounds) {
+      // Add in the child's bounds
+      nsRect  kidBounds;
+      kidFrame->GetRect(kidBounds);
+      aChildBounds->UnionRect(*aChildBounds, kidBounds);
 
-    // If the frame has visible overflow, then take it into account, too.
-    nsFrameState  kidFrameState;
-    kidFrame->GetFrameState(&kidFrameState);
-    if (kidFrameState & NS_FRAME_OUTSIDE_CHILDREN) {
-      // Get the property
-      nsRect* overflowArea = ::GetOverflowAreaProperty(aPresContext, kidFrame);
+      // If the frame has visible overflow, then take it into account, too.
+      nsFrameState  kidFrameState;
+      kidFrame->GetFrameState(&kidFrameState);
+      if (kidFrameState & NS_FRAME_OUTSIDE_CHILDREN) {
+        // Get the property
+        nsRect* overflowArea = ::GetOverflowAreaProperty(aPresContext, kidFrame);
 
-      if (overflowArea) {
-        // The overflow area is in the child's coordinate space, so translate
-        // it into the parent's coordinate space
-        nsRect  rect(*overflowArea);
+        if (overflowArea) {
+          // The overflow area is in the child's coordinate space, so translate
+          // it into the parent's coordinate space
+          nsRect  rect(*overflowArea);
 
-        rect.MoveBy(kidBounds.x, kidBounds.y);
-        aChildBounds.UnionRect(aChildBounds, rect);
+          rect.MoveBy(kidBounds.x, kidBounds.y);
+          aChildBounds->UnionRect(*aChildBounds, rect);
+        }
       }
     }
   }
@@ -253,6 +277,9 @@ void
 nsAbsoluteContainingBlock::CalculateChildBounds(nsIPresContext* aPresContext,
                                                 nsRect&         aChildBounds)
 {
+  // Initialize the OUT parameters
+  aChildBounds.SetRect(0, 0, 0, 0);
+
   for (nsIFrame* f = mAbsoluteFrames.FirstChild(); f; f->GetNextSibling(&f)) {
     // Add in the child's bounds
     nsRect  bounds;
@@ -284,12 +311,12 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
                                              const nsHTMLReflowState& aReflowState,
                                              nscoord                  aContainingBlockWidth,
                                              nscoord                  aContainingBlockHeight,
-                                             PRBool&                  aWasHandled,
-                                             nsRect&                  aChildBounds)
+                                             PRBool&                  aWasHandled)
 {
-  // Initialize the OUT paremeters
-  aWasHandled = PR_FALSE;
-  aChildBounds.SetRect(0, 0, 0, 0);
+  // Initialize the OUT parameters
+  // Assume we handled the incremental reflow
+  // until we find something that requires a full block reflow
+  aWasHandled = PR_TRUE;
 
   // See if the reflow command is targeted at us.
   nsReflowPath *path = aReflowState.path;
@@ -300,7 +327,7 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
     nsCOMPtr<nsIAtom> listName;
     command->GetChildListName(*getter_AddRefs(listName));
 
-    if (nsLayoutAtoms::absoluteList == listName) {
+    if (GetChildListName() == listName) {
       nsReflowType  type;
 
       // Get the type of reflow command
@@ -308,7 +335,7 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
 
       // The only type of reflow command we expect is that we have dirty
       // child frames to reflow
-      NS_ASSERTION(eReflowType_ReflowDirty, "unexpected reflow type");
+      NS_ASSERTION(type == eReflowType_ReflowDirty, "unexpected reflow type");
 
       // Walk the positioned frames and reflow the dirty frames
       for (nsIFrame* f = mAbsoluteFrames.FirstChild(); f; f->GetNextSibling(&f)) {
@@ -328,12 +355,11 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
                               reason, status);
         }
       }
-
-      // Indicate we handled the reflow command.
-      aWasHandled = PR_TRUE;
-      
-      // Calculate the total child bounds.
-      CalculateChildBounds(aPresContext, aChildBounds);
+    } else {
+      // A reflow command is targeted directly at this block.
+      // The block will have to do a proper reflow.
+      // But keep going to make sure that we process any dirty absolute frames, below.
+      aWasHandled = PR_FALSE;
     }
   }
 
@@ -341,10 +367,6 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
   nsReflowPath::iterator end = path->EndChildren();
 
   if (iter != end && mAbsoluteFrames.NotEmpty()) {
-    // One of our children has been targeted for reflow. By default,
-    // assume we've handled it.
-    aWasHandled = PR_TRUE;
-
     for ( ; iter != end; ++iter) {
       // See if it's one of our absolutely positioned child frames
       if (mAbsoluteFrames.ContainsFrame(*iter)) {
@@ -359,9 +381,6 @@ nsAbsoluteContainingBlock::IncrementalReflow(nsIFrame*                aDelegatin
         // repainting, and because it has a view if it changes size
         // the view manager will damage the dirty area
 
-        // Calculate the total child bounds
-        CalculateChildBounds(aPresContext, aChildBounds);
-
         // Prune the path so we don't flow the block frame _again_
         // when returning to the caller.
         aReflowState.path->Remove(iter);
@@ -392,6 +411,11 @@ nsAbsoluteContainingBlock::DestroyFrames(nsIFrame*       aDelegatingFrame,
 // XXX Optimize the case where it's a resize reflow and the absolutely
 // positioned child has the exact same size and position and skip the
 // reflow...
+
+// When bug 154892 is checked in, make sure that when 
+// GetChildListName() == nsLayoutAtoms::fixedList, the height is unconstrained.
+// since we don't allow replicated frames to split.
+
 nsresult
 nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegatingFrame,
                                                nsIPresContext*          aPresContext,
@@ -402,14 +426,52 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
                                                nsReflowReason           aReason,
                                                nsReflowStatus&          aStatus)
 {
+#ifdef DEBUG
+  if (nsBlockFrame::gNoisyReflow) {
+    nsFrame::IndentBy(stdout,nsBlockFrame::gNoiseIndent);
+    printf("abs pos ");
+    if (nsnull != aKidFrame) {
+      nsIFrameDebug*  frameDebug;
+      if (NS_SUCCEEDED(CallQueryInterface(aKidFrame, &frameDebug))) {
+        nsAutoString name;
+        frameDebug->GetFrameName(name);
+        printf("%s ", NS_LossyConvertUCS2toASCII(name).get());
+      }
+    }
+    printf("r=%d",aReflowState.reason);
+
+    if (aReflowState.reason == eReflowReason_Incremental) {
+      nsHTMLReflowCommand *command = aReflowState.path->mReflowCommand;
+
+      if (command) {
+        // We're the target.
+        nsReflowType type;
+        command->GetType(type);
+        printf("(%d)", type);      
+      }
+    }
+    char width[16];
+    char height[16];
+    PrettyUC(aReflowState.availableWidth, width);
+    PrettyUC(aReflowState.availableHeight, height);
+    printf(" a=%s,%s ", width, height);
+    PrettyUC(aReflowState.mComputedWidth, width);
+    PrettyUC(aReflowState.mComputedHeight, height);
+    printf("c=%s,%s \n", width, height);
+  }
+  if (nsBlockFrame::gNoisy) {
+    nsBlockFrame::gNoiseIndent++;
+  }
+#endif // DEBUG
+
+
   nsresult  rv;
   nsMargin  border;
-
   // Get the border values
   if (!aReflowState.mStyleBorder->GetBorder(border)) {
     NS_NOTYETIMPLEMENTED("percentage border");
   }
-  
+
   nsFrameState        kidFrameState;
   nsSize              availSize(aReflowState.mComputedWidth, NS_UNCONSTRAINEDSIZE);
   nsHTMLReflowMetrics kidDesiredSize(nsnull);
@@ -494,10 +556,48 @@ nsAbsoluteContainingBlock::ReflowAbsoluteFrame(nsIFrame*                aDelegat
     // Get the property (creating a rect struct if necessary)
     nsRect* overflowArea = ::GetOverflowAreaProperty(aPresContext, aKidFrame, PR_TRUE);
 
+    NS_ASSERTION(overflowArea, "should have created rect");
     if (overflowArea) {
       *overflowArea = kidDesiredSize.mOverflowArea;
     }
   }
-
+#ifdef DEBUG
+  if (nsBlockFrame::gNoisy) {
+    nsBlockFrame::gNoiseIndent--;
+  }
+  if (nsBlockFrame::gNoisyReflow) {
+    nsFrame::IndentBy(stdout,nsBlockFrame::gNoiseIndent);
+    printf("abs pos ");
+    if (nsnull != aKidFrame) {
+      nsIFrameDebug*  frameDebug;
+      if (NS_SUCCEEDED(CallQueryInterface(aKidFrame, &frameDebug))) {
+        nsAutoString name;
+        frameDebug->GetFrameName(name);
+        printf("%s ", NS_LossyConvertUCS2toASCII(name).get());
+      }
+    }
+    printf("%p rect=%d,%d,%d,%d", aKidFrame, rect.x, rect.y, rect.width, rect.height);
+    printf("\n");
+  }
+#endif
   return rv;
 }
+
+#ifdef DEBUG
+ void nsAbsoluteContainingBlock::PrettyUC(nscoord aSize,
+                        char*   aBuf)
+{
+  if (NS_UNCONSTRAINEDSIZE == aSize) {
+    strcpy(aBuf, "UC");
+  }
+  else {
+    if(0xdeadbeef == aSize)
+    {
+      strcpy(aBuf, "deadbeef");
+    }
+    else {
+      sprintf(aBuf, "%d", aSize);
+    }
+  }
+}
+#endif
