@@ -180,14 +180,14 @@ public:
 
 #endif
 
-  virtual void DeleteChildsNextInFlow(nsIPresContext* aPresContext,
-                                      nsIFrame* aNextInFlow);
+  virtual void DeleteNextInFlowChild(nsIPresContext* aPresContext,
+                                     nsIFrame*       aNextInFlow);
 
   /** return the topmost block child based on y-index.
     * almost always the first or second line, if there is one.
     * accounts for lines that hold only compressed white space, etc.
     */
-  nsIFrame* GetTopBlockChild();
+  nsIFrame* GetTopBlockChild(nsIPresContext *aPresContext);
 
   // Returns the line containing aFrame, or end_lines() if the frame
   // isn't in the block.
@@ -199,6 +199,13 @@ public:
 
   inline nscoord GetAscent() { return mAscent; }
 
+  // Create a contination for aPlaceholder and its out of flow frame and
+  // add it to the list of overflow floaters
+  nsresult SplitPlaceholder(nsIPresContext& aPresContext, nsIFrame& aPlaceholder);
+
+  void UndoSplitPlaceholders(nsBlockReflowState& aState,
+                             nsIFrame*           aLastPlaceholder);
+  
 protected:
   nsBlockFrame();
   virtual ~nsBlockFrame();
@@ -245,8 +252,15 @@ protected:
   virtual PRIntn GetSkipSides() const;
 
   virtual void ComputeFinalSize(const nsHTMLReflowState& aReflowState,
-                                nsBlockReflowState&  aState,
-                                nsHTMLReflowMetrics& aMetrics);
+                                nsBlockReflowState&      aState,
+                                nsHTMLReflowMetrics&     aMetrics);
+
+  void ComputeCombinedArea(const nsHTMLReflowState& aReflowState,
+                           nsHTMLReflowMetrics& aMetrics);
+
+  // Calls |nsLineBox::IsEmpty| with the correct arguments.
+  PRBool IsLineEmpty(nsIPresContext* aPresContext,
+                     const nsLineBox* aLine) const;
 
   /** add the frames in aFrameList to this block after aPrevSibling
     * this block thinks in terms of lines, but the frame construction code
@@ -425,11 +439,6 @@ protected:
                                     nsIFrame*           aLastPlaceholder,
                                     PRBool&             aKeepReflowGoing);
 
-  // Create a contination for aPlaceholder and its out of flow frame and
-  // add it to the list of overflow floaters
-  nsresult SplitPlaceholder(nsBlockReflowState& aState,
-                            nsIFrame&           aPlaceholder);
-
   nsresult SplitLine(nsBlockReflowState& aState,
                      nsLineLayout& aLineLayout,
                      line_iterator aLine,
@@ -496,6 +505,12 @@ protected:
   nsresult SetOverflowLines(nsIPresContext* aPresContext,
                             nsLineList*     aOverflowLines);
 
+  nsFrameList* GetOverflowPlaceholders(nsIPresContext* aPresContext,
+                                       PRBool          aRemoveProperty) const;
+
+  nsresult SetOverflowPlaceholders(nsIPresContext* aPresContext,
+                                   nsFrameList*    aOverflowPlaceholders);
+
   nsIFrame* LastChild();
 
 #ifdef NS_DEBUG
