@@ -46,6 +46,7 @@
 #include "nsIPresContext.h"
 #include "nsLayoutAtoms.h"
 #include "nsIFrame.h"
+#include "nsIFrameManager.h"
 
 #include "nsINameSpaceManager.h"
 #include "nsHTMLAtoms.h"
@@ -120,11 +121,10 @@ nsBlockReflowState::nsBlockReflowState(const nsHTMLReflowState& aReflowState,
     }
     else {
       nscoord lr = borderPadding.left + borderPadding.right;
-      mContentArea.width = aReflowState.availableWidth - lr;
+      mContentArea.width = PR_MAX(0, aReflowState.availableWidth - lr);
     }
   }
   mHaveRightFloaters = PR_FALSE;
-  mOverflowFloaters.SetFrames(nsnull);
 
   // Compute content area height. Unlike the width, if we have a
   // specified style height we ignore it since extra content is
@@ -137,7 +137,7 @@ nsBlockReflowState::nsBlockReflowState(const nsHTMLReflowState& aReflowState,
     // the bottom border and padding. The content area height doesn't
     // include either border or padding edge.
     mBottomEdge = aReflowState.availableHeight - borderPadding.bottom;
-    mContentArea.height = mBottomEdge - borderPadding.top;
+    mContentArea.height = PR_MAX(0, mBottomEdge - borderPadding.top);
   }
   else {
     // When we are not in a paginated situation then we always use
@@ -448,11 +448,10 @@ nsBlockReflowState::ReconstructMarginAbove(nsLineList::iterator aLine)
   mPrevBottomMargin.Zero();
   nsBlockFrame *block = mBlock;
 
-  const nsStyleText* styleText = NS_STATIC_CAST(const nsStyleText*,
-      block->mStyleContext->GetStyleData(eStyleStruct_Text));
-  PRBool isPre =
-      ((NS_STYLE_WHITESPACE_PRE == styleText->mWhiteSpace) ||
-       (NS_STYLE_WHITESPACE_MOZ_PRE_WRAP == styleText->mWhiteSpace));
+  const nsStyleText* styleText;
+  ::GetStyleData(block, &styleText);
+  PRBool isPre = NS_STYLE_WHITESPACE_PRE == styleText->mWhiteSpace ||
+                 NS_STYLE_WHITESPACE_MOZ_PRE_WRAP == styleText->mWhiteSpace;
 
   nsCompatibility mode;
   mPresContext->GetCompatibilityMode(&mode);
@@ -848,7 +847,8 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
   // content.
   nscoord saveY = mY;
 
-  nsIFrame* floater = aFloaterCache->mPlaceholder->GetOutOfFlowFrame();
+  nsPlaceholderFrame* placeholder = aFloaterCache->mPlaceholder;
+  nsIFrame*           floater = placeholder->GetOutOfFlowFrame();
 
   // Grab the floater's display information
   const nsStyleDisplay* floaterDisplay;
@@ -880,8 +880,8 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
   }
 
   // Reflow the floater
-  mBlock->ReflowFloater(*this, aFloaterCache->mPlaceholder, aFloaterCache->mCombinedArea,
-			                  aFloaterCache->mMargins, aFloaterCache->mOffsets, aReflowStatus);
+  mBlock->ReflowFloater(*this, placeholder, aFloaterCache->mCombinedArea,
+                        aFloaterCache->mMargins, aFloaterCache->mOffsets, aReflowStatus);
 
   // Get the floaters bounding box and margin information
   floater->GetRect(region);
@@ -964,7 +964,7 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
       mY += mAvailSpaceRect.height;
       GetAvailableSpace();
       // reflow the floater again now since we have more space
-      mBlock->ReflowFloater(*this, aFloaterCache->mPlaceholder, aFloaterCache->mCombinedArea,
+      mBlock->ReflowFloater(*this, placeholder, aFloaterCache->mCombinedArea,
                             aFloaterCache->mMargins, aFloaterCache->mOffsets, aReflowStatus);
       // Get the floaters bounding box and margin information
       floater->GetRect(region);
@@ -975,12 +975,33 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
 
     }
   }
-  // If the floater is continued, it will get the same x value as its prev-in-flow
-  nsRect prevInFlowRect(0,0,0,0);
+  // If the floater is continued, it will get the same absolute x value as its prev-in-flow
+  nsRect prevRect(0,0,0,0);
   nsIFrame* prevInFlow;
   floater->GetPrevInFlow(&prevInFlow);
   if (prevInFlow) {
-    prevInFlow->GetRect(prevInFlowRect);
+    prevInFlow->GetRect(prevRect);
+
+    nsCOMPtr<nsIPresShell> presShell;
+    mPresContext->GetShell(getter_AddRefs(presShell));
+    nsCOMPtr<nsIFrameManager> frameManager;
+    presShell->GetFrameManager(getter_AddRefs(frameManager));
+
+    nsIFrame *placeParent, *placeParentPrev, *prevPlace, *prevPlaceParent;
+    // If prevInFlow's placeholder is in a block that wasn't continued, we need to adjust 
+    // prevRect.x to account for the missing frame offsets.
+    placeholder->GetParent(&placeParent);
+    placeParent->GetPrevInFlow(&placeParentPrev);
+    frameManager->GetPlaceholderFrameFor(prevInFlow, &prevPlace);
+    prevPlace->GetParent(&prevPlaceParent);
+
+    for (nsIFrame* ancestor = prevPlaceParent; 
+         ancestor && (ancestor != placeParentPrev); 
+         ancestor->GetParent(&ancestor)) {
+      nsRect rect;
+      ancestor->GetRect(rect);
+      prevRect.x += rect.x;
+    }        
   }
   // Assign an x and y coordinate to the floater. Note that the x,y
   // coordinates are computed <b>relative to the translation in the
@@ -990,15 +1011,15 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
   PRBool isLeftFloater;
   if (NS_STYLE_FLOAT_LEFT == floaterDisplay->mFloats) {
     isLeftFloater = PR_TRUE;
-    region.x = (prevInFlow) ? prevInFlowRect.x : mAvailSpaceRect.x;
+    region.x = (prevInFlow) ? prevRect.x : mAvailSpaceRect.x;
   }
   else {
     isLeftFloater = PR_FALSE;
-    if (NS_UNCONSTRAINEDSIZE != mAvailSpaceRect.XMost()) {
+    if (NS_UNCONSTRAINEDSIZE != mAvailSpaceRect.width) {
       nsIFrame* prevInFlow;
       floater->GetPrevInFlow(&prevInFlow);
       if (prevInFlow) {
-        region.x = prevInFlowRect.x;
+        region.x = prevRect.x;
       }
       else if (!keepFloaterOnSameLine) {
         region.x = mAvailSpaceRect.XMost() - region.width;
@@ -1011,7 +1032,7 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
     }
     else {
       okToAddRectRegion = PR_FALSE;
-      region.x = mAvailSpaceRect.x;
+      region.x = NS_UNCONSTRAINEDSIZE - region.width;
     }
   }
   *aIsLeftFloater = isLeftFloater;
@@ -1031,7 +1052,7 @@ nsBlockReflowState::FlowAndPlaceFloater(nsFloaterCache* aFloaterCache,
     // if the floater split, then take up all of the vertical height 
     if (NS_FRAME_IS_NOT_COMPLETE(aReflowStatus) && 
         (NS_UNCONSTRAINEDSIZE != mContentArea.height)) {
-      region.height += PR_MAX(region.height, mContentArea.height);
+      region.height = PR_MAX(region.height, mContentArea.height);
     }
 #ifdef DEBUG
     nsresult rv =
@@ -1161,7 +1182,7 @@ nsBlockReflowState::PlaceBelowCurrentLineFloaters(nsFloaterCacheList& aList)
       }
       else if (NS_FRAME_IS_NOT_COMPLETE(reflowStatus)) {
         // Create a continuation for the incomplete floater and its placeholder.
-        nsresult rv = mBlock->SplitPlaceholder(*this, *fc->mPlaceholder);
+        nsresult rv = mBlock->SplitPlaceholder(*mPresContext, *fc->mPlaceholder);
         if (NS_FAILED(rv)) 
           return PR_FALSE;
       }
