@@ -54,6 +54,7 @@
 #include "nsIPresShell.h"
 #include "nsCOMPtr.h"
 #include "nsLayoutAtoms.h"
+#include "nsCSSAnonBoxes.h"
 #include "nsIViewManager.h"
 #include "nsIWidget.h"
 #include "nsGfxCIID.h"
@@ -61,6 +62,7 @@
 #include "nsCSSRendering.h"
 #include "nsTransform2D.h"
 #include "nsRegion.h"
+#include "nsLayoutErrors.h"
 
 static NS_DEFINE_CID(kRegionCID, NS_REGION_CID);
 
@@ -854,7 +856,7 @@ nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
 
   nsCOMPtr<nsIAtom>  pseudoTag;
   aStyleContext->GetPseudoType(*getter_AddRefs(pseudoTag));
-  if (pseudoTag == nsLayoutAtoms::scrolledContentPseudo) {
+  if (pseudoTag == nsCSSAnonBoxes::scrolledContent) {
     return PR_TRUE;
   }
 
