@@ -44,6 +44,7 @@
 #include "pldhash.h"
 #include "nsPlaceholderFrame.h"
 #include "nsLayoutAtoms.h"
+#include "nsCSSAnonBoxes.h"
 #include "nsHTMLAtoms.h"
 #ifdef NS_DEBUG
 #include "nsISupportsArray.h"
@@ -73,6 +74,7 @@
 #include "nsUnicharUtils.h"
 #include "nsPrintfCString.h"
 #include "nsDummyLayoutRequest.h"
+#include "nsLayoutErrors.h"
 
   #ifdef DEBUG
     //#define NOISY_DEBUG
@@ -369,10 +371,10 @@ public:
                               nsIAtom*  aPropertyName,
                               PRUint32  aOptions,
                               void**    aPropertyValue);
-  NS_IMETHOD SetFrameProperty(nsIFrame*            aFrame,
-                              nsIAtom*             aPropertyName,
-                              void*                aPropertyValue,
-                              NSFMPropertyDtorFunc aPropDtorFunc);
+  NS_IMETHOD SetFrameProperty(nsIFrame*               aFrame,
+                              nsIAtom*                aPropertyName,
+                              void*                   aPropertyValue,
+                              NSFramePropertyDtorFunc aPropDtorFunc);
   NS_IMETHOD RemoveFrameProperty(nsIFrame* aFrame,
                                  nsIAtom*  aPropertyName);
 
@@ -381,13 +383,13 @@ public:
 #endif
 
   struct PropertyList {
-    nsCOMPtr<nsIAtom>    mName;          // property name
-    PLDHashTable         mFrameValueMap; // map of frame/value pairs
-    NSFMPropertyDtorFunc mDtorFunc;      // property specific value dtor function
-    PropertyList*        mNext;
+    nsCOMPtr<nsIAtom>       mName;          // property name
+    PLDHashTable            mFrameValueMap; // map of frame/value pairs
+    NSFramePropertyDtorFunc mDtorFunc;      // property specific value dtor function
+    PropertyList*           mNext;
 
-    PropertyList(nsIAtom*             aName,
-                 NSFMPropertyDtorFunc aDtorFunc);
+    PropertyList(nsIAtom*                aName,
+                 NSFramePropertyDtorFunc aDtorFunc);
     ~PropertyList();
 
     // Removes the property associated with the given frame, and destroys
@@ -783,6 +785,8 @@ FrameManager::RegisterPlaceholderFrame(nsPlaceholderFrame* aPlaceholderFrame)
                               PL_DHASH_ADD));
   if (!entry)
     return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ASSERTION(!entry->placeholderFrame, "Registering a placeholder for a frame that already has a placeholder!");
   entry->placeholderFrame = aPlaceholderFrame;
   return NS_OK;
 }
@@ -1729,7 +1733,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
     
     // do primary context
     nsIStyleContext* newContext = nsnull;
-    if (pseudoTag == nsHTMLAtoms::mozNonElementPseudo) {
+    if (pseudoTag == nsCSSAnonBoxes::mozNonElement) {
       NS_ASSERTION(localContent,
                    "non pseudo-element frame without content node");
       aPresContext->ResolveStyleContextForNonElement(parentContext,
@@ -1785,7 +1789,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
         }
       }
       else {
-        if (pseudoTag && pseudoTag != nsHTMLAtoms::mozNonElementPseudo &&
+        if (pseudoTag && pseudoTag != nsCSSAnonBoxes::mozNonElement &&
             aAttribute &&
             (!(aMinChange &
                (nsChangeHint_ReflowFrame | nsChangeHint_ReconstructFrame
@@ -1812,7 +1816,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
           nsIStyleContext* newExtraContext = nsnull;
           oldExtraContext->GetPseudoType(pseudoTag);
           NS_ASSERTION(pseudoTag &&
-                       pseudoTag != nsHTMLAtoms::mozNonElementPseudo,
+                       pseudoTag != nsCSSAnonBoxes::mozNonElement,
                        "extra style context is not pseudo element");
           result = aPresContext->ResolvePseudoStyleContextFor(content, pseudoTag, newContext,
                                                               &newExtraContext);
@@ -1829,7 +1833,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
 #if 0
               // XXXldb |oldContext| is null by this point, so this will
               // never do anything.
-              if (pseudoTag && pseudoTag != nsHTMLAtoms::mozNonElementPseudo &&
+              if (pseudoTag && pseudoTag != nsCSSAnonBoxes::mozNonElement &&
                   aAttribute && (aMinChange < NS_STYLE_HINT_REFLOW) &&
                   HasAttributeContent(oldContext, aAttrNameSpaceID, aAttribute)) {
                 aChangeList.AppendChange(aFrame, content, NS_STYLE_HINT_REFLOW);
@@ -1857,7 +1861,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
                                                newContext, 
                                                &undisplayedContext);
         }
-        else if (pseudoTag == nsHTMLAtoms::mozNonElementPseudo) {
+        else if (pseudoTag == nsCSSAnonBoxes::mozNonElement) {
           aPresContext->ResolveStyleContextForNonElement(newContext, 
                                                          &undisplayedContext);
         }
@@ -2448,10 +2452,10 @@ FrameManager::GetFrameProperty(nsIFrame* aFrame,
 }
 
 NS_IMETHODIMP
-FrameManager::SetFrameProperty(nsIFrame*            aFrame,
-                               nsIAtom*             aPropertyName,
-                               void*                aPropertyValue,
-                               NSFMPropertyDtorFunc aPropDtorFunc)
+FrameManager::SetFrameProperty(nsIFrame*               aFrame,
+                               nsIAtom*                aPropertyName,
+                               void*                   aPropertyValue,
+                               NSFramePropertyDtorFunc aPropDtorFunc)
 {
   NS_ENSURE_TRUE(mPresShell, NS_ERROR_NOT_AVAILABLE);
   NS_PRECONDITION(aPropertyName && aFrame, "unexpected null param");
@@ -2663,8 +2667,8 @@ UndisplayedMap::Clear(void)
 
 //----------------------------------------------------------------------
     
-FrameManager::PropertyList::PropertyList(nsIAtom*             aName,
-                                         NSFMPropertyDtorFunc aDtorFunc)
+FrameManager::PropertyList::PropertyList(nsIAtom*                aName,
+                                         NSFramePropertyDtorFunc aDtorFunc)
   : mName(aName), mDtorFunc(aDtorFunc), mNext(nsnull)
 {
   PL_DHashTableInit(&mFrameValueMap, PL_DHashGetStubOps(), this,
