@@ -38,15 +38,16 @@
 #include "nsCOMPtr.h"
 #include "nsImageFrame.h"
 #include "nsString.h"
+#include "nsPrintfCString.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
 #include "nsIPresShell.h"
-#include "nsHTMLIIDs.h"
 #include "nsIImage.h"
 #include "nsIWidget.h"
 #include "nsHTMLAtoms.h"
 #include "nsIHTMLContent.h"
 #include "nsIDocument.h"
+#include "nsINodeInfo.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsImageMap.h"
@@ -104,16 +105,16 @@
 #endif
 
 
-// Value's for mSuppress
-#define SUPPRESS_UNSET   0
-#define DONT_SUPPRESS    1
-#define SUPPRESS         2
-#define DEFAULT_SUPPRESS 3
-
 // values for image loading icons
 #define NS_ICON_LOADING_IMAGE (0)
 #define NS_ICON_BROKEN_IMAGE  (1)
 
+// sizes (pixels) for image icon, padding and border frame
+#define ICON_SIZE        (16)
+#define ICON_PADDING     (3)
+#define ALT_BORDER_WIDTH (1)
+
+
 // Default alignment value (so we can tell an unset value from a set value)
 #define ALIGN_UNSET PRUint8(-1)
 
@@ -141,7 +142,20 @@ static PRBool HaveFixedSize(const struct nsStylePosition& aStylePosition)
 inline PRBool HaveFixedSize(const nsHTMLReflowState& aReflowState)
 { 
   NS_ASSERTION(aReflowState.mStylePosition, "crappy reflowState - null stylePosition");
-  return HaveFixedSize(*(aReflowState.mStylePosition)); 
+  // when an image has percent css style height or width, but mComputedHeight 
+  // or mComputedWidth of reflow state is  NS_UNCONSTRAINEDSIZE  
+  // it needs to return PR_FALSE to cause an incremental reflow later
+  // if an image is inside table like bug 156731 simple testcase III, 
+  // during pass 1 reflow, mComputedWidth is NS_UNCONSTRAINEDSIZE
+  // in pass 2 reflow, mComputedWidth is 0, it also needs to return PR_FALSE
+  // see bug 156731
+  nsStyleUnit heightUnit = (*(aReflowState.mStylePosition)).mHeight.GetUnit();
+  nsStyleUnit widthUnit = (*(aReflowState.mStylePosition)).mWidth.GetUnit();
+  return ((eStyleUnit_Percent == heightUnit && NS_UNCONSTRAINEDSIZE == aReflowState.mComputedHeight) ||
+          (eStyleUnit_Percent == widthUnit && (NS_UNCONSTRAINEDSIZE == aReflowState.mComputedWidth ||
+           0 == aReflowState.mComputedWidth)))
+          ? PR_FALSE
+          : HaveFixedSize(*(aReflowState.mStylePosition)); 
 }
 
 nsresult
@@ -297,7 +311,7 @@ nsImageFrame::Init(nsIPresContext*  aPresContext,
   // See if we have a SRC attribute
   nsAutoString src;
   nsresult ca;
-  ca = mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::src, src);
+  ca = mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, src);
   if ((NS_CONTENT_ATTR_HAS_VALUE != ca) || (src.Length() == 0))
   {
     // Let's see if this is an object tag and we have a DATA attribute
@@ -305,7 +319,7 @@ nsImageFrame::Init(nsIPresContext*  aPresContext,
     mContent->GetTag(tag);
 
     if(tag == nsHTMLAtoms::object)
-      mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::data, src);
+      mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::data, src);
   
     NS_IF_RELEASE(tag);
   }
@@ -614,26 +628,30 @@ NS_IMETHODIMP nsImageFrame::OnStopDecode(imgIRequest *aRequest, nsIPresContext *
 
       mLoads[1].mRequest = nsnull;
 
-      if (!mSizeConstrained && (mLoads[0].mIntrinsicSize != mIntrinsicSize)) {
-        nsCOMPtr<nsIPresShell> presShell;
-        aPresContext->GetShell(getter_AddRefs(presShell));
-        NS_ASSERTION(mParent, "No parent to pass the reflow request up to.");
-        NS_ASSERTION(presShell, "No PresShell.");
-        if (mParent && presShell && mGotInitialReflow) { // don't reflow if we havn't gotten the inital reflow yet
-          mState |= NS_FRAME_IS_DIRTY;
-          mParent->ReflowDirtyChild(presShell, NS_STATIC_CAST(nsIFrame*, this));
-        }
-      } else {
-        nsSize s;
-        GetSize(s);
-        nsRect r(0,0, s.width, s.height);
-        if (!r.IsEmpty()) {
-          Invalidate(aPresContext, r, PR_FALSE);
-        }
-      }
-
     } else {
+      // load failed, we need to reset mLoads[0] to get an broken icon
+      // since the src attribute changed
+      mLoads[0].mRequest = nsnull;
       mLoads[1].mRequest = nsnull;
+      imageFailedToLoad = PR_TRUE;
+    }
+
+    if (!mSizeConstrained && (mLoads[0].mIntrinsicSize != mIntrinsicSize)) {
+      nsCOMPtr<nsIPresShell> presShell;
+      aPresContext->GetShell(getter_AddRefs(presShell));
+      NS_ASSERTION(mParent, "No parent to pass the reflow request up to.");
+      NS_ASSERTION(presShell, "No PresShell.");
+      if (mParent && presShell && mGotInitialReflow) { // don't reflow if we havn't gotten the inital reflow yet
+        mState |= NS_FRAME_IS_DIRTY;
+        mParent->ReflowDirtyChild(presShell, NS_STATIC_CAST(nsIFrame*, this));
+      }
+    } else {
+      nsSize s;
+      GetSize(s);
+      nsRect r(0,0, s.width, s.height);
+      if (!r.IsEmpty()) {
+        Invalidate(aPresContext, r, PR_FALSE);
+      }
     }
 
   } else if (NS_FAILED(aStatus)) {
@@ -650,10 +668,10 @@ NS_IMETHODIMP nsImageFrame::OnStopDecode(imgIRequest *aRequest, nsIPresContext *
     if (mFailureReplace) {
       // first check for image map
       nsAutoString usemap;
-      mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::usemap, usemap);    
+      mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::usemap, usemap);    
       if (usemap.IsEmpty()) {
         // check if we want to honor the ALT text in the IMG frame, or let the preShell make it into inline text
-        //  - if QuirksMode, and the IMG has a size, then render the ALT text in the ING frame
+        //  - if QuirksMode, and the IMG has a size, then render the ALT text in the IMG frame
         //    UNLESS there is a pref set to force inline alt text
         PRBool useSizedBox = PR_FALSE;
         PRBool prefForceInlineAltText = mIconLoad ? mIconLoad->mPrefForceInlineAltText : PR_FALSE;
@@ -667,11 +685,24 @@ NS_IMETHODIMP nsImageFrame::OnStopDecode(imgIRequest *aRequest, nsIPresContext *
         nsCompatibility mode;
         aPresContext->GetCompatibilityMode(&mode);
         
-        // wrap it all up
-        useSizedBox = !prefForceInlineAltText &&
-                      HaveFixedSize(*stylePosition) && 
-                      mode == eCompatibility_NavQuirks &&
-                      !mImageBlocked;
+        // check for being in Composer: if we are, we always show the size-box
+        PRBool forceIcon = PR_FALSE;
+
+        // check for style property that indicates the icon should always be shown
+        const nsStyleUIReset* styleData;
+        GetStyleData(eStyleStruct_UIReset, (const nsStyleStruct*&) styleData);
+        if (styleData->mForceBrokenImageIcon) {
+          forceIcon = PR_TRUE;
+        }
+
+        // wrap it all up: use the size box if
+        // - in editor
+        // - not forcing inline alt text, have fixed size, in quirks mode, image not blocked
+        useSizedBox = forceIcon ||
+                      (!prefForceInlineAltText &&
+                       HaveFixedSize(*stylePosition) && 
+                       mode == eCompatibility_NavQuirks &&
+                       !mImageBlocked);
 
         if (!useSizedBox) {
           // let the presShell handle converting this into the inline alt text frame
@@ -708,6 +739,12 @@ NS_IMETHODIMP nsImageFrame::FrameChanged(imgIContainer *aContainer, nsIPresConte
   if (!mLoads[0].mRequest)
     return NS_OK; // if mLoads[0].mRequest is null, this isn't for the first one, so we don't care about it.
 
+  const nsStyleVisibility* vis;
+  ::GetStyleData(this, &vis);
+
+  if (!vis->IsVisible())
+    return NS_OK;
+
   nsCOMPtr<imgIContainer> con;
   mLoads[0].mRequest->GetImage(getter_AddRefs(con));
   if (aContainer == con.get()) {
@@ -722,6 +759,8 @@ NS_IMETHODIMP nsImageFrame::FrameChanged(imgIContainer *aContainer, nsIPresConte
 
     mLoads[0].mTransform.TransformCoord(&r.x, &r.y, &r.width, &r.height);
     if (!r.IsEmpty()) {
+      r.x += mBorderPadding.left;
+      r.y += mBorderPadding.top;
       Invalidate(aPresContext, r, PR_FALSE);
     }
   }
@@ -745,12 +784,39 @@ nsImageFrame::GetDesiredSize(nsIPresContext* aPresContext,
   // check to see if the size is already known by the image container.
   if (mLoads[0].mIntrinsicSize.width == 0 &&
       mLoads[0].mIntrinsicSize.height == 0) {
+    PRBool needMinIconSizing = PR_FALSE;
+    float p2t;
+    aPresContext->GetPixelsToTwips(&p2t);
     if (mLoads[0].mRequest) {
       nsCOMPtr<imgIContainer> con;
       mLoads[0].mRequest->GetImage(getter_AddRefs(con));
       if (con) {
-        con->GetWidth(&mLoads[0].mIntrinsicSize.width);
-        con->GetHeight(&mLoads[0].mIntrinsicSize.height);
+        nscoord width, height;
+
+        con->GetWidth(&width);
+        con->GetHeight(&height);
+        mLoads[0].mIntrinsicSize.width = NSIntPixelsToTwips(width, p2t);
+        mLoads[0].mIntrinsicSize.height = NSIntPixelsToTwips(height, p2t);
+      } else {
+        // no image container, so we may need to synthesize a minimum size for the image icons
+        needMinIconSizing = PR_TRUE;
+      }
+    } else {
+      // no request, so we may need to synthesize a minimum size for the image icons
+      needMinIconSizing = PR_TRUE;
+    }
+
+    if (needMinIconSizing) {
+      // XXX:  we need this in composer, but it is also good for general quirks mode to always
+      // XXX:  have room for the icon
+      // check for quirks mode
+      nsCompatibility mode;
+      aPresContext->GetCompatibilityMode(&mode);
+      if (mode == eCompatibility_NavQuirks) {
+        // image request is null or image size not known, probably an invalid image specified
+        // - make the image big enough for the icon (it may not be used if inline alt expansion is used instead)
+        mLoads[0].mIntrinsicSize.width = NSIntPixelsToTwips(ICON_SIZE+(2*(ICON_PADDING+ALT_BORDER_WIDTH)), p2t);
+        mLoads[0].mIntrinsicSize.height = NSIntPixelsToTwips(ICON_SIZE+(2*(ICON_PADDING+ALT_BORDER_WIDTH)), p2t);
       }
     }
   }
@@ -908,14 +974,22 @@ nsImageFrame::Reflow(nsIPresContext*          aPresContext,
   //  in Paginated mode, we need to have a constrained height, and have a height larger than our available height
   PRBool isPaginated;
   aPresContext->IsPaginated(&isPaginated);
-  if ((isPaginated) && 
+  PRUint32 loadStatus = imgIRequest::STATUS_NONE;
+  if (mLoads[0].mRequest) {
+    mLoads[0].mRequest->GetImageStatus(&loadStatus);
+  }
+  if (isPaginated &&
+      ((loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE) || mSizeConstrained) &&
       (NS_UNCONSTRAINEDSIZE != aReflowState.availableHeight) && 
       (aMetrics.height > aReflowState.availableHeight)) { 
     nsCOMPtr<nsIAtom> fType;
     GetFrameType(getter_AddRefs(fType));
     // split an image frame but not an image control frame
     if (nsLayoutAtoms::imageFrame == fType.get()) {
-      aMetrics.height = aReflowState.availableHeight;
+      float p2t;
+      aPresContext->GetScaledPixelsToTwips(&p2t);
+      // our desired height was greater than 0, so to avoid infinite splitting, use 1 pixel as the min
+      aMetrics.height = PR_MAX(NSToCoordRound(p2t), aReflowState.availableHeight);
       aStatus = NS_FRAME_NOT_COMPLETE;
     }
   }
@@ -938,6 +1012,7 @@ nsImageFrame::Reflow(nsIPresContext*          aPresContext,
   NS_FRAME_TRACE(NS_FRAME_TRACE_CALLS,
                   ("exit nsImageFrame::Reflow: size=%d,%d",
                   aMetrics.width, aMetrics.height));
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aMetrics);
   return NS_OK;
 }
 
@@ -1015,12 +1090,9 @@ nsImageFrame::DisplayAltText(nsIPresContext*      aPresContext,
 {
   const nsStyleColor* color =
     (const nsStyleColor*)mStyleContext->GetStyleData(eStyleStruct_Color);
-  const nsStyleFont* font =
-    (const nsStyleFont*)mStyleContext->GetStyleData(eStyleStruct_Font);
-
   // Set font and color
   aRenderingContext.SetColor(color->mColor);
-  aRenderingContext.SetFont(font->mFont);
+  SetFontFromStyle(&aRenderingContext, mStyleContext);
 
   // Format the text to display within the formatting rect
   nsIFontMetrics* fm;
@@ -1079,10 +1151,6 @@ struct nsRecessedBorder : public nsStyleBorder {
   }
 };
 
-#define ICON_SIZE        (16)
-#define ICON_PADDING     (6)
-#define ALT_BORDER_WIDTH (1)
-
 void
 nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
                                  nsIRenderingContext& aRenderingContext,
@@ -1098,6 +1166,12 @@ nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
   aPresContext->GetScaledPixelsToTwips(&p2t);
   borderEdgeWidth = NSIntPixelsToTwips(ALT_BORDER_WIDTH, p2t);
 
+  // if inner area is empty, then make it big enough for at least the icon
+  if (inner.IsEmpty()){
+    inner.SizeBy(2*(NSIntPixelsToTwips(ICON_SIZE+ICON_PADDING+ALT_BORDER_WIDTH,p2t)),
+                 2*(NSIntPixelsToTwips(ICON_SIZE+ICON_PADDING+ALT_BORDER_WIDTH,p2t)));
+  }
+
   // Make sure we have enough room to actually render the border within
   // our frame bounds
   if ((inner.width < 2 * borderEdgeWidth) || (inner.height < 2 * borderEdgeWidth)) {
@@ -1133,46 +1207,51 @@ nsImageFrame::DisplayAltFeedback(nsIPresContext*      aPresContext,
   aRenderingContext.PushState();
   aRenderingContext.SetClipRect(inner, nsClipCombine_kIntersect, clipState);
 
-  PRInt32 size = NSIntPixelsToTwips(ICON_SIZE, p2t);
+  PRBool dispIcon = mIconLoad ? mIconLoad->mPrefShowPlaceholders : PR_TRUE;
 
-  PRBool iconUsed = PR_FALSE;
+  // Check if we should display image placeholders
+  if (dispIcon) {
+    PRInt32 size = NSIntPixelsToTwips(ICON_SIZE, p2t);
 
-  // see if the icon images are present...
-  if (mIconLoad && mIconLoad->mIconsLoaded) {
-    // pick the correct image
-    NS_ASSERTION( aIconId == NS_ICON_LOADING_IMAGE ||
-                  aIconId == NS_ICON_BROKEN_IMAGE, "Invalid Icon ID in DisplayAltFeedback");
+    PRBool iconUsed = PR_FALSE;
 
-    nsCOMPtr<imgIContainer> imgCon;
-    if (mIconLoad->mIconLoads[aIconId].mRequest) {
-      mIconLoad->mIconLoads[aIconId].mRequest->GetImage(getter_AddRefs(imgCon));
-    }
-    if (imgCon) { 
-      // draw it
-      nsPoint p(inner.x, inner.y);
-      nsRect r(0,0,size,size);
-      aRenderingContext.DrawImage(imgCon, &r, &p);
-      iconUsed = PR_TRUE;
+    // see if the icon images are present...
+    if (mIconLoad && mIconLoad->mIconsLoaded) {
+      // pick the correct image
+      NS_ASSERTION( aIconId == NS_ICON_LOADING_IMAGE ||
+                    aIconId == NS_ICON_BROKEN_IMAGE, "Invalid Icon ID in DisplayAltFeedback");
+
+      nsCOMPtr<imgIContainer> imgCon;
+      if (mIconLoad->mIconLoads[aIconId].mRequest) {
+        mIconLoad->mIconLoads[aIconId].mRequest->GetImage(getter_AddRefs(imgCon));
+      }
+      if (imgCon) { 
+        // draw it
+        nsPoint p(inner.x, inner.y);
+        nsRect r(0,0,size,size);
+        aRenderingContext.DrawImage(imgCon, &r, &p);
+        iconUsed = PR_TRUE;
+      }
     }
+
+    // if we could not draw the image, then just draw some grafitti
+    if (!iconUsed) {
+      nscolor oldColor;
+      aRenderingContext.DrawRect(0,0,size,size);
+      aRenderingContext.GetColor(oldColor);
+      aRenderingContext.SetColor( aIconId == NS_ICON_BROKEN_IMAGE ? NS_RGB(0xFF,0,0) : NS_RGB(0,0xFF,0));
+      aRenderingContext.FillEllipse(NS_STATIC_CAST(int,size/2),NS_STATIC_CAST(int,size/2),
+                                    NS_STATIC_CAST(int,(size/2)-(2*p2t)),NS_STATIC_CAST(int,(size/2)-(2*p2t)));
+      aRenderingContext.SetColor(oldColor);
+    }  
+
+    // Reduce the inner rect by the width of the icon, and leave an
+    // additional ICON_PADDING pixels for padding
+    PRInt32 iconWidth = NSIntPixelsToTwips(ICON_SIZE + ICON_PADDING, p2t);
+    inner.x += iconWidth;
+    inner.width -= iconWidth;
   }
 
-  // if we could not draw the image, then just draw some grafitti
-  if (!iconUsed) {
-    nscolor oldColor;
-    aRenderingContext.DrawRect(0,0,size,size);
-    aRenderingContext.GetColor(oldColor);
-    aRenderingContext.SetColor( aIconId == NS_ICON_BROKEN_IMAGE ? NS_RGB(0xFF,0,0) : NS_RGB(0,0xFF,0));
-    aRenderingContext.FillEllipse(NS_STATIC_CAST(int,size/2),NS_STATIC_CAST(int,size/2),
-                                  NS_STATIC_CAST(int,(size/2)-(2*p2t)),NS_STATIC_CAST(int,(size/2)-(2*p2t)));
-    aRenderingContext.SetColor(oldColor);
-  }  
-
-  // Reduce the inner rect by the width of the icon, and leave an
-  // additional ICON_PADDING pixels for padding
-  PRInt32 iconWidth = NSIntPixelsToTwips(ICON_SIZE + ICON_PADDING, p2t);
-  inner.x += iconWidth;
-  inner.width -= iconWidth;
-
   // If there's still room, display the alt-text
   if (!inner.IsEmpty()) {
     nsCOMPtr<nsIContent> content;
@@ -1212,24 +1291,30 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
       return NS_OK;
 
 
-    // First paint background and borders
-    if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+    // First paint background and borders, which should be in the
+    // FOREGROUND or BACKGROUND paint layer if the element is
+    // inline-level or block-level, respectively.  (See CSS2 9.5, which
+    // is the rationale for paint layers.)
+    const nsStyleDisplay *display;
+    ::GetStyleData(NS_STATIC_CAST(nsIStyleContext*, mStyleContext), &display);
+    nsFramePaintLayer backgroundLayer = display->IsBlockLevel()
+                                            ? NS_FRAME_PAINT_LAYER_BACKGROUND
+                                            : NS_FRAME_PAINT_LAYER_FOREGROUND;
+    if (aWhichLayer == backgroundLayer) {
       const nsStyleVisibility* vis = 
         (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
       if (vis->IsVisibleOrCollapsed()) {
         const nsStyleBorder* myBorder = (const nsStyleBorder*)
           mStyleContext->GetStyleData(eStyleStruct_Border);
-        const nsStyleOutline* myOutline = (const nsStyleOutline*)
-          mStyleContext->GetStyleData(eStyleStruct_Outline);
+        const nsStylePadding* myPadding = (const nsStylePadding*)
+          mStyleContext->GetStyleData(eStyleStruct_Padding);
         nsRect rect(0, 0, mRect.width, mRect.height);
         nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *myBorder, 0, 0);
+                                        aDirtyRect, rect, *myBorder, *myPadding,
+                                        0, 0);
         nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
                                     aDirtyRect, rect, *myBorder,
                                     mStyleContext, 0);
-        nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
-                                     aDirtyRect, rect, *myBorder,
-                                     *myOutline, mStyleContext, 0);
       }
     }
 
@@ -1245,9 +1330,10 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
     if (loadStatus & imgIRequest::STATUS_ERROR || !imgCon) {
       // No image yet, or image load failed. Draw the alt-text and an icon
       // indicating the status (unless image is blocked, in which case we show nothing)
+
 #ifndef SUPPRESS_LOADING_ICON
       if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer &&
-          !mImageBlocked) {
+          (!mImageBlocked || mIconLoad->mPrefAllImagesBlocked)) {
 #else
       if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer &&
           mInitialLoadCompleted) {
@@ -1259,6 +1345,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
       }
     }
     else {
+      PRBool paintOutline   = PR_FALSE;
       mInitialLoadCompleted = PR_TRUE;
       if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
         // Now render the image into our content area (the area inside the
@@ -1321,6 +1408,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
 
             aRenderingContext.DrawScaledImage(imgCon, &r, &d);
           }
+          paintOutline = PR_TRUE;
         }
       }
 
@@ -1335,6 +1423,20 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
         aRenderingContext.Translate(inner.x, inner.y);
         map->Draw(aPresContext, aRenderingContext);
         aRenderingContext.PopState(clipState);
+        paintOutline = PR_TRUE;
+      }
+
+      // paint the outline in the overlay layer (or if there is an image map) until the 
+      // general problem of painting it outside the border box is solved.
+      if (paintOutline) {
+        const nsStyleBorder* myBorder = (const nsStyleBorder*)
+          mStyleContext->GetStyleData(eStyleStruct_Border);
+        const nsStyleOutline* myOutline = (const nsStyleOutline*)
+          mStyleContext->GetStyleData(eStyleStruct_Outline);
+        nsRect rect(0, 0, mRect.width, mRect.height);
+        nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
+                                     aDirtyRect, rect, *myBorder,
+                                     *myOutline, mStyleContext, 0);
       }
 
 #ifdef DEBUG
@@ -1369,7 +1471,14 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
   if (!(displaySelection & nsISelectionDisplay::DISPLAY_IMAGES))
     return NS_OK;//no need to check the blue border, we cannot be drawn selected
   
-  return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, nsISelectionDisplay::DISPLAY_IMAGES);
+}
+
+NS_IMETHODIMP
+nsImageFrame::GetImageMap(nsIPresContext *aPresContext, nsIImageMap **aImageMap)
+{
+  nsImageMap *map = GetImageMap(aPresContext);
+  return map->QueryInterface(NS_GET_IID(nsIImageMap), (void**)aImageMap);
 }
 
 nsImageMap*
@@ -1383,7 +1492,7 @@ nsImageFrame::GetImageMap(nsIPresContext* aPresContext)
     }
 
     nsAutoString usemap;
-    mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::usemap, usemap);
+    mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::usemap, usemap);
 
     nsCOMPtr<nsIDOMHTMLMapElement> map;
     if (NS_SUCCEEDED(nsImageMapUtils::FindImageMap(doc,usemap,getter_AddRefs(map))) && map) {
@@ -1403,13 +1512,13 @@ nsImageFrame::GetImageMap(nsIPresContext* aPresContext)
 
 void
 nsImageFrame::TriggerLink(nsIPresContext* aPresContext,
-                          const nsString& aURLSpec,
+                          nsIURI* aURI,
                           const nsString& aTargetSpec,
                           PRBool aClick)
 {
   // We get here with server side image map
-  nsILinkHandler* handler = nsnull;
-  aPresContext->GetLinkHandler(&handler);
+  nsCOMPtr<nsILinkHandler> handler;
+  aPresContext->GetLinkHandler(getter_AddRefs(handler));
   if (nsnull != handler) {
     if (aClick) {
       nsresult proceed = NS_OK;
@@ -1429,22 +1538,18 @@ nsImageFrame::TriggerLink(nsIPresContext* aPresContext,
       nsCOMPtr<nsIURI> baseURI;
       if (NS_SUCCEEDED(rv) && doc) 
         doc->GetDocumentURL(getter_AddRefs(baseURI));
-      nsCOMPtr<nsIURI> absURI;
-      if (NS_SUCCEEDED(rv)) 
-        rv = NS_NewURI(getter_AddRefs(absURI), aURLSpec, baseURI);
       
       if (NS_SUCCEEDED(rv)) 
-        proceed = securityManager->CheckLoadURI(baseURI, absURI, nsIScriptSecurityManager::STANDARD);
+        proceed = securityManager->CheckLoadURI(baseURI, aURI, nsIScriptSecurityManager::STANDARD);
 
       // Only pass off the click event if the script security manager
       // says it's ok.
       if (NS_SUCCEEDED(proceed))
-        handler->OnLinkClick(mContent, eLinkVerb_Replace, aURLSpec.get(), aTargetSpec.get());
+        handler->OnLinkClick(mContent, eLinkVerb_Replace, aURI, aTargetSpec.get());
     }
     else {
-      handler->OnOverLink(mContent, aURLSpec.get(), aTargetSpec.get());
+      handler->OnOverLink(mContent, aURI, aTargetSpec.get());
     }
-    NS_RELEASE(handler);
   }
 }
 
@@ -1453,22 +1558,7 @@ nsImageFrame::IsServerImageMap()
 {
   nsAutoString ismap;
   return NS_CONTENT_ATTR_HAS_VALUE ==
-    mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::ismap, ismap);
-}
-
-PRIntn
-nsImageFrame::GetSuppress()
-{
-  nsAutoString s;
-  if (NS_CONTENT_ATTR_HAS_VALUE ==
-      mContent->GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::suppress, s)) {
-    if (s.EqualsIgnoreCase("true")) {
-      return SUPPRESS;
-    } else if (s.EqualsIgnoreCase("false")) {
-      return DONT_SUPPRESS;
-    }
-  }
-  return DEFAULT_SUPPRESS;
+    mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::ismap, ismap);
 }
 
 //XXX the event come's in in view relative coords, but really should
@@ -1567,12 +1657,10 @@ nsImageFrame::GetContentForEvent(nsIPresContext* aPresContext,
     nsPoint p;
     TranslateEventCoords(aPresContext, aEvent->point, p);
     nsAutoString absURL, target, altText;
-    PRBool suppress;
     PRBool inside = PR_FALSE;
     nsCOMPtr<nsIContent> area;
     inside = map->IsInside(p.x, p.y, getter_AddRefs(area),
-                           absURL, target, altText,
-                           &suppress);
+                           absURL, target, altText);
     if (inside && area) {
       *aContent = area;
       NS_ADDREF(*aContent);
@@ -1602,7 +1690,6 @@ nsImageFrame::HandleEvent(nsIPresContext* aPresContext,
         nsPoint p;
         TranslateEventCoords(aPresContext, aEvent->point, p);
         nsAutoString absURL, target, altText;
-        PRBool suppress;
         PRBool inside = PR_FALSE;
         // Even though client-side image map triggering happens
         // through content, we need to make sure we're not inside
@@ -1611,12 +1698,10 @@ nsImageFrame::HandleEvent(nsIPresContext* aPresContext,
         if (nsnull != map) {
           nsCOMPtr<nsIContent> area;
           inside = map->IsInside(p.x, p.y, getter_AddRefs(area),
-                                 absURL, target, altText,
-                                 &suppress);
+                                 absURL, target, altText);
         }
 
         if (!inside && isServerMap) {
-          suppress = GetSuppress();
           nsCOMPtr<nsIURI> baseURL;
           GetBaseURI(getter_AddRefs(baseURL));
           
@@ -1625,7 +1710,20 @@ nsImageFrame::HandleEvent(nsIPresContext* aPresContext,
             // element to provide the basis for the destination url.
             nsAutoString src;
             if (GetAnchorHREFAndTarget(src, target)) {
-              NS_MakeAbsoluteURI(absURL, src, baseURL);
+              nsCOMPtr<nsINodeInfo> nodeInfo;
+              mContent->GetNodeInfo(*getter_AddRefs(nodeInfo));
+              NS_ASSERTION(nodeInfo, "Image content without a nodeinfo?");
+              nsCOMPtr<nsIDocument> doc;
+              nodeInfo->GetDocument(*getter_AddRefs(doc));
+              nsAutoString charset;
+              if (doc) {
+                doc->GetDocumentCharacterSet(charset);
+              } 
+              nsCOMPtr<nsIURI> uri;
+              nsresult rv = NS_NewURI(getter_AddRefs(uri), src,
+                                      NS_LossyConvertUCS2toASCII(charset).get(),
+                                      baseURL);
+              NS_ENSURE_SUCCESS(rv, rv);
             
               // XXX if the mouse is over/clicked in the border/padding area
               // we should probably just pretend nothing happened. Nav4
@@ -1634,15 +1732,22 @@ nsImageFrame::HandleEvent(nsIPresContext* aPresContext,
               // mouse is over the border.
               if (p.x < 0) p.x = 0;
               if (p.y < 0) p.y = 0;
-              char cbuf[50];
-              PR_snprintf(cbuf, sizeof(cbuf), "?%d,%d", p.x, p.y);
-              absURL.AppendWithConversion(cbuf);
+              nsCOMPtr<nsIURL> url = do_QueryInterface(uri);
+              if (url) {
+                url->SetQuery(nsPrintfCString("%d,%d", p.x, p.y));
+              } else {
+                nsCAutoString spec;
+                uri->GetSpec(spec);
+                spec += nsPrintfCString("?%d,%d", p.x, p.y);
+                uri->SetSpec(spec);                
+              }
+              
               PRBool clicked = PR_FALSE;
               if (aEvent->message == NS_MOUSE_LEFT_BUTTON_UP) {
                 *aEventStatus = nsEventStatus_eConsumeDoDefault; 
                 clicked = PR_TRUE;
               }
-              TriggerLink(aPresContext, absURL, target, clicked);
+              TriggerLink(aPresContext, uri, target, clicked);
             }
           } 
           else 
@@ -1703,26 +1808,25 @@ nsImageFrame::AttributeChanged(nsIPresContext* aPresContext,
     nsAutoString newSRC;
     aChild->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, newSRC);
 
-    PRUint32 loadStatus = imgIRequest::STATUS_ERROR;
 
-    if (mLoads[0].mRequest)
+    if (mLoads[0].mRequest) {
+      PRUint32 loadStatus = imgIRequest::STATUS_ERROR;
       mLoads[0].mRequest->GetImageStatus(&loadStatus);
 
-    if (!(loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
-      if (mLoads[0].mRequest) {
+      if (!(loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
         mFailureReplace = PR_FALSE; // don't cause a CantRenderReplacedElement call
         mLoads[0].mRequest->Cancel(NS_ERROR_FAILURE);
         mLoads[0].mRequest = nsnull;
       }
-
-      mCanSendLoadEvent = PR_TRUE;
     }
 
+    mCanSendLoadEvent = PR_TRUE;
+
     if (mLoads[1].mRequest) {
       mLoads[1].mRequest->Cancel(NS_ERROR_FAILURE);
       mLoads[1].mRequest = nsnull;
     }
-    
+
     nsCOMPtr<imgIRequest> req(do_CreateInstance("@mozilla.org/image/request;1"));
     if (!mLoads[0].mRequest) {
       mLoads[0].mRequest = req;
@@ -1775,12 +1879,13 @@ mRect.height);
   fprintf(out, " [content=%p]", mContent);
 
   // output the img src url
-  nsCOMPtr<nsIURI> uri;
-  mLoads[0].mRequest->GetURI(getter_AddRefs(uri));
-  nsXPIDLCString uristr;
-  uri->GetSpec(getter_Copies(uristr));
-  fprintf(out, " [src=%s]", uristr.get());
-
+  if (mLoads[0].mRequest) {
+    nsCOMPtr<nsIURI> uri;
+    mLoads[0].mRequest->GetURI(getter_AddRefs(uri));
+    nsCAutoString uristr;
+    uri->GetAsciiSpec(uristr);
+    fprintf(out, " [src=%s]", uristr.get());
+  }
   fputs("\n", out);
   return NS_OK;
 }
@@ -1841,9 +1946,9 @@ nsImageFrame::IsImageComplete(PRBool* aComplete)
 }
 
 nsresult
-nsImageFrame::LoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest)
+nsImageFrame::LoadImage(const nsAString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest, PRBool aCheckContentPolicy)
 {
-  nsresult rv = RealLoadImage(aSpec, aPresContext, aRequest);
+  nsresult rv = RealLoadImage(aSpec, aPresContext, aRequest, aCheckContentPolicy);
 
   if (NS_FAILED(rv)) {
     int whichLoad = GetImageLoad(aRequest);
@@ -1855,7 +1960,7 @@ nsImageFrame::LoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresCon
 }
 
 nsresult
-nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest)
+nsImageFrame::RealLoadImage(const nsAString& aSpec, nsIPresContext *aPresContext, imgIRequest *aRequest, PRBool aCheckContentPolicy)
 {
   nsresult rv = NS_OK;
 
@@ -1868,8 +1973,11 @@ nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPre
   nsCOMPtr<nsIURI> realURI;
 
   /* don't load the image if some security check fails... */
-  GetRealURI(aSpec, getter_AddRefs(realURI));
-  if (!CanLoadImage(realURI)) return NS_ERROR_FAILURE;
+  nsCOMPtr<nsIIOService> ioService;
+  aPresContext->GetIOService(getter_AddRefs(ioService));
+  GetRealURI(aSpec, ioService, getter_AddRefs(realURI));
+  if (aCheckContentPolicy)
+    if (!CanLoadImage(realURI)) return NS_ERROR_FAILURE;
 
   if (!mListener) {
     nsImageListener *listener = new nsImageListener(this);
@@ -1894,20 +2002,49 @@ nsImageFrame::RealLoadImage(const nsAReadableString& aSpec, nsIPresContext *aPre
   /* get the URI, convert internal-gopher-stuff if needed */
   nsCOMPtr<nsIURI> uri;
   GetURI(aSpec, getter_AddRefs(uri));
-  if (!uri)
+  if (!uri) {
+    if (!realURI) 
+      return NS_ERROR_FAILURE;
     uri = realURI;
-
+  }
+  
   /* set this back to FALSE before we do the real load */
   mInitialLoadCompleted = PR_FALSE;
 
+  nsCOMPtr<nsIURI> baseURI;
+  rv = aPresContext->GetBaseURL(getter_AddRefs(baseURI));
+
+  // Get the document URI for the referrer...
+  nsCOMPtr<nsIURI> documentURI;
+  nsCOMPtr<nsIDocument> doc;
+  if (mContent) {
+    (void) mContent->GetDocument(*getter_AddRefs(doc));
+    if (doc) {
+      doc->GetDocumentURL(getter_AddRefs(documentURI));
+    }
+  }
+
   nsCOMPtr<imgIRequest> tempRequest;
-  return il->LoadImage(uri, loadGroup, mListener, aPresContext, loadFlags, nsnull, aRequest, getter_AddRefs(tempRequest));
+  return il->LoadImage(uri, baseURI, documentURI, loadGroup, mListener, aPresContext, loadFlags, nsnull, aRequest, getter_AddRefs(tempRequest));
 }
 
 #define INTERNAL_GOPHER_LENGTH 16 /* "internal-gopher-" length */
 
 void
-nsImageFrame::GetURI(const nsAReadableString& aSpec, nsIURI **aURI)
+nsImageFrame::GetDocumentCharacterSet(nsAString& aCharset) const
+{
+  nsresult rv;
+  nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(mContent, &rv));
+  if (NS_SUCCEEDED(rv)) {
+    nsCOMPtr<nsIDocument> doc;
+    rv = htmlContent->GetDocument(*getter_AddRefs(doc));
+    if (NS_SUCCEEDED(rv))
+      (void) doc->GetDocumentCharacterSet(aCharset);
+  }
+}
+
+void
+nsImageFrame::GetURI(const nsAString& aSpec, nsIURI **aURI)
 {
   *aURI = nsnull;
 
@@ -1924,16 +2061,22 @@ nsImageFrame::GetURI(const nsAReadableString& aSpec, nsIURI **aURI)
     newURI.Assign(NS_LITERAL_STRING("resource:/res/html/gopher-") +
                   Substring(aSpec, INTERNAL_GOPHER_LENGTH, aSpec.Length() - INTERNAL_GOPHER_LENGTH) +
                   NS_LITERAL_STRING(".gif"));
-    GetRealURI(newURI, aURI);
+    // XXXldb Using GetRealURI seems silly.
+    GetRealURI(newURI, nsnull, aURI);
   }
 }
 
 void
-nsImageFrame::GetRealURI(const nsAReadableString& aSpec, nsIURI **aURI)
+nsImageFrame::GetRealURI(const nsAString& aSpec, nsIIOService *aIOService,
+                         nsIURI **aURI)
 {
   nsCOMPtr<nsIURI> baseURI;
   GetBaseURI(getter_AddRefs(baseURI));
-  NS_NewURI(aURI, aSpec, baseURI);
+  nsAutoString charset;
+  GetDocumentCharacterSet(charset);
+  NS_NewURI(aURI, aSpec, 
+            charset.IsEmpty() ? nsnull : NS_ConvertUCS2toUTF8(charset).get(), 
+            baseURI, aIOService);
 }
 
 void
@@ -2011,7 +2154,7 @@ nsImageFrame::CanLoadImage(nsIURI *aURI)
     nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalScript));
 
     rv = NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
-                                 aURI, element, domWin, &shouldLoad);
+                                   aURI, element, domWin, &shouldLoad);
     if (NS_SUCCEEDED(rv) && !shouldLoad) {
       // this image has been blocked, so flag it
       mImageBlocked = PR_TRUE;
@@ -2059,14 +2202,14 @@ nsresult nsImageFrame::LoadIcons(nsIPresContext *aPresContext)
 #ifdef NOISY_ICON_LOADING
       printf( "Loading request %p\n", mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest.get() );
 #endif
-      rv = LoadImage(loadingSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest);
+      rv = LoadImage(loadingSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_LOADING_IMAGE].mRequest, PR_FALSE);
       if (NS_SUCCEEDED(rv)) {
         mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest = do_CreateInstance("@mozilla.org/image/request;1");
         if (mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest) {
 #ifdef NOISY_ICON_LOADING
           printf( "Loading request %p\n", mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest.get() );
 #endif
-          rv = LoadImage(brokenSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest);
+          rv = LoadImage(brokenSrc, aPresContext, mIconLoad->mIconLoads[NS_ICON_BROKEN_IMAGE].mRequest, PR_FALSE);
           // ImageLoader will callback into OnStartContainer, which will handle the mIconsLoaded flag
         }
       }
@@ -2122,7 +2265,7 @@ void nsImageFrame::InvalidateIcon(nsIPresContext *aPresContext)
   Invalidate(aPresContext, rect, PR_FALSE);
 }
 
-void nsImageFrame::IconLoad::GetAltModePref(nsIPresContext *aPresContext)
+void nsImageFrame::IconLoad::GetPrefs(nsIPresContext *aPresContext)
 {
   NS_ASSERTION(aPresContext, "null presContext is not allowed in GetAltModePref");
   // NOTE: the presContext could be used to fetch a cached pref if needed, but is not for now
@@ -2130,11 +2273,22 @@ void nsImageFrame::IconLoad::GetAltModePref(nsIPresContext *aPresContext)
   nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID);
   if (prefs) {
     PRBool boolPref;
+    PRInt32 intPref;
     if (NS_SUCCEEDED(prefs->GetBoolPref("browser.display.force_inline_alttext", &boolPref))) {
       mPrefForceInlineAltText = boolPref;
     } else {
       mPrefForceInlineAltText = PR_FALSE;
     }
+    if (NS_SUCCEEDED(prefs->GetIntPref("network.image.imageBehavior", &intPref)) && intPref == 2) {
+      mPrefAllImagesBlocked = PR_TRUE;
+    } else {
+      mPrefAllImagesBlocked = PR_FALSE;
+    }
+    if (NS_SUCCEEDED(prefs->GetBoolPref("browser.display.show_image_placeholders", &boolPref))) {
+      mPrefShowPlaceholders = boolPref;
+    } else {
+      mPrefShowPlaceholders = PR_TRUE;
+    }
   }
 }
 
