@@ -208,7 +208,7 @@ private:
   
   // Overloaded new operator. Uses an arena (which comes from the presShell)
   // to perform the allocation.
-  void* operator new(size_t sz, nsIPresShell* aPresShell);
+  void* operator new(size_t sz, nsIPresShell* aPresShell) CPP_THROW_NEW;
   void operator delete(void* aPtr, size_t sz);
 
 public:
@@ -300,7 +300,8 @@ public:
     return (PRInt32) mFlags.mChildCount;
   }
   void SetChildCount(PRInt32 aNewCount) {
-    if (NS_WARN_IF_FALSE(aNewCount >= 0, "negative child count")) {
+    if (aNewCount < 0) {
+      NS_WARNING("negative child count");
       aNewCount = 0;
     }
     if (aNewCount > LINE_MAX_CHILD_COUNT) {
@@ -333,9 +334,11 @@ public:
   void FreeFloaters(nsFloaterCacheFreeList& aFreeList);
   void AppendFloaters(nsFloaterCacheFreeList& aFreeList);
   PRBool RemoveFloater(nsIFrame* aFrame);
-  void RemoveFloatersFromSpaceManager(nsSpaceManager* aSpaceManager);
 
-  // Combined area
+  // Combined area is the area of the line that should influence the
+  // overflow area of its parent block.  The combined area should be
+  // used for painting-related things, but should never be used for
+  // layout (except for handling of 'overflow').
   void SetCombinedArea(const nsRect& aCombinedArea);
   void GetCombinedArea(nsRect* aResult);
   PRBool CombinedAreaIntersects(const nsRect& aDamageRect) {
@@ -389,7 +392,7 @@ public:
   }
 
   // whether the line box is "logically" empty (just like nsIFrame::IsEmpty)
-  nsresult IsEmpty(PRBool aIsQuirkMode, PRBool aParentIsPre,
+  nsresult IsEmpty(nsCompatibility aCompatMode, PRBool aParentIsPre,
                    PRBool *aResult) const;
 
 #ifdef DEBUG
