@@ -41,7 +41,6 @@
 #include "nsIStyleContext.h"
 #include "nsIRenderingContext.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsLayoutAtoms.h"
 #include "nsIStyleSet.h"
 #include "nsIPresShell.h"
@@ -49,6 +48,8 @@
 #include "nsReadableUtils.h"
 #include "nsIPrintPreviewContext.h"
 #include "nsIPrintContext.h"
+#include "nsPageContentFrame.h"
+#include "nsTextFrame.h" // for function BinarySearchForPosition
 
 #include "nsIView.h" // view flags for clipping
 #include "nsCSSRendering.h"
@@ -56,10 +57,17 @@
 #include "nsHTMLContainerFrame.h" // view creation
 
 #include "nsSimplePageSequence.h" // for nsSharedPageData
+#include "nsRegion.h"
+#include "nsIViewManager.h"
 
 // for page number localization formatting
 #include "nsTextFormatter.h"
 
+#ifdef IBMBIDI
+#include "nsBidiUtils.h"
+#include "nsBidiPresUtils.h"
+#endif
+
 // Temporary
 #include "nsIFontMetrics.h"
 
@@ -67,34 +75,32 @@
 #include "nsIPrintSettings.h"
 #include "nsGfxCIID.h"
 #include "nsIServiceManager.h"
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
+
+// Widget Creation
+#include "nsIWidget.h"
+#include "nsWidgetsCID.h"
+static NS_DEFINE_CID(kCChildCID, NS_CHILD_CID);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
 //#define DEBUG_PRINTING
 #endif
 
-#ifdef DEBUG_PRINTING
-#define PRINT_DEBUG_MSG1(_msg1) fprintf(mDebugFD, (_msg1))
-#define PRINT_DEBUG_MSG2(_msg1, _msg2) fprintf(mDebugFD, (_msg1), (_msg2))
-#define PRINT_DEBUG_MSG3(_msg1, _msg2, _msg3) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3))
-#define PRINT_DEBUG_MSG4(_msg1, _msg2, _msg3, _msg4) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3), (_msg4))
-#define PRINT_DEBUG_MSG5(_msg1, _msg2, _msg3, _msg4, _msg5) fprintf(mDebugFD, (_msg1), (_msg2), (_msg3), (_msg4), (_msg5))
-#else //--------------
-#define PRINT_DEBUG_MSG1(_msg) 
-#define PRINT_DEBUG_MSG2(_msg1, _msg2) 
-#define PRINT_DEBUG_MSG3(_msg1, _msg2, _msg3) 
-#define PRINT_DEBUG_MSG4(_msg1, _msg2, _msg3, _msg4) 
-#define PRINT_DEBUG_MSG5(_msg1, _msg2, _msg3, _msg4, _msg5) 
+#include "prlog.h"
+#ifdef PR_LOGGING 
+extern PRLogModuleInfo * kLayoutPrintingLogMod;
+#define PR_PL(_p1)  PR_LOG(kLayoutPrintingLogMod, PR_LOG_DEBUG, _p1)
+#else
+#define PR_PL(_p1)
 #endif
 
+// XXX Part of Temporary fix for Bug 127263
+PRBool nsPageFrame::mDoCreateWidget = PR_TRUE;
 
 nsresult
 NS_NewPageFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
 {
   NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
+
   nsPageFrame* it = new (aPresShell) nsPageFrame;
   if (nsnull == it) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -107,45 +113,30 @@ nsPageFrame::nsPageFrame() :
   mSupressHF(PR_FALSE),
   mClipRect(-1, -1, -1, -1)
 {
-#ifdef NS_DEBUG
-  mDebugFD = stdout;
-#endif
 }
 
 nsPageFrame::~nsPageFrame()
 {
 }
 
+
 NS_IMETHODIMP
 nsPageFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                       nsIAtom*        aListName,
                                       nsIFrame*       aChildList)
 {
-  // only create a view for the area frame if we are printing the selection
-  // (Also skip it if we are doing PrintPreview)
-  nsCOMPtr<nsIPrintPreviewContext> ppContext = do_QueryInterface(aPresContext);
-  if (!ppContext) {
-    nsCOMPtr<nsIPrintContext> prtContext = do_QueryInterface(aPresContext);
-    if (prtContext) {
-      nsCOMPtr<nsIPrintSettings> printSettings;
-      prtContext->GetPrintSettings(getter_AddRefs(printSettings));
-      if (printSettings) {
-        PRInt16 printRangeType = nsIPrintSettings::kRangeAllPages; 
-        printSettings->GetPrintRange(&printRangeType);
-        // make sure we are printing the selection
-        if (printRangeType == nsIPrintSettings::kRangeSelection) {
-          nsIView* view;
-          aChildList->GetView(aPresContext, &view);
-          if (view == nsnull) {
-            nsCOMPtr<nsIStyleContext> styleContext;
-            aChildList->GetStyleContext(getter_AddRefs(styleContext));
-            nsHTMLContainerFrame::CreateViewForFrame(aPresContext, aChildList,
-                                                     styleContext, nsnull, PR_TRUE);
-          }
-        }
-      }
+  nsIView * view;
+  aChildList->GetView(aPresContext, &view);
+  if (view != nsnull && mDoCreateWidget) {
+    nscoord dx, dy;
+    nsCOMPtr<nsIWidget> widget;
+    view->GetOffsetFromWidget(&dx, &dy, *getter_AddRefs(widget));
+    nsCOMPtr<nsIPrintPreviewContext> ppContext = do_QueryInterface(aPresContext);
+    if (ppContext && widget) {
+      view->CreateWidget(kCChildCID);  
     }
   }
+
   return nsContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
 }
 
@@ -161,10 +152,20 @@ NS_IMETHODIMP nsPageFrame::Reflow(nsIPresContext*          aPresContext,
   if (eReflowReason_Incremental != aReflowState.reason) {
     // Do we have any children?
     // XXX We should use the overflow list instead...
-    if (mFrames.IsEmpty() && (nsnull != mPrevInFlow)) {
-      nsPageFrame*  prevPage = (nsPageFrame*)mPrevInFlow;
+    nsIFrame*           firstFrame  = mFrames.FirstChild();
+    nsPageContentFrame* contentPage = NS_STATIC_CAST(nsPageContentFrame*, firstFrame);
+    NS_ASSERTION(contentPage, "There should always be a content page");
 
-      nsIFrame* prevLastChild = prevPage->mFrames.LastChild();
+#ifdef NS_DEBUG
+    nsCOMPtr<nsIAtom> type;
+    firstFrame->GetFrameType(getter_AddRefs(type));
+    NS_ASSERTION(type.get() == nsLayoutAtoms::pageContentFrame, "This frame isn't a pageContentFrame");
+#endif
+
+    if (contentPage && contentPage->mFrames.IsEmpty() && nsnull != mPrevInFlow) {
+      nsPageFrame*        prevPage        = NS_STATIC_CAST(nsPageFrame*, mPrevInFlow);
+      nsPageContentFrame* prevContentPage = NS_STATIC_CAST(nsPageContentFrame*, prevPage->mFrames.FirstChild());
+      nsIFrame*           prevLastChild   = prevContentPage->mFrames.LastChild();
 
       // Create a continuing child of the previous page's last child
       nsIPresShell* presShell;
@@ -174,21 +175,40 @@ NS_IMETHODIMP nsPageFrame::Reflow(nsIPresContext*          aPresContext,
       aPresContext->GetShell(&presShell);
       presShell->GetStyleSet(&styleSet);
       NS_RELEASE(presShell);
-      styleSet->CreateContinuingFrame(aPresContext, prevLastChild, this, &newFrame);
+      styleSet->CreateContinuingFrame(aPresContext, prevLastChild, contentPage, &newFrame);
       NS_RELEASE(styleSet);
-      mFrames.SetFrames(newFrame);
+      contentPage->mFrames.SetFrames(newFrame);
     }
 
     // Resize our frame allowing it only to be as big as we are
     // XXX Pay attention to the page's border and padding...
     if (mFrames.NotEmpty()) {
       nsIFrame* frame = mFrames.FirstChild();
+      // When availableHeight is NS_UNCONSTRAINEDSIZE it means we are reflowing a single page
+      // to print selection. So this means we want to use NS_UNCONSTRAINEDSIZE without altering it
+      nscoord avHeight;
+      if (aReflowState.availableHeight == NS_UNCONSTRAINEDSIZE) {
+        avHeight = NS_UNCONSTRAINEDSIZE;
+      } else {
+        avHeight = mPD->mReflowRect.height - mPD->mReflowMargin.top - mPD->mReflowMargin.bottom;
+      }
       nsSize  maxSize(mPD->mReflowRect.width - mPD->mReflowMargin.right - mPD->mReflowMargin.left, 
-                      mPD->mReflowRect.height - mPD->mReflowMargin.top - mPD->mReflowMargin.bottom);
+                      avHeight);
+      // Get the number of Twips per pixel from the PresContext
+      float p2t;
+      aPresContext->GetScaledPixelsToTwips(&p2t);
+      nscoord onePixelInTwips = NSToCoordRound(p2t);
+      NS_ASSERTION(maxSize.width >= onePixelInTwips, "maxSize.width must be >= 1 pixel");
+      NS_ASSERTION(maxSize.height >= onePixelInTwips, "maxSize.height must be >= 1 pixel");
+      // insurance against infinite reflow, when reflowing less than a pixel
+      if (maxSize.width < onePixelInTwips || maxSize.height < onePixelInTwips) {
+        aDesiredSize.width  = 0;
+        aDesiredSize.height = 0;
+        return NS_OK;
+      }
+
       nsHTMLReflowState kidReflowState(aPresContext, aReflowState, frame, maxSize);
       kidReflowState.mFlags.mIsTopOfPage = PR_TRUE;
-      kidReflowState.availableWidth  = maxSize.width;
-      kidReflowState.availableHeight = maxSize.height;
 
       // calc location of frame
       nscoord xc = mPD->mReflowMargin.left + mPD->mDeadSpaceMargin.left + mPD->mExtraMargin.left;
@@ -202,10 +222,22 @@ NS_IMETHODIMP nsPageFrame::Reflow(nsIPresContext*          aPresContext,
       FinishReflowChild(frame, aPresContext, &kidReflowState, aDesiredSize, xc, yc, 0);
 
       // Make sure the child is at least as tall as our max size (the containing window)
-      if (aDesiredSize.height < aReflowState.availableHeight) {
+      if (aDesiredSize.height < aReflowState.availableHeight &&
+          aReflowState.availableHeight != NS_UNCONSTRAINEDSIZE) {
         aDesiredSize.height = aReflowState.availableHeight;
       }
 
+      nsIView * view;
+      frame->GetView(aPresContext, &view);
+      if (view) {
+        nsCOMPtr<nsIViewManager> vm;
+        view->GetViewManager(*getter_AddRefs(vm));
+        nsRegion region;
+        region.Copy(nsRect(0, 0, aDesiredSize.width, aDesiredSize.height));
+        vm->SetViewChildClipRegion(view, &region);
+      }
+
+#ifdef NS_DEBUG
       // Is the frame complete?
       if (NS_FRAME_IS_COMPLETE(aStatus)) {
         nsIFrame* childNextInFlow;
@@ -213,20 +245,34 @@ NS_IMETHODIMP nsPageFrame::Reflow(nsIPresContext*          aPresContext,
         frame->GetNextInFlow(&childNextInFlow);
         NS_ASSERTION(nsnull == childNextInFlow, "bad child flow list");
       }
+#endif
     }
-    PRINT_DEBUG_MSG2("PageFrame::Reflow %p ", this);
-    PRINT_DEBUG_MSG5("[%d,%d][%d,%d]\n", aDesiredSize.width, aDesiredSize.height, aReflowState.availableWidth, aReflowState.availableHeight);
+    PR_PL(("PageFrame::Reflow %p ", this));
+    PR_PL(("[%d,%d][%d,%d]\n", aDesiredSize.width, aDesiredSize.height, aReflowState.availableWidth, aReflowState.availableHeight));
 
     // Return our desired size
     aDesiredSize.width = aReflowState.availableWidth;
-    aDesiredSize.height = aReflowState.availableHeight;
+    if (aReflowState.availableHeight != NS_UNCONSTRAINEDSIZE) {
+      aDesiredSize.height = aReflowState.availableHeight;
+    }
   }
-  PRINT_DEBUG_MSG2("PageFrame::Reflow %p ", this);
-  PRINT_DEBUG_MSG3("[%d,%d]\n", aReflowState.availableWidth, aReflowState.availableHeight);
+  PR_PL(("PageFrame::Reflow %p ", this));
+  PR_PL(("[%d,%d]\n", aReflowState.availableWidth, aReflowState.availableHeight));
 
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
+void nsPageFrame::SetClipRect(nsRect* aClipRect) 
+{
+  mClipRect = *aClipRect; 
+  nsIFrame*           firstFrame  = mFrames.FirstChild();
+  nsPageContentFrame* contentPage = NS_STATIC_CAST(nsPageContentFrame*, firstFrame);
+  NS_ASSERTION(contentPage, "There should always be a content page");
+  contentPage->SetClipRect(aClipRect);
+}
+
+
 NS_IMETHODIMP
 nsPageFrame::GetFrameType(nsIAtom** aType) const
 {
@@ -251,29 +297,21 @@ nsPageFrame::IsPercentageBase(PRBool& aBase) const
   return NS_OK;
 }
 
-//------------------------------------------------------------------------------
-// helper function for converting from char * to unichar
-static PRUnichar *
-GetUStr(const char * aCStr)
-{
-  return ToNewUnicode(nsDependentCString(aCStr));
-}
-
 // Remove fix below when string gets fixed
 #define WORKAROUND_FOR_BUG_110335
 // replace the &<code> with the value, but if the value is empty
 // set the string to zero length
 static void
-SubstValueForCode(nsString& aStr, PRUnichar * aUKey, PRUnichar * aUStr)
+SubstValueForCode(nsString& aStr, const PRUnichar * aUKey, const PRUnichar * aUStr)
 {
 #ifdef WORKAROUND_FOR_BUG_110335
-  PRUnichar* uKeyStr = aUKey;
+  const PRUnichar* uKeyStr = aUKey;
 
   // Check to make sure our subst code &<code> isn't in the data string
   // for example &T for title is in QB&T
   nsAutoString dataStr(aUStr);
   nsAutoString newKey(aUKey);
-  PRBool fixingSubstr = dataStr.Find(newKey) > -1;
+  PRBool fixingSubstr = (dataStr.Find(newKey) != kNotFound);
   if (fixingSubstr) {
     // well, the code is in the data str so make up a new code
     // but make sure it it isn't in either substs string or the data string
@@ -292,14 +330,11 @@ SubstValueForCode(nsString& aStr, PRUnichar * aUKey, PRUnichar * aUStr)
 
     // Ok, we have the new code, so repplace the old code 
     // in the dest str with the new code
-    nsAutoString oldKey(aUKey);
-    aStr.ReplaceSubstring(oldKey, newKey);
+    aStr.ReplaceSubstring(aUKey, newKey.get());
     uKeyStr = ToNewUnicode(newKey);
   }
 
-  nsAutoString str;
-  str = aUStr;
-  if (str.Length() == 0) {
+  if (!aUStr || !*aUStr) {
     aStr.SetLength(0);
   } else {
     aStr.ReplaceSubstring(uKeyStr, aUStr);
@@ -307,12 +342,11 @@ SubstValueForCode(nsString& aStr, PRUnichar * aUKey, PRUnichar * aUStr)
 
   // Free uKeyStr only if we fixed the string.
   if (fixingSubstr) {
-    nsMemory::Free(uKeyStr);
+    nsMemory::Free(NS_CONST_CAST(PRUnichar*, uKeyStr));
   }
 #else
-  nsAutoString str;
-  str = aUStr;
-  if (str.Length() == 0) {
+
+  if (!aUStr || !*aUStr) {
     aStr.SetLength(0);
   } else {
     aStr.ReplaceSubstring(aUKey, aUStr);
@@ -331,74 +365,49 @@ nsPageFrame::ProcessSpecialCodes(const nsString& aStr, nsString& aNewStr)
 
   // Search to see if the &D code is in the string 
   // then subst in the current date/time
-  PRUnichar * kDate = GetUStr("&D");
-  if (kDate != nsnull) {
-    if (aStr.Find(kDate) > -1) {
-      if (mPD->mDateTimeStr != nsnull) {
-        aNewStr.ReplaceSubstring(kDate, mPD->mDateTimeStr);
-      } else {
-        aNewStr.ReplaceSubstring(kDate, NS_LITERAL_STRING("").get());
-      }
-      nsMemory::Free(kDate);
-      return;
+  NS_NAMED_LITERAL_STRING(kDate, "&D");
+  if (aStr.Find(kDate) != kNotFound) {
+    if (mPD->mDateTimeStr != nsnull) {
+      aNewStr.ReplaceSubstring(kDate.get(), mPD->mDateTimeStr);
+    } else {
+      aNewStr.ReplaceSubstring(kDate.get(), NS_LITERAL_STRING("").get());
     }
-    nsMemory::Free(kDate);
+    return;
   }
 
   // NOTE: Must search for &PT before searching for &P
   //
   // Search to see if the "page number and page" total code are in the string
-  // and replace the page number and page total code with the actual values
-  PRUnichar * kPage = GetUStr("&PT");
-  if (kPage != nsnull) {
-    if (aStr.Find(kPage) > -1) {
-      PRUnichar * uStr = nsTextFormatter::smprintf(mPD->mPageNumAndTotalsFormat, mPageNum, mTotNumPages);
-      aNewStr.ReplaceSubstring(kPage, uStr);
-      nsMemory::Free(uStr);
-      nsMemory::Free(kPage);
-      return;
-    }
-    nsMemory::Free(kPage);
+  // and replace the page number and page total code with the actual
+  // values
+  NS_NAMED_LITERAL_STRING(kPageAndTotal, "&PT");
+  if (aStr.Find(kPageAndTotal) != kNotFound) {
+    PRUnichar * uStr = nsTextFormatter::smprintf(mPD->mPageNumAndTotalsFormat, mPageNum, mTotNumPages);
+    aNewStr.ReplaceSubstring(kPageAndTotal.get(), uStr);
+    nsMemory::Free(uStr);
+    return;
   }
 
   // Search to see if the page number code is in the string
-  // and replace the page number code with the actual values
-  kPage = GetUStr("&P");
-  if (kPage != nsnull) {
-    if (aStr.Find(kPage) > -1) {
-      PRUnichar * uStr = nsTextFormatter::smprintf(mPD->mPageNumFormat, mPageNum);
-      aNewStr.ReplaceSubstring(kPage, uStr);
-      nsMemory::Free(uStr);
-      nsMemory::Free(kPage);
-      return;
-    }
-    nsMemory::Free(kPage);
+  // and replace the page number code with the actual value
+  NS_NAMED_LITERAL_STRING(kPage, "&P");
+  if (aStr.Find(kPage) != kNotFound) {
+    PRUnichar * uStr = nsTextFormatter::smprintf(mPD->mPageNumFormat, mPageNum);
+    aNewStr.ReplaceSubstring(kPage.get(), uStr);
+    nsMemory::Free(uStr);
+    return;
   }
 
-  PRUnichar * kTitle = GetUStr("&T");
-  if (kTitle != nsnull) {
-    if (aStr.Find(kTitle) > -1) {
-      PRUnichar * uTitle;
-      mPD->mPrintOptions->GetTitle(&uTitle);   // creates memory
-      SubstValueForCode(aNewStr, kTitle, uTitle);
-      nsMemory::Free(uTitle);
-      nsMemory::Free(kTitle);
-      return;
-    }
-    nsMemory::Free(kTitle);
+  NS_NAMED_LITERAL_STRING(kTitle, "&T");
+  if (aStr.Find(kTitle) != kNotFound) {
+    SubstValueForCode(aNewStr, kTitle.get(), mPD->mDocTitle);
+    return;
   }
 
-  PRUnichar * kDocURL = GetUStr("&U");
-  if (kDocURL != nsnull) {
-    if (aStr.Find(kDocURL) > -1) {
-      PRUnichar * uDocURL;
-      mPD->mPrintOptions->GetDocURL(&uDocURL);   // creates memory
-      SubstValueForCode(aNewStr, kDocURL, uDocURL);
-      nsMemory::Free(uDocURL);
-      nsMemory::Free(kDocURL);
-      return;
-    }
-    nsMemory::Free(kDocURL);
+  NS_NAMED_LITERAL_STRING(kDocURL, "&U");
+  if (aStr.Find(kDocURL) != kNotFound) {
+    SubstValueForCode(aNewStr, kDocURL.get(), mPD->mDocURL);
+    return;
   }
 }
 
@@ -415,7 +424,7 @@ nscoord nsPageFrame::GetXPosition(nsIRenderingContext& aRenderingContext,
   nscoord x = aRect.x;
   switch (aJust) {
     case nsIPrintSettings::kJustLeft:
-      x += mPD->mExtraMargin.left + mPD->mHeadFooterGap;
+      x += mPD->mExtraMargin.left + mPD->mEdgePaperMargin.left;
       break;
 
     case nsIPrintSettings::kJustCenter:
@@ -423,7 +432,7 @@ nscoord nsPageFrame::GetXPosition(nsIRenderingContext& aRenderingContext,
       break;
 
     case nsIPrintSettings::kJustRight:
-      x += aRect.width - width - mPD->mExtraMargin.right - mPD->mHeadFooterGap;
+      x += aRect.width - width - mPD->mExtraMargin.right - mPD->mEdgePaperMargin.right;
       break;
   } // switch
 
@@ -443,7 +452,8 @@ nscoord nsPageFrame::GetXPosition(nsIRenderingContext& aRenderingContext,
 // @parm aUseHalfThePage - indicates whether the text should limited to  the width
 //                         of the entire page or just half the page
 void
-nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
+nsPageFrame::DrawHeaderFooter(nsIPresContext*      aPresContext,
+                              nsIRenderingContext& aRenderingContext,
                               nsIFrame *           aFrame,
                               nsHeaderFooterEnum   aHeaderFooter,
                               PRInt32              aJust,
@@ -463,13 +473,13 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
   nscoord strSpace = aRect.width / numStrs;
 
   if (!aStr1.IsEmpty()) {
-    DrawHeaderFooter(aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustLeft, aStr1, aRect, aAscent, aHeight, strSpace);
+    DrawHeaderFooter(aPresContext, aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustLeft, aStr1, aRect, aAscent, aHeight, strSpace);
   }
   if (!aStr2.IsEmpty()) {
-    DrawHeaderFooter(aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustCenter, aStr2, aRect, aAscent, aHeight, strSpace);
+    DrawHeaderFooter(aPresContext, aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustCenter, aStr2, aRect, aAscent, aHeight, strSpace);
   }
   if (!aStr3.IsEmpty()) {
-    DrawHeaderFooter(aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustRight, aStr3, aRect, aAscent, aHeight, strSpace);
+    DrawHeaderFooter(aPresContext, aRenderingContext, aFrame, aHeaderFooter, nsIPrintSettings::kJustRight, aStr3, aRect, aAscent, aHeight, strSpace);
   }
 }
 
@@ -483,7 +493,8 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
 // @parm aHeight - the height of the text
 // @parm aWidth - available width for any one of the strings
 void
-nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
+nsPageFrame::DrawHeaderFooter(nsIPresContext*      aPresContext,
+                              nsIRenderingContext& aRenderingContext,
                               nsIFrame *           aFrame,
                               nsHeaderFooterEnum   aHeaderFooter,
                               PRInt32              aJust,
@@ -494,31 +505,33 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
                               nscoord              aWidth)
 {
 
-  nscoord contentWidth = aWidth - (mPD->mHeadFooterGap * 2);
+  nscoord contentWidth = aWidth - (mPD->mEdgePaperMargin.left + mPD->mEdgePaperMargin.right);
 
   // first make sure we have a vaild string and that the height of the
   // text will fit in the margin
   if (aStr.Length() > 0 && 
       ((aHeaderFooter == eHeader && aHeight < mMargin.top) ||
        (aHeaderFooter == eFooter && aHeight < mMargin.bottom))) {
-    // measure the width of the text
     nsAutoString str;
     ProcessSpecialCodes(aStr, str);
 
-    PRInt32 width;
-    aRenderingContext.GetWidth(str, width);
-    PRBool addEllipse = PR_FALSE;
+    PRInt32 indx;
+    PRInt32 textWidth = 0;
+    const PRUnichar* text = str.get();
 
-    // trim the text and add the elipses if it won't fit
-    while (width >= contentWidth && str.Length() > 1) {
-      str.SetLength(str.Length()-1);
-      aRenderingContext.GetWidth(str, width);
-      addEllipse = PR_TRUE;
+    PRInt32 len = (PRInt32)str.Length();
+    if (len == 0) {
+      return; // bail is empty string
     }
-    if (addEllipse && str.Length() > 3) {
-      str.SetLength(str.Length()-3);
-      str.Append(NS_LITERAL_STRING("..."));
-      aRenderingContext.GetWidth(str, width);
+    // find how much text fits, the "position" is the size of the avilable area
+    if (BinarySearchForPosition(&aRenderingContext, text, 0, 0, 0, len,
+                                PRInt32(contentWidth), indx, textWidth)) {
+      if (indx < len-1 && len > 3) {
+        str.SetLength(indx-3);
+        str.Append(NS_LITERAL_STRING("..."));
+      }
+    } else { 
+      return; // bail if couldn't find the correct length
     }
 
     // cacl the x and y positions of the text
@@ -526,9 +539,9 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
     nscoord x = GetXPosition(aRenderingContext, rect, aJust, str);
     nscoord y;
     if (aHeaderFooter == eHeader) {
-      y = rect.y + mPD->mExtraMargin.top + mPD->mHeadFooterGap;
+      y = rect.y + mPD->mExtraMargin.top + mPD->mEdgePaperMargin.top;
     } else {
-      y = rect.y + rect.height - aHeight - mPD->mExtraMargin.bottom - mPD->mHeadFooterGap;
+      y = rect.y + rect.height - aHeight - mPD->mExtraMargin.bottom - mPD->mEdgePaperMargin.bottom;
     }
 
     // set up new clip and draw the text
@@ -536,13 +549,34 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
     aRenderingContext.PushState();
     aRenderingContext.SetColor(NS_RGB(0,0,0));
     aRenderingContext.SetClipRect(rect, nsClipCombine_kReplace, clipEmpty);
+#ifdef IBMBIDI
+    nsresult rv = NS_ERROR_FAILURE;
+
+    PRBool isBidiEnabled = PR_FALSE;
+    aPresContext->GetBidiEnabled(&isBidiEnabled);
+    if (isBidiEnabled) {
+      nsBidiPresUtils* bidiUtils;
+      aPresContext->GetBidiUtils(&bidiUtils);
+      
+      if (bidiUtils) {
+        PRUnichar* buffer = (PRUnichar*)str.get();
+        // Base direction is always LTR for now. If bug 139337 is fixed, 
+        // that should change.
+        rv = bidiUtils->RenderText(buffer, str.Length(), NSBIDI_LTR,
+                                   aPresContext, aRenderingContext,
+                                   x, y + aAscent);
+      }
+    }
+    if (NS_FAILED(rv))
+#endif // IBMBIDI
     aRenderingContext.DrawString(str, x, y + aAscent);
     aRenderingContext.PopState(clipEmpty);
+
 #ifdef DEBUG_PRINTING
-    PRINT_DEBUG_MSG2("Page: %p", this);
+    PR_PL(("Page: %p", this));
     const char * s = NS_ConvertUCS2toUTF8(str).get();
     if (s) {
-      PRINT_DEBUG_MSG2(" [%s]", s);
+      PR_PL((" [%s]", s));
     }
     char justStr[64];
     switch (aJust) {
@@ -550,10 +584,10 @@ nsPageFrame::DrawHeaderFooter(nsIRenderingContext& aRenderingContext,
       case nsIPrintSettings::kJustCenter:strcpy(justStr, "Center");break;
       case nsIPrintSettings::kJustRight:strcpy(justStr, "Right");break;
     } // switch
-    PRINT_DEBUG_MSG2(" HF: %s ", aHeaderFooter==eHeader?"Header":"Footer");
-    PRINT_DEBUG_MSG2(" JST: %s ", justStr);
-    PRINT_DEBUG_MSG3(" x,y: %d,%d", x, y);
-    PRINT_DEBUG_MSG2(" Hgt: %d \n", aHeight);
+    PR_PL((" HF: %s ", aHeaderFooter==eHeader?"Header":"Footer"));
+    PR_PL((" JST: %s ", justStr));
+    PR_PL((" x,y: %d,%d", x, y));
+    PR_PL((" Hgt: %d \n", aHeight));
 #endif
   }
 }
@@ -579,8 +613,6 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
       printf("*** ClipRect: %5d,%5d,%5d,%5d\n", mClipRect.x, mClipRect.y, mClipRect.width, mClipRect.height);
     }
 #endif
-    mClipRect.x = 0;
-    mClipRect.y = 0;
     aRenderingContext.SetClipRect(mClipRect, nsClipCombine_kReplace, clipEmpty);
     rect = mClipRect;
   } else {
@@ -591,15 +623,19 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
 
     nsCOMPtr<nsIPrintPreviewContext> ppContext = do_QueryInterface(aPresContext);
     if (ppContext) {
+      // fill page with White
       aRenderingContext.SetColor(NS_RGB(255,255,255));
       rect.x = 0;
       rect.y = 0;
       rect.width  -= mPD->mShadowSize.width;
       rect.height -= mPD->mShadowSize.height;
       aRenderingContext.FillRect(rect);
+      // draw line around outside of page
+      aRenderingContext.SetColor(NS_RGB(0,0,0));
+      aRenderingContext.DrawRect(rect);
 
       if (mPD->mShadowSize.width > 0 && mPD->mShadowSize.height > 0) {
-        aRenderingContext.SetColor(NS_RGB(0,0,0));
+        aRenderingContext.SetColor(NS_RGB(51,51,51));
         nsRect r(0,0, mRect.width, mRect.height);
         nsRect shadowRect;
         shadowRect.x = r.x + r.width - mPD->mShadowSize.width;
@@ -617,17 +653,18 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
     }
   }
 
-  DrawBackground(aPresContext,aRenderingContext,aDirtyRect) ;
+  if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+    DrawBackground(aPresContext,aRenderingContext,aDirtyRect);
+  }
 
   nsresult rv = nsContainerFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 
 #if defined(DEBUG_rods) || defined(DEBUG_dcone)
   if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
-    nsRect r;
-    fprintf(mDebugFD, "PF::Paint    -> %p  SupHF: %s  Rect: [%5d,%5d,%5d,%5d] SC:%s\n", this, 
-            mSupressHF?"Yes":"No", mRect.x, mRect.y, mRect.width, mRect.height, specialClipIsSet?"Yes":"No");
-    fprintf(stdout, "PF::Paint    -> %p  SupHF: %s  Rect: [%5d,%5d,%5d,%5d] SC:%s\n", this, 
-            mSupressHF?"Yes":"No", mRect.x, mRect.y, mRect.width, mRect.height, specialClipIsSet?"Yes":"No");
+    PR_PL(("PF::Paint    -> %p  SupHF: %s  Rect: [%5d,%5d,%5d,%5d] SC:%s\n", this, 
+            mSupressHF?"Yes":"No", mRect.x, mRect.y, mRect.width, mRect.height, specialClipIsSet?"Yes":"No"));
+    PR_PL(("PF::Paint    -> %p  SupHF: %s  Rect: [%5d,%5d,%5d,%5d] SC:%s\n", this, 
+            mSupressHF?"Yes":"No", mRect.x, mRect.y, mRect.width, mRect.height, specialClipIsSet?"Yes":"No"));
   }
 #endif
 
@@ -646,25 +683,7 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
 
     rect.SetRect(0, 0, mRect.width - mPD->mShadowSize.width, mRect.height - mPD->mShadowSize.height);
 
-#if defined(DEBUG_rods) || defined(DEBUG_dcone)
-    {
-    nsRect rct = rect;
-    // XXX Paint a one-pixel border around the page so it's easy to see where
-    // each page begins and ends when we're
-    rct.Deflate(mMargin);
-    rct.Deflate(mPD->mDeadSpaceMargin);
-    //float   p2t;
-    //aPresContext->GetPixelsToTwips(&p2t);
-    //rect.Deflate(NSToCoordRound(p2t), NSToCoordRound(p2t));
-    aRenderingContext.SetColor(NS_RGB(0, 0, 0));
-    aRenderingContext.DrawRect(rct);
-    //rect.Inflate(NSToCoordRound(p2t), NSToCoordRound(p2t));
-    fprintf(mDebugFD, "PageFr::PaintChild -> Painting Frame %p Page No: %d\n", this, mPageNum);
-    }
-#endif
-
-   
-    aRenderingContext.SetFont(*mPD->mHeadFootFont);
+    aRenderingContext.SetFont(*mPD->mHeadFootFont, nsnull);
     aRenderingContext.SetColor(NS_RGB(0,0,0));
 
     // Get the FontMetrics to determine width.height of strings
@@ -672,7 +691,7 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
     aPresContext->GetDeviceContext(getter_AddRefs(deviceContext));
     NS_ASSERTION(deviceContext, "Couldn't get the device context"); 
     nsCOMPtr<nsIFontMetrics> fontMet;
-    deviceContext->GetMetricsFor(*mPD->mHeadFootFont, *getter_AddRefs(fontMet));
+    deviceContext->GetMetricsFor(*mPD->mHeadFootFont, nsnull, *getter_AddRefs(fontMet));
     nscoord ascent = 0;
     nscoord visibleHeight = 0;
     if (fontMet) {
@@ -685,7 +704,7 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
     mPD->mPrintSettings->GetHeaderStrLeft(&headers[0]);   // creates memory
     mPD->mPrintSettings->GetHeaderStrCenter(&headers[1]); // creates memory
     mPD->mPrintSettings->GetHeaderStrRight(&headers[2]);  // creates memory
-    DrawHeaderFooter(aRenderingContext, this, eHeader, nsIPrintSettings::kJustLeft, 
+    DrawHeaderFooter(aPresContext, aRenderingContext, this, eHeader, nsIPrintSettings::kJustLeft, 
                      nsAutoString(headers[0]), nsAutoString(headers[1]), nsAutoString(headers[2]), 
                      rect, ascent, visibleHeight);
     PRInt32 i;
@@ -695,7 +714,7 @@ nsPageFrame::Paint(nsIPresContext*      aPresContext,
     mPD->mPrintSettings->GetFooterStrLeft(&footers[0]);   // creates memory
     mPD->mPrintSettings->GetFooterStrCenter(&footers[1]); // creates memory
     mPD->mPrintSettings->GetFooterStrRight(&footers[2]);  // creates memory
-    DrawHeaderFooter(aRenderingContext, this, eFooter, nsIPrintSettings::kJustRight, 
+    DrawHeaderFooter(aPresContext, aRenderingContext, this, eFooter, nsIPrintSettings::kJustRight, 
                      nsAutoString(footers[0]), nsAutoString(footers[1]), nsAutoString(footers[2]), 
                      rect, ascent, visibleHeight);
     for (i=0;i<3;i++) nsMemory::Free(footers[i]);
@@ -724,15 +743,120 @@ nsPageFrame::DrawBackground(nsIPresContext*      aPresContext,
 {
   nsSimplePageSequenceFrame* seqFrame = NS_STATIC_CAST(nsSimplePageSequenceFrame*, mParent);
   if (seqFrame != nsnull) {
-    nsRect rect = mPD->mReflowRect;
-    rect.Deflate(mPD->mReflowMargin);
-    rect.Deflate(mPD->mExtraMargin);
+    nsIFrame* pageContentFrame  = mFrames.FirstChild();
+    NS_ASSERTION(pageContentFrame, "Must always be there.");
 
-    const nsStyleBorder* border = NS_STATIC_CAST(const nsStyleBorder*,
-                             mStyleContext->GetStyleData(eStyleStruct_Border));
+    nsRect rect;
+    pageContentFrame->GetRect(rect);
+    const nsStyleBorder* border = 
+      NS_STATIC_CAST(const nsStyleBorder*,
+                     mStyleContext->GetStyleData(eStyleStruct_Border));
+    const nsStylePadding* padding = 
+      NS_STATIC_CAST(const nsStylePadding*,
+                     mStyleContext->GetStyleData(eStyleStruct_Padding));
 
     nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                    aDirtyRect, rect, *border, 0, 0);      
+                                    aDirtyRect, rect, *border, *padding,
+                                    0, 0, PR_TRUE);
   }
 }
 
+void
+nsPageFrame::SetSharedPageData(nsSharedPageData* aPD) 
+{ 
+  mPD = aPD;
+  // Set the shared data into the page frame before reflow
+  nsPageContentFrame * pcf = NS_STATIC_CAST(nsPageContentFrame*, mFrames.FirstChild());
+  if (pcf) {
+    pcf->SetSharedPageData(mPD);
+  }
+
+}
+
+nsresult
+NS_NewPageBreakFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
+{
+  NS_PRECONDITION(aPresShell && aNewFrame, "null PresShell or OUT ptr");
+#ifdef DEBUG
+  //check that we are only creating page break frames when printing
+  nsCOMPtr<nsIPresContext> presContext;    
+  aPresShell->GetPresContext(getter_AddRefs(presContext));
+  PRBool isPaginated;
+  presContext->IsPaginated(&isPaginated);
+  NS_ASSERTION(isPaginated, "created a page break frame while not printing");
+#endif
+
+  nsPageBreakFrame* it = new (aPresShell) nsPageBreakFrame;
+  if (nsnull == it) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  *aNewFrame = it;
+  return NS_OK;
+}
+
+nsPageBreakFrame::nsPageBreakFrame()
+: mHaveReflowed(PR_FALSE)
+{
+}
+
+nsPageBreakFrame::~nsPageBreakFrame()
+{
+}
+
+void 
+nsPageBreakFrame::GetDesiredSize(nsIPresContext*          aPresContext,
+                                 const nsHTMLReflowState& aReflowState,
+                                 nsHTMLReflowMetrics&     aDesiredSize)
+{
+  NS_PRECONDITION(aPresContext, "null pres context");
+  float p2t;
+  aPresContext->GetScaledPixelsToTwips(&p2t);
+  nscoord onePixel = NSToCoordRound(p2t);
+
+  aDesiredSize.width  = onePixel;
+  if (mHaveReflowed) {
+    // If blocks reflow us a 2nd time trying to put us on a new page, then return
+    // a desired height of 0 to avoid an extra page break. 
+    aDesiredSize.height = 0;
+  }
+  else {
+    aDesiredSize.height = aReflowState.availableHeight;
+    // round the height down to the nearest pixel
+    aDesiredSize.height -= aDesiredSize.height % onePixel;
+  }
+
+  if (aDesiredSize.maxElementSize) {
+    aDesiredSize.maxElementSize->width  = onePixel;
+    aDesiredSize.maxElementSize->height = aDesiredSize.height;
+  }
+  aDesiredSize.ascent  = 0;
+  aDesiredSize.descent = 0;
+}
+
+nsresult 
+nsPageBreakFrame::Reflow(nsIPresContext*          aPresContext,
+                         nsHTMLReflowMetrics&     aDesiredSize,
+                         const nsHTMLReflowState& aReflowState,
+                         nsReflowStatus&          aStatus)
+{
+  NS_PRECONDITION(aPresContext, "null pres context");
+  DO_GLOBAL_REFLOW_COUNT("nsTableFrame", aReflowState.reason);
+  DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
+
+  aStatus = NS_FRAME_COMPLETE; 
+  GetDesiredSize(aPresContext, aReflowState, aDesiredSize);
+  mHaveReflowed = PR_TRUE;
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPageBreakFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::pageBreakFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
+
