@@ -53,6 +53,7 @@
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIComponentManager.h"
+#include "nsIFrameManager.h"
 #include "nsIStreamListener.h"
 #include "nsIURL.h"
 #include "nsNetUtil.h"
@@ -93,6 +94,7 @@
 #include "nsPIDOMWindow.h"
 #include "nsIRenderingContext.h"
 #include "nsIFrameFrame.h"
+#include "nsIPluginViewer.h"
 
 // For Accessibility
 #ifdef ACCESSIBILITY
@@ -363,7 +365,34 @@ nsHTMLFrameOuterFrame::Init(nsIPresContext*  aPresContext,
   nsIView* view = nsnull;
   GetView(aPresContext, &view);
   if (!view) {
-    nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,mStyleContext,nsnull,PR_TRUE); 
+    // To properly initialize the view we need to know the frame for the content
+    // that is the parent of content for this frame. This might not be our actual
+    // frame parent if we are out of flow (e.g., positioned) so our parent frame
+    // may have been set to some other ancestor.
+    // We look for a content parent frame in the frame property list, where it
+    // will have been set by nsCSSFrameConstructor if necessary.
+    nsCOMPtr<nsIAtom> contentParentAtom = do_GetAtom("contentParent");
+    nsIFrame* contentParent = nsnull;
+    nsCOMPtr<nsIPresShell> presShell;
+    aPresContext->GetShell(getter_AddRefs(presShell));
+
+    if (presShell) {
+      nsCOMPtr<nsIFrameManager> frameManager;
+      presShell->GetFrameManager(getter_AddRefs(frameManager));
+
+      if (frameManager) {
+        void* value;
+        rv = frameManager->GetFrameProperty(this,
+                                           contentParentAtom, 
+                                           NS_IFRAME_MGR_REMOVE_PROP,
+                                           &value);
+        if (NS_SUCCEEDED(rv)) {
+          contentParent = (nsIFrame*)value;
+        }
+      }
+    }
+  
+    nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,mStyleContext,contentParent,PR_TRUE); 
     GetView(aPresContext, &view);
   }
 
@@ -655,7 +684,10 @@ nsHTMLFrameOuterFrame::AttributeChanged(nsIPresContext* aPresContext,
       nsCOMPtr<nsIDocShellTreeOwner> parentTreeOwner;
       parentAsItem->GetTreeOwner(getter_AddRefs(parentTreeOwner));
       if (parentTreeOwner) {
+        PRInt32 parentType;
+        parentAsItem->GetItemType(&parentType);
         PRBool is_primary_content =
+          parentType == nsIDocShellTreeItem::typeChrome &&
           value.EqualsIgnoreCase("content-primary");
 
         parentTreeOwner->ContentShellAdded(docShellAsItem, is_primary_content,
@@ -693,6 +725,9 @@ nsHTMLFrameInnerFrame::nsHTMLFrameInnerFrame()
 NS_IMETHODIMP
 nsHTMLFrameInnerFrame::Destroy(nsIPresContext* aPresContext)
 {
+  
+  PRBool containsPluginViewer = PR_FALSE;
+
   if (mFrameLoader) {
     // Get the content viewer through the docshell, but don't call
     // GetDocShell() since we don't want to create one if we don't
@@ -714,15 +749,37 @@ nsHTMLFrameInnerFrame::Destroy(nsIPresContext* aPresContext)
         // Hide the content viewer now that the frame is going away...
 
         content_viewer->Hide();
+
+        // Full-page plugins don't have a DOM so they always need to be destroyed
+        // here, before this frame and window are destroyed. See bug 173938.
+        // When bug 90256 is fixed this code can probably go away.
+        nsCOMPtr<nsIPluginViewer> pluginViewer (do_QueryInterface(content_viewer));
+        if (pluginViewer)
+          containsPluginViewer = PR_TRUE;
       }
     }
   }
 
-  if (mFrameLoader && mOwnsFrameLoader) {
-    // We own this frame loader, and we're going away, so destroy our
-    // frame loader.
+  if (mFrameLoader) {
+    if (mOwnsFrameLoader) {
+      // We own this frame loader, and we're going away, so destroy our
+      // frame loader.
 
-    mFrameLoader->Destroy();
+      mFrameLoader->Destroy();
+
+    } else if (containsPluginViewer) {
+      // If this is a full-page plugin, we need to notify content to let go
+      // of the frame loader so that it will be re-recreated if this frame
+      // is shown later.
+      nsCOMPtr<nsIContent> content;
+      GetParentContent(getter_AddRefs(content));
+      nsCOMPtr<nsIFrameLoaderOwner> frame_loader_owner (do_QueryInterface(content));
+      if (frame_loader_owner)
+        frame_loader_owner->SetFrameLoader(nsnull);
+
+      mFrameLoader->Destroy();
+      mFrameLoader = nsnull;
+    }
   }
 
   return nsLeafFrame::Destroy(aPresContext);
@@ -1118,6 +1175,9 @@ nsHTMLFrameInnerFrame::CreateViewAndWidget(nsIPresContext* aPresContext,
   rv = view->CreateWidget(kCChildCID, &initData);
   SetView(aPresContext, view);
 
+  nsContainerFrame::SyncFrameViewProperties(aPresContext, this, nsnull, view);
+
+  // XXX the following should be unnecessary, given the above Sync call
   // if the visibility is hidden, reflect that in the view
   const nsStyleVisibility* vis;
   GetStyleData(eStyleStruct_Visibility, ((const nsStyleStruct *&)vis));
@@ -1288,7 +1348,7 @@ nsHTMLFrameInnerFrame::GetDesiredSize(nsIPresContext* aPresContext,
  *****************************************************************************/
 FrameLoadingInfo::FrameLoadingInfo(const nsSize& aSize)
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 
   mFrameSize = aSize;
 }
