@@ -281,6 +281,7 @@ protected:
   nsSize                mCacheSize;
   nsSize                mCachedMaxElementSize;
   nsSize                mCachedAvailableSize;
+  nscoord               mCachedAscent;
 
   nsSize                mCachedUncDropdownSize;
   nsSize                mCachedUncComboSize;
@@ -303,7 +304,6 @@ protected:
   // static class data member for Bug 32920
   // only one control can be focused at a time
   static nsComboboxControlFrame * mFocused;
-  static nscoord                  mCachedScrollbarWidth;
 
 #ifdef DO_REFLOW_COUNTER
   PRInt32 mReflowId;
