@@ -53,9 +53,9 @@
 #include "nsIHTMLContent.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLAtoms.h"
+#include "nsLayoutAtoms.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsStyleUtil.h"
 #include "nsFont.h"
 #include "nsCOMPtr.h"
 #ifdef ACCESSIBILITY
@@ -107,6 +107,8 @@ public:
                                const nsPoint&    aPoint, 
                                nsFramePaintLayer aWhichLayer,
                                nsIFrame**        aFrame);
+  NS_IMETHOD GetFrameType(nsIAtom** aType) const;
+
 #ifdef ACCESSIBILITY  
   NS_IMETHOD  GetAccessible(nsIAccessible** aAccessible);
 #endif
@@ -158,6 +160,15 @@ nsFieldSetFrame::nsFieldSetFrame()
 }
 
 
+NS_IMETHODIMP
+nsFieldSetFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::fieldSetFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsFieldSetFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                      nsIAtom*        aListName,
@@ -188,6 +199,8 @@ nsFieldSetFrame::Paint(nsIPresContext*      aPresContext,
       PRIntn skipSides = GetSkipSides();
       const nsStyleBorder* borderStyle = 
         (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
+      const nsStylePadding* paddingStyle = 
+        (const nsStylePadding*)mStyleContext->GetStyleData(eStyleStruct_Padding);
        
         nsMargin border;
         if (!borderStyle->GetBorder(border)) {
@@ -204,7 +217,8 @@ nsFieldSetFrame::Paint(nsIPresContext*      aPresContext,
         nsRect rect(0, yoff, mRect.width, mRect.height - yoff);
 
         nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *borderStyle, 0, 0);
+                                        aDirtyRect, rect, *borderStyle,
+                                        *paddingStyle, 0, 0);
 
         if (mLegendFrame) {
 
@@ -464,7 +478,8 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
             nsHTMLReflowMetrics kidDesiredSize(aDesiredSize.maxElementSize, aDesiredSize.mFlags);
             // Reflow the frame
             ReflowChild(mContentFrame, aPresContext, kidDesiredSize, kidReflowState,
-                        kidReflowState.mComputedMargin.left, kidReflowState.mComputedMargin.top,
+                        borderPadding.left + kidReflowState.mComputedMargin.left,
+                        borderPadding.top + mLegendSpace + kidReflowState.mComputedMargin.top,
                         0, aStatus);
 #ifdef NOISY_REFLOW
             printf("  returned (%d, %d)\n", kidDesiredSize.width, kidDesiredSize.height);
@@ -522,10 +537,13 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
         }
     }
 
+    // use the computed width if the inner content does not fill it
+    if (aReflowState.mComputedWidth != NS_INTRINSICSIZE &&
+        aReflowState.mComputedWidth > contentRect.width) {
+      contentRect.width = aReflowState.mComputedWidth;
+    }
+
     if (mLegendFrame) {
-      // use the computed width if the inner content does not fill it
-      if (aReflowState.mComputedWidth != NS_INTRINSICSIZE && aReflowState.mComputedWidth > contentRect.width ) 
-        contentRect.width = aReflowState.mComputedWidth;
       // if the content rect is larger then the  legend we can align the legend
       if (contentRect.width > mLegendRect.width) {
         PRInt32 align = ((nsLegendFrame*)mLegendFrame)->GetAlign();
