@@ -43,7 +43,6 @@
 #include "nsIContent.h"
 #include "nsIComponentManager.h"
 #include "nsHTMLAtoms.h"
-#include "nsINameSpaceManager.h"
 #include "nsIPresState.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
@@ -54,6 +53,7 @@
 #include "nsIServiceManager.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMHTMLInputElement.h"
+#include "nsITheme.h"
 
 
 //------------------------------------------------------------
@@ -187,6 +187,14 @@ nsGfxCheckboxControlFrame::PaintCheckBox(nsIPresContext* aPresContext,
                                          const nsRect& aDirtyRect,
                                          nsFramePaintLayer aWhichLayer)
 {
+  const nsStyleDisplay* disp = (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
+  if (disp->mAppearance) {
+    nsCOMPtr<nsITheme> theme;
+    aPresContext->GetTheme(getter_AddRefs(theme));
+    if (theme && theme->ThemeSupportsWidget(aPresContext, this, disp->mAppearance))
+      return; // No need to paint the checkbox. The theme will do it.
+  }
+
   aRenderingContext.PushState();
 
   float p2t;
@@ -237,6 +245,8 @@ nsGfxCheckboxControlFrame::Paint(nsIPresContext*   aPresContext,
       if (myColor->mBackgroundImage.Length() > 0) {
         const nsStyleBorder* myBorder = (const nsStyleBorder*)
             mCheckButtonFaceStyle->GetStyleData(eStyleStruct_Border);
+        const nsStylePadding* myPadding = (const nsStylePadding*)
+            mCheckButtonFaceStyle->GetStyleData(eStyleStruct_Padding);
         const nsStylePosition* myPosition = (const nsStylePosition*)
             mCheckButtonFaceStyle->GetStyleData(eStyleStruct_Position);
 
@@ -248,7 +258,8 @@ nsGfxCheckboxControlFrame::Paint(nsIPresContext*   aPresContext,
         nsRect rect(x, y, width, height); 
 
         nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                          aDirtyRect, rect, *myBorder, 0, 0);
+                                        aDirtyRect, rect, *myBorder, *myPadding,
+                                        0, 0);
         nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
                                     aDirtyRect, rect, *myBorder, mCheckButtonFaceStyle, 0);
         doDefaultPainting = PR_FALSE;
