@@ -48,6 +48,7 @@
 #include "nsIHTMLContent.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLAtoms.h"
+#include "nsLayoutAtoms.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsFont.h"
@@ -80,6 +81,15 @@ nsLegendFrame::~nsLegendFrame()
 {
 }
 
+NS_IMETHODIMP
+nsLegendFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::legendFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsLegendFrame::Destroy(nsIPresContext *aPresContext)
 {
