@@ -53,7 +53,6 @@
 #include "nsHTMLAtoms.h"
 #include "nsVoidArray.h"
 #include "nsIView.h"
-#include "nsStyleUtil.h"
 #include "nsLayoutAtoms.h"
 #include "nsIFrameManager.h"
 #include "nsIPresShell.h"
@@ -325,8 +324,7 @@ nsresult nsTableCellFrame::SetColIndex(PRInt32 aColIndex)
     return rv;
 
   nsIHTMLTableCellElement* cellContent = nsnull;
-  rv = cell->QueryInterface(NS_GET_IID(nsIHTMLTableCellElement), 
-                            (void **)&cellContent);  // cellContent: REFCNT++
+  rv = CallQueryInterface(cell, &cellContent); // cellContent: REFCNT++
   if (cellContent && NS_SUCCEEDED(rv)) { // it's a table cell
     cellContent->SetColIndex(aColIndex);
     NS_RELEASE(cellContent);
@@ -419,13 +417,15 @@ nsTableCellFrame::PaintUnderlay(nsIPresContext&           aPresContext,
                                 PRUint32&                 aFlags,
                                 const nsStyleTableBorder& aCellTableStyle,
                                 const nsStyleBorder&      aStyleBorder,
+                                const nsStylePadding&     aStylePadding,
                                 PRBool                    aVisibleBackground,
                                 PRBool&                   aPaintChildren)
 {
   if (aVisibleBackground) {
     nsRect rect(0, 0, mRect.width, mRect.height);
     nsCSSRendering::PaintBackground(&aPresContext, aRenderingContext, this,
-                                    aDirtyRect, rect, aStyleBorder, 0, 0, PR_TRUE);
+                                    aDirtyRect, rect, aStyleBorder, aStylePadding,
+                                    0, 0, PR_TRUE);
     // draw the border only when there is content or showing empty cells
     if (!GetContentEmpty() || NS_STYLE_TABLE_EMPTY_CELLS_SHOW == aCellTableStyle.mEmptyCells) {
       PRIntn skipSides = GetSkipSides();
@@ -456,11 +456,15 @@ nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
   if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
     PRBool paintBackground = PR_FALSE;
     const nsStyleBorder*      myBorder       = nsnull;
+    const nsStylePadding*     myPadding      = nsnull;
     const nsStyleTableBorder* cellTableStyle = nsnull;
     const nsStyleVisibility* vis = 
        (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
     if (vis->IsVisibleOrCollapsed()) {
-      myBorder = (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border); NS_ENSURE_TRUE(myBorder, NS_ERROR_NULL_POINTER);
+      myBorder = (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
+      NS_ENSURE_TRUE(myBorder, NS_ERROR_NULL_POINTER);
+      myPadding = (const nsStylePadding*)mStyleContext->GetStyleData(eStyleStruct_Border);
+      NS_ENSURE_TRUE(myPadding, NS_ERROR_NULL_POINTER);
 
       GetStyleData(eStyleStruct_TableBorder, ((const nsStyleStruct *&)cellTableStyle));
 
@@ -471,7 +475,7 @@ nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
     }
   
     PaintUnderlay(*aPresContext, aRenderingContext, aDirtyRect, aFlags, *cellTableStyle,
-                  *myBorder, paintBackground, paintChildren);
+                  *myBorder, *myPadding, paintBackground, paintChildren);
 
     if (vis->IsVisibleOrCollapsed()) {
       const nsStyleBackground* myColor = 
@@ -1015,8 +1019,7 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
   if (0 == kidSize.height) {
     if ((pos->mHeight.GetUnit() != eStyleUnit_Coord) &&
         (pos->mHeight.GetUnit() != eStyleUnit_Percent)) {
-      // Standard mode should probably be 0 pixels high instead of 1
-      PRInt32 pixHeight = (eCompatibility_NavQuirks == compatMode) ? 2 : 1;
+      PRInt32 pixHeight = (eCompatibility_NavQuirks == compatMode) ? 1 : 0;
       kidSize.height = NSIntPixelsToTwips(pixHeight, p2t);
       if ((nsnull != aDesiredSize.maxElementSize) && (0 == pMaxElementSize->height))
         pMaxElementSize->height = kidSize.height;
@@ -1283,7 +1286,6 @@ NS_IMETHODIMP nsTableCellFrame::GetAccessible(nsIAccessible** aAccessible)
   nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
 
   if (accService) {
-    nsIAccessible* acc = nsnull;
     return accService->CreateHTMLTableCellAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
   }
 
@@ -1323,11 +1325,10 @@ nsTableCellFrame::GetPreviousCellInColumn(nsITableCellLayout **aCellLayout)
   {
     // Get the cellframe at previous colIndex
     nsTableCellFrame *cellFrame = tableFrame->GetCellFrameAt(rowIndex, colIndex-1);
-    if (cellFrame)
-      cellFrame->QueryInterface(NS_GET_IID(nsITableCellLayout), (void **)aCellLayout);  
+    if (!cellFrame) return NS_ERROR_FAILURE;
+    return CallQueryInterface(cellFrame, aCellLayout);
   }
-
-  return NS_OK;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -1347,10 +1348,8 @@ nsTableCellFrame::GetNextCellInColumn(nsITableCellLayout **aCellLayout)
 
   // Get the cellframe at next colIndex
   nsTableCellFrame *cellFrame = tableFrame->GetCellFrameAt(rowIndex, colIndex+1);
-  if (cellFrame)
-    cellFrame->QueryInterface(NS_GET_IID(nsITableCellLayout), (void **)aCellLayout);  
-
-  return NS_OK;
+  if (!cellFrame) return NS_ERROR_FAILURE;
+  return CallQueryInterface(cellFrame, aCellLayout);
 }
 
 nsresult 
@@ -1538,6 +1537,7 @@ nsBCTableCellFrame::PaintUnderlay(nsIPresContext&           aPresContext,
                                   PRUint32&                 aFlags,
                                   const nsStyleTableBorder& aCellTableStyle,
                                   const nsStyleBorder&      aStyleBorder,
+                                  const nsStylePadding&     aStylePadding,
                                   PRBool                    aVisibleBackground,
                                   PRBool&                   aPaintChildren)
 {
@@ -1562,7 +1562,8 @@ nsBCTableCellFrame::PaintUnderlay(nsIPresContext&           aPresContext,
 
     nsRect rect(0, 0, mRect.width, mRect.height);
     nsCSSRendering::PaintBackground(&aPresContext, aRenderingContext, this,
-                                    aDirtyRect, rect, myBorder, 0, 0, PR_TRUE);
+                                    aDirtyRect, rect, myBorder, aStylePadding,
+                                    0, 0, PR_TRUE);
     // borders are painted by nsTableFrame
   }
 
