@@ -21,23 +21,19 @@
  */
 
 #include "nsCOMPtr.h"
-#include "nsHTMLParts.h"
-#include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
 #include "nsIPresContext.h"
-#include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsINameSpaceManager.h"
 #include "nsString.h"
 #include "nsUnicharUtils.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
+#include "nsISupportsPrimitives.h"
 #include "nsIComponentManager.h"
 #include "nsIPersistentProperties2.h"
 #include "nsIServiceManager.h"
@@ -154,11 +150,11 @@ AlertMissingFonts(nsString& aMissingFonts)
   sb->FormatStringFromName(NS_LITERAL_STRING("mathfont_missing_dialog_message").get(),
                            strings, 1, getter_Copies(message));
 
-  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+  nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (!wwatch)
     return;
 
-  nsCOMPtr<nsIDialogParamBlock> paramBlock(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> paramBlock(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!paramBlock)
     return;
 
@@ -310,7 +306,7 @@ public:
             nsRect*              aClipRect = nsnull);
 
 private:
-  PRUnichar   GetAnnotation(nsMathMLChar* aChar, PRInt32 aPosition);
+  char   GetAnnotation(nsMathMLChar* aChar, PRInt32 aPosition);
   nsGlyphCode ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUint32 aPosition);
 
   // The type is either NS_TABLE_TYPE_UNICODE or NS_TABLE_TYPE_GLYPH_INDEX
@@ -347,7 +343,7 @@ private:
   PRUnichar mCharCache;
 };
 
-PRUnichar
+char
 nsGlyphTable::GetAnnotation(nsMathMLChar* aChar, PRInt32 aPosition)
 {
   NS_ASSERTION(aChar->mDirection == NS_STRETCH_DIRECTION_VERTICAL ||
@@ -359,9 +355,9 @@ nsGlyphTable::GetAnnotation(nsMathMLChar* aChar, PRInt32 aPosition)
     // return an ASCII digit for the size=0,1,2,...
     return PRUnichar('0' + aPosition - 4);
   }
-  return (aChar->mDirection == NS_STRETCH_DIRECTION_VERTICAL)
-    ? PRUnichar(kVertical[aPosition])
-    : PRUnichar(kHorizontal[aPosition]);
+  return (aChar->mDirection == NS_STRETCH_DIRECTION_VERTICAL) ?
+      kVertical[aPosition] :
+      kHorizontal[aPosition];
 }
 
 nsGlyphCode
@@ -391,9 +387,10 @@ nsGlyphTable::ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUin
     mState = NS_TABLE_STATE_READY;
 
     // see if there are external fonts needed for certain chars in this table
-    nsAutoString key, value;
+    nsCAutoString key;
+    nsAutoString value;
     for (PRInt32 i = 1; ; i++) {
-      key.Assign(NS_LITERAL_STRING("external."));
+      key.Assign(NS_LITERAL_CSTRING("external."));
       key.AppendInt(i, 10);
       rv = mGlyphProperties->GetStringProperty(key, value);
       if (NS_FAILED(rv)) break;
@@ -409,9 +406,12 @@ nsGlyphTable::ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUin
   // Update our cache if it is not associated to this character
   PRUnichar uchar = aChar->mData[0];
   if (mCharCache != uchar) {
-    // The key in the property file is interpreted as ASCII and kept as such ...
+    // The key in the property file is interpreted as ASCII and kept
+    // as such ...
     char cbuf[10]; PR_snprintf(cbuf, sizeof(cbuf), "\\u%04X", uchar);
-    nsAutoString key, value; key.AssignWithConversion(cbuf);
+    nsDependentCString key(cbuf);
+    
+    nsAutoString value;
     nsresult rv = mGlyphProperties->GetStringProperty(key, value);
     if (NS_FAILED(rv)) return kNullGlyph;
     Clean(value);
@@ -421,7 +421,8 @@ nsGlyphTable::ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUin
     // as combined pairs of 'code@font', excluding the '@' separator. This means that
     // mGlyphCache[2*k] will later be rendered with mFontName[mGlyphCache[2*k+1]-'0']
     // Note: font identifier is internally an ASCII digit to avoid the null char issue
-    nsAutoString buffer, puaKey, puaValue;
+    nsAutoString buffer, puaValue;
+    nsCAutoString puaKey;
     PRInt32 length = value.Length();
     for (PRInt32 i = 0, j = 0; i < length; i++, j++) {
       PRUnichar code = value[i];
@@ -435,7 +436,7 @@ nsGlyphTable::ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUin
       // the PUA, and lookup "key.[TLMBRG1-9]" in the PUA
       else if (code == PRUnichar(0xF8FF)) {
         puaKey.Assign(key);
-        puaKey.Append(PRUnichar('.'));
+        puaKey.Append('.');
         puaKey.Append(GetAnnotation(aChar, j));
         rv = gPUAProperties->GetStringProperty(puaKey, puaValue);
         if (NS_FAILED(rv) || !puaValue.Length()) return kNullGlyph;
@@ -446,9 +447,9 @@ nsGlyphTable::ElementAt(nsIPresContext* aPresContext, nsMathMLChar* aChar, PRUin
       else if ((i+2 < length) && (value[i+1] == PRUnichar('.'))) {
         i += 2;
         PR_snprintf(cbuf, sizeof(cbuf), "\\u%04X", code);
-        puaKey.AssignWithConversion(cbuf);
-        puaKey.Append(PRUnichar('.'));
-        puaKey.Append(value[i]);
+        puaKey.Assign(cbuf);
+        puaKey.Append('.');
+        puaKey.Append(char(value[i])); // safe cast, it's ascii
         rv = gPUAProperties->GetStringProperty(puaKey, puaValue);
         if (NS_FAILED(rv) || !puaValue.Length()) return kNullGlyph;
         code = puaValue[0];
@@ -1028,12 +1029,12 @@ PreferredFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
 
 // Store the list of preferred extension fonts for this char
 static void
-SetPreferredTableList(PRUnichar aChar, nsString& aExtension, nsString& aFamilyList)
+SetPreferredTableList(PRUnichar aChar, nsACString& aExtension, nsString& aFamilyList)
 {
   PRBool isFontForParts;
-  if (aExtension.Equals(NS_LITERAL_STRING(".parts")))
+  if (aExtension.Equals(NS_LITERAL_CSTRING(".parts")))
     isFontForParts = PR_TRUE;
-  else if (aExtension.Equals(NS_LITERAL_STRING(".variants")))
+  else if (aExtension.Equals(NS_LITERAL_CSTRING(".variants")))
     isFontForParts = PR_FALSE;
   else return; // input is not applicable
 
@@ -1126,7 +1127,8 @@ InitGlobals(nsIPresContext* aPresContext)
     It will be deleted at shutdown, even if a failure happens below.
   */
 
-  nsAutoString key, value;
+  nsCAutoString key;
+  nsAutoString value;
   nsCOMPtr<nsIPersistentProperties> mathfontProp;
 
   // Add the math fonts in the gGlyphTableList in order of preference ...
@@ -1145,16 +1147,22 @@ InitGlobals(nsIPresContext* aPresContext)
 
   // Get the default list of mathfonts to be used for stretchy characters
   nsFont font(nsnull, 0, 0, 0, 0, 0);
-  nsXPIDLString familyList;
-  nsCOMPtr<nsIPref> pref;
-  pref = do_GetService(NS_PREF_CONTRACTID);
-  if (pref) {
+  nsAutoString familyList;
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (prefBranch) {
     // first, try to see if the user has a value in the pref
-    pref->CopyUnicharPref("font.mathfont-family", getter_Copies(familyList));
+    nsCOMPtr<nsISupportsString> prefString;
+    prefBranch->GetComplexValue("font.mathfont-family",
+                                NS_GET_IID(nsISupportsString),
+                                getter_AddRefs(prefString));
+    if (prefString) {
+      prefString->GetData(familyList);
+    }
   }
   if (familyList.IsEmpty()) {
     // fallback to the default list
-    rv = mathfontProp->GetStringProperty(NS_LITERAL_STRING("mathfont-family"), value);
+    rv = mathfontProp->GetStringProperty(NS_LITERAL_CSTRING("mathfont-family"),
+                                         value);
     if (NS_FAILED(rv)) return rv;
     font.name.Assign(value);
   }
@@ -1176,15 +1184,13 @@ InitGlobals(nsIPresContext* aPresContext)
 
   // Let the particular characters have their preferred extension tables
   nsCOMPtr<nsISimpleEnumerator> iterator;
-  if (NS_SUCCEEDED(mathfontProp->SimpleEnumerateProperties(getter_AddRefs(iterator)))) {
+  if (NS_SUCCEEDED(mathfontProp->Enumerate(getter_AddRefs(iterator)))) {
     PRBool more;
     while ((NS_SUCCEEDED(iterator->HasMoreElements(&more))) && more) {
       nsCOMPtr<nsIPropertyElement> element;
       if (NS_SUCCEEDED(iterator->GetNext(getter_AddRefs(element)))) {
-        nsXPIDLString xkey, xvalue;
-        if (NS_SUCCEEDED(element->GetKey(getter_Copies(xkey))) &&
-            NS_SUCCEEDED(element->GetValue(getter_Copies(xvalue)))) {
-          key.Assign(xkey);
+        if (NS_SUCCEEDED(element->GetKey(key)) &&
+            NS_SUCCEEDED(element->GetValue(value))) {
           // expected key: "mathfont-family.\uNNNN.parts" or
           // "mathfont-family.\uNNNN.variants"
           if ((22 <= key.Length()) && (0 == key.Find("mathfont-family.\\u"))) {
@@ -1193,7 +1199,6 @@ InitGlobals(nsIPresContext* aPresContext)
             PRUnichar uchar = key.ToInteger(&error, 16);
             if (error) continue;
             key.Cut(0, 4); // the digits of the unicode point ("NNNN")
-            value.Assign(xvalue);
             Clean(value);
             SetPreferredTableList(uchar, key, value);
           }
@@ -1470,7 +1475,7 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   // mStyleContext is a leaf context used only when stretching happens.
   // For the base size, the default font should come from the parent context
   nsAutoString fontName;
-  nsCOMPtr<nsIStyleContext> parentContext(dont_AddRef(mStyleContext->GetParent()));
+  nsCOMPtr<nsIStyleContext> parentContext(mStyleContext->GetParent());
   const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
     parentContext->GetStyleData(eStyleStruct_Font));
   nsFont theFont(font->mFont);
@@ -1870,7 +1875,7 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
                     const nsRect*        aSelectedRect)
 {
   nsresult rv = NS_OK;
-  nsCOMPtr<nsIStyleContext> parentContext(dont_AddRef(mStyleContext->GetParent()));
+  nsCOMPtr<nsIStyleContext> parentContext(mStyleContext->GetParent());
   nsIStyleContext* styleContext = mStyleContext;
 
   if (NS_STRETCH_DIRECTION_UNSUPPORTED == mDirection) {
@@ -1903,17 +1908,20 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
     else if (mRect.width && mRect.height) {
       const nsStyleBorder *border = NS_STATIC_CAST(const nsStyleBorder*,
         styleContext->GetStyleData(eStyleStruct_Border));
+      const nsStylePadding *padding = NS_STATIC_CAST(const nsStylePadding*,
+        styleContext->GetStyleData(eStyleStruct_Padding));
       const nsStyleBackground *backg = NS_STATIC_CAST(const nsStyleBackground*,
         styleContext->GetStyleData(eStyleStruct_Background));
       nsRect rect(mRect); //0, 0, mRect.width, mRect.height);
       if (styleContext != parentContext.get() &&
           0 == (backg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT))
         nsCSSRendering::PaintBackgroundWithSC(aPresContext, aRenderingContext, aForFrame,
-                                              aDirtyRect, rect, *backg, *border, 0, 0);
+                                              aDirtyRect, rect, *backg, *border, *padding,
+                                              0, 0);
       //else
       //  our container frame will take care of painting its background
       //  nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, aForFrame,
-      //                                  aDirtyRect, rect, *border, 0, 0);
+      //                                  aDirtyRect, rect, *border, *padding, 0, 0);
 #if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
       // for visual debug
       PRIntn skipSides = 0; //aForFrame->GetSkipSides();
