@@ -25,17 +25,15 @@
 #include "nsHTMLParts.h"
 #include "nsIHTMLContent.h"
 #include "nsFrame.h"
-#include "nsLineLayout.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsHTMLAtoms.h"
+#include "nsCSSAnonBoxes.h"
 #include "nsUnitConversion.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsINameSpaceManager.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
-#include "nsStyleUtil.h"
 
 #include "nsIDOMText.h"
 #include "nsITextContent.h"
@@ -598,7 +596,7 @@ nsMathMLContainerFrame::PropagateScriptStyleFor(nsIPresContext* aPresContext,
 
     nsCOMPtr<nsIStyleContext> oldStyleContext;
     aFrame->GetStyleContext(getter_AddRefs(oldStyleContext));
-    nsCOMPtr<nsIStyleContext> parentContext(dont_AddRef(oldStyleContext->GetParent()));
+    nsCOMPtr<nsIStyleContext> parentContext(oldStyleContext->GetParent());
 
     nsCOMPtr<nsIContent> content;
     aFrame->GetContent(getter_AddRefs(content));
@@ -671,7 +669,7 @@ nsMathMLContainerFrame::PropagateScriptStyleFor(nsIPresContext* aPresContext,
       nsCOMPtr<nsIFrameManager> fm;
       presShell->GetFrameManager(getter_AddRefs(fm));
       if (fm) {
-        PRInt32 maxChange, minChange = NS_STYLE_HINT_NONE;
+        nsChangeHint maxChange = NS_STYLE_HINT_NONE, minChange = NS_STYLE_HINT_NONE;
         nsStyleChangeList changeList;
         fm->ComputeStyleChangeFor(aPresContext, aFrame,
                                   kNameSpaceID_None, nsMathMLAtoms::fontsize,
@@ -728,7 +726,8 @@ nsMathMLContainerFrame::WrapForeignFrames(nsIPresContext* aPresContext)
       nsresult rv = NS_NewMathMLForeignFrameWrapper(shell, &wrapper);
       if (NS_FAILED(rv)) return rv;
       nsCOMPtr<nsIStyleContext> newStyleContext;
-      aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::mozAnonymousBlock,
+      aPresContext->ResolvePseudoStyleContextFor(mContent,
+                                                 nsCSSAnonBoxes::mozAnonymousBlock,
                                                  mStyleContext,
                                                  getter_AddRefs(newStyleContext));
       rv = wrapper->Init(aPresContext, mContent, this, newStyleContext, nsnull);
@@ -736,7 +735,7 @@ nsMathMLContainerFrame::WrapForeignFrames(nsIPresContext* aPresContext)
         wrapper->Destroy(aPresContext);
         return rv;
       }
-      mFrames.ReplaceFrame(this, child, wrapper);
+      mFrames.ReplaceFrame(aPresContext, this, child, wrapper, PR_FALSE);
       child->SetParent(wrapper);
       child->SetNextSibling(nsnull);
       aPresContext->ReParentStyleContext(child, newStyleContext);
@@ -788,37 +787,6 @@ nsMathMLContainerFrame::Paint(nsIPresContext*      aPresContext,
   return rv;
 }
 
-static void
-CompressWhitespace(nsIContent* aContent)
-{
-  nsCOMPtr<nsIAtom> tag;
-  aContent->GetTag(*getter_AddRefs(tag));
-  if (tag.get() == nsMathMLAtoms::mo_ ||
-      tag.get() == nsMathMLAtoms::mi_ ||
-      tag.get() == nsMathMLAtoms::mn_ ||
-      tag.get() == nsMathMLAtoms::ms_ ||
-      tag.get() == nsMathMLAtoms::mtext_) {
-    PRInt32 numKids;
-    aContent->ChildCount(numKids);
-    for (PRInt32 kid = 0; kid < numKids; kid++) {
-      nsCOMPtr<nsIContent> kidContent;
-      aContent->ChildAt(kid, *getter_AddRefs(kidContent));
-      if (kidContent.get()) {       
-        nsCOMPtr<nsIDOMText> kidText(do_QueryInterface(kidContent));
-        if (kidText.get()) {
-          nsCOMPtr<nsITextContent> tc(do_QueryInterface(kidContent));
-          if (tc) {
-            nsAutoString text;
-            tc->CopyText(text);
-            text.CompressWhitespace();
-            tc->SetText(text, PR_FALSE); // not meant to be used if notify is needed
-          }
-        }
-      }
-    }
-  }
-}
-
 // This method is called in a top-down manner, as we descend the frame tree
 // during its construction
 NS_IMETHODIMP
@@ -828,11 +796,6 @@ nsMathMLContainerFrame::Init(nsIPresContext*  aPresContext,
                              nsIStyleContext* aContext,
                              nsIFrame*        aPrevInFlow)
 {
-  // leading and trailing whitespace doesn't count -- bug 15402
-  // brute force removal for people who do <mi> a </mi> instead of <mi>a</mi>
-  // XXX the best fix is to skip these in nsTextFrame
-  CompressWhitespace(aContent);
-
   MapAttributesIntoCSS(aPresContext, aContent);
 
   // let the base class do its Init()
@@ -1051,13 +1014,7 @@ nsMathMLContainerFrame::ReplaceFrame(nsIPresContext* aPresContext,
     return NS_ERROR_INVALID_ARG;
   }
   // Replace the old frame with the new frame in the list
-  mFrames.ReplaceFrame(this, aOldFrame, aNewFrame);
-
-  // XXX now destroy the old frame, really? the usage of ReplaceFrame() vs.
-  // XXX ReplaceFrameAndDestroy() is ambiguous - see bug 122748
-  // XXX The style system doesn't call ReplaceFrame() and that's why
-  // XXX nobody seems to have been biten by the ambiguity yet
-  aOldFrame->Destroy(aPresContext);
+  mFrames.ReplaceFrame(aPresContext, this, aOldFrame, aNewFrame, PR_TRUE);
 
   return ChildListChanged(aPresContext, nsIDOMMutationEvent::MODIFICATION);
 }
@@ -1085,132 +1042,6 @@ nsMathMLContainerFrame::AttributeChanged(nsIPresContext* aPresContext,
   return ReflowDirtyChild(presShell, nsnull);
 }
 
-// helper function to reflow token elements
-// note that mBoundingMetrics is computed here
-nsresult
-nsMathMLContainerFrame::ReflowTokenFor(nsIFrame*                aFrame,
-                                       nsIPresContext*          aPresContext,
-                                       nsHTMLReflowMetrics&     aDesiredSize,
-                                       const nsHTMLReflowState& aReflowState,
-                                       nsReflowStatus&          aStatus)
-{
-  NS_PRECONDITION(aFrame, "null arg");
-  nsresult rv = NS_OK;
-
-  // See if this is an incremental reflow
-  if (aReflowState.reason == eReflowReason_Incremental) {
-#ifdef MATHML_NOISY_INCREMENTAL_REFLOW
-printf("nsMathMLContainerFrame::ReflowTokenFor:IncrementalReflow received by: ");
-nsFrame::ListTag(stdout, aFrame);
-printf("\n");
-#endif
-  }
-
-  // initializations needed for empty markup like <mtag></mtag>
-  aDesiredSize.width = aDesiredSize.height = 0;
-  aDesiredSize.ascent = aDesiredSize.descent = 0;
-  aDesiredSize.mBoundingMetrics.Clear();
-
-  // ask our children to compute their bounding metrics
-  nsHTMLReflowMetrics childDesiredSize(aDesiredSize.maxElementSize,
-                      aDesiredSize.mFlags | NS_REFLOW_CALC_BOUNDING_METRICS);
-  nsSize availSize(aReflowState.mComputedWidth, aReflowState.mComputedHeight);
-  PRInt32 count = 0;
-  nsIFrame* childFrame;
-  aFrame->FirstChild(aPresContext, nsnull, &childFrame);
-  while (childFrame) {
-    nsHTMLReflowState childReflowState(aPresContext, aReflowState,
-                                       childFrame, availSize);
-    rv = NS_STATIC_CAST(nsMathMLContainerFrame*,
-                        aFrame)->ReflowChild(childFrame,
-                                             aPresContext, childDesiredSize,
-                                             childReflowState, aStatus);
-    //NS_ASSERTION(NS_FRAME_IS_COMPLETE(aStatus), "bad status");
-    if (NS_FAILED(rv)) return rv;
-
-    // origins are used as placeholders to store the child's ascent and descent.
-    childFrame->SetRect(aPresContext,
-                        nsRect(childDesiredSize.descent, childDesiredSize.ascent,
-                               childDesiredSize.width, childDesiredSize.height));
-    // compute and cache the bounding metrics
-    if (0 == count)
-      aDesiredSize.mBoundingMetrics  = childDesiredSize.mBoundingMetrics;
-    else
-      aDesiredSize.mBoundingMetrics += childDesiredSize.mBoundingMetrics;
-
-    count++;
-    childFrame->GetNextSibling(&childFrame);
-  }
-
-  // cache the frame's mBoundingMetrics
-  NS_STATIC_CAST(nsMathMLContainerFrame*,
-                 aFrame)->SetBoundingMetrics(aDesiredSize.mBoundingMetrics);
-
-  // place and size children
-  NS_STATIC_CAST(nsMathMLContainerFrame*,
-                 aFrame)->FinalizeReflow(aPresContext, *aReflowState.rendContext,
-                                         aDesiredSize);
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
-  return NS_OK;
-}
-
-// helper function to place token elements
-// mBoundingMetrics is computed at the ReflowToken pass, it is
-// not computed here because our children may be text frames that
-// do not implement the GetBoundingMetrics() interface.
-nsresult
-nsMathMLContainerFrame::PlaceTokenFor(nsIFrame*            aFrame,
-                                      nsIPresContext*      aPresContext,
-                                      nsIRenderingContext& aRenderingContext,
-                                      PRBool               aPlaceOrigin,
-                                      nsHTMLReflowMetrics& aDesiredSize)
-{
-  aDesiredSize.width = aDesiredSize.height = 0;
-  aDesiredSize.ascent = aDesiredSize.descent = 0;
-
-  const nsStyleFont* font;
-  aFrame->GetStyleData(eStyleStruct_Font, (const nsStyleStruct*&)font);
-  nsCOMPtr<nsIFontMetrics> fm;
-  aPresContext->GetMetricsFor(font->mFont, getter_AddRefs(fm));
-  nscoord ascent, descent;
-  fm->GetMaxAscent(ascent);
-  fm->GetMaxDescent(descent);
-
-  nsBoundingMetrics bm;
-  NS_STATIC_CAST(nsMathMLContainerFrame*, aFrame)->GetBoundingMetrics(bm);
-  aDesiredSize.mBoundingMetrics = bm;
-  aDesiredSize.width = bm.width;
-  aDesiredSize.ascent = PR_MAX(bm.ascent, ascent);
-  aDesiredSize.descent = PR_MAX(bm.descent, descent);
-  aDesiredSize.height = aDesiredSize.ascent + aDesiredSize.descent;
-
-  if (aPlaceOrigin) {
-    nscoord dy, dx = 0;
-    nsRect rect;
-    nsIFrame* childFrame;
-    aFrame->FirstChild(aPresContext, nsnull, &childFrame);
-    while (childFrame) {
-      childFrame->GetRect(rect);
-      nsHTMLReflowMetrics childSize(nsnull);
-      childSize.width = rect.width;
-      childSize.height = aDesiredSize.height; //rect.height;
-
-      // place and size the child
-      dy = aDesiredSize.ascent - rect.y;
-      NS_STATIC_CAST(nsMathMLContainerFrame*,
-                     aFrame)->FinishReflowChild(childFrame, aPresContext, nsnull,
-                                                childSize, dx, dy, 0);
-      dx += rect.width;
-      childFrame->GetNextSibling(&childFrame);
-    }
-  }
-
-  NS_STATIC_CAST(nsMathMLContainerFrame*,
-                 aFrame)->SetReference(nsPoint(0, aDesiredSize.ascent));
-
-  return NS_OK;
-}
-
 // We are an inline frame, so we handle dirty request like nsInlineFrame
 NS_IMETHODIMP
 nsMathMLContainerFrame::ReflowDirtyChild(nsIPresShell* aPresShell, nsIFrame* aChild)
@@ -1345,24 +1176,13 @@ nsMathMLContainerFrame::GetFrameType(nsIAtom** aType) const
 {
   NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
   // see if this is an embellished operator (mapped to 'Op' in TeX)
-  if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags)) {
-    *aType = nsMathMLAtoms::operatorMathMLFrame;
-  }
-  else {
-    nsCOMPtr<nsIAtom> tag;
-    mContent->GetTag(*getter_AddRefs(tag));
-    // see if this a token element (mapped to 'Ord'in TeX)
-    if (tag.get() == nsMathMLAtoms::mi_ ||
-        tag.get() == nsMathMLAtoms::mn_ ||
-        tag.get() == nsMathMLAtoms::ms_ ||
-        tag.get() == nsMathMLAtoms::mtext_) {
-      *aType = nsMathMLAtoms::ordinaryMathMLFrame;
-    }
-    else {
-      // everything else is a schematta element (mapped to 'Inner' in TeX)
-      *aType = nsMathMLAtoms::schemataMathMLFrame;
-    }
+  if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags) &&
+      mEmbellishData.coreFrame) {
+    return mEmbellishData.coreFrame->GetFrameType(aType);
   }
+
+  // everything else is a schematta element (mapped to 'Inner' in TeX)
+  *aType = nsMathMLAtoms::schemataMathMLFrame;
   NS_ADDREF(*aType);
   return NS_OK;
 }
@@ -1370,63 +1190,105 @@ nsMathMLContainerFrame::GetFrameType(nsIAtom** aType) const
 enum eMathMLFrameType {
   eMathMLFrameType_UNKNOWN = -1,
   eMathMLFrameType_Ordinary,
-  eMathMLFrameType_Operator,
-  eMathMLFrameType_Punctuation,
+  eMathMLFrameType_OperatorVisible,
+  eMathMLFrameType_OperatorInvisible,
   eMathMLFrameType_Inner
 };
 
 // see spacing table in Chapter 18, TeXBook (p.170)
-static PRInt32 interFrameSpacingTable[4][4] =
+// Our table isn't quite identical to TeX because operators have 
+// built-in values for lspace & rspace in the Operator Dictionary.
+static PRInt32 kInterFrameSpacingTable[4][4] =
 {
   // in units of muspace.
   // upper half of the byte is set if the
   // spacing is not to be used for scriptlevel > 0
-  /*          Ord   Op    Punc  Inner */
-  /*Ord  */  {0x01, 0x00, 0x00, 0x01},
-  /*Op   */  {0x00, 0x00, 0x00, 0x00},
-  /*Punc */  {0x11, 0x00, 0x11, 0x11},
-  /*Inner*/  {0x01, 0x00, 0x11, 0x01}
+  /*          Ord   OpVis OpInv Inner */
+  /*Ord  */  {0x01, 0x00, 0x01, 0x01},
+  /*OpVis*/  {0x00, 0x00, 0x00, 0x00},
+  /*OpInv*/  {0x01, 0x00, 0x00, 0x01},
+  /*Inner*/  {0x01, 0x00, 0x01, 0x01}
 };
 
-// XXX more tuning of the result as in TeX, maybe depending on fence:true, etc
+#define GET_INTERSPACE(scriptlevel_, frametype1_, frametype2_, space_)  \
+   /* no space if there is a frame that we know nothing about */        \
+   if (frametype1_ == eMathMLFrameType_UNKNOWN ||                       \
+       frametype2_ == eMathMLFrameType_UNKNOWN)                         \
+    space_ = 0;                                                         \
+  else {                                                                \
+    space_ = kInterFrameSpacingTable[frametype1_][frametype2_];         \
+    space_ = (scriptlevel_ > 0 && (space_ & 0xF0))                      \
+      ? 0 /* spacing is disabled */                                     \
+      : space_ & 0x0F;                                                  \
+  }                                                                     \
+
+#define MAP_FRAMETYPE(atomtype_, enumtype_)                             \
+  if (atomtype_ == nsMathMLAtoms::ordinaryMathMLFrame)                  \
+    enumtype_ = eMathMLFrameType_Ordinary;                              \
+  else if (atomtype_ == nsMathMLAtoms::schemataMathMLFrame)             \
+    enumtype_ = eMathMLFrameType_Inner;                                 \
+  else if (atomtype_ == nsMathMLAtoms::operatorVisibleMathMLFrame)      \
+    enumtype_ = eMathMLFrameType_OperatorVisible;                       \
+  else if (atomtype_ == nsMathMLAtoms::operatorInvisibleMathMLFrame)    \
+    enumtype_ = eMathMLFrameType_OperatorInvisible;                     \
+  else                                                                  \
+    enumtype_ = eMathMLFrameType_UNKNOWN;
+
+// This function computes the inter-space between two frames. However, 
+// since invisible operators need special treatment, the inter-space may
+// be delayed when an invisible operator is encountered. In this case,
+// the function will carry the inter-space forward until it is determined
+// that it can be applied properly (i.e., until we encounter a visible
+// frame where to decide whether to accept or reject the inter-space).
+// aFromFrameType: remembers the frame when the carry-forward initiated.
+// aCarrySpace: keeps track of the inter-space that is delayed.
+// @returns: current inter-space (which is 0 when the true inter-space is
+// delayed -- and thus has no effect since the frame is invisible anyway).
 static nscoord
-GetInterFrameSpacing(PRInt32  aScriptLevel,
-                     nsIAtom* aFirstFrameType,
-                     nsIAtom* aSecondFrameType)
+GetInterFrameSpacing(PRInt32           aScriptLevel,
+                     nsIAtom*          aFirstFrameType,
+                     nsIAtom*          aSecondFrameType,
+                     eMathMLFrameType* aFromFrameType, // IN/OUT
+                     PRInt32*          aCarrySpace)    // IN/OUT
 {
-  eMathMLFrameType firstType = eMathMLFrameType_UNKNOWN;
-  eMathMLFrameType secondType = eMathMLFrameType_UNKNOWN;
+  eMathMLFrameType firstType, secondType;
+  MAP_FRAMETYPE(aFirstFrameType, firstType);
+  MAP_FRAMETYPE(aSecondFrameType, secondType);
 
-  // do the mapping for the first frame
-  if (aFirstFrameType == nsMathMLAtoms::ordinaryMathMLFrame)
-    firstType = eMathMLFrameType_Ordinary;
-  else if (aFirstFrameType == nsMathMLAtoms::operatorMathMLFrame)
-    firstType = eMathMLFrameType_Operator;
-  else if (aFirstFrameType == nsMathMLAtoms::schemataMathMLFrame)
-    firstType = eMathMLFrameType_Inner;
+  PRInt32 space;
+  GET_INTERSPACE(aScriptLevel, firstType, secondType, space);
 
-  // do the mapping for the second frame
-  if (aSecondFrameType == nsMathMLAtoms::ordinaryMathMLFrame)
-    secondType = eMathMLFrameType_Ordinary;
-  else if (aSecondFrameType == nsMathMLAtoms::operatorMathMLFrame)
-    secondType = eMathMLFrameType_Operator;
-  else if (aSecondFrameType == nsMathMLAtoms::schemataMathMLFrame)
-    secondType = eMathMLFrameType_Inner;
+  // feedback control to avoid the inter-space to be added when not necessary
+  if (secondType == eMathMLFrameType_OperatorInvisible) {
+    // see if we should start to carry the space forward until we
+    // encounter a visible frame
+    if (*aFromFrameType == eMathMLFrameType_UNKNOWN) {
+      *aFromFrameType = firstType;
+      *aCarrySpace = space;
+    }
+    // keep carrying *aCarrySpace forward, while returning 0 for this stage
+    space = 0;
+  }
+  else if (*aFromFrameType != eMathMLFrameType_UNKNOWN) {
+    // no carry-forward anymore, get the real inter-space between
+    // the two frames of interest
+    GET_INTERSPACE(aScriptLevel, *aFromFrameType, secondType, space);
 
-  // return 0 if there is a frame that we know nothing about
-  if (firstType == eMathMLFrameType_UNKNOWN ||
-      secondType == eMathMLFrameType_UNKNOWN) {
-    return 0;
+    // Now, we have two values: the computed space and the space that
+    // has been carried forward until now. Which value do we pick?
+    // If the second type is an operator (e.g., fence), it already has
+    // built-in lspace & rspace, so we let them win. Otherwise we pick
+    // the max between the two values that we have.
+    if (secondType != eMathMLFrameType_OperatorVisible &&
+        space < *aCarrySpace)
+      space = *aCarrySpace;
+
+    // reset everything now that the carry-forward is done
+    *aFromFrameType = eMathMLFrameType_UNKNOWN;
+    *aCarrySpace = 0;
   }
 
-  PRInt32 space = interFrameSpacingTable[firstType][secondType];
-  if (aScriptLevel > 0 && (space & 0xF0)) {
-    // spacing is disabled
-    return 0;
-  }
-  else {
-    return (space & 0x0F);
-  }
+  return space;
 }
 
 NS_IMETHODIMP
@@ -1446,6 +1308,8 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
   nscoord thinSpace = NSToCoordRound(float(font->mFont.size)*float(3) / float(18));
 
   PRInt32 count = 0;
+  PRInt32 carrySpace = 0;
+  eMathMLFrameType fromFrameType = eMathMLFrameType_UNKNOWN;
   nsHTMLReflowMetrics childSize (nsnull);
   nsBoundingMetrics bmChild;
   nscoord leftCorrection = 0, italicCorrection = 0;
@@ -1477,7 +1341,7 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
         aDesiredSize.ascent = childSize.ascent;
       // add inter frame spacing
       nscoord space = GetInterFrameSpacing(mPresentationData.scriptLevel,
-        prevFrameType, childFrameType);
+        prevFrameType, childFrameType, &fromFrameType, &carrySpace);
       mBoundingMetrics.width += space * thinSpace;
       // add the child size
       mBoundingMetrics += bmChild;
@@ -1510,6 +1374,8 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
     count = 0;
     nscoord dx = 0, dy = 0;
     italicCorrection = 0;
+    carrySpace = 0;
+    fromFrameType = eMathMLFrameType_UNKNOWN;
     childFrame = mFrames.FirstChild();
     while (childFrame) {
       nsCOMPtr<nsIAtom> childFrameType;
@@ -1527,7 +1393,7 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
       else {
         // add inter frame spacing
         nscoord space = GetInterFrameSpacing(mPresentationData.scriptLevel,
-          prevFrameType, childFrameType);
+          prevFrameType, childFrameType, &fromFrameType, &carrySpace);
         dx += space * thinSpace;
       }
       count++;
@@ -1544,8 +1410,48 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
   return NS_OK;
 }
 
-// helper to fix the inter-spacing when <math> is the only parent
+// helpers to fix the inter-spacing when <math> is the only parent
 // e.g., it fixes <math> <mi>f</mi> <mo>q</mo> <mi>f</mi> <mo>I</mo> </math>
+
+static nscoord
+GetInterFrameSpacingFor(nsIPresContext* aPresContext,
+                        PRInt32         aScriptLevel,
+                        nsIFrame*       aParentFrame,
+                        nsIFrame*       aChildFrame)
+{
+  nsIFrame* childFrame;
+  aParentFrame->FirstChild(aPresContext, nsnull, &childFrame);
+  if (!childFrame || aChildFrame == childFrame)
+    return 0;
+
+  PRInt32 carrySpace = 0;
+  eMathMLFrameType fromFrameType = eMathMLFrameType_UNKNOWN;
+  nsCOMPtr<nsIAtom> childFrameType;
+  nsCOMPtr<nsIAtom> prevFrameType;
+  childFrame->GetFrameType(getter_AddRefs(childFrameType));
+  childFrame->GetNextSibling(&childFrame);
+  while (childFrame) {
+    prevFrameType = childFrameType;
+    childFrame->GetFrameType(getter_AddRefs(childFrameType));
+    nscoord space = GetInterFrameSpacing(aScriptLevel,
+      prevFrameType, childFrameType, &fromFrameType, &carrySpace);
+    if (aChildFrame == childFrame) {
+      // get thinspace
+      nsCOMPtr<nsIStyleContext> parentContext;
+      aParentFrame->GetStyleContext(getter_AddRefs(parentContext));
+      const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
+        parentContext->GetStyleData(eStyleStruct_Font));
+      nscoord thinSpace = NSToCoordRound(float(font->mFont.size)*float(3) / float(18));
+      // we are done
+      return space * thinSpace;
+    }
+    childFrame->GetNextSibling(&childFrame);
+  }
+
+  NS_NOTREACHED("child not in the childlist of its parent");
+  return 0;
+}
+
 nsresult
 nsMathMLContainerFrame::FixInterFrameSpacing(nsIPresContext*      aPresContext,
                                              nsHTMLReflowMetrics& aDesiredSize)
@@ -1554,38 +1460,17 @@ nsMathMLContainerFrame::FixInterFrameSpacing(nsIPresContext*      aPresContext,
   nsCOMPtr<nsIContent> parentContent;
   mParent->GetContent(getter_AddRefs(parentContent));
   parentContent->GetTag(*getter_AddRefs(parentTag));
-  if (parentTag.get() == nsMathMLAtoms::math) {
-    nsIFrame* childFrame;
-    mParent->FirstChild(aPresContext, nsnull, &childFrame);
-    nsFrameList frameList(childFrame);
-    nsIFrame* prevSibling = frameList.GetPrevSiblingFor(this);
-    nscoord gap = 0, leftCorrection, italicCorrection;
-    if (prevSibling) {
-      nsIMathMLFrame* mathMLFrame;
-      prevSibling->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-      if (mathMLFrame) {
-        // get thinspace
-        nsCOMPtr<nsIStyleContext> parentContext;
-        mParent->GetStyleContext(getter_AddRefs(parentContext));
-        const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
-          parentContext->GetStyleData(eStyleStruct_Font));
-        nscoord thinSpace = NSToCoordRound(float(font->mFont.size)*float(3) / float(18));
-        // add inter frame spacing to our width
-        nsCOMPtr<nsIAtom> frameType;
-        GetFrameType(getter_AddRefs(frameType));
-        nsCOMPtr<nsIAtom> prevFrameType;
-        prevSibling->GetFrameType(getter_AddRefs(prevFrameType));
-        nscoord space = GetInterFrameSpacing(mPresentationData.scriptLevel,
-          prevFrameType, frameType);
-        gap += space * thinSpace;
-      }
-    }
+  if (parentTag == nsMathMLAtoms::math ||
+      parentTag == nsMathMLAtoms::mtd_) {
+    nscoord gap = GetInterFrameSpacingFor(aPresContext,
+      mPresentationData.scriptLevel, mParent, this);
     // add our own italic correction
+    nscoord leftCorrection = 0, italicCorrection = 0;
     GetItalicCorrection(mBoundingMetrics, leftCorrection, italicCorrection);
     gap += leftCorrection;
     // see if we should shift our children to account for the correction
     if (gap) {
-      childFrame = mFrames.FirstChild();
+      nsIFrame* childFrame = mFrames.FirstChild();
       while (childFrame) {
         nsPoint origin;
         childFrame->GetOrigin(origin);
