@@ -653,6 +653,8 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
       // that may come from reconstructing the frame tree. Our rules only need
       // a re-resolve of style data and a reflow, not a reconstruct-all...
       sheet->SetOwningDocument(nsnull);
+      // We're about to manipulate the CSSOM, so we better do this
+      sheet->SetComplete();
     }
 
     // check for duplicate, if a similar rule is already there, don't bother to add another one
@@ -710,7 +712,7 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
     nsCOMPtr<nsIFrameManager> fm;
     presShell->GetFrameManager(getter_AddRefs(fm));
     if (fm) {
-      PRInt32 maxChange, minChange = NS_STYLE_HINT_NONE;
+      nsChangeHint maxChange = NS_STYLE_HINT_NONE, minChange = NS_STYLE_HINT_NONE;
       nsStyleChangeList changeList;
       fm->ComputeStyleChangeFor(aPresContext, aFrame,
                                 kNameSpaceID_None, nsnull,
