@@ -90,6 +90,7 @@
 #include "nsIScrollableFrame.h"
 #include "nsWidgetsCID.h"
 #include "nsLayoutAtoms.h"
+#include "nsCSSAnonBoxes.h"
 #include "nsViewsCID.h"
 #include "nsIScrollableView.h"
 #include "nsHTMLContainerFrame.h"
@@ -98,6 +99,10 @@
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsITheme.h"
+#include "nsTransform2D.h"
+#include "nsIEventListenerManager.h"
+#include "nsIEventStateManager.h"
+#include "nsIDOMEvent.h"
 
 // Needed for Print Preview
 #include "nsIDocument.h"
@@ -119,102 +124,8 @@ static NS_DEFINE_IID(kWidgetCID, NS_CHILD_CID);
 //#define DO_NOISY_REFLOW
 #endif
 
-/**
- * The boxes private implementation
- */
-class nsBoxFrameInner
-{
-public:
-  nsBoxFrameInner(nsIPresShell* aPresShell, nsBoxFrame* aThis)
-  {
-    mOuter = aThis;
-  }
-
-  ~nsBoxFrameInner()
-  {
-  }
-
-  //----- Sibling/Child ----
-
-  nsresult SetDebug(nsIPresContext* aPresContext, PRBool aDebug);
-
-  void Recycle(nsIPresShell* aPresShell);
-
-  // Overloaded new operator. Initializes the memory to 0 and relies on an arena
-  // (which comes from the presShell) to perform the allocation.
-  void* operator new(size_t sz, nsIPresShell* aPresShell);
-
-  // Overridden to prevent the global delete from being called, since the memory
-  // came out of an nsIArena instead of the global delete operator's heap.  
-  // XXX Would like to make this private some day, but our UNIX compilers can't 
-  // deal with it.
-  void operator delete(void* aPtr, size_t sz);
-
-  // helper methods
-
-
-   void TranslateEventCoords(nsIPresContext* aPresContext,
-                                    const nsPoint& aPoint,
-                                    nsPoint& aResult);
-
-    static PRBool AdjustTargetToScope(nsIFrame* aParent, nsIFrame*& aTargetFrame);
-
-
-    PRBool GetInitialDebug(PRBool& aDebug);
-
-    void GetDebugPref(nsIPresContext* aPresContext);
-
-    /*
-    nsresult PaintDebug(nsIBox* aBox, 
-                        nsIPresContext* aPresContext,
-                        nsIRenderingContext& aRenderingContext,
-                        const nsRect& aDirtyRect,
-                        nsFramePaintLayer aWhichLayer);
-*/
-
-    nsresult DisplayDebugInfoFor(nsIBox* aBox, 
-                                 nsIPresContext* aPresContext,
-                                 nsPoint&        aPoint,
-                                 PRInt32&        aCursor);
-
-    nsresult GetFrameSizeWithMargin(nsIBox* aBox, nsSize& aSize);
-
-    void GetDebugBorder(nsMargin& aInset);
-    void GetDebugPadding(nsMargin& aInset);
-    void GetDebugMargin(nsMargin& aInset);
-    void PixelMarginToTwips(nsIPresContext* aPresContext, nsMargin& aMarginPixels);
-
-    void GetValue(nsIPresContext* aPresContext, const nsSize& a, const nsSize& b, char* value);
-    void GetValue(nsIPresContext* aPresContext, PRInt32 a, PRInt32 b, char* value);
-    void DrawSpacer(nsIPresContext* aPresContext, nsIRenderingContext& aRenderingContext, PRBool aHorizontal, PRInt32 flex, nscoord x, nscoord y, nscoord size, nscoord spacerSize);
-    void DrawLine(nsIRenderingContext& aRenderingContext,  PRBool aHorizontal, nscoord x1, nscoord y1, nscoord x2, nscoord y2);
-    void FillRect(nsIRenderingContext& aRenderingContext,  PRBool aHorizontal, nscoord x, nscoord y, nscoord width, nscoord height);
-    void UpdateMouseThrough();
-
-    void CacheAttributes();
-
-    nsIBox* GetBoxForFrame(nsIFrame* aFrame, PRBool& aIsAdaptor);
-
-    nsBoxFrame::Halignment GetHAlign();
-    nsBoxFrame::Valignment GetVAlign();
-
-    // instance variables.
-    nsBoxFrame* mOuter;
-
-    nsBoxFrame::Valignment mValign;
-    nsBoxFrame::Halignment mHalign;
-
-    PRBool mAttributesCached;
-    
-    nsIPresContext* mPresContext;
-
-    static PRBool gDebug;
-    static nsIBox* mDebugChild;
-
-};
-
-PRBool nsBoxFrameInner::gDebug = PR_FALSE;
-nsIBox* nsBoxFrameInner::mDebugChild = nsnull;
+PRBool nsBoxFrame::gDebug = PR_FALSE;
+nsIBox* nsBoxFrame::mDebugChild = nsnull;
 
 nsresult
 NS_NewBoxFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame, PRBool aIsRoot, nsIBoxLayout* aLayoutManager)
@@ -237,21 +148,15 @@ NS_NewBoxFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame, PRBool aIsRoot, n
 nsBoxFrame::nsBoxFrame(nsIPresShell* aPresShell, PRBool aIsRoot, nsIBoxLayout* aLayoutManager)
 :nsContainerBox(aPresShell)
 {
-  mInner = new (aPresShell) nsBoxFrameInner(aPresShell, this);
-
-  mInner->mAttributesCached = PR_FALSE;
-
   mState |= NS_STATE_IS_HORIZONTAL;
   mState |= NS_STATE_AUTO_STRETCH;
 
   if (aIsRoot) 
      mState |= NS_STATE_IS_ROOT;
 
-  mInner->mValign = nsBoxFrame::vAlign_Top;
-  mInner->mHalign = nsBoxFrame::hAlign_Left;
+  mValign = vAlign_Top;
+  mHalign = hAlign_Left;
   
-  NeedsRecalc();
-
   // if no layout manager specified us the static sprocket layout
   nsCOMPtr<nsIBoxLayout> layout = aLayoutManager;
 
@@ -262,12 +167,10 @@ nsBoxFrame::nsBoxFrame(nsIPresShell* aPresShell, PRBool aIsRoot, nsIBoxLayout* a
   SetLayoutManager(layout);
 
   NeedsRecalc();
-
 }
 
 nsBoxFrame::~nsBoxFrame()
 {
-  NS_ASSERTION(mInner == nsnull,"Error Destroy was never called on this Frame!!!");
 }
 
 NS_IMETHODIMP nsBoxFrame::SetParent(const nsIFrame* aParent)
@@ -295,21 +198,21 @@ NS_IMETHODIMP nsBoxFrame::SetParent(const nsIFrame* aParent)
 NS_IMETHODIMP
 nsBoxFrame::GetVAlign(Valignment& aAlign)
 {
-   aAlign = mInner->mValign;
+   aAlign = mValign;
    return NS_OK;
 }
 
 NS_IMETHODIMP
 nsBoxFrame::GetHAlign(Halignment& aAlign)
 {
-   aAlign = mInner->mHalign;
+   aAlign = mHalign;
    return NS_OK;
 }
 
 NS_IMETHODIMP
 nsBoxFrame::SetInitialChildList(nsIPresContext* aPresContext,
-                                              nsIAtom*        aListName,
-                                              nsIFrame*       aChildList)
+                                nsIAtom*        aListName,
+                                nsIFrame*       aChildList)
 {
   SanityCheck(mFrames);
 
@@ -323,7 +226,7 @@ nsBoxFrame::SetInitialChildList(nsIPresContext* aPresContext,
     InitChildren(state, aChildList);
     CheckFrameOrder();
   } else {
-    printf("Warning add child failed!!\n");
+    NS_WARNING("Warning add child failed!!\n");
   }
 
   SanityCheck(mFrames);
@@ -348,21 +251,20 @@ nsBoxFrame::IsNormalDirection() const
  */
 NS_IMETHODIMP
 nsBoxFrame::Init(nsIPresContext*  aPresContext,
-              nsIContent*      aContent,
-              nsIFrame*        aParent,
-              nsIStyleContext* aContext,
-              nsIFrame*        aPrevInFlow)
+                 nsIContent*      aContent,
+                 nsIFrame*        aParent,
+                 nsIStyleContext* aContext,
+                 nsIFrame*        aPrevInFlow)
 {
   SetParent(aParent);
 
-  mInner->mPresContext = aPresContext;
+  mPresContext = aPresContext;
 
   nsresult  rv = nsContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
   // see if we need a widget. Get our parent. Querty interface the parent we are given. 
-  nsCOMPtr<nsIBox> parent (do_QueryInterface(aParent));
-
-  if (parent) {
+  nsIBox *parent;
+  if (aParent && NS_SUCCEEDED(CallQueryInterface(aParent, &parent))) {
     PRBool needsWidget = PR_FALSE;
     parent->ChildrenMustHaveWidgets(needsWidget);
     if (needsWidget) {
@@ -382,16 +284,16 @@ nsBoxFrame::Init(nsIPresContext*  aPresContext,
     }
   }
 
-  mInner->CacheAttributes();
+  CacheAttributes();
 
     // if we are root and this
   if (mState & NS_STATE_IS_ROOT) 
-      mInner->GetDebugPref(aPresContext);
+      GetDebugPref(aPresContext);
 
 
   mMouseThrough = unset;
 
-  mInner->UpdateMouseThrough();
+  UpdateMouseThrough();
 
   // register access key
   rv = RegUnregAccessKey(aPresContext, PR_TRUE);
@@ -399,84 +301,84 @@ nsBoxFrame::Init(nsIPresContext*  aPresContext,
   return rv;
 }
 
-void nsBoxFrameInner::UpdateMouseThrough()
+void nsBoxFrame::UpdateMouseThrough()
 {
-  if (mOuter->mContent) {
+  if (mContent) {
     nsAutoString value;
-    if (NS_CONTENT_ATTR_HAS_VALUE == mOuter->mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::mousethrough, value)) {
+    if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::mousethrough, value)) {
         if (value.EqualsIgnoreCase("never")) 
-          mOuter->mMouseThrough = mOuter->never;
+          mMouseThrough = never;
         else if (value.EqualsIgnoreCase("always")) 
-          mOuter->mMouseThrough = mOuter->always;
+          mMouseThrough = always;
       
     }
   }
 }
 
 void
-nsBoxFrameInner::CacheAttributes()
+nsBoxFrame::CacheAttributes()
 {
   /*
   printf("Caching: ");
-  mOuter->DumpBox(stdout);
+  DumpBox(stdout);
   printf("\n");
    */
 
-  mValign = nsBoxFrame::vAlign_Top;
-  mHalign = nsBoxFrame::hAlign_Left;
+  mValign = vAlign_Top;
+  mHalign = hAlign_Left;
 
   PRBool orient = PR_FALSE;
-  mOuter->GetInitialOrientation(orient); 
+  GetInitialOrientation(orient); 
   if (orient)
-    mOuter->mState |= NS_STATE_IS_HORIZONTAL;
+    mState |= NS_STATE_IS_HORIZONTAL;
   else
-    mOuter->mState &= ~NS_STATE_IS_HORIZONTAL;
+    mState &= ~NS_STATE_IS_HORIZONTAL;
 
   PRBool normal = PR_TRUE;
-  mOuter->GetInitialDirection(normal); 
+  GetInitialDirection(normal); 
   if (normal)
-    mOuter->mState |= NS_STATE_IS_DIRECTION_NORMAL;
+    mState |= NS_STATE_IS_DIRECTION_NORMAL;
   else
-    mOuter->mState &= ~NS_STATE_IS_DIRECTION_NORMAL;
+    mState &= ~NS_STATE_IS_DIRECTION_NORMAL;
 
-  mOuter->GetInitialVAlignment(mValign);
-  mOuter->GetInitialHAlignment(mHalign);
+  GetInitialVAlignment(mValign);
+  GetInitialHAlignment(mHalign);
   
   PRBool equalSize = PR_FALSE;
-  mOuter->GetInitialEqualSize(equalSize); 
+  GetInitialEqualSize(equalSize); 
   if (equalSize)
-        mOuter->mState |= NS_STATE_EQUAL_SIZE;
+        mState |= NS_STATE_EQUAL_SIZE;
     else
-        mOuter->mState &= ~NS_STATE_EQUAL_SIZE;
+        mState &= ~NS_STATE_EQUAL_SIZE;
 
-  PRBool autostretch = mOuter->mState & NS_STATE_AUTO_STRETCH;
-  mOuter->GetInitialAutoStretch(autostretch);
+  PRBool autostretch = mState & NS_STATE_AUTO_STRETCH;
+  GetInitialAutoStretch(autostretch);
   if (autostretch)
-        mOuter->mState |= NS_STATE_AUTO_STRETCH;
+        mState |= NS_STATE_AUTO_STRETCH;
      else
-        mOuter->mState &= ~NS_STATE_AUTO_STRETCH;
+        mState &= ~NS_STATE_AUTO_STRETCH;
 
 
-  PRBool debug = mOuter->mState & NS_STATE_SET_TO_DEBUG;
+  PRBool debug = mState & NS_STATE_SET_TO_DEBUG;
   PRBool debugSet = GetInitialDebug(debug); 
   if (debugSet) {
-        mOuter->mState |= NS_STATE_DEBUG_WAS_SET;
+        mState |= NS_STATE_DEBUG_WAS_SET;
         if (debug)
-            mOuter->mState |= NS_STATE_SET_TO_DEBUG;
+            mState |= NS_STATE_SET_TO_DEBUG;
         else
-            mOuter->mState &= ~NS_STATE_SET_TO_DEBUG;
+            mState &= ~NS_STATE_SET_TO_DEBUG;
   } else {
-        mOuter->mState &= ~NS_STATE_DEBUG_WAS_SET;
+        mState &= ~NS_STATE_DEBUG_WAS_SET;
   }
 }
 
 PRBool
-nsBoxFrameInner::GetInitialDebug(PRBool& aDebug)
+nsBoxFrame::GetInitialDebug(PRBool& aDebug)
 {
   nsAutoString value;
 
   nsCOMPtr<nsIContent> content;
-  mOuter->GetContentOf(getter_AddRefs(content));
+  GetContentOf(getter_AddRefs(content));
 
   if (!content)
     return PR_FALSE;
@@ -940,8 +842,7 @@ nsBoxFrame::Reflow(nsIPresContext*          aPresContext,
 
   nsMargin m;
   m = aReflowState.mComputedBorderPadding;
-
-  //GetBorderAndPadding(m);
+  // GetBorderAndPadding(m);
 
   // this happens sometimes. So lets handle it gracefully.
   if (aReflowState.mComputedHeight == 0) {
@@ -1131,14 +1032,6 @@ nsBoxFrame::GetMaxSize(nsBoxLayoutState& aBoxLayoutState, nsSize& aSize)
      return NS_OK;
   }
 
-  /*
-    // @@@ hack to fix bug in xbl where it won't set flex -EDV
-  if ((mState & NS_FRAME_FIRST_REFLOW) && !mInner->mAttributesCached) {
-    mInner->CacheAttributes();
-    mInner->mAttributesCached = PR_TRUE;
-  }
-  */
-
   PropagateDebug(aBoxLayoutState);
 
   nsresult rv = NS_OK;
@@ -1178,7 +1071,7 @@ nsBoxFrame::PropagateDebug(nsBoxLayoutState& aState)
       else
           SetDebug(aState, PR_FALSE);
   } else if (mState & NS_STATE_IS_ROOT) {
-    SetDebug(aState, nsBoxFrameInner::gDebug);
+    SetDebug(aState, gDebug);
   }
 }
 
@@ -1207,36 +1100,18 @@ nsBoxFrame::DoLayout(nsBoxLayoutState& aState)
   return nsContainerBox::DoLayout(aState);
 }
 
-nsBoxFrame::Valignment
-nsBoxFrameInner::GetVAlign()
-{
-   return mValign;
-}
-
-nsBoxFrame::Halignment
-nsBoxFrameInner::GetHAlign()
-{
-    return mHalign;
-}
-
 NS_IMETHODIMP
 nsBoxFrame::Destroy(nsIPresContext* aPresContext)
 {
 // if we are root remove 1 from the debug count.
   if (mState & NS_STATE_IS_ROOT)
-      mInner->GetDebugPref(aPresContext);
+      GetDebugPref(aPresContext);
 
   // unregister access key
   RegUnregAccessKey(aPresContext, PR_FALSE);
 
   SetLayoutManager(nsnull);
 
-  // recycle the Inner via the shell's arena.
-  nsCOMPtr<nsIPresShell> shell;
-  aPresContext->GetShell(getter_AddRefs(shell));
-  mInner->Recycle(shell);
-  mInner = nsnull;
-
   return nsContainerFrame::Destroy(aPresContext);
 } 
 
@@ -1267,21 +1142,19 @@ nsBoxFrame::SetDebug(nsBoxLayoutState& aState, PRBool aDebug)
 NS_IMETHODIMP
 nsBoxFrame::NeedsRecalc()
 {
-  if (mInner) {
-    SizeNeedsRecalc(mPrefSize);
-    SizeNeedsRecalc(mMinSize);
-    SizeNeedsRecalc(mMaxSize);
-    CoordNeedsRecalc(mFlex);
-    CoordNeedsRecalc(mAscent);
-  }
+  SizeNeedsRecalc(mPrefSize);
+  SizeNeedsRecalc(mMinSize);
+  SizeNeedsRecalc(mMaxSize);
+  CoordNeedsRecalc(mFlex);
+  CoordNeedsRecalc(mAscent);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsBoxFrame::RemoveFrame(nsIPresContext* aPresContext,
-                           nsIPresShell& aPresShell,
-                           nsIAtom* aListName,
-                           nsIFrame* aOldFrame)
+                        nsIPresShell&   aPresShell,
+                        nsIAtom*        aListName,
+                        nsIFrame*       aOldFrame)
 {
   SanityCheck(mFrames);
 
@@ -1302,10 +1175,10 @@ nsBoxFrame::RemoveFrame(nsIPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsBoxFrame::InsertFrames(nsIPresContext* aPresContext,
-                            nsIPresShell& aPresShell,
-                            nsIAtom* aListName,
-                            nsIFrame* aPrevFrame,
-                            nsIFrame* aFrameList)
+                         nsIPresShell&   aPresShell,
+                         nsIAtom*        aListName,
+                         nsIFrame*       aPrevFrame,
+                         nsIFrame*       aFrameList)
 {
    SanityCheck(mFrames);
 
@@ -1340,9 +1213,9 @@ nsBoxFrame::InsertFrames(nsIPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsBoxFrame::AppendFrames(nsIPresContext* aPresContext,
-                           nsIPresShell&   aPresShell,
-                           nsIAtom*        aListName,
-                           nsIFrame*       aFrameList)
+                         nsIPresShell&   aPresShell,
+                         nsIAtom*        aListName,
+                         nsIFrame*       aFrameList)
 {
    SanityCheck(mFrames);
 
@@ -1399,8 +1272,8 @@ nsBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
         aAttribute == nsXULAtoms::dir     ||
         aAttribute == nsXULAtoms::debug) {
 
-      mInner->mValign = nsBoxFrame::vAlign_Top;
-      mInner->mHalign = nsBoxFrame::hAlign_Left;
+      mValign = nsBoxFrame::vAlign_Top;
+      mHalign = nsBoxFrame::hAlign_Left;
 
       PRBool orient = PR_TRUE;
       GetInitialOrientation(orient); 
@@ -1416,8 +1289,8 @@ nsBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
       else
         mState &= ~NS_STATE_IS_DIRECTION_NORMAL;
 
-      GetInitialVAlignment(mInner->mValign);
-      GetInitialHAlignment(mInner->mHalign);
+      GetInitialVAlignment(mValign);
+      GetInitialHAlignment(mHalign);
 
       PRBool equalSize = PR_FALSE;
       GetInitialEqualSize(equalSize); 
@@ -1427,7 +1300,7 @@ nsBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
         mState &= ~NS_STATE_EQUAL_SIZE;
 
       PRBool debug = mState & NS_STATE_SET_TO_DEBUG;
-      PRBool debugSet = mInner->GetInitialDebug(debug);
+      PRBool debugSet = GetInitialDebug(debug);
       if (debugSet) {
         mState |= NS_STATE_DEBUG_WAS_SET;
 
@@ -1451,7 +1324,7 @@ nsBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
       mState &= ~NS_STATE_STACK_NOT_POSITIONED;
     }
     else if (aAttribute == nsXULAtoms::mousethrough) {
-      mInner->UpdateMouseThrough();
+      UpdateMouseThrough();
     }
   }
   else if (aAttribute == nsXULAtoms::ordinal) {
@@ -1492,12 +1365,12 @@ nsBoxFrame::GetInset(nsMargin& margin)
      nsMargin debugMargin(0,0,0,0);
      nsMargin debugBorder(0,0,0,0);
      nsMargin debugPadding(0,0,0,0);
-     mInner->GetDebugBorder(debugBorder);
-     mInner->PixelMarginToTwips(mInner->mPresContext, debugBorder);
-     mInner->GetDebugMargin(debugMargin);
-     mInner->PixelMarginToTwips(mInner->mPresContext, debugMargin);
-     mInner->GetDebugMargin(debugPadding);
-     mInner->PixelMarginToTwips(mInner->mPresContext, debugPadding);
+     GetDebugBorder(debugBorder);
+     PixelMarginToTwips(mPresContext, debugBorder);
+     GetDebugMargin(debugMargin);
+     PixelMarginToTwips(mPresContext, debugMargin);
+     GetDebugMargin(debugPadding);
+     PixelMarginToTwips(mPresContext, debugPadding);
      margin += debugBorder;
      margin += debugMargin;
      margin += debugPadding;
@@ -1564,7 +1437,7 @@ nsBoxFrame::CheckFrameOrder()
 }
 
 void
-nsBoxFrameInner::GetDebugPref(nsIPresContext* aPresContext)
+nsBoxFrame::GetDebugPref(nsIPresContext* aPresContext)
 {
     gDebug = PR_FALSE;
     nsCOMPtr<nsIPref> pref(do_GetService(NS_PREF_CONTRACTID));
@@ -1600,12 +1473,15 @@ nsBoxFrame::Paint(nsIPresContext*      aPresContext,
       PRIntn skipSides = GetSkipSides();
       const nsStyleBorder* border = (const nsStyleBorder*)
         mStyleContext->GetStyleData(eStyleStruct_Border);
+      const nsStylePadding* padding = (const nsStylePadding*)
+        mStyleContext->GetStyleData(eStyleStruct_Padding);
       const nsStyleOutline* outline = (const nsStyleOutline*)
         mStyleContext->GetStyleData(eStyleStruct_Outline);
 
       nsRect  rect(0, 0, mRect.width, mRect.height);
       nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                      aDirtyRect, rect, *border, 0, 0);
+                                      aDirtyRect, rect, *border, *padding,
+                                      0, 0);
       nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
                                   aDirtyRect, rect, *border, mStyleContext, skipSides);
       nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, this,
@@ -1644,7 +1520,7 @@ nsBoxFrame::Paint(nsIPresContext*      aPresContext,
   const nsStyleDisplay* display = 
       (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
   if (!(display->mAppearance && nsBox::gTheme && 
-        gTheme->ThemeSupportsWidget(aPresContext, display->mAppearance) &&
+        gTheme->ThemeSupportsWidget(aPresContext, this, display->mAppearance) &&
         !gTheme->WidgetIsContainer(display->mAppearance)))
     PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 
@@ -1691,12 +1567,22 @@ nsBoxFrame::PaintChild(nsIPresContext*      aPresContext,
       // coordinate system.
       damageArea.x -= kidRect.x;
       damageArea.y -= kidRect.y;
-      aRenderingContext.Translate(kidRect.x, kidRect.y);
 
+      // Save the transformation matrix's translation components.
+      float xMatrix;
+      float yMatrix;
+      nsTransform2D *theTransform; 
+      aRenderingContext.GetCurrentTransform(theTransform);
+      NS_ASSERTION(theTransform != nsnull, "The rendering context transform is null");
+      theTransform->GetTranslation(&xMatrix, &yMatrix);
+
+      aRenderingContext.Translate(kidRect.x, kidRect.y);
+     
       // Paint the kid
       aFrame->Paint(aPresContext, aRenderingContext, damageArea, aWhichLayer);
-      // don't use PushState and PopState, because they're slow
-      aRenderingContext.Translate(-kidRect.x, -kidRect.y);
+
+      // Restore the transformation matrix's translation components.
+      theTransform->SetTranslation(xMatrix, yMatrix);
     }
   }
 }
@@ -1725,14 +1611,14 @@ nsBoxFrame::PaintChildren(nsIPresContext*      aPresContext,
         aPresContext->GetScaledPixelsToTwips(&p2t);
         onePixel = NSIntPixelsToTwips(1, p2t);
 
-        mInner->GetDebugBorder(debugBorder);
-        mInner->PixelMarginToTwips(aPresContext, debugBorder);
+        GetDebugBorder(debugBorder);
+        PixelMarginToTwips(aPresContext, debugBorder);
 
-        mInner->GetDebugMargin(debugMargin);
-        mInner->PixelMarginToTwips(aPresContext, debugMargin);
+        GetDebugMargin(debugMargin);
+        PixelMarginToTwips(aPresContext, debugMargin);
 
-        mInner->GetDebugPadding(debugPadding);
-        mInner->PixelMarginToTwips(aPresContext, debugPadding);
+        GetDebugPadding(debugPadding);
+        PixelMarginToTwips(aPresContext, debugPadding);
 
         GetContentRect(inner);
         inner.Deflate(debugMargin);
@@ -1845,8 +1731,8 @@ nsBoxFrame::PaintChildren(nsIPresContext*      aPresContext,
     GetContentRect(r);
 
     if (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) {
-      mInner->GetDebugMargin(debugMargin);
-      mInner->PixelMarginToTwips(aPresContext, debugMargin);
+      GetDebugMargin(debugMargin);
+      PixelMarginToTwips(aPresContext, debugMargin);
       r.Deflate(debugMargin);
     }
 
@@ -1908,7 +1794,7 @@ nsBoxFrame::PaintChildren(nsIPresContext*      aPresContext,
           else 
               borderSize = cr.height;
         
-          mInner->DrawSpacer(aPresContext, aRenderingContext, isHorizontal, flex, x, y, borderSize, spacerSize);
+          DrawSpacer(aPresContext, aRenderingContext, isHorizontal, flex, x, y, borderSize, spacerSize);
         }
 
         kid->GetNextBox(&kid);
@@ -1947,13 +1833,13 @@ nsBoxFrame::GetFrame(nsIFrame** aFrame)
   return NS_OK;
 }
 
+#ifdef DEBUG_LAYOUT
 void
 nsBoxFrame::GetBoxName(nsAutoString& aName)
 {
-#ifdef DEBUG
    GetFrameName(aName);
-#endif
 }
+#endif
 
 #ifdef DEBUG
 NS_IMETHODIMP
@@ -1980,10 +1866,10 @@ nsBoxFrame::GetDebug(PRBool& aDebug)
 }
 
 NS_IMETHODIMP  
-nsBoxFrame::GetFrameForPoint(nsIPresContext* aPresContext,
-                             const nsPoint& aPoint, 
+nsBoxFrame::GetFrameForPoint(nsIPresContext*   aPresContext,
+                             const nsPoint&    aPoint, 
                              nsFramePaintLayer aWhichLayer,    
-                             nsIFrame**     aFrame)
+                             nsIFrame**        aFrame)
 {   
 
   if ((aWhichLayer != NS_FRAME_PAINT_LAYER_FOREGROUND))
@@ -2041,7 +1927,7 @@ nsBoxFrame::GetFrameForPoint(nsIPresContext* aPresContext,
         // if the kid had a child before see if this child has mouse
         // though. 
         PRBool isAdaptor = PR_FALSE;
-        nsCOMPtr<nsIBox> box = mInner->GetBoxForFrame(hit, isAdaptor);
+        nsIBox *box = GetBoxForFrame(hit, isAdaptor);
         if (box) {
           PRBool mouseThrough = PR_FALSE;
           box->GetMouseThrough(mouseThrough);
@@ -2069,7 +1955,7 @@ nsBoxFrame::GetFrameForPoint(nsIPresContext* aPresContext,
 }
 
 nsIBox*
-nsBoxFrameInner::GetBoxForFrame(nsIFrame* aFrame, PRBool& aIsAdaptor)
+nsBoxFrame::GetBoxForFrame(nsIFrame* aFrame, PRBool& aIsAdaptor)
 {
   if (aFrame == nsnull)
     return nsnull;
@@ -2128,8 +2014,8 @@ nsBoxFrame::GetMouseThrough(PRBool& aMouseThrough)
 
 NS_IMETHODIMP
 nsBoxFrame::GetCursor(nsIPresContext* aPresContext,
-                           nsPoint&        aPoint,
-                           PRInt32&        aCursor)
+                      nsPoint&        aPoint,
+                      PRInt32&        aCursor)
 {
   /*
     #ifdef NS_DEBUG
@@ -2141,16 +2027,18 @@ nsBoxFrame::GetCursor(nsIPresContext* aPresContext,
  */
 
     nsPoint newPoint;
-    mInner->TranslateEventCoords(aPresContext, aPoint, newPoint);
+    TranslateEventCoords(aPresContext, aPoint, newPoint);
     
+#ifdef DEBUG_LAYOUT
     // if we are in debug and we are in the debug area
     // return our own cursor and dump the debug information.
     if (mState & NS_STATE_CURRENTLY_IN_DEBUG) 
     {
-          nsresult rv = mInner->DisplayDebugInfoFor(this, aPresContext, newPoint, aCursor);
+          nsresult rv = DisplayDebugInfoFor(this, aPresContext, newPoint, aCursor);
           if (rv == NS_OK)
              return rv;
     }
+#endif
 
     nsresult rv = nsContainerFrame::GetCursor(aPresContext, aPoint, aCursor);
 
@@ -2165,9 +2053,9 @@ nsBoxFrame::GetCursor(nsIPresContext* aPresContext,
 // view) into a localized pixel coordinate that is relative to the
 // content area of this frame (inside the border+padding).
 void
-nsBoxFrameInner::TranslateEventCoords(nsIPresContext* aPresContext,
-                                      const nsPoint& aPoint,
-                                      nsPoint& aResult)
+nsBoxFrame::TranslateEventCoords(nsIPresContext* aPresContext,
+                                 const nsPoint& aPoint,
+                                 nsPoint& aResult)
 {
   nscoord x = aPoint.x;
   nscoord y = aPoint.y;
@@ -2176,10 +2064,10 @@ nsBoxFrameInner::TranslateEventCoords(nsIPresContext* aPresContext,
   // to this frame; otherwise we have to adjust the coordinates
   // appropriately.
   nsIView* view;
-  mOuter->GetView(aPresContext, &view);
+  GetView(aPresContext, &view);
   if (nsnull == view) {
     nsPoint offset;
-    mOuter->GetOffsetFromView(aPresContext, offset, &view);
+    GetOffsetFromView(aPresContext, offset, &view);
     if (nsnull != view) {
       x -= offset.x;
       y -= offset.y;
@@ -2211,45 +2099,17 @@ nsBoxFrame::GetContentOf(nsIContent** aContent)
     return NS_OK;
 }
 
-
-
-void* 
-nsBoxFrameInner::operator new(size_t sz, nsIPresShell* aPresShell)
-{
-    return nsBoxLayoutState::Allocate(sz,aPresShell);
-}
-
-void 
-nsBoxFrameInner::Recycle(nsIPresShell* aPresShell)
-{
-  nsBoxLayoutState state(aPresShell);
-  mOuter->ClearChildren(state);
-
-  delete this;
-  nsBoxLayoutState::RecycleFreedMemory(aPresShell, this);
-}
-
-
-// Overridden to prevent the global delete from being called, since the memory
-// came out of an nsIArena instead of the global delete operator's heap.
-void 
-nsBoxFrameInner::operator delete(void* aPtr, size_t sz)
-{
-    nsBoxLayoutState::Free(aPtr, sz);
-}
-
 /*
-
 nsresult
-nsBoxFrameInner::PaintDebug(nsIBox* aBox, 
-                        nsIPresContext* aPresContext,
-                        nsIRenderingContext& aRenderingContext,
-                        const nsRect& aDirtyRect,
-                        nsFramePaintLayer aWhichLayer)
+nsBoxFrame::PaintDebug(nsIBox*              aBox, 
+                       nsIPresContext*      aPresContext,
+                       nsIRenderingContext& aRenderingContext,
+                       const nsRect&        aDirtyRect,
+                       nsFramePaintLayer    aWhichLayer)
 
 {
     
-        PRBool isHorizontal = mOuter->IsHorizontal();
+        PRBool isHorizontal = IsHorizontal();
 
         float p2t;
         aPresContext->GetScaledPixelsToTwips(&p2t);
@@ -2308,9 +2168,9 @@ nsBoxFrameInner::PaintDebug(nsIBox* aBox,
         // if we have dirty children or we are dirty 
         // place a green border around us.
         PRBool dirty = PR_FALSE;
-        mOuter->IsDirty(dirty);
+        IsDirty(dirty);
         PRBool dirtyc = PR_FALSE;
-        mOuter->HasDirtyChildren(dirty);
+        HasDirtyChildren(dirty);
 
         if (dirty || dirtyc) {
            nsRect dirtyr(inner);
@@ -2368,7 +2228,7 @@ nsBoxFrameInner::PaintDebug(nsIBox* aBox,
 */
 
 void
-nsBoxFrameInner::DrawLine(nsIRenderingContext& aRenderingContext, PRBool aHorizontal, nscoord x1, nscoord y1, nscoord x2, nscoord y2)
+nsBoxFrame::DrawLine(nsIRenderingContext& aRenderingContext, PRBool aHorizontal, nscoord x1, nscoord y1, nscoord x2, nscoord y2)
 {
     if (aHorizontal)
        aRenderingContext.DrawLine(x1,y1,x2,y2);
@@ -2377,7 +2237,7 @@ nsBoxFrameInner::DrawLine(nsIRenderingContext& aRenderingContext, PRBool aHorizo
 }
 
 void
-nsBoxFrameInner::FillRect(nsIRenderingContext& aRenderingContext, PRBool aHorizontal, nscoord x, nscoord y, nscoord width, nscoord height)
+nsBoxFrame::FillRect(nsIRenderingContext& aRenderingContext, PRBool aHorizontal, nscoord x, nscoord y, nscoord width, nscoord height)
 {
     if (aHorizontal)
        aRenderingContext.FillRect(x,y,width,height);
@@ -2386,7 +2246,7 @@ nsBoxFrameInner::FillRect(nsIRenderingContext& aRenderingContext, PRBool aHorizo
 }
 
 void 
-nsBoxFrameInner::DrawSpacer(nsIPresContext* aPresContext, nsIRenderingContext& aRenderingContext, PRBool aHorizontal, PRInt32 flex, nscoord x, nscoord y, nscoord size, nscoord spacerSize)
+nsBoxFrame::DrawSpacer(nsIPresContext* aPresContext, nsIRenderingContext& aRenderingContext, PRBool aHorizontal, PRInt32 flex, nscoord x, nscoord y, nscoord size, nscoord spacerSize)
 {    
         float p2t;
         aPresContext->GetScaledPixelsToTwips(&p2t);
@@ -2426,31 +2286,31 @@ nsBoxFrameInner::DrawSpacer(nsIPresContext* aPresContext, nsIRenderingContext& a
 }
 
 void
-nsBoxFrameInner::GetDebugBorder(nsMargin& aInset)
+nsBoxFrame::GetDebugBorder(nsMargin& aInset)
 {
     aInset.SizeTo(2,2,2,2);
 
-    if (mOuter->IsHorizontal()) 
+    if (IsHorizontal()) 
        aInset.top = 10;
     else 
        aInset.left = 10;
 }
 
 void
-nsBoxFrameInner::GetDebugMargin(nsMargin& aInset)
+nsBoxFrame::GetDebugMargin(nsMargin& aInset)
 {
     aInset.SizeTo(2,2,2,2);
 }
 
 void
-nsBoxFrameInner::GetDebugPadding(nsMargin& aPadding)
+nsBoxFrame::GetDebugPadding(nsMargin& aPadding)
 {
     aPadding.SizeTo(2,2,2,2);
 }
 
 
 void 
-nsBoxFrameInner::PixelMarginToTwips(nsIPresContext* aPresContext, nsMargin& aMarginPixels)
+nsBoxFrame::PixelMarginToTwips(nsIPresContext* aPresContext, nsMargin& aMarginPixels)
 {
   float p2t;
   aPresContext->GetScaledPixelsToTwips(&p2t);
@@ -2462,8 +2322,9 @@ nsBoxFrameInner::PixelMarginToTwips(nsIPresContext* aPresContext, nsMargin& aMar
 }
 
 
+#ifdef DEBUG_LAYOUT
 void
-nsBoxFrameInner::GetValue(nsIPresContext* aPresContext, const nsSize& a, const nsSize& b, char* ch) 
+nsBoxFrame::GetValue(nsIPresContext* aPresContext, const nsSize& a, const nsSize& b, char* ch) 
 {
     float p2t;
     aPresContext->GetScaledPixelsToTwips(&p2t);
@@ -2488,7 +2349,7 @@ nsBoxFrameInner::GetValue(nsIPresContext* aPresContext, const nsSize& a, const n
 }
 
 void
-nsBoxFrameInner::GetValue(nsIPresContext* aPresContext, PRInt32 a, PRInt32 b, char* ch) 
+nsBoxFrame::GetValue(nsIPresContext* aPresContext, PRInt32 a, PRInt32 b, char* ch) 
 {
     if (a == NS_INTRINSICSIZE)
       sprintf(ch, "%d[SET]", b);             
@@ -2497,10 +2358,10 @@ nsBoxFrameInner::GetValue(nsIPresContext* aPresContext, PRInt32 a, PRInt32 b, ch
 }
 
 nsresult
-nsBoxFrameInner::DisplayDebugInfoFor(nsIBox* aBox,
-                                     nsIPresContext* aPresContext,
-                                     nsPoint&        aPoint,
-                                     PRInt32&        aCursor)
+nsBoxFrame::DisplayDebugInfoFor(nsIBox*         aBox,
+                                nsIPresContext* aPresContext,
+                                nsPoint&        aPoint,
+                                PRInt32&        aCursor)
 {
     nsBoxLayoutState state(aPresContext);
 
@@ -2517,7 +2378,7 @@ nsBoxFrameInner::DisplayDebugInfoFor(nsIBox* aBox,
     aBox->GetBorderAndPadding(border);
     insideBorder.Deflate(border);
 
-    PRBool isHorizontal = mOuter->IsHorizontal();
+    PRBool isHorizontal = IsHorizontal();
 
     if (!insideBorder.Contains(nsPoint(x,y)))
         return NS_ERROR_FAILURE;
@@ -2561,7 +2422,7 @@ nsBoxFrameInner::DisplayDebugInfoFor(nsIBox* aBox,
 
                             if (content) {                             
                               printf("---------------\n");
-                              mOuter->DumpBox(stdout);
+                              DumpBox(stdout);
                               printf("\n");
                             }
 
@@ -2640,9 +2501,10 @@ nsBoxFrameInner::DisplayDebugInfoFor(nsIBox* aBox,
 
         return NS_OK;
 }
+#endif
 
 nsresult
-nsBoxFrameInner::GetFrameSizeWithMargin(nsIBox* aBox, nsSize& aSize)
+nsBoxFrame::GetFrameSizeWithMargin(nsIBox* aBox, nsSize& aSize)
 {
   nsRect rect(0,0,0,0);
   aBox->GetBounds(rect);
@@ -2658,10 +2520,10 @@ nsBoxFrameInner::GetFrameSizeWithMargin(nsIBox* aBox, nsSize& aSize)
  * Boxed don't support fixed positionioning of their children.
  */
 nsresult
-nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
-                             nsIFrame* aFrame,
-                             nsIStyleContext* aStyleContext,
-                             PRBool aForce)
+nsBoxFrame::CreateViewForFrame(nsIPresContext*  aPresContext,
+                               nsIFrame*        aFrame,
+                               nsIStyleContext* aStyleContext,
+                               PRBool           aForce)
 {
   nsIView* view;
   aFrame->GetView(aPresContext, &view);
@@ -2695,7 +2557,7 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
     if (!aForce) {
       nsIAtom*  pseudoTag;
       aStyleContext->GetPseudoType(pseudoTag);
-      if (pseudoTag == nsLayoutAtoms::scrolledContentPseudo) {
+      if (pseudoTag == nsCSSAnonBoxes::scrolledContent) {
         NS_FRAME_LOG(NS_FRAME_TRACE_CALLS,
           ("nsBoxFrame::CreateViewForFrame: scrolled frame=%p", aFrame));
         aForce = PR_TRUE;
@@ -2716,12 +2578,8 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
 
       // Create a view
       static NS_DEFINE_IID(kViewCID, NS_VIEW_CID);
-
-      nsresult result = nsComponentManager::CreateInstance(kViewCID, 
-                                                     nsnull, 
-                                                     NS_GET_IID(nsIView), 
-                                                     (void **)&view);
-      if (NS_OK == result) {
+      nsresult result = CallCreateInstance(kViewCID, &view);
+      if (NS_SUCCEEDED(result)) {
         nsIViewManager* viewManager;
         parentView->GetViewManager(viewManager);
         NS_ASSERTION(nsnull != viewManager, "null view manager");
@@ -2740,7 +2598,7 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
         // Insert the view into the view hierarchy. If the parent view is a
         // scrolling view we need to do this differently
         nsIScrollableView*  scrollingView;
-        if (NS_SUCCEEDED(parentView->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollingView))) {
+        if (NS_SUCCEEDED(CallQueryInterface(parentView, &scrollingView))) {
           scrollingView->SetScrolledView(view);
         } else {
           viewManager->SetViewZIndex(view, autoZIndex, zIndex);
@@ -2822,8 +2680,7 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
 // If you make changes to this function, check its counterparts
 // in nsTextBoxFrame and nsAreaFrame
 nsresult
-nsBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext,
-                              PRBool aDoReg)
+nsBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext, PRBool aDoReg)
 {
   // if we have no content, we can't do anything
   if (!mContent)
@@ -2837,7 +2694,10 @@ nsBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext,
 
   // only support accesskeys for the following elements
   if (atom != nsXULAtoms::button &&
+      atom != nsXULAtoms::toolbarbutton &&
       atom != nsXULAtoms::checkbox &&
+      atom != nsXULAtoms::textbox &&
+      atom != nsXULAtoms::tab &&
       atom != nsXULAtoms::radio) {
 
     return NS_OK;
@@ -2868,3 +2728,23 @@ nsBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext,
 }
 
 
+void
+nsBoxFrame::FireDOMEvent(nsIPresContext *aPresContext, const nsAString& aDOMEventName)
+{
+  if (mContent) {
+    // Fire a DOM event for the title change.
+    nsCOMPtr<nsIDOMEvent> event;
+    nsCOMPtr<nsIEventListenerManager> manager;
+    mContent->GetListenerManager(getter_AddRefs(manager));
+    if (manager &&
+        NS_SUCCEEDED(manager->CreateEvent(aPresContext, nsnull, NS_LITERAL_STRING("Events"), getter_AddRefs(event)))) {
+      event->InitEvent(aDOMEventName, PR_TRUE, PR_TRUE);
+      PRBool noDefault;
+      nsCOMPtr<nsIEventStateManager> esm;
+      aPresContext->GetEventStateManager(getter_AddRefs(esm));
+      if (esm)
+        esm->DispatchNewEvent(mContent, event, &noDefault);
+    }
+  }
+}
+
