@@ -282,8 +282,7 @@ nsImageBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
     Redraw(state);
   }
 
-
-  return NS_OK;
+  return rv;
 }
 
 nsImageBoxFrame::nsImageBoxFrame(nsIPresShell* aShell):nsLeafBoxFrame(aShell), mIntrinsicSize(0,0),
@@ -365,7 +364,7 @@ nsImageBoxFrame::GetImageSource()
     const nsStyleDisplay* disp =
       (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
     if (disp->mAppearance && nsBox::gTheme && 
-        nsBox::gTheme->ThemeSupportsWidget(nsnull, disp->mAppearance))
+        nsBox::gTheme->ThemeSupportsWidget(nsnull, this, disp->mAppearance))
       return;
 
     // get the list-style-image
@@ -452,11 +451,23 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
 
   nsresult rv;
   nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1", &rv));
+  if (NS_FAILED(rv)) return;
 
   nsCOMPtr<nsILoadGroup> loadGroup;
   GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
 
-  il->LoadImage(srcURI, nsnull, loadGroup, mListener, aPresContext, mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
+  // Get the document URI for the referrer...
+  nsCOMPtr<nsIURI> documentURI;
+  nsCOMPtr<nsIDocument> doc;
+  if (mContent) {
+    (void) mContent->GetDocument(*getter_AddRefs(doc));
+    if (doc) {
+      doc->GetDocumentURL(getter_AddRefs(documentURI));
+    }
+  }
+
+  // XXX: initialDocumentURI is NULL!
+  il->LoadImage(srcURI, nsnull, documentURI, loadGroup, mListener, aPresContext, mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
 
   aResize = PR_TRUE;
 }
@@ -510,7 +521,7 @@ nsImageBoxFrame::PaintImage(nsIPresContext* aPresContext,
   mImageRequest->GetImage(getter_AddRefs(imgCon));
 
   if (imgCon) {
-    PRBool hasSubRect = mSubRect.width > 0 || mSubRect.height > 0;
+    PRBool hasSubRect = !mUseSrcAttr && (mSubRect.width > 0 || mSubRect.height > 0);
     PRBool sizeMatch = hasSubRect ? 
                        mSubRect.width == rect.width && mSubRect.height == rect.height :
                        mImageSize.width == rect.width && mImageSize.height == rect.height;
@@ -561,7 +572,7 @@ nsImageBoxFrame::DidSetStyleContext( nsIPresContext* aPresContext )
   const nsStyleDisplay* disp =
     (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
   if (disp->mAppearance && nsBox::gTheme && 
-      nsBox::gTheme->ThemeSupportsWidget(nsnull, disp->mAppearance))
+      nsBox::gTheme->ThemeSupportsWidget(nsnull, this, disp->mAppearance))
     return NS_OK;
 
   // If list-style-image changes, we have a new image.
@@ -631,7 +642,7 @@ nsImageBoxFrame::GetPrefSize(nsBoxLayoutState& aState, nsSize& aSize)
      CacheImageSize(aState);
   }
 
-  if (mSubRect.width > 0 || mSubRect.height > 0)
+  if (!mUseSrcAttr && (mSubRect.width > 0 || mSubRect.height > 0))
     aSize = nsSize(mSubRect.width, mSubRect.height);
   else
     aSize = mImageSize;
