@@ -66,7 +66,6 @@
 #include "nsIDeviceContext.h"
 #include "nsIFontMetrics.h"
 #include "nsITimer.h"
-#include "nsITimerCallback.h"
 
 /////////////// nsListScrollSmoother //////////////////
 
@@ -103,7 +102,7 @@ public:
   virtual ~nsListScrollSmoother();
 
   // nsITimerCallback
-  NS_IMETHOD_(void) Notify(nsITimer *timer);
+  NS_DECL_NSITIMERCALLBACK
 
   void Start();
   void Stop();
@@ -116,7 +115,6 @@ public:
 
 nsListScrollSmoother::nsListScrollSmoother(nsListBoxBodyFrame* aOuter)
 {
-  NS_INIT_REFCNT();
   mDelta = 0;
   mOuter = aOuter;
 }
@@ -126,16 +124,17 @@ nsListScrollSmoother::~nsListScrollSmoother()
   Stop();
 }
 
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 nsListScrollSmoother::Notify(nsITimer *timer)
 {
   Stop();
 
   NS_ASSERTION(mOuter, "mOuter is null, see bug #68365");
-  if (!mOuter) return;
+  if (!mOuter) return NS_OK;
 
   // actually do some work.
   mOuter->InternalPositionChangedCallback();
+  return NS_OK;
 }
 
 PRBool
@@ -149,7 +148,7 @@ nsListScrollSmoother::Start()
 {
   Stop();
   mRepeatTimer = do_CreateInstance("@mozilla.org/timer;1");
-  mRepeatTimer->Init(this, SMOOTH_INTERVAL);
+  mRepeatTimer->InitWithCallback(this, SMOOTH_INTERVAL, nsITimer::TYPE_ONE_SHOT);
 }
 
 void
@@ -203,13 +202,13 @@ nsListBoxBodyFrame::~nsListBoxBodyFrame()
 NS_IMETHODIMP_(nsrefcnt) 
 nsListBoxBodyFrame::AddRef(void)
 {
-  return NS_OK;
+  return 2;
 }
 
 NS_IMETHODIMP_(nsrefcnt)
 nsListBoxBodyFrame::Release(void)
 {
-  return NS_OK;
+  return 1;
 }
 
 //
@@ -750,7 +749,6 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
   if (firstRowContent) {
     nsCOMPtr<nsIStyleContext> styleContext;
     aBoxLayoutState.GetPresContext()->ResolveStyleContextFor(firstRowContent, nsnull,
-                                                             PR_FALSE, 
                                                              getter_AddRefs(styleContext));
 
     nscoord width = 0;
@@ -932,18 +930,17 @@ nsListBoxBodyFrame::InternalPositionChanged(PRBool aUp, PRInt32 aDelta, PRBool a
     // go off screen, so blow them all away. Weeee!
     nsIBox* currBox;
     GetChildBox(&currBox);
+    nsBoxLayoutState state(mPresContext);
     while (currBox) {
       nsIBox* nextBox;
       currBox->GetNextBox(&nextBox);
       nsIFrame* frame;
       currBox->QueryInterface(NS_GET_IID(nsIFrame), (void**)&frame); 
       mFrameConstructor->RemoveMappingsForFrameSubtree(mPresContext, frame, nsnull);
+      Remove(state, frame);
+      mFrames.DestroyFrame(mPresContext, frame);
       currBox = nextBox;
     }
-
-    nsBoxLayoutState state(mPresContext);
-    ClearChildren(state);
-    mFrames.DestroyFrames(mPresContext);
   }
 
   // clear frame markers so that CreateRows will re-create
@@ -1140,7 +1137,8 @@ nsListBoxBodyFrame::GetFirstItemBox(PRInt32 aOffset, PRBool* aCreated)
   mBottomFrame = mTopFrame;
 
   if (mTopFrame) {
-    nsCOMPtr<nsIBox> box(do_QueryInterface(mTopFrame));
+    nsIBox *box;
+    CallQueryInterface(mTopFrame, &box);
     return box;
   }
 
@@ -1149,7 +1147,8 @@ nsListBoxBodyFrame::GetFirstItemBox(PRInt32 aOffset, PRBool* aCreated)
   mBottomFrame = mTopFrame;
 
   if (mTopFrame && mRowsToPrepend <= 0) {
-    nsCOMPtr<nsIBox> box(do_QueryInterface(mTopFrame));
+    nsIBox *box;
+    CallQueryInterface(mTopFrame, &box);
     return box;
   }
 
@@ -1191,7 +1190,8 @@ nsListBoxBodyFrame::GetFirstItemBox(PRInt32 aOffset, PRBool* aCreated)
 
       mBottomFrame = mTopFrame;
 
-      nsCOMPtr<nsIBox> box(do_QueryInterface(mTopFrame));
+      nsIBox *box;
+      CallQueryInterface(mTopFrame, &box);
       return box;
     } else
       return GetFirstItemBox(++aOffset, 0);
@@ -1224,7 +1224,7 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset, PRBool* aCreat
     prevContent->GetParent(*getter_AddRefs(parentContent));
     parentContent->IndexOf(prevContent, i);
     parentContent->ChildCount(childCount);
-    if (i+1 < childCount) {
+    if (i+aOffset+1 < childCount) {
       // There is a content node that wants a frame.
       nsCOMPtr<nsIContent> nextContent;
       parentContent->ChildAt(i+aOffset+1, *getter_AddRefs(nextContent));
@@ -1246,10 +1246,13 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset, PRBool* aCreat
     }
   }
 
-  if (result)
-    mBottomFrame = result;
+  if (!result)
+    return nsnull;
 
-  nsCOMPtr<nsIBox> box(do_QueryInterface(result));
+  mBottomFrame = result;
+
+  nsIBox *box;
+  CallQueryInterface(result, &box);
   return box;
 }
 
@@ -1323,6 +1326,18 @@ nsListBoxBodyFrame::OnContentInserted(nsIPresContext* aPresContext, nsIContent*
 {
   ++mRowCount;
 
+  nsCOMPtr<nsIPresShell> shell;
+  aPresContext->GetShell(getter_AddRefs(shell));
+  // The RDF content builder will build content nodes such that they are all 
+  // ready when OnContentInserted is first called, meaning the first call
+  // to CreateRows will create all the frames, but OnContentInserted will
+  // still be called again for each content node - so we need to make sure
+  // that the frame for each content node hasn't already been created.
+  nsIFrame* childFrame = nsnull;
+  shell->GetPrimaryFrameFor(aChildContent, &childFrame);
+  if (childFrame)
+    return;
+
   PRInt32 siblingIndex;
   nsCOMPtr<nsIContent> nextSiblingContent;
   GetListItemNextSibling(aChildContent, getter_AddRefs(nextSiblingContent), siblingIndex);
@@ -1334,8 +1349,6 @@ nsListBoxBodyFrame::OnContentInserted(nsIPresContext* aPresContext, nsIContent*
     mRowsToPrepend = 1;
   } else if (nextSiblingContent) {
     // we may be inserting before a frame that is on screen
-    nsCOMPtr<nsIPresShell> shell;
-    aPresContext->GetShell(getter_AddRefs(shell));
     nsIFrame* nextSiblingFrame = nsnull;
     shell->GetPrimaryFrameFor(nextSiblingContent, &nextSiblingFrame);
     mLinkupFrame = nextSiblingFrame;
@@ -1343,8 +1356,6 @@ nsListBoxBodyFrame::OnContentInserted(nsIPresContext* aPresContext, nsIContent*
   
   nsBoxLayoutState state(aPresContext);
   MarkDirtyChildren(state);
-  nsCOMPtr<nsIPresShell> shell;
-  aPresContext->GetShell(getter_AddRefs(shell));
   shell->FlushPendingNotifications(PR_FALSE);
 }
 
