@@ -83,9 +83,8 @@
 #include "nsUnicharUtils.h"
 #include "nsIStringBundle.h"
 #include "nsGUIEvent.h"
-#include "nsIEventListenerManager.h"
 #include "nsIEventStateManager.h"
-
+#include "nsITimerInternal.h"
 #define NS_MENU_POPUP_LIST_INDEX   0
 
 #ifdef XP_PC
@@ -171,8 +170,7 @@ nsMenuFrame::SetParent(const nsIFrame* aParent)
   nsIFrame* currFrame = (nsIFrame*)aParent;
   while (!mMenuParent && currFrame) {
     // Set our menu parent.
-    nsCOMPtr<nsIMenuParent> menuparent(do_QueryInterface(currFrame));
-    mMenuParent = menuparent.get();
+    CallQueryInterface(currFrame, &mMenuParent);
 
     currFrame->GetParent(&currFrame);
   }
@@ -194,8 +192,7 @@ nsMenuFrame::Init(nsIPresContext*  aPresContext,
   nsIFrame* currFrame = aParent;
   while (!mMenuParent && currFrame) {
     // Set our menu parent.
-    nsCOMPtr<nsIMenuParent> menuparent(do_QueryInterface(currFrame));
-    mMenuParent = menuparent.get();
+    CallQueryInterface(currFrame, &mMenuParent);
 
     currFrame->GetParent(&currFrame);
   }
@@ -290,7 +287,8 @@ nsMenuFrame::SetInitialChildList(nsIPresContext* aPresContext,
     // the popup frame list.
     nsIFrame* frame = frames.FirstChild();
     while (frame) {
-      nsCOMPtr<nsIMenuParent> menuPar(do_QueryInterface(frame));
+      nsIMenuParent *menuPar;
+      CallQueryInterface(frame, &menuPar);
       if (menuPar) {
         // Remove this frame from the list and place it in the other list.
         frames.RemoveFrame(frame);
@@ -358,7 +356,6 @@ nsMenuFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                               nsFramePaintLayer aWhichLayer,    
                               nsIFrame**     aFrame)
 {
-
   if ((aWhichLayer != NS_FRAME_PAINT_LAYER_FOREGROUND))
     return NS_ERROR_FAILURE;
 
@@ -460,28 +457,12 @@ nsMenuFrame::HandleEvent(nsIPresContext* aPresContext,
     mMenuParent->GetIsContextMenu(isContextMenu);
     if ( isContextMenu ) {
       *aEventStatus = nsEventStatus_eConsumeNoDefault;
-      Execute();
+      Execute(aEvent);
     }
   }
   else if (aEvent->message == NS_MOUSE_LEFT_BUTTON_UP && !IsMenu() && mMenuParent && !IsDisabled()) {
-    // First, flip "checked" state if we're a checkbox menu, or
-    // an un-checked radio menu
-    if (mType == eMenuType_Checkbox ||
-        (mType == eMenuType_Radio && !mChecked)) {
-      if (mChecked) {
-        mContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::checked,
-                            PR_TRUE);
-      }
-      else {
-        mContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::checked, NS_LITERAL_STRING("true"),
-                          PR_TRUE);
-      }
-        
-      /* the AttributeChanged code will update all the internal state */
-    }
-
     // Execute the execute event handler.
-    Execute();
+    Execute(aEvent);
   }
   else if (aEvent->message == NS_MOUSE_EXIT_SYNTH) {
     // Kill our timer if one is active.
@@ -535,7 +516,12 @@ nsMenuFrame::HandleEvent(nsIPresContext* aPresContext,
 
       // We're a menu, we're built, we're closed, and no timer has been kicked off.
       mOpenTimer = do_CreateInstance("@mozilla.org/timer;1");
-      mOpenTimer->Init(this, menuDelay, NS_PRIORITY_HIGHEST);
+
+      nsCOMPtr<nsITimerInternal> ti = do_QueryInterface(mOpenTimer);
+      ti->SetIdle(PR_FALSE);
+
+      mOpenTimer->InitWithCallback(this, menuDelay, nsITimer::TYPE_ONE_SHOT);
+
     }
   }
   
@@ -555,24 +541,6 @@ nsMenuFrame::ToggleMenuState()
   return NS_OK;
 }
 
-void
-nsMenuFrame::FireMenuDOMEvent(const nsAString& aDOMEventName)
-{
-  // Fire a DOM event for the title change.
-  nsCOMPtr<nsIDOMEvent> event;
-  nsCOMPtr<nsIEventListenerManager> manager;
-  mContent->GetListenerManager(getter_AddRefs(manager));
-  if (manager &&
-      NS_SUCCEEDED(manager->CreateEvent(mPresContext, nsnull, NS_LITERAL_STRING("Events"), getter_AddRefs(event)))) {
-    event->InitEvent(aDOMEventName, PR_TRUE, PR_TRUE);
-    PRBool noDefault;
-    nsCOMPtr<nsIEventStateManager> esm;
-    mPresContext->GetEventStateManager(getter_AddRefs(esm));
-    if (esm)
-      esm->DispatchNewEvent(mContent, event, &noDefault);
-  }
-}
-
 NS_IMETHODIMP
 nsMenuFrame::SelectMenu(PRBool aActivateFlag)
 {
@@ -594,7 +562,7 @@ nsMenuFrame::SelectMenu(PRBool aActivateFlag)
     domEventToFire.Assign(NS_LITERAL_STRING("DOMMenuItemInactive"));
   }
 
-  FireMenuDOMEvent(domEventToFire);
+  FireDOMEvent(mPresContext, domEventToFire);
   return NS_OK;
 }
 
@@ -742,6 +710,9 @@ nsMenuFrame::AttributeChanged(nsIPresContext* aPresContext,
 NS_IMETHODIMP
 nsMenuFrame::OpenMenu(PRBool aActivateFlag)
 {
+  if (!mContent)
+    return NS_OK;
+
   nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(mContent));
   if (aActivateFlag) {
     // Now that the menu is opened, we should have a menupopup child built.
@@ -865,7 +836,8 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
 
       ActivateMenu(PR_TRUE);
 
-      nsCOMPtr<nsIMenuParent> childPopup(do_QueryInterface(frame));
+      nsIMenuParent *childPopup = nsnull;
+      CallQueryInterface(frame, &childPopup);
       UpdateDismissalListener(childPopup);
 
       OnCreated();
@@ -907,6 +879,18 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
         mMenuParent->RemoveKeyboardNavigator();
       else if (!mMenuParent)
         menuPopup->RemoveKeyboardNavigator();
+
+      // XXX, bug 137033, In Windows, if mouse is outside the window when the menupopup closes, no
+      // mouse_enter/mouse_exit event will be fired to clear current hover state, we should clear it manually.
+      // This code may not the best solution, but we can leave it here untill we find the better approach.
+      nsCOMPtr<nsIContent> menuPopupContent;
+      menuPopup->GetContent(getter_AddRefs(menuPopupContent));
+      nsCOMPtr<nsIEventStateManager> esm;
+      mPresContext->GetEventStateManager(getter_AddRefs(esm));
+      PRInt32 state;
+      esm->GetContentState(menuPopupContent, state);
+      if (state & NS_EVENT_STATE_HOVER)
+        esm->SetContentState(nsnull, NS_EVENT_STATE_HOVER);
     }
 
     // activate false will also set the mMenuOpen to false.
@@ -1232,12 +1216,12 @@ nsMenuFrame::ShortcutNavigation(nsIDOMKeyEvent* aKeyEvent, PRBool& aHandledFlag)
 }
 
 NS_IMETHODIMP
-nsMenuFrame::KeyboardNavigation(PRUint32 aDirection, PRBool& aHandledFlag)
+nsMenuFrame::KeyboardNavigation(PRUint32 aKeyCode, PRBool& aHandledFlag)
 {
   nsIFrame* frame = mPopupFrames.FirstChild();
   if (frame) {
     nsMenuPopupFrame* popup = (nsMenuPopupFrame*)frame;
-    popup->KeyboardNavigation(aDirection, aHandledFlag);
+    popup->KeyboardNavigation(aKeyCode, aHandledFlag);
   }
 
   return NS_OK;
@@ -1280,7 +1264,7 @@ nsMenuFrame::Enter()
   if (!mMenuOpen) {
     // The enter key press applies to us.
     if (!IsMenu() && mMenuParent)
-      Execute();          // Execute our event handler
+      Execute(0);          // Execute our event handler
     else {
       OpenMenu(PR_TRUE);
       SelectFirstItem();
@@ -1318,7 +1302,7 @@ nsMenuFrame::IsMenu()
   return mIsMenu;
 }
 
-NS_IMETHODIMP_(void)
+NS_IMETHODIMP
 nsMenuFrame::Notify(nsITimer* aTimer)
 {
   // Our timer has fired.
@@ -1338,6 +1322,7 @@ nsMenuFrame::Notify(nsITimer* aTimer)
   }
   
   mOpenTimer = nsnull;
+  return NS_OK;
 }
 
 PRBool 
@@ -1623,8 +1608,25 @@ nsMenuFrame::BuildAcceleratorText()
 }
 
 void
-nsMenuFrame::Execute()
+nsMenuFrame::Execute(nsGUIEvent *aEvent)
 {
+  // flip "checked" state if we're a checkbox menu, or an un-checked radio menu
+  if (mType == eMenuType_Checkbox || (mType == eMenuType_Radio && !mChecked)) {
+    nsAutoString value;
+    mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::autocheck, value);
+    if (!value.Equals(NS_LITERAL_STRING("false"))) {
+      if (mChecked) {
+        mContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::checked,
+                            PR_TRUE);
+      }
+      else {
+        mContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::checked, NS_LITERAL_STRING("true"),
+                          PR_TRUE);
+      }        
+      /* the AttributeChanged code will update all the internal state */
+    }
+  }
+
   // Temporarily disable rollup events on this menu.  This is
   // to suppress this menu getting removed in the case where
   // the oncommand handler opens a dialog, etc.
@@ -1646,10 +1648,20 @@ nsMenuFrame::Execute()
   nsMouseEvent event;
   event.eventStructType = NS_EVENT;
   event.message = NS_XUL_COMMAND;
-  event.isShift = PR_FALSE;
-  event.isControl = PR_FALSE;
-  event.isAlt = PR_FALSE;
-  event.isMeta = PR_FALSE;
+  if (aEvent && (aEvent->eventStructType == NS_MOUSE_EVENT ||
+                 aEvent->eventStructType == NS_KEY_EVENT ||
+                 aEvent->eventStructType == NS_ACCESSIBLE_EVENT)) {
+
+    event.isShift = NS_STATIC_CAST(nsInputEvent *, aEvent)->isShift;
+    event.isControl = NS_STATIC_CAST(nsInputEvent *, aEvent)->isControl;
+    event.isAlt = NS_STATIC_CAST(nsInputEvent *, aEvent)->isAlt;
+    event.isMeta = NS_STATIC_CAST(nsInputEvent *, aEvent)->isMeta;
+  } else {
+    event.isShift = PR_FALSE;
+    event.isControl = PR_FALSE;
+    event.isAlt = PR_FALSE;
+    event.isMeta = PR_FALSE;
+  }
   event.clickCount = 0;
   event.widget = nsnull;
   // The order of the nsIViewManager and nsIPresShell COM pointers is
@@ -1935,9 +1947,10 @@ nsMenuFrame::InsertFrames(nsIPresContext* aPresContext,
   nsCOMPtr<nsIAtom> tag;
   nsresult          rv;
 
-  nsCOMPtr<nsIMenuParent> menuPar(do_QueryInterface(aFrameList));
-  if (menuPar) {
-    nsCOMPtr<nsIBox> menupopup(do_QueryInterface(aFrameList));
+  nsIMenuParent *menuPar;
+  if (aFrameList && NS_SUCCEEDED(CallQueryInterface(aFrameList, &menuPar))) {
+    nsIBox *menupopup;
+    CallQueryInterface(aFrameList, &menupopup);
     NS_ASSERTION(menupopup,"Popup is not a box!!!");
     menupopup->SetParentBox(this);
     mPopupFrames.InsertFrames(nsnull, nsnull, aFrameList);
@@ -1964,9 +1977,10 @@ nsMenuFrame::AppendFrames(nsIPresContext* aPresContext,
   nsCOMPtr<nsIAtom> tag;
   nsresult          rv;
 
-  nsCOMPtr<nsIMenuParent> menuPar(do_QueryInterface(aFrameList));
-  if (menuPar) {
-    nsCOMPtr<nsIBox> menupopup(do_QueryInterface(aFrameList));
+  nsIMenuParent *menuPar;
+  if (aFrameList && NS_SUCCEEDED(CallQueryInterface(aFrameList, &menuPar))) {
+    nsIBox *menupopup;
+    CallQueryInterface(aFrameList, &menupopup);
     NS_ASSERTION(menupopup,"Popup is not a box!!!");
     menupopup->SetParentBox(this);
 
@@ -2141,7 +2155,8 @@ nsMenuFrame::GetBoxInfo(nsIPresContext* aPresContext, const nsHTMLReflowState& a
         frame = mPopupFrames.FirstChild();
       }
       
-      nsCOMPtr<nsIBox> box(do_QueryInterface(frame));
+      nsIBox *box;
+      CallQueryInterface(frame, &box);
       nsCalculatedBoxInfo childInfo(frame);
       box->GetBoxInfo(aPresContext, aReflowState, childInfo);
       GetRedefinedMinPrefMax(aPresContext, this, childInfo);
