@@ -176,8 +176,7 @@ nsMenuPopupFrame::Init(nsIPresContext*  aPresContext,
   // lookup if we're allowed to overlap the OS bar (menubar/taskbar) from the
   // look&feel object
   nsCOMPtr<nsILookAndFeel> lookAndFeel;
-  nsComponentManager::CreateInstance(kLookAndFeelCID, nsnull, NS_GET_IID(nsILookAndFeel), 
-                                      getter_AddRefs(lookAndFeel));
+  aPresContext->GetLookAndFeel(getter_AddRefs(lookAndFeel));
   if ( lookAndFeel ) {
     PRBool tempBool;
     lookAndFeel->GetMetric(nsILookAndFeel::eMetric_MenusCanOverlapOSBar, tempBool);
@@ -626,8 +625,12 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
         shell->GetPrimaryFrameFor(targetAsContent, &targetFrame);
         nsIView* parentView = nsnull;
         if (targetFrame) {
-          GetRootViewForPopup(mPresContext, targetFrame, &parentView);
-          GetWidgetForView(parentView, *getter_AddRefs(targetDocumentWidget));
+          nsCOMPtr<nsIPresContext> targetContext;
+          shell->GetPresContext(getter_AddRefs(targetContext));
+          if (targetContext) {
+            GetRootViewForPopup(targetContext, targetFrame, &parentView);
+            GetWidgetForView(parentView, *getter_AddRefs(targetDocumentWidget));
+          }
         }
         if (!targetDocumentWidget) {
           // We aren't inside a popup. This means we should use the root view's
@@ -1423,6 +1426,54 @@ NS_IMETHODIMP nsMenuPopupFrame::GetCurrentMenuItem(nsIMenuFrame** aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMenuPopupFrame::ConsumeOutsideClicks(PRBool& aConsumeOutsideClicks)
+{
+  /*
+   * When this popup is open, should clicks outside of it be consumed?
+   * Return PR_TRUE if the popup hould rollup on an outside click, 
+   * but consume that click so it can't be used for anything else.
+   * Return PR_FALSE to allow clicks outside the popup to activate content 
+   * even when the popup is open.
+   * ---------------------------------------------------------------------
+   * 
+   * Should clicks outside of a popup be eaten?
+   *
+   *       Menus     Autocomplete     Comboboxes
+   * Mac     Eat           No              Eat
+   * Win     No            No              Eat     
+   * Unix    Eat           No              Eat
+   *
+   */
+
+  aConsumeOutsideClicks = PR_TRUE;
+
+  nsCOMPtr<nsIContent> parentContent;
+  mContent->GetParent(*getter_AddRefs(parentContent));
+
+  if (parentContent) {
+    nsCOMPtr<nsIAtom> parentTag;
+    parentContent->GetTag(*getter_AddRefs(parentTag));
+    if (parentTag == nsXULAtoms::menulist)
+      return NS_OK;  // Consume outside clicks for combo boxes on all platforms
+    if (parentTag == nsXULAtoms::menu) {
+#ifdef XP_WIN
+      // Don't consume outside clicks for menus in Windows
+      aConsumeOutsideClicks = PR_FALSE;
+#endif
+      return NS_OK;
+    }
+    if (parentTag == nsXULAtoms::textbox) {
+      // Don't consume outside clicks for autocomplete widget
+      nsAutoString typeString;
+      parentContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::type, typeString);
+      if (typeString.EqualsIgnoreCase("autocomplete"))
+        aConsumeOutsideClicks = PR_FALSE;
+    }
+  }
+
+  return NS_OK;
+}
+
 nsIScrollableView* nsMenuPopupFrame::GetScrollableView(nsIFrame* aStart)
 {
   if ( ! aStart )
@@ -2093,7 +2144,7 @@ nsMenuPopupFrame::AttributeChanged(nsIPresContext* aPresContext,
   if (aAttribute == nsXULAtoms::left || aAttribute == nsXULAtoms::top)
     MoveToAttributePosition();
   
-  return NS_OK;
+  return rv;
 }
 
 void 
