@@ -70,7 +70,6 @@
 #include "nsIXMLContent.h"
 #include "nsXULAtoms.h"
 #include "nsHTMLAtoms.h"
-#include "nsINameSpaceManager.h"
 #include "nsIScrollableView.h"
 #include "nsRepeatService.h"
 #include "nsBoxLayoutState.h"
@@ -79,7 +78,8 @@
 #include "nsIServiceManager.h"
 #include "nsGUIEvent.h"
 
-#define DEBUG_SLIDER PR_FALSE
+// Turn this on if you want to debug slider frames.
+#undef DEBUG_SLIDER
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
@@ -448,11 +448,11 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
   SyncLayout(aState);
 
-  if (DEBUG_SLIDER) {
-     PRInt32 c = GetCurrentPosition(scrollbar);
-     PRInt32 m = GetMaxPosition(scrollbar);
-     printf("Current=%d, max=%d\n",c,m);
-  }
+#ifdef DEBUG_SLIDER
+  PRInt32 c = GetCurrentPosition(scrollbar);
+  PRInt32 m = GetMaxPosition(scrollbar);
+  printf("Current=%d, max=%d\n", c, m);
+#endif
 
   // redraw only if thumb changed size.
   if (oldThumbRect != thumbRect)
@@ -888,8 +888,9 @@ nsSliderFrame::SetCurrentPosition(nsIContent* scrollbar, nsIFrame* aThumbFrame,
   // set the new position
   scrollbar->SetAttr(kNameSpaceID_None, nsXULAtoms::curpos, NS_ConvertASCIItoUCS2(ch), PR_TRUE);
 
-  if (DEBUG_SLIDER)
-     printf("Current Pos=%s\n",ch);
+#ifdef DEBUG_SLIDER
+  printf("Current Pos=%s\n",ch);
+#endif
 
 }
 
@@ -1152,26 +1153,27 @@ nsSliderFrame::RemoveListener()
 
 NS_IMETHODIMP
 nsSliderFrame::HandlePress(nsIPresContext* aPresContext,
-                     nsGUIEvent*     aEvent,
-                     nsEventStatus*  aEventStatus)
+                           nsGUIEvent*     aEvent,
+                           nsEventStatus*  aEventStatus)
 {
-  PRBool isHorizontal = IsHorizontal();
-
   nsIFrame* thumbFrame = mFrames.FirstChild();
+  if (!thumbFrame) // display:none?
+    return NS_OK;
+  
   nsRect thumbRect;
   thumbFrame->GetRect(thumbRect);
+  
+  nscoord change = 1;
+  if (IsHorizontal() ? aEvent->point.x < thumbRect.x 
+                     : aEvent->point.y < thumbRect.y)
+    change = -1;
 
-    nscoord change = 1;
-
-    if ((isHorizontal && aEvent->point.x < thumbRect.x) || (!isHorizontal && aEvent->point.y < thumbRect.y))
-        change = -1;
-
-    mChange = change;
-    mClickPoint = aEvent->point;
-    PageUpDown(thumbFrame, change);
-
-    nsRepeatService::GetInstance()->Start(mMediator);
-
+  mChange = change;
+  mClickPoint = aEvent->point;
+  PageUpDown(thumbFrame, change);
+  
+  nsRepeatService::GetInstance()->Start(mMediator);
+  
   return NS_OK;
 }
 
